#include <linux/ctype.h>
#include <linux/string.h>
#include <linux/stddef.h>

#include <common.h>
#include <parse_h.h>
#include <asm/arch/bootparam.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>
#include <exports.h>
#include <factorylib_ro.h>
#include <factorylib.h>
#include <platform_lib/board/pcb.h>
#include <drivers/otp/rtk_otp_sw_crc.h>
#include <gpt_parse.h>

bootparam_t bootparam;
char oem_key[OEM_KEY_MAX_SIZE+1];

#define DEBUG(msg,arg...)   do { printf("[%s %3d]:", __FILE__, __LINE__); printf(msg,##arg); } while(0)
//#define DEBUG(msg,arg...)

#define BOOT_PARAM_FILE_NAME	FACTORY_HEADER_FILE_NAME"bin_panel/000BootParam.h"
#define BOOT_PART_FILE_NAME		FACTORY_HEADER_FILE_NAME"layout.txt"
#define BOOT_OEMKEY_FILE_NAME	FACTORY_HEADER_FILE_NAME"oem.key"
#define BOOT_PARAM_SNAP_FILE_NAME FACTORY_HEADER_FILE_NAME"snap.txt"
extern int boot_power_on_mode;
extern uchar boot_ac_on;
extern uchar boot_music_enable;
extern uchar boot_logo_enable;
extern uint custom_video1_leng;
extern uint custom_video2_leng;
extern uint custom_audio1_leng;
extern uint custom_audio2_leng;
extern int prev_power_on_status;
extern uint  custom_audio1_length ;
extern uint  custom_video1_length ;
extern uint  custom_audio2_length ;
extern uint  custom_video2_length ;

extern uint boot_part;
#ifdef CONFIG_SYS_RTK_SPI_FLASH	// from layout.txt
extern layout_fw_desc KernelEntry;
extern layout_fw_desc AudioFWEntry;
extern layout_fw_desc VideoFWEntry;
extern layout_fw_desc VideoFWEntry2;
extern int layout_part_no;
extern layout_part_desc layout_part_info[8];
#endif //CONFIG_SYS_RTK_SPI_FLASH
#ifdef CONFIG_SYS_SNAPSHOT
extern layout_part_desc std_part_info;
#endif
#if CONFIG_ANDROID_RECOVERY
extern layout_part_desc misc_part_info;
#endif

#ifdef Config_Boot_ATF_TRUE
extern char share_cmdline[SHARE_CMDLINE_SIZE];
#endif

typedef struct
{
    unsigned int magic_num;
    unsigned int scaler_resource_offset;
    unsigned int throttle_resource_offset;
    unsigned int cpu_resource_offset;
    unsigned int check_sum;
}RESOURCE_TABLE;

RESOURCE_TABLE *resource_table = (RESOURCE_TABLE *)POWER_ON_RESOURCE_ADDR;

/************************************************************************
**
** get boot info in factory area of flash
**
*************************************************************************/
void get_bootparam(void)
{
	char *dst_addr = NULL;
	char *retAddr = NULL, *firstpos = NULL, *lastpos = NULL;
	int dst_length;
	uint retVal;

	printf("%s:\n", __FUNCTION__);

//	display_evaluate_time("get_bootparam-0");

	memset(&bootparam, 0, sizeof(bootparam));

	if (factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length)) {
		printf("------------can't find %s\n", BOOT_PARAM_FILE_NAME);
		bootparam.prev_power_state = POWER_ON;
		bootparam.boot_music_enable = 0;
		bootparam.boot_logo_enable = 1;
		bootparam.backlight_value = 50;
		bootparam.local_upgrade_en = 0;
		bootparam.panel_ssc_enable = BOOTPARAM_PANEL_SSC_UNKNOWN;
		bootparam.panel_ssc_step = 0;
		bootparam.panel_ssc_span = 0;
	}
	else {
		printf("------------%s found\n", BOOT_PARAM_FILE_NAME);

		//irda
		{
#if defined(Config_TV030_JP_IRDA_TRUE)
			memcpy(bootparam.ir_default_remoter[0],"aeha(AEHA)",10);
			DEBUG("can't find irda default remoter, use [aeha(AEHA)]\n");
#elif defined(Config_Board_TV002_V001)
			memcpy(bootparam.ir_default_remoter[0],"sony(RAW_SONY)",14);
			DEBUG("can't find irda default remoter, use [sony(RAW_SONY)]\n");
#else

			if (parse_h("IRDA_DEFAULT_REMOTER", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0)
			{
				memcpy(bootparam.ir_default_remoter[0],retAddr+1, retVal-2);
				printf("IRDA_DEFAULT_REMOTER=%s\n",bootparam.ir_default_remoter[0]);
			}
			else
			{
#if defined(Config_Board_TV030)
				memcpy(bootparam.ir_default_remoter[0],"rca(RCA)",8);
				DEBUG("can't find irda default remoter, use [rca(RCA)]\n");
#else
                                memcpy(bootparam.ir_default_remoter[0],"hikeen(NEC)",11);
				DEBUG("can't find irda default remoter, use [hikeen(NEC)]\n");
#endif
			}
#endif
			if (parse_h("IRDA_POWER_UP_KEY", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
				memcpy(bootparam.ir_power_up_key[0],retAddr+1, retVal-2);
				printf("IRDA_POWER_UP_KEY=%s\n",bootparam.ir_power_up_key[0]);
			}
			else {
				memcpy(bootparam.ir_power_up_key[0],"74,221,189,16e,2f5,2f6",22);
				DEBUG("can't find power up key, use 0x74 0x221 0x189 0x16e 0x2f5 0x2f6\n");
			}
			if (parse_h("IRDA_FILTER", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0)
			{
				memcpy(bootparam.irda_filter,retAddr+1, retVal-2);
				printf("IRDA_FILTER=%s\n",bootparam.irda_filter);
			}
	 		if (parse_h("IRDA_DEFAULT_REMOTER1", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
				memcpy(bootparam.ir_default_remoter[1],retAddr+1, retVal-2);
				printf("IRDA_DEFAULT_REMOTER1=%s\n",bootparam.ir_default_remoter[1]);
			}
			else {
				memcpy(bootparam.ir_default_remoter[1],"DefaultSetting(none)",18);
				DEBUG("can't find irda default remoter, use [Clear setting]\n");
			}

			if (parse_h("IRDA_POWER_UP_KEY1", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0)
			{
				memcpy(bootparam.ir_power_up_key[1],retAddr+1, retVal-2);
				printf("IRDA_POWER_UP_KEY1=%s\n",bootparam.ir_power_up_key[1]);
			}
			else {
				memcpy(bootparam.ir_power_up_key[1],"none",4);
				DEBUG("can't find power up key1, use none\n");
			}
		}


		// Tuner selection
		// remove "", e.g. "DEFUALT" => DEFAULT
		char tp_str[50];
		int tp_index;
		memset(tp_str, 0, 50);
		for (tp_index = 0; tp_index < TP_MAX_NUM; tp_index++) {
			snprintf(tp_str, 50, "BOOT_TP%d_FE", tp_index);
			if (parse_h(tp_str, dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
				memcpy(bootparam.tp_fe_setting[tp_index], retAddr+1, retVal-2);
				printf("%s=%s\n", tp_str, bootparam.tp_fe_setting[tp_index]);
			}
			else {
				printf("can't find tp%d fe setting\n", tp_index);
			}
		}

		if (parse_h("BOOT_POWER_ON_MODE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.prev_power_state = retVal;
			printf("BOOT_POWER_ON_MODE=%d\n", retVal);
		}
		else {
			bootparam.prev_power_state = POWER_ON;
			DEBUG("can't find power state\n");
		}

	    if (parse_h("BOOT_LOGO_ENABLE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.boot_logo_enable = 1;//retVal;
			printf("BOOT_LOGO_ENABLE=%d\n", retVal);
		}
		else {
			bootparam.boot_logo_enable = 1;
			DEBUG("can't find boot_logo_enable\n");
		}
	    if(!bootparam.boot_logo_enable)
	    {
		resource_table = (RESOURCE_TABLE *)POWER_ON_RESOURCE_LOGO_OFF_ADDR;
	    }
	    if (parse_h("CUSTOM_VIDEO1_LENG", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.custom_video1_leng = retVal;
			printf("CUSTOM_VIDEO1_LENG=%d\n", retVal);
		}
		else {
			DEBUG("can't find custom_video1_leng\n");
		}

	    if (parse_h("CUSTOM_VIDEO2_LENG", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.custom_video2_leng = retVal;
			printf("CUSTOM_VIDEO2_LENG=%d\n", retVal);
		}
		else {
			DEBUG("can't find custom_video2_leng\n");
		}

		if (parse_h("BOOT_MUSIC_ENABLE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.boot_music_enable = retVal;
			printf("BOOT_MUSIC_ENABLE=%d\n", retVal);
		}
		else {
			bootparam.boot_music_enable = 0;
			DEBUG("can't find music enable state\n");
		}

		if (parse_h("CUSTOM_AUDIO1_LENG", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.custom_audio1_leng = retVal;
			printf("CUSTOM_AUDIO1_LENG=%d\n", retVal);
		}
		else {
			DEBUG("can't find custom_audio1_leng\n");
		}

		if (parse_h("CUSTOM_AUDIO2_LENG", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.custom_audio2_leng = retVal;
			printf("CUSTOM_AUDIO2_LENG=%d\n", retVal);
		}
		else {
			DEBUG("can't find custom_audio2_leng\n");
		}

#if 0 // BUG, wait for jerry fix
		if (parse_h("BOOT_MAC_ADDRESS", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			memcpy(bootparam.ethaddr,retAddr, dst_length);
			printf("BOOT_MAC_ADDRESS=%s\n", retAddr);
		}
		else
		{
			char cmdname[]={" 10:20:30:40:50:60 \n"};
			memcpy(bootparam.ethaddr,cmdname,sizeof(cmdname));
		}
#endif

		if (parse_h("LOCAL_UPGRADE_EN", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.local_upgrade_en = retVal;
			printf("LOCAL_UPGRADE_EN=%d\n", retVal);
		}
		else {
			bootparam.local_upgrade_en = 0;
			DEBUG("can't find local_upgrade_en\n");
		}

		// Panel SSC features
		if (parse_h("PANEL_SSC_ENABLE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.panel_ssc_enable = retVal;
			printf("PANEL_SSC_ENABLE=%d\n", retVal);

			// Panel SSC Percentage
			if (parse_h("PANEL_SSC_STEP", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
				bootparam.panel_ssc_step = retVal;
				printf("PANEL_SSC_STEP=%d\n", retVal);
			}
			else {
				bootparam.panel_ssc_enable = BOOTPARAM_PANEL_SSC_UNKNOWN;
				bootparam.panel_ssc_step = 0;
				DEBUG("can't find PANEL_SSC_STEP\n");
			}

			// Panel SSC Modulation
			if (parse_h("PANEL_SSC_SPAN", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
				bootparam.panel_ssc_span = retVal;
				printf("PANEL_SSC_SPAN=%d\n", retVal);
			}
			else {
				bootparam.panel_ssc_enable = BOOTPARAM_PANEL_SSC_UNKNOWN;
				bootparam.panel_ssc_span = 0;
				DEBUG("can't find PANEL_SSC_SPAN\n");
			}
		}
		else {
			bootparam.panel_ssc_enable = BOOTPARAM_PANEL_SSC_UNKNOWN;
			bootparam.panel_ssc_step = 0;
			bootparam.panel_ssc_span = 0;
			DEBUG("can't find PANEL_SSC_ENABLE\n");
		}

#ifdef ENABLE_BOOTUP_POLICY
		//0:bootup whatever AC on/off; 1:do not bootup whatever AC on/off; 2:bootup decide by previous power state
		if (parse_h("BOOTUP_POLICY", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			bootparam.bootup_policy = retVal;
			printf("BOOTUP_POLICY=%d\n", retVal);
		}
		else {
			bootparam.bootup_policy = 2; //auto by prev_status
			DEBUG("can't find bootup_policy\n");
		}
#endif

		memset(bootparam.install_file_name, 0, sizeof(bootparam.install_file_name));
		if (parse_h("IMGFILE_PATTERN", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
			firstpos = strchr(retAddr, '\"');
			if (firstpos)
				lastpos = strchr(firstpos+1, '\"');
			if (firstpos && lastpos)
				strncpy((char *)bootparam.install_file_name, firstpos+1, lastpos-firstpos-1);
			else
				strncpy((char *)bootparam.install_file_name, retAddr, retVal);
			// add by cfyeh, remove unprintable char in bootparam.install_file_name
			{
				int str_len = strlen((char *)bootparam.install_file_name);
				int i;
				for (i = str_len - 1; i > 0 ; i--) {
					if (isprint(bootparam.install_file_name[i]))
						break;
					else
						bootparam.install_file_name[i] = '\0';
				}
			}
			printf("IMGFILE_PATTERN=%s\n", bootparam.install_file_name);
#ifdef Config_Boot_ATF_TRUE
			strcat(share_cmdline, " fimg=");
			strcat(share_cmdline,(const char *) bootparam.install_file_name);
#endif
		}
      	else {
			DEBUG("can't IMGFILE_PATTERN\n");
		}

		CusApi_get_bootparam(dst_addr, dst_length); // call customized function
	}

	boot_music_enable = bootparam.boot_music_enable;
	boot_logo_enable = bootparam.boot_logo_enable;

	custom_video1_length = bootparam.custom_video1_leng;
	custom_video2_length = bootparam.custom_video2_leng;
	custom_audio1_length = bootparam.custom_audio1_leng;
	custom_audio2_length = bootparam.custom_audio2_leng;

	if(bootparam.local_upgrade_en)
		boot_mode = BOOT_RESCUE_MODE;

	CusApi_decide_power_on_mode(); // call customized function

	printf("boot_power_on_mode=%d\n", boot_power_on_mode);

//	display_evaluate_time("get_bootparam-1");
}

#define DBG_PRINTF printf
void set_bootparam_power_state(uint val)
{
	char *dst_addr;
	char *retAddr;
	int dst_length;
	uint retVal;

	printf("%s(%d)\n",__FUNCTION__,val);
//	if(&bootparam == NULL)
//		return;

	if(factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length))
		return;

	if(parse_h("BOOT_POWER_ON_MODE", dst_addr, (uint)dst_length, &retVal, &retAddr))
		return;

	if(val == retVal)
		return;

	if (val == 0)
		*retAddr = 0x30;
	else if (val == 1)
		*retAddr = 0x31;
	else
		return;

	if(factory_write(BOOT_PARAM_FILE_NAME, dst_addr, dst_length)!=0) {
		DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
	}

	if(factory_save()!=0) {
		DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
	}
}

void SetBootParamSNPART(uint val)
{
    char *dst_addr;
    char *retAddr;
    char *tmp_cmdline;
    char cmdname[]={"#define SNP_ART"};
    int dst_length;
    int len = 0;
    uint retVal;

    printf("%s(%c)\n",__FUNCTION__,val);

    if(factory_read(BOOT_PARAM_SNAP_FILE_NAME, &dst_addr, &dst_length))
    {
        len = strlen(cmdname) + 4;
        tmp_cmdline = (char *)malloc(len);
        if(!tmp_cmdline)
        {
            printf("[ERR] %s: Malloc failed\n", __func__);
        }
        else
        {
            memset(tmp_cmdline, 0, len);
            sprintf(tmp_cmdline, "%s %c\n", cmdname,val);
            dst_length = strlen(tmp_cmdline);
            if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, tmp_cmdline, dst_length)!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
            }
            free(tmp_cmdline);
            if(factory_save()!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
            }
        }
        return;
    }
    if(parse_h("SNP_ART", dst_addr, (uint)dst_length, &retVal, &retAddr))
        return;

    if(val == retVal)
        return;
    *retAddr = val;

    if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0) {
		DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
	}

	if(factory_save()!=0) {
		DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
	}
}

uint GetBootParamSNPART(void)
{
    char *dst_addr;
    char *retAddr;
    char *tmp_cmdline;
    char cmdname[]={"#define SNP_ART"};
    int dst_length;
    int len = 0;
    uint retVal;

    if(factory_read(BOOT_PARAM_SNAP_FILE_NAME, &dst_addr, &dst_length))
    {
        len = strlen(cmdname) + 4;
        tmp_cmdline = (char *)malloc(len);
        if(!tmp_cmdline)
        {
            printf("[ERR] %s: Malloc failed\n", __func__);
        }
        else
        {
            memset(tmp_cmdline, 0, len);
            sprintf(tmp_cmdline, "%s N\n", cmdname);
            dst_length = strlen(tmp_cmdline);
            if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, tmp_cmdline, dst_length)!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
            }
            free(tmp_cmdline);
            if(factory_save()!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
            }
        }
        return 0;
    }

    if(parse_h("SNP_ART", dst_addr, (uint)dst_length, &retVal, &retAddr))
    {
        printf("set SNP_ART if not define \n");
        tmp_cmdline = dst_addr;
        tmp_cmdline += dst_length;
        sprintf(tmp_cmdline, "%s N\n", cmdname);
        dst_length += (strlen(cmdname)+4);
        if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
        }
        if(factory_save()!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
        }
        return 0;
    }
    return *retAddr;
}

void SetBootparamSnpArtState(unsigned char val)
{
    char *dst_addr;
    char *retAddr;
    char *tmp_cmdline;
    char cmdname[]={"#define SNP_ART_STATE"};
    int dst_length;
    uint retVal;

    printf("%s(%c)\n",__FUNCTION__,val);

    if(factory_read(BOOT_PARAM_SNAP_FILE_NAME, &dst_addr, &dst_length))
        return;
    if(parse_h("SNP_ART_STATE", dst_addr, (uint)dst_length, &retVal, &retAddr))
    {
        tmp_cmdline = dst_addr;
        tmp_cmdline += dst_length;
        sprintf(tmp_cmdline, "%s %c\n", cmdname,val);
        dst_length += (strlen(cmdname) + 4);
        if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
        }
        if(factory_save()!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
        }
        return ;
    }

    if(val == retVal)
        return;
    *retAddr = val;
    if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0) {
		DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
	}

	if(factory_save()!=0) {
		DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
	}
}

unsigned char GetBootparamSnpArtState(void)
{
    char *dst_addr;
    char *retAddr;
    char *tmp_cmdline;
    char cmdname[]={"#define SNP_ART_STATE"};
    int dst_length;
    uint retVal;

    printf("%s\n",__FUNCTION__);

    if(factory_read(BOOT_PARAM_SNAP_FILE_NAME, &dst_addr, &dst_length))
        return 0;
    if(parse_h("SNP_ART_STATE", dst_addr, (uint)dst_length, &retVal, &retAddr))
    {
        tmp_cmdline = dst_addr;
        tmp_cmdline += dst_length;
        sprintf(tmp_cmdline, "%s r\n", cmdname);
        dst_length += (strlen(cmdname) + 4);
        if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
        }
        if(factory_save()!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
        }
        return 0;
    }
    return *retAddr;
}

uint GetBootParamSNPSupport(void)
{
    char *dst_addr;
    char *retAddr;
    char *tmp_cmdline;
    char cmdname[]={"#define SNP_SUPPORT"};
    int dst_length;
    int len = 0;
    uint retVal;

    DBG_PRINTF("%s\n",__FUNCTION__);

    if(factory_read(BOOT_PARAM_SNAP_FILE_NAME, &dst_addr, &dst_length))
    {
        len = strlen(cmdname) + 4;
        tmp_cmdline = (char *)malloc(len);
        if(!tmp_cmdline)
        {
            printf("[ERR] %s: Malloc failed\n", __func__);
        }
        else
        {
            memset(tmp_cmdline, 0, len);
#ifdef Config_RTD_STD_TRUE
            sprintf(tmp_cmdline, "%s 1\n", cmdname);
#else
            sprintf(tmp_cmdline, "%s 0\n", cmdname);
#endif
            dst_length = strlen(tmp_cmdline);
            if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, tmp_cmdline, dst_length)!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
            }
            free(tmp_cmdline);
            if(factory_save()!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
            }
        }
        return 0;
    }

    if(parse_h("SNP_SUPPORT", dst_addr, (uint)dst_length, &retVal, &retAddr))
    {
        tmp_cmdline = dst_addr;
        tmp_cmdline += dst_length;
#ifdef Config_RTD_STD_TRUE
        sprintf(tmp_cmdline, "%s 1\n", cmdname);
#else
        sprintf(tmp_cmdline, "%s 0\n", cmdname);
#endif
        dst_length += (strlen(cmdname)+4);

        if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
        }

        if(factory_save()!=0)
        {
            DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
        }
        return 0;
    }
    return retVal;
}

void SetBootParamSNPSupport(uint val)
{
    char *dst_addr;
    char *retAddr;
    char *tmp_cmdline;
    char cmdname[]={"#define SNP_SUPPORT"};
    int dst_length;
    int len = 0;
    uint retVal;

    printf("%s(%d)\n",__FUNCTION__,val);

    if(factory_read(BOOT_PARAM_SNAP_FILE_NAME, &dst_addr, &dst_length))
    {
        len = strlen(cmdname) + 4;
        tmp_cmdline = (char *)malloc(len);
        if(!tmp_cmdline)
        {
            printf("[ERR] %s: Malloc failed\n", __func__);
        }
        else
        {
            memset(tmp_cmdline, 0, len);
            if(val)
                sprintf(tmp_cmdline, "%s 1\n", cmdname);
            else
                sprintf(tmp_cmdline, "%s 0\n", cmdname);
            dst_length = strlen(tmp_cmdline);
            if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, tmp_cmdline, dst_length)!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
            }
            free(tmp_cmdline);
            if(factory_save()!=0)
            {
                DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
            }
        }
        return;
    }
    printf("get  define value %x \n",dst_length);
    if(parse_h("SNP_SUPPORT", dst_addr, (uint)dst_length, &retVal, &retAddr))
        return;
    printf("get  define value1 %x \n",dst_length);
    if(val == retVal)
        return;

    if (val == 0)
        *retAddr = 0x30;
    else if (val == 1)
        *retAddr = 0x31;
    else
        return;
    if(factory_write(BOOT_PARAM_SNAP_FILE_NAME, dst_addr, dst_length)!=0) {
		DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
	}

	if(factory_save()!=0) {
		DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
	}
}

void set_bootparam_upgrade(int val)
{
	char *dst_addr;
	char *retAddr;
	int dst_length;
	uint retVal;

	printf("%s(%d)\n",__FUNCTION__,val);
//	if(&bootparam == NULL)
//		return;

	if(factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length))
		return;

	if(parse_h("LOCAL_UPGRADE_EN", dst_addr, (uint)dst_length, &retVal, &retAddr))
		return;

	if(val == retVal)
		return;

	if (val == 0)
		*retAddr = 0x30;
	else if (val == 1)
		*retAddr = 0x31;
	else
		return;

	if(factory_write(BOOT_PARAM_FILE_NAME, dst_addr, dst_length)!=0) {
		DBG_PRINTF("[DEBUG]%s factory_write error\n",__FUNCTION__);
	}

	if(factory_save()!=0) {
		DBG_PRINTF("[DEBUG]%s factory_save error\n",__FUNCTION__);
	}
}

/************************************************************************
**
** get layout.txt in factory area of flash
**
*************************************************************************/
int get_partition_layout(layout_part_desc *layout_part, char *part_name)
{
	int ret = 0;
	gpt_entry entry; 
	unsigned long long start = 0, end = 0;
	unsigned int dev_index = 0;

	memset(&entry, 0x00, sizeof(gpt_entry));
	ret = get_gpt_entry_by_name(&entry, part_name);
	if(ret < 0){
		DBG_MSG("Fail to find %s partition!\n", part_name);
		return -1;
	}
		
	start = get_partition_starting_by_gpt_entry(entry);
	end = get_partition_ending_by_gpt_entry(entry);
	ret = get_dev_index_by_name(part_name, &dev_index);
	if(start == 0 || end == 0 || ret < 0){
		DBG_MSG("Fail to get %s partition info!\n", part_name);
		return -1;
	}
	
	layout_part->offset = start;
	layout_part->size = end - start;
	snprintf(layout_part->mount_dev, sizeof(layout_part->mount_dev), "/dev/block/mmcblk0p%d", dev_index);

	DBG_MSG("%s partition offset 0x%llx size 0x%llx mount_dev %s\r\n", \
		part_name, layout_part->offset, layout_part->size, layout_part->mount_dev);
	
	return 0;
}

void get_layout(void)
{
	char *dst_addr;
	int dst_length;
	
#if !defined(CONFIG_CUSTOMER_TV002)	
	uint retVal;
	char *retAddr;
#endif

	DBG_CKP("!\n");

#ifdef CONFIG_SYS_RTK_EMMC_FLASH
#ifdef CONFIG_SYS_SNAPSHOT
	get_partition_layout(&std_part_info, "snaps");	
#endif

#if CONFIG_ANDROID_RECOVERY
	get_partition_layout(&misc_part_info, "misc");	
#endif	
#endif

//	display_evaluate_time("get_layout-0");

	if (factory_read(BOOT_PART_FILE_NAME, &dst_addr, &dst_length))
	{
		DBG_WAR("------------can't find %s!\n", BOOT_PART_FILE_NAME);
		boot_part = 0;
	}
	else
	{
		#ifdef CONFIG_SYS_RTK_SPI_FLASH	// from layout.txt
		char part_name[64];
		#endif

#if !defined(CONFIG_CUSTOMER_TV002)
		if (parse_h("BOOTPART", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0)
		{
			boot_part = retVal;
			DBG_MSG("BOOTPART=%d\n", retVal);
		}
		else
		{
			DBG_ERR("can't find boot part!\n");
		}

		if (parse_h("TAG", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0)
		{
			boot_tag = retVal;
			DBG_MSG("TAG=%d!\n", retVal);
		}
		else
		{
			DBG_ERR("can't find Tag!\n");
		}
#else
		printf("boot_part value = %d boot_tag value = %d \r\n", boot_part, boot_tag);
#endif

		#ifdef CONFIG_SYS_RTK_SPI_FLASH	// from layout.txt
		memset(&KernelEntry, 0, sizeof(KernelEntry));
		if (parse_fw("FW_KERNEL", dst_addr, (uint)dst_length, &KernelEntry) == 0)
		{
			DBG_MSG("%s, %s, target:0x%x, offset:0x%x, size:0x%x!\n", KernelEntry.name, KernelEntry.type, KernelEntry.target, KernelEntry.offset, KernelEntry.size);
		}
		else
		{
			DBG_ERR("can't find Kernel!\n");
		}

		memset(&AudioFWEntry, 0, sizeof(AudioFWEntry));
		if (parse_fw("FW_AKERNEL", dst_addr, (uint)dst_length, &AudioFWEntry) == 0)
		{
			DBG_MSG("%s, %s, target:0x%x, offset:0x%x, size:0x%x!\n", AudioFWEntry.name, AudioFWEntry.type, AudioFWEntry.target, AudioFWEntry.offset, AudioFWEntry.size);
		}
		else
		{
			DBG_ERR("can't find Audio FW!\n");
		}

		memset(&VideoFWEntry, 0, sizeof(VideoFWEntry));
		if (parse_fw("FW_VKERNEL", dst_addr, (uint)dst_length, &VideoFWEntry) == 0)
		{
			DBG_MSG("%s, %s, target:0x%x, offset:0x%x, size:0x%x!\n", VideoFWEntry.name, VideoFWEntry.type, VideoFWEntry.target, VideoFWEntry.offset, VideoFWEntry.size);
		}
		else
		{
			DBG_ERR("can't find Video FW!\n");
		}

		memset(&VideoFWEntry2, 0, sizeof(VideoFWEntry2));
		if (parse_fw("FW_VKERNEL2", dst_addr, (uint)dst_length, &VideoFWEntry2) == 0)
		{
			DBG_MSG("%s, %s, target:0x%x, offset:0x%x, size:0x%x!\n", VideoFWEntry2.name, VideoFWEntry2.type, VideoFWEntry2.target, VideoFWEntry2.offset, VideoFWEntry2.size);
		}
		else
		{
			DBG_ERR("can't find Video FW 2!\n");
		}

		memset(&layout_part_info, 0, sizeof(layout_part_info));
		for (layout_part_no=0; layout_part_no < 8; layout_part_no++)
		{
			sprintf(part_name, "PART%d", layout_part_no);
			if (parse_part(part_name, dst_addr, (uint)dst_length, &layout_part_info[layout_part_no]) == 0)
			{
				DBG_MSG("%d:%s, offset:0x%x, size=0x%x mount_point=%s filesystem=%s partname=%s type=%s!\n", layout_part_no, part_name, layout_part_info[layout_part_no].offset, layout_part_info[layout_part_no].size, layout_part_info[layout_part_no].mount_point, layout_part_info[layout_part_no].filesystem, layout_part_info[layout_part_no].partname, layout_part_info[layout_part_no].type);
			}
			else
			{
				DBG_MSG("Fail to find %s!\n", part_name);
				break;
			}
		}

		#endif //CONFIG_SYS_RTK_SPI_FLASH
	}

//	display_evaluate_time("get_layout-1");
}

/************************************************************************
**
** get oem.key in factory area of flash
**
*************************************************************************/
void get_oemkey(void)
{
	char *dst_addr;
	int dst_length = 0;

	if (factory_read(BOOT_OEMKEY_FILE_NAME, &dst_addr, &dst_length))
	{
		DEBUG("------------can't find %s!\n", BOOT_OEMKEY_FILE_NAME);
	}
	else
	{
		memset(&oem_key, 0, sizeof(oem_key));

		if (dst_length <= OEM_KEY_MAX_SIZE)
			strncpy(oem_key, dst_addr, dst_length);
		else
			strncpy(oem_key, dst_addr, OEM_KEY_MAX_SIZE);

		printf("[%s] %d: get oem.key = %s \n", __FUNCTION__, __LINE__, oem_key);
	}

}

/***************************************************************/
#define RESOURCE_MAGIC_NUM  0x1234abcd

/*********************************************/
// parse throttle_resource.xml
#define THROTTLE_RESOURCE_FILE  FACTORY_HEADER_FILE_NAME"throttle_resource.xml"
#define THROTTLE_LINE_MAX  10
#define THROTTLE_CONTENT_MAX  8
typedef enum
{
        MODULE_NAME_KEY_E = 0,
        THROTTLE_ENABLE_KEY_E,
        THROTTLE_0_KEY_E,
        THROTTLE_1_KEY_E,
        THROTTLE_2_KEY_E,
        THROTTLE_3_KEY_E,
        THROTTLE_MAX_KEY_E,
}THROTTLE_KEY_E;

char throttle_key_string[THROTTLE_MAX_KEY_E][16] =
{
        "module_name=\"",
        "enable=\"",
        "throttle_0=\"",
        "throttle_1=\"",
        "throttle_2=\"",
        "throttle_3=\"",
};

typedef struct
{
        char module_name_content[THROTTLE_CONTENT_MAX];
        int content[THROTTLE_MAX_KEY_E-1];
}THROTTLE_CONTENT_T;

typedef struct
{
        unsigned int magic_num;
        int check_sum;
        bool valid;
        char reserve1;
        char reserve2;
        char reserve3;
        THROTTLE_CONTENT_T throttle_content[THROTTLE_LINE_MAX];
}THROTTLE_RESOURCE_T;


extern int sscanf(const char *buf, const char *fmt, ...);
int parse_platform_throttle_resource(char *src_addr, int length)
{
         char line[256];
         char line_bak[256];
         int i=0, j=0, line_num = 0;
         char * start = NULL;
         char * end = NULL;
         int module_name_index = 0;
         THROTTLE_RESOURCE_T throttle_resource;
         int *p_tmp = NULL;
         int check_sum = 0;
         unsigned int size = 0;

         memset(&throttle_resource, 0, sizeof(THROTTLE_RESOURCE_T));
         throttle_resource.magic_num = RESOURCE_MAGIC_NUM;
         throttle_resource.check_sum = 0;
         throttle_resource.valid = false;
         //printf("test: 0x%x size:0x%x", (unsigned int)throttle_resource.throttle_content, sizeof(throttle_resource.throttle_content));

         while(i < length)
        {
                memset(line, 0, sizeof(line));
                i = i + parse_getline(line,  &src_addr[i]) + 1;
                line_num++;

                 if(!(strstr(line, "<pattern") &&  strstr(line, "/>")))
                {
                        DBG_MSG("line_num(%d): skip\n", line_num);
                        continue;
                }

                // printf("%s_%d: %s\n", __FUNCTION__, __LINE__, line);

                for(j=0; j< THROTTLE_MAX_KEY_E; j++)
                {
                        start = strstr(line, throttle_key_string[j]);
                        if(NULL == start)
                        {
                                printf("line_num(%d): cannot find %s\n", line_num, throttle_key_string[j]);
                                goto error;
                        }
                        start += strlen(throttle_key_string[j]);

                        end = strstr(start, " ");
                        if(NULL == end)
                        {
                                end = strstr(start, "/>");
                                if(end == NULL)
                                {
                                        printf("line_num(%d):  %s not match\n", line_num, throttle_key_string[j]);
                                        goto error;
                                }
                        }
                        size = end -start;

                        if(size > (THROTTLE_CONTENT_MAX-1))
                        {
                                printf("line_num(%d):   %s content is beyond %d, so error\n", line_num, throttle_key_string[j], THROTTLE_CONTENT_MAX);
                                goto error;
                        }
                        if(j == MODULE_NAME_KEY_E)
                        {
                                strncpy(throttle_resource.throttle_content[module_name_index].module_name_content, start, size-1);
                        }
                        else
                        {
                                if((*start >='0') && (*start <= '9'))
                                {
                                        sscanf(start, "%d%s", &(throttle_resource.throttle_content[module_name_index].content[j-1]), line_bak);
                                }
                                else
                                {
                                        printf("line_num(%d):   %s content: the first char is not number, so error\n", line_num, throttle_key_string[j]);
                                        goto error;
                                }
                        }
                }

                module_name_index++;
                if(module_name_index >= THROTTLE_LINE_MAX)
                {
                        printf("line_num(%d):  beyond the THROTTLE_LINE_MAX %d, so error\n", line_num, THROTTLE_LINE_MAX);
                        goto error;
                }
        }

        throttle_resource.magic_num = RESOURCE_MAGIC_NUM;
        throttle_resource.valid = true;
        throttle_resource.check_sum = 0;

        p_tmp = (int *)throttle_resource.throttle_content;
        for(i=0; i<sizeof(throttle_resource.throttle_content)/4; i++)
        {
                check_sum += p_tmp[i];
        }
        throttle_resource.check_sum =   check_sum;
         memcpy((void*)((unsigned int)resource_table+resource_table->throttle_resource_offset), (void*)&throttle_resource, sizeof(THROTTLE_RESOURCE_T));

#if 0
        for(i=0; i<THROTTLE_LINE_MAX; i++)
        {
                if(!strlen(throttle_resource.throttle_content[i].module_name_content))
                        continue;

                printf("%s ", throttle_resource.throttle_content[i].module_name_content);

                for(j=THROTTLE_ENABLE_KEY_E; j<THROTTLE_MAX_KEY_E; j++)
                        printf("%d ", throttle_resource.throttle_content[i].content[j-1]);
                printf("\n");
        }
#endif

        return 0;
 error:
        throttle_resource.magic_num = RESOURCE_MAGIC_NUM;
        throttle_resource.valid = false;
        throttle_resource.check_sum = 0;
        memcpy((void*)((unsigned int)resource_table+resource_table->throttle_resource_offset), (void*)&throttle_resource, sizeof(THROTTLE_RESOURCE_T));
        return -1;
}

/*********************************************/
// parse cpu_resource.xml
#define CPU_RESOURCE_FILE  FACTORY_HEADER_FILE_NAME"cpu_resource.xml"
#define CPU_LINE_MAX  10
#define CPU_CONTENT_MAX  8
typedef enum
{
        CPU_NAME_KEY_E = 0,
        CPU_ENABLE_KEY_E,
        CPU_0_KEY_E,
        CPU_1_KEY_E,
        CPU_2_KEY_E,
        CPU_3_KEY_E,
        CPU_4_KEY_E,
        CPU_MAX_KEY_E,
}CPU_KEY_E;

char cpu_key_string[CPU_MAX_KEY_E][16] =
{
        "module_name=\"",
        "enable=\"",
        "freq_0=\"",
        "freq_1=\"",
        "freq_2=\"",
        "freq_3=\"",
        "freq_4=\"",
};

typedef struct
{
        char module_name_content[CPU_CONTENT_MAX];
        int content[CPU_MAX_KEY_E-1];
}CPU_CONTENT_T;

typedef struct
{
        unsigned int magic_num;
        int check_sum;
        bool valid;
        char reserve1;
        char reserve2;
        char reserve3;
        CPU_CONTENT_T cpu_content[CPU_LINE_MAX];
}CPU_RESOURCE_T;

int parse_platform_cpu_resource(char *src_addr, int length)
{
         char line[256];
         char line_bak[256];
         int i=0, j=0, line_num = 0;
         char * start = NULL;
         char * end = NULL;
         int module_name_index = 0;
         CPU_RESOURCE_T cpu_resource;
         int *p_tmp = NULL;
         int check_sum = 0;
         unsigned int size = 0;

         memset(&cpu_resource, 0, sizeof(CPU_RESOURCE_T));
         cpu_resource.magic_num = RESOURCE_MAGIC_NUM;
         cpu_resource.check_sum = 0;
         cpu_resource.valid = false;
         //printf("test: 0x%x size:0x%x", (unsigned int)cpu_resource.cpu_content, sizeof(cpu_resource.cpu_content));

         while(i < length)
        {
                memset(line, 0, sizeof(line));
                i = i + parse_getline(line,  &src_addr[i]) + 1;
                line_num++;

                 if(!(strstr(line, "<pattern") &&  strstr(line, "/>")))
                {
                        DBG_MSG("line_num(%d): skip\n", line_num);
                        continue;
                }

                // printf("%s_%d: %s\n", __FUNCTION__, __LINE__, line);

                for(j=0; j< CPU_MAX_KEY_E; j++)
                {
                        start = strstr(line, cpu_key_string[j]);
                        if(NULL == start)
                        {
                                printf("line_num(%d): cannot find %s\n", line_num, cpu_key_string[j]);
                                goto error;
                        }
                        start += strlen(cpu_key_string[j]);

                        end = strstr(start, " ");
                        if(NULL == end)
                        {
                                end = strstr(start, "/>");
                                if(end == NULL)
                                {
                                        printf("line_num(%d):  %s not match\n", line_num, cpu_key_string[j]);
                                        goto error;
                                }
                        }
                        size = end -start;

                        if(size > (CPU_CONTENT_MAX-1))
                        {
                                printf("line_num(%d):   %s content is beyond %d, so error\n", line_num, cpu_key_string[j], CPU_CONTENT_MAX);
                                goto error;
                        }
                        if(j == CPU_NAME_KEY_E)
                        {
                                strncpy(cpu_resource.cpu_content[module_name_index].module_name_content, start, size-1);
                        }
                        else
                        {
                                if((*start >='0') && (*start <= '9'))
                                {
                                        sscanf(start, "%d%s", &(cpu_resource.cpu_content[module_name_index].content[j-1]), line_bak);
                                }
                                else
                                {
                                        printf("line_num(%d):   %s content: the first char is not number, so error\n", line_num, cpu_key_string[j]);
                                        goto error;
                                }
                        }
                }

                module_name_index++;
                if(module_name_index >= CPU_LINE_MAX)
                {
                        printf("line_num(%d):  beyond the CPU_LINE_MAX %d, so error\n", line_num, CPU_LINE_MAX);
                        goto error;
                }
        }

        cpu_resource.magic_num = RESOURCE_MAGIC_NUM;
        cpu_resource.valid = true;
        cpu_resource.check_sum = 0;

        p_tmp = (int *)cpu_resource.cpu_content;
        for(i=0; i<sizeof(cpu_resource.cpu_content)/4; i++)
        {
                check_sum += p_tmp[i];
        }
        cpu_resource.check_sum =   check_sum;
        memcpy((void*)((unsigned int)resource_table+resource_table->cpu_resource_offset), (void*)&cpu_resource, sizeof(CPU_RESOURCE_T));

#if 0
        for(i=0; i<CPU_LINE_MAX; i++)
        {
                if(!strlen(cpu_resource.cpu_content[i].module_name_content))
                        continue;

                printf("%s ", cpu_resource.cpu_content[i].module_name_content);

                for(j=CPU_ENABLE_KEY_E; j<CPU_MAX_KEY_E; j++)
                        printf("%d ", cpu_resource.cpu_content[i].content[j-1]);
                printf("\n");
        }
#endif

        return 0;
 error:
        cpu_resource.magic_num = RESOURCE_MAGIC_NUM;
        cpu_resource.valid = false;
        cpu_resource.check_sum = 0;
        memcpy((void*)((unsigned int)resource_table+resource_table->cpu_resource_offset), (void*)&cpu_resource, sizeof(CPU_RESOURCE_T));
        return -1;
}

/*********************************************/
// parse vsc_resource.xml
#define VSC_RESOURCE_FILE  FACTORY_HEADER_FILE_NAME"vsc_resource.xml"
#define VSC_LINE_MAX  12

/*********************************************/
// parse vsc_resource.xml
#define VSC_RESOURCE_FILE  FACTORY_HEADER_FILE_NAME"vsc_resource.xml"
#define VSC_LINE_MAX  12
#define VSC_CONTENT_MAX  20
typedef enum
{
        VSC_KEY_TIMING_E,
        VSC_KEY_RTNR_MODE_E,
        VSC_KEY_OD_E,
        VSC_KEY_DEXC_E,
        VSC_KEY_FREE_RUN_E,
        VSC_KEY_VE_COMP_E,
        VSC_KEYDIPQC_BIT_E,
        VSC_KEY_MA_SNR_E,
        VSC_KEY_MAX_E,
}VSC_KEY_E;

typedef struct{
        unsigned int magic_num;
        unsigned int check_sum;
        bool valid;
        char reserve1;
        char reserve2;
        char reserve3;
        char vsc_content[VSC_LINE_MAX][VSC_KEY_MAX_E][VSC_CONTENT_MAX];
}VSC_RESOURCE;

char vsc_key_string[VSC_KEY_MAX_E][16] =
{
        "timing=\"",
        "rtnr_mode=\"",
        "od=\"",
        "dexc=\"",
        "free_run=\"",
        "ve_comp=\"",
        "dipqc_bit=\"",
        "ma_snr=\"",
};

int parse_platform_vsc_resource(char *src_addr, int length)
{
        int i =0,  line_num = 0, j = 0, timing_index = 0;
        char line[256];
        char * start = NULL;
        char * end = NULL;
        unsigned int size = 0;
        VSC_RESOURCE vsc_resource;
        int *p_tmp = NULL;
        int check_sum = 0;

        memset(&vsc_resource, 0, sizeof(VSC_RESOURCE));
        //printf("test: 0x%x size:0x%x", (unsigned int)vsc_resource.vsc_content, sizeof(vsc_resource.vsc_content));

        while(i < length)
        {
                memset(line, 0, sizeof(line));

                i = i + parse_getline(line,  &src_addr[i]) + 1;
                line_num++;

                if(!(strstr(line, "<pattern") &&  strstr(line, "/>")))
                {
                        DBG_MSG("line_num(%d): skip\n", line_num);
                        continue;
                }

                // printf("LINE_NUM:(%d) %s\n", line_num, line);
                for(j=0; j< VSC_KEY_MAX_E; j++)
                {
                        start = strstr(line, vsc_key_string[j]);
                        if(NULL == start)
                        {
                                printf("line_num(%d): cannot find %s\n", line_num, vsc_key_string[j]);
                                goto error;
                        }
                        start += strlen(vsc_key_string[j]);
                        end = strstr(start, " ");
                        if(NULL == end)
                        {
                                end = strstr(start, "/>");
                                if(end == NULL)
                                {
                                        printf("line_num(%d):  %s not match\n", line_num, vsc_key_string[j]);
                                        goto error;
                                }
                        }
                        size = end -start -1;
                        if(size > (VSC_CONTENT_MAX-1))
                        {
                                printf("line_num(%d):   %s content is beyond %d, so error\n", line_num, vsc_key_string[j], VSC_CONTENT_MAX);
                                goto error;
                        }
                        strncpy(vsc_resource.vsc_content[timing_index][j], start, size);
                }
                timing_index ++;
                if(timing_index >= VSC_LINE_MAX)
                {
                        printf("line_num(%d):  beyond the VSC_LINE_MAX %d, so error\n", line_num, VSC_LINE_MAX);
                        goto error;
                }
        }

        p_tmp = (int *)vsc_resource.vsc_content;
        for(i=0; i<sizeof(vsc_resource.vsc_content)/4; i++)
        {
                check_sum += p_tmp[i];
        }

#if 0
        // printf("%s_%d: check_sum:0x%x\n", __FUNCTION__, __LINE__, check_sum);
        for(i=0; i<VSC_LINE_MAX; i++)
        {
                for(j=0; j<VSC_KEY_MAX_E; j++)
                {
                        printf("%s ", vsc_resource.vsc_content[i][j]);
                }
                printf("\n");
        }
 #endif

        vsc_resource.magic_num = RESOURCE_MAGIC_NUM;
        vsc_resource.valid = true;
        vsc_resource.check_sum = check_sum;
        memcpy((void*)((unsigned int)resource_table+resource_table->scaler_resource_offset), (void*)&vsc_resource, sizeof(VSC_RESOURCE));
        return 0;

error:
         vsc_resource.magic_num = RESOURCE_MAGIC_NUM;
         vsc_resource.valid = false;
         vsc_resource.check_sum = 0;
         memcpy((void*)((unsigned int)resource_table+resource_table->scaler_resource_offset), (void*)&vsc_resource, sizeof(VSC_RESOURCE));
         return -1;
}


#define FACTORY_RO_IR_TABLE_FIEL_PATH  FACTORY_RO_HEADER_FILE_NAME"ir_table.bin"
#define FACTORY_IR_TABLE_FIEL_PATH  FACTORY_HEADER_FILE_NAME"ir_table.bin"
typedef struct {
	unsigned short protocol;
	unsigned short keycode;
	unsigned int scancode;
}IR_USER_KEY;

static int get_factory_ir_table_from_factory(char *file_path, unsigned int factory_ro)
{
	int ret = -1;
	char* dst_addr = NULL;
        int dst_length = 0;
	unsigned int *p_share_mem = (unsigned int *)POWER_ON_IR_TABLE_ADDR;

	if(factory_ro)
                ret = factory_ro_read(file_path, &dst_addr, &dst_length);
        else
	        ret = factory_read(file_path, &dst_addr, &dst_length);
	if (ret || !dst_addr || !dst_length)
        {
                DBG_ERR("------------can't find %s!, so parse fail\n", file_path);
                ret = -1;
        } else if( dst_length > POWER_ON_IR_TABLE_SIZE)
        {
        	DBG_ERR("%s parse fail, size too large:%d\n", file_path, dst_length);
                ret = -1;
        } else if((dst_length % (sizeof(IR_USER_KEY))) != 0)
        {
        	DBG_ERR("%s parse fail, size not align :%d\n", file_path, dst_length);
                ret = -1;
        } else
        {
                unsigned int *p_buf = (unsigned int *)dst_addr;
                #ifndef CONFIG_RUN_FLASH_WRITER
                unsigned int crc = 0x0;
                #endif
                if((p_buf[0] * sizeof(IR_USER_KEY) >= POWER_ON_IR_TABLE_SIZE) ||
                    (dst_length != (p_buf[0] + 1) * sizeof(IR_USER_KEY))) {
                    DBG_ERR("%s parse fail, size error:%d\n", file_path, p_buf[0]);
                    ret = -1;
                    goto EXIT;
                }
                #ifndef CONFIG_RUN_FLASH_WRITER
                count_sw_crc(CRC_TYPE_CRC16, (unsigned char *)(p_buf + 2), sizeof(IR_USER_KEY) * p_buf[0], &crc);
                if(crc != p_buf[1]) {
                    ret = -1;
                    DBG_ERR("%s parse fail, crc error\n", file_path);
                    goto EXIT;
                }
		  #endif
		p_share_mem[0] = 0x49525442;
                p_share_mem[1] = p_buf[0];

		memcpy(p_share_mem + 2, p_buf + 2, sizeof(IR_USER_KEY) * p_buf[0]);

		DBG_ERR("%s parse suc, size:%d\n", file_path, p_buf[0]);
                ret = 0;
	}
 EXIT:
        return ret;
}

void get_factory_ir_table(void)
{
	int ret = -1;
        ret = get_factory_ir_table_from_factory(FACTORY_RO_IR_TABLE_FIEL_PATH, 1);
        if(ret != 0)
                get_factory_ir_table_from_factory(FACTORY_IR_TABLE_FIEL_PATH, 0);
}


void get_factory_platform_resource(void)
{
        char* dst_addr = NULL;
        int dst_length = 0;
        int ret = -1;

        resource_table->magic_num = RESOURCE_MAGIC_NUM;
        resource_table->scaler_resource_offset = 0x100;  // size 0x780, don't modified
        resource_table->throttle_resource_offset= 0xa00; // size 0x118, don't modified
        resource_table->cpu_resource_offset= 0xd00;	 // size 0x140, don't modified
        resource_table->check_sum = resource_table->magic_num + resource_table->scaler_resource_offset +  resource_table->throttle_resource_offset + resource_table->cpu_resource_offset;

        ret = factory_read(VSC_RESOURCE_FILE, &dst_addr, &dst_length);
        if (ret || (NULL == dst_addr) || (0 == dst_length))
        {
                DBG_ERR("------------can't find %s!, so parse fail\n", VSC_RESOURCE_FILE);
        }
        else
        {
                if(-1 == parse_platform_vsc_resource(dst_addr, dst_length))
               {
                        DBG_ERR("\033[1m------------parse %s fail!\033[0m\n", VSC_RESOURCE_FILE);
                }
                else
                {
                        printf("------------parse %s ok!\n", VSC_RESOURCE_FILE);
                }
        }

        ret = factory_read(THROTTLE_RESOURCE_FILE, &dst_addr, &dst_length);
        if (ret || (NULL == dst_addr) || (0 == dst_length))
        {
                DBG_ERR("\033[1m------------can't find %s!, so parse fail\033[0m\n", THROTTLE_RESOURCE_FILE);
        }
        else
        {
                if(-1 == parse_platform_throttle_resource(dst_addr, dst_length))
               {
                        DBG_ERR("\033[1m------------parse %s fail!\033[0m\n", THROTTLE_RESOURCE_FILE);
                }
                else
                {
                        printf("------------parse %s ok!\n", THROTTLE_RESOURCE_FILE);
                }
        }

        ret = factory_read(CPU_RESOURCE_FILE, &dst_addr, &dst_length);
        if (ret || (NULL == dst_addr) || (0 == dst_length))
        {
                DBG_ERR("\033[1m------------can't find %s!, so parse fail\033[0m\n", CPU_RESOURCE_FILE);
        }
        else
        {
                if(-1 == parse_platform_cpu_resource(dst_addr, dst_length))
               {
                        DBG_ERR("\033[1m------------parse %s fail!\033[0m\n", CPU_RESOURCE_FILE);
                }
                else
                {
                        printf("------------parse %s ok!\n", CPU_RESOURCE_FILE);
                }
        }
}
