#ifndef __BOOT_PARAM_H__
#define __BOOT_PARAM_H__

#define OEM_KEY_MAX_SIZE 32
#define TP_MAX_NUM 2

typedef enum {
	POWER_OFF = 0,
	POWER_ON,
	BACKLIGHT_OFF,
} POWER_MODE_T;

typedef enum {
	BOOTPARAM_PANEL_SSC_DISABLE = 0x0,
	BOOTPARAM_PANEL_SSC_ENABLE = 0x1,
	BOOTPARAM_PANEL_SSC_UNKNOWN = 0xFF
} BOOTPARAM_PANEL_SSC_MODE_T;

typedef struct {
	unsigned int prev_power_state;
	unsigned int boot_logo_enable;
	unsigned int boot_music_enable;
	unsigned int backlight_value;
	unsigned int custom_video1_leng;
	unsigned int custom_video2_leng;
	unsigned int custom_audio1_leng;
	unsigned int custom_audio2_leng;
	unsigned int local_upgrade_en;
	unsigned int bootup_policy;
	unsigned char install_file_name[64];
	char ethaddr[20];
	char ir_power_up_key[2][100];
	char ir_default_remoter[2][50];
	char irda_filter[100];
	unsigned char panel_ssc_enable; // 0:disable 1:enable 0xff:unknown setting
	unsigned short panel_ssc_step; // Percentage
	unsigned int panel_ssc_span; // Modulation
	char tp_fe_setting[TP_MAX_NUM][100];          // index1: TP0, TP1, index2: string
} bootparam_t;

#if defined(CONFIG_FSW)
typedef struct _bootloader_message {
    char command[32];
    char status[32];
    char recovery[768];
    char stage[32];
    char reserved[1184];
} bootloader_message;
#endif

void get_bootparam(void);
void get_layout(void);
void get_oemkey(void);
void set_bootparam_power_state(uint val);
void get_factory_platform_resource(void);
void get_factory_ir_table(void);

extern bootparam_t bootparam;
extern char oem_key[OEM_KEY_MAX_SIZE+1];

extern void CusApi_get_bootparam(char *dst_addr, int dst_length); // call customized function
extern void CusApi_decide_power_on_mode(void); // call customized function

#endif /* #define __BOOT_PARAM_H__ */

