#include "cmd_hwmonitor.h"

#define DCMT_CMD_BUF_LEN (256)
#define DCMT_ENV_MEM_SET        "dcmt_set"
#define DCMT_ENV_MEM_UNSET      "dcmt_unset"
#define DCMT_ENV_DBG_LOG        "dcmt_log"
#define DCMT_ENV_KEEP_GO        "dcmt_keepgo"
#define DCMT_ENV_DRIVER_DISABLE "dcmt_disable"

char dcmt_cmd_buf[DCMT_CMD_BUF_LEN];
static const char* prot_str[4] = { "na", "r", "w", "rw"};
extern setmenu dcmt_mem_set[];
extern setmenu dcmt_mem_unset[];

/*Merlin5 module str*/
#define MAX_MODULE_NAME_LEN     (32)
typedef struct
{
        unsigned char id;
        const char      name[MAX_MODULE_NAME_LEN];
} MODULE_INFO;

typedef struct
{
        char * key;
        unsigned char * table;
        int table_len;
} MODULE_SEARCH_TABLE;

#define MODULE_ID(bridge, module)       ((bridge & 0xF)<<4 |(module & 0xf))
#define BRIDGE_ID(module)               ((module>>4) & 0xF)
#define SUB_MODULE_ID(module)           (module & 0xf)

#define MODULE_TVSB2_DIW                        MODULE_ID(0, 0)
#define MODULE_TVSB2_DIR                        MODULE_ID(0, 1)
#define MODULE_TVSB2_M_CAP                      MODULE_ID(0, 2)
#define MODULE_TVSB2_M_DISP                     MODULE_ID(0, 3)
#define MODULE_TVSB2_S_CAP                      MODULE_ID(0, 4)
#define MODULE_TVSB2_S_DISP                     MODULE_ID(0, 5)
#define MODULE_TVSB2_VO1_Y                      MODULE_ID(0, 6)
#define MODULE_TVSB2_VO1_C                      MODULE_ID(0, 7)
#define MODULE_TVSB2_VO2_Y                      MODULE_ID(0, 8)
#define MODULE_TVSB2_VO2_C                      MODULE_ID(0, 9)
#define MODULE_TVSB2_I3DDMA_Y                     MODULE_ID(0, 0xa)
#define MODULE_TVSB2_DE_XC_W                    MODULE_ID(0, 0xb)
#define MODULE_TVSB2_DE_XC_R                    MODULE_ID(0, 0xc)
#define MODULE_TVSB2_SNR_R                      MODULE_ID(0, 0xd)
#define MODULE_TVSB2_3DTABLE                    MODULE_ID(0, 0xe)
#define MODULE_TVSB2                            MODULE_ID(0, 0xf)

#define MODULE_TVSB1_VD                         MODULE_ID(1, 0)
#define MODULE_TVSB1_AUDIO                      MODULE_ID(1, 1)
#define MODULE_TVSB1_AUDIO_2                    MODULE_ID(1, 2)
#define MODULE_TVSB1_VBI                        MODULE_ID(1, 3)
#define MODULE_TVSB1                            MODULE_ID(1, 0xf)

#define MODULE_SB1_MD_KC                          MODULE_ID(2, 1)
#define MODULE_SB1_USB                          MODULE_ID(2, 2)
#define MODULE_SB1_RBUSDMA                      MODULE_ID(2, 3)
#define MODULE_SB1_MD_BBS                      MODULE_ID(2, 4)
#define MODULE_SB1_ETN                          MODULE_ID(2, 5)
#define MODULE_SB1_UART_DMA                        MODULE_ID(2, 6)
#define MODULE_SB1_EMMC_KC                      MODULE_ID(2, 7)
#define MODULE_SB1_MD_NKC_NVC                       MODULE_ID(2, 9)
#define MODULE_SB1_MD_VC                       MODULE_ID(2, 0xa)
#define MODULE_SB1_EMMC_NKC                     MODULE_ID(2, 0xe)
#define MODULE_SB1                              MODULE_ID(2, 0xf)

#define MODULE_SB2_VCPU                         MODULE_ID(3, 3)
#define MODULE_SB2_LZMA                         MODULE_ID(3, 4)
#define MODULE_SB2_UART                         MODULE_ID(3, 5)
#define MODULE_SB2_HOST_IF                      MODULE_ID(3, 6)
#define MODULE_SB2                              MODULE_ID(3, 0xf)

#define MODULE_SB1_SSB1_NAG_DMA_KC   MODULE_ID(4, 0)
#define MODULE_SB1_SSB1_TP2_KC              MODULE_ID(4, 2)
#define MODULE_SB1_SSB1_TP_KC               MODULE_ID(4, 3)
#define MODULE_SB1_SSB1_NF_KC                MODULE_ID(4, 4)
#define MODULE_SB1_SSB1_NAG_DMA_SC   MODULE_ID(4, 8)
#define MODULE_SB1_SSB1_TP2_NKC              MODULE_ID(4, 9)
#define MODULE_SB1_SSB1_TP_NKC               MODULE_ID(4, 0xa)
#define MODULE_SB1_SSB1_NF_NKC                MODULE_ID(4, 0xb)
#define MODULE_SB1_SSB1_NAG_DMA_PB      MODULE_ID(4, 0xc)
#define MODULE_SB1_SSB1_TP2_PB              MODULE_ID(4, 0xd)
#define MODULE_SB1_SSB1_TP_PB               MODULE_ID(4, 0xe)
#define MODULE_SB1_SSB1                              MODULE_ID(4, 0xf)

#define MODULE_VE_VM1                           MODULE_ID(5, 0)
#define MODULE_VE_VM2                           MODULE_ID(5, 1)
#define MODULE_VE_PRD                           MODULE_ID(5, 2)
#define MODULE_VE_DBK                           MODULE_ID(5, 3)
#define MODULE_VE_MVD                           MODULE_ID(5, 4)
#define MODULE_VE_NDBP2                       MODULE_ID(5, 5)
#define MODULE_VE_R1SEQ                         MODULE_ID(5, 6)
#define MODULE_VE_BS                            MODULE_ID(5, 7)
#define MODULE_VE_R1BLK                         MODULE_ID(5, 8)
#define MODULE_VE_VEP2COM                       MODULE_ID(5, 9)
#define MODULE_VE_VECOM                         MODULE_ID(5, 0xa)
#define MODULE_VE_CABP2                         MODULE_ID(5, 0xb)
#define MODULE_VE_BSP2                          MODULE_ID(5, 0xc)
#define MODULE_VE_LR_SAO                        MODULE_ID(5, 0xd)
#define MODULE_VE_SEG                           MODULE_ID(5, 0xe)
#define MODULE_VE_CMP                           MODULE_ID(5, 0xf)

#define MODULE_TVSB3_SEQ0                       MODULE_ID(6, 0x0)
#define MODULE_TVSB3_SEQ1                       MODULE_ID(6, 0x1)
#define MODULE_TVSB3_CP_KC                      MODULE_ID(6, 0x2)
#define MODULE_TVSB3_CP_NKC                     MODULE_ID(6, 0x3)
#define MODULE_TVSB3_DDR_TR_GEN                 MODULE_ID(6, 0x4)
#define MODULE_TVSB3                            MODULE_ID(6, 0xf)

#define MODULE_GPU_NONK                         MODULE_ID(7, 0)
#define MODULE_GPU_KCPU                         MODULE_ID(7, 1)
#define MODULE_GPU                              MODULE_ID(7, 0xf)

#define MODULE_TVSB5_DEMOD                      MODULE_ID(8, 0)
#define MODULE_TVSB5_DEBUGDMA                   MODULE_ID(8, 1)
#define MODULE_TVSB5_DEMOD2                     MODULE_ID(8, 2)
#define MODULE_TVSB5_USB3                       MODULE_ID(8, 3)
#define MODULE_TVSB5_DDR_TR_GEN                 MODULE_ID(8, 4)
#define MODULE_TVSB5                            MODULE_ID(8, 0xf)

#define MODULE_NN_NONK                          MODULE_ID(9, 0)
#define MODULE_NN_KCPU                          MODULE_ID(9, 1)
#define MODULE_NN                               MODULE_ID(9, 0xf)

#define MODULE_TVSB4_ODW                        MODULE_ID(0xb, 0)
#define MODULE_TVSB4_ODR                        MODULE_ID(0xb, 1)
#define MODULE_TVSB4_DC2H                       MODULE_ID(0xb, 2)
#define MODULE_TVSB4_DDR_TR_GEN                 MODULE_ID(0xb, 3)
#define MODULE_TVSB4_OSD1                       MODULE_ID(0xb, 4)
#define MODULE_TVSB4_OSD2                       MODULE_ID(0xb, 5)
#define MODULE_TVSB4_OSD3                       MODULE_ID(0xb, 6)
#define MODULE_TVSB4_DEMURA                     MODULE_ID(0xb, 7)
#define MODULE_TVSB4_DMATO3DLUT                 MODULE_ID(0xb, 8)
#define MODULE_TVSB4_DMATOSIP1                 MODULE_ID(0xb, 9)
#define MODULE_TVSB4_DMATOSIP2                 MODULE_ID(0xb, 0xa)
#define MODULE_TVSB4                            MODULE_ID(0xb, 0xf)

#define MODULE_MEMC_ME_W                        MODULE_ID(0xc, 0)
#define MODULE_MEMC_ME_R                        MODULE_ID(0xc, 1)
#define MODULE_MEMC_MC_LF_W                 MODULE_ID(0xc, 2)
#define MODULE_MEMC_MC_LFI_R                MODULE_ID(0xc, 4)
#define MODULE_MEMC_MC_HFI_R                MODULE_ID(0xc, 5)
#define MODULE_MEMC_MC_LFP_R                MODULE_ID(0xc, 6)
#define MODULE_MEMC_MC_HFP_R                MODULE_ID(0xc, 7)
#define MODULE_MEMC_ME_SHR_W                MODULE_ID(0xc, 8)
#define MODULE_MEMC_ME_SHR_R                MODULE_ID(0xc, 9)
#define MODULE_MEMC_MVINFO_W                MODULE_ID(0xc, 0xa)
#define MODULE_MEMC_MVINFO_R                MODULE_ID(0xc, 0xb)
#define MODULE_MEMC                             MODULE_ID(0xc, 0xf)

#define MODULE_TVSB7_ME                         MODULE_ID(0xd, 0)
#define MODULE_TVSB7_VDE                        MODULE_ID(0xd, 1)
#define MODULE_TVSB7_MF2                        MODULE_ID(0xd, 2)
#define MODULE_TVSB7_CDF_ALF                    MODULE_ID(0xd, 3)
#define MODULE_TVSB7_MF1                        MODULE_ID(0xd, 4)
#define MODULE_TVSB7_MNHP2                       MODULE_ID(0xd, 5)
#define MODULE_TVSB7_MF0                        MODULE_ID(0xd, 7)
#define MODULE_TVSB7_DDR_TR_GEN                 MODULE_ID(0xd, 0xe)
#define MODULE_TVSB7                            MODULE_ID(0xd, 0xf)

#define MODULE_ANY                              MODULE_ID(0xf, 0xf)

const MODULE_INFO module_info[] =
{
        {MODULE_TVSB2_DIW,      "TVSB2_DIW"},
        {MODULE_TVSB2_DIR,      "TVSB2_DIR"},
        {MODULE_TVSB2_M_CAP,    "TVSB2_M_CAP"},
        {MODULE_TVSB2_M_DISP,   "TVSB2_M_DISP"},
        {MODULE_TVSB2_S_CAP,    "TVSB2_S_CAP"},
        {MODULE_TVSB2_S_DISP,   "TVSB2_S_DISP"},
        {MODULE_TVSB2_VO1_Y,    "TVSB2_VO1_Y"},
        {MODULE_TVSB2_VO1_C,    "TVSB2_VO1_C"},
        {MODULE_TVSB2_VO2_Y,    "TVSB2_VO2_Y"},
        {MODULE_TVSB2_VO2_C,    "TVSB2_VO2_C"},
        {MODULE_TVSB2_I3DDMA_Y,   "TVSB2_I3DDMA_Y"},
        {MODULE_TVSB2_DE_XC_W,  "TVSB2_DE_XC_W"},
        {MODULE_TVSB2_DE_XC_R,  "TVSB2_DE_XC_R"},
        {MODULE_TVSB2_SNR_R,    "TVSB2_SNR_R"},
        {MODULE_TVSB2_3DTABLE,  "TVSB2_3DTABLE"},
        {MODULE_TVSB2,  "TVSB2"},

        {MODULE_TVSB1_VD,       "TVSB1_VD"},
        {MODULE_TVSB1_AUDIO,    "TVSB1_AUDIO"},
        {MODULE_TVSB1_AUDIO_2,  "TVSB1_AUDIO_2"},
        {MODULE_TVSB1_VBI,      "TVSB1_VBI"},
        {MODULE_TVSB1,  "TVSB1"},
        
        {MODULE_SB1_MD_KC,    "SB1_MD_KC"},
        {MODULE_SB1_USB,        "SB1_USB"},
        {MODULE_SB1_RBUSDMA,    "SB1_RBUSDMA"},
        {MODULE_SB1_MD_BBS,     "SB1_MD_BBS"},
        {MODULE_SB1_ETN,        "SB1_ETN"},
        {MODULE_SB1_UART_DMA,      "SB1_UART_DMA"},
        {MODULE_SB1_EMMC_KC,    "SB1_EMMC_KC"},
        {MODULE_SB1_MD_NKC_NVC,      "SB1_MD_NKC_NVC"},
        {MODULE_SB1_MD_VC,     "SB1_MD_VC"},
        {MODULE_SB1_EMMC_NKC,   "SB1_EMMC_NKC"},
        {MODULE_SB1,    "SB1"},
        
        {MODULE_SB2_VCPU,       "SB2_VCPU"},
        {MODULE_SB2_LZMA,       "SB2_LZMA"},
        {MODULE_SB2_UART,       "SB2_UART"},
        {MODULE_SB2_HOST_IF,    "SB2_HOST_IF"},
        {MODULE_SB2,    "SB2"},
        
        {MODULE_SB1_SSB1_NAG_DMA_KC,        "SB1_SSB1_NAG_DMA_KC"},
        {MODULE_SB1_SSB1_TP2_KC,        "SB1_SSB1_TP2_KC"},
        {MODULE_SB1_SSB1_TP_KC,      "SB1_SSB1_TP_KC"},
        {MODULE_SB1_SSB1_NF_KC,   "SB1_SSB1_NF_KC"},
        {MODULE_SB1_SSB1_NAG_DMA_SC,     "SB1_SSB1_NAG_DMA_SC"},
        {MODULE_SB1_SSB1_TP2_NKC,      "SB1_SSB1_TP2_NKC"},
        {MODULE_SB1_SSB1_TP_NKC,    "SB1_SSB1_TP_NKC"},
        {MODULE_SB1_SSB1_NF_NKC,   "SB1_SSB1_NF_NKC"},
        {MODULE_SB1_SSB1_NAG_DMA_PB,     "SB1_SSB1_NAG_DMA_PB"},
        {MODULE_SB1_SSB1_TP2_PB,      "SB1_SSB1_TP2_PB"},
        {MODULE_SB1_SSB1_TP_PB,      "SB1_SSB1_TP_PB"},
        {MODULE_SB1_SSB1,    "SB1_SSB1"},

        {MODULE_VE_VM1, "VE_VM1"},
        {MODULE_VE_VM2, "VE_VM2"},
        {MODULE_VE_PRD, "VE_PRD"},
        {MODULE_VE_DBK, "VE_DBK"},
        {MODULE_VE_MVD, "VE_MVD"},
        {MODULE_VE_NDBP2, "VE_NDBP2"},
        {MODULE_VE_R1SEQ,       "VE_R1SEQ"},
        {MODULE_VE_BS,  "VE_BS"},
        {MODULE_VE_R1BLK,       "VE_R1BLK"},
        {MODULE_VE_VEP2COM,     "VE_VEP2COM"},
        {MODULE_VE_VECOM,       "VE_VECOM"},
        {MODULE_VE_CABP2,       "VE_CABP2"},
        {MODULE_VE_BSP2,        "VE_BSP2"},
        {MODULE_VE_LR_SAO,      "VE_LR_SAO"},
        {MODULE_VE_SEG, "VE_SEG"},
        {MODULE_VE_CMP, "VE_CMP"},

        {MODULE_TVSB3_SEQ0,     "TVSB3_SEQ0"},
        {MODULE_TVSB3_SEQ1,     "TVSB3_SEQ1"},
        {MODULE_TVSB3_CP_KC,    "TVSB3_CP_KC"},
        {MODULE_TVSB3_CP_NKC,   "TVSB3_CP_NKC"},
        {MODULE_TVSB3_DDR_TR_GEN,       "TVSB3_DDR_TR_GEN"},
        {MODULE_TVSB3,  "TVSB3"},

        {MODULE_GPU_NONK,       "GPU_NONK"},
        {MODULE_GPU_KCPU,       "GPU_KCPU"},
        {MODULE_GPU,    "GPU"},

        {MODULE_TVSB5_DEMOD,    "TVSB5_DEMOD"},
        {MODULE_TVSB5_DEBUGDMA, "TVSB5_DEBUGDMA"},
        {MODULE_TVSB5_DEMOD2,   "TVSB5_DEMOD2"},
        {MODULE_TVSB5_USB3,     "TVSB5_USB3"},
        {MODULE_TVSB5_DDR_TR_GEN,       "TVSB5_DDR_TR_GEN"},
        {MODULE_TVSB5,  "TVSB5"},

        {MODULE_NN_NONK,        "NN_NONK"},
        {MODULE_NN_KCPU,        "NN_KCPU"},
        {MODULE_NN,     "NN"},

        {MODULE_TVSB4_ODW,      "TVSB4_ODW"},
        {MODULE_TVSB4_ODR,      "TVSB4_ODR"},
        {MODULE_TVSB4_DC2H,     "TVSB4_DC2H"},
        {MODULE_TVSB4_DDR_TR_GEN,       "TVSB4_DDR_TR_GEN"},
        {MODULE_TVSB4_OSD1,     "TVSB4_OSD1"},
        {MODULE_TVSB4_OSD2,     "TVSB4_OSD2"},
        {MODULE_TVSB4_OSD3,     "TVSB4_OSD3"},
        {MODULE_TVSB4_DEMURA,   "TVSB4_DEMURA"},
        {MODULE_TVSB4_DMATO3DLUT,       "TVSB4_DMATO3DLUT"},
        {MODULE_TVSB4_DMATOSIP1,       "TVSB4_DMATOSIP1"},
        {MODULE_TVSB4_DMATOSIP2,       "TVSB4_DMATOSIP2"},
        {MODULE_TVSB4,  "TVSB4"},

        {MODULE_MEMC_ME_W,      "MEMC_ME_W"},
        {MODULE_MEMC_ME_R,      "MEMC_ME_R"},
        {MODULE_MEMC_MC_LF_W,     "MEMC_MC_LF_W"},
        {MODULE_MEMC_MC_LFI_R,     "MEMC_MC_LFI_R"},
        {MODULE_MEMC_MC_HFI_R,     "MEMC_MC_HFI_R"},
        {MODULE_MEMC_MC_LFP_R,     "MEMC_MC_LFP_R"},
        {MODULE_MEMC_MC_HFP_R,     "MEMC_MC_HFP_R"},
        {MODULE_MEMC_ME_SHR_W,     "MEMC_ME_SHR_W"},
        {MODULE_MEMC_ME_SHR_R,     "MEMC_ME_SHR_R"},
        {MODULE_MEMC_MVINFO_W,     "MEMC_MVINFO_W"},
        {MODULE_MEMC_MVINFO_R,     "MEMC_MVINFO_R"},
        {MODULE_MEMC,   "MEMC"},

        {MODULE_TVSB7_ME,       "TVSB7_ME"},
        {MODULE_TVSB7_VDE,      "TVSB7_VDE"},
        {MODULE_TVSB7_MF2,      "TVSB7_MF2"},
        {MODULE_TVSB7_CDF_ALF,  "TVSB7_CDF_ALF"},
        {MODULE_TVSB7_MF1,      "TVSB7_MF1"},
        {MODULE_TVSB7_MNHP2,     "TVSB7_MNHP2"},
        {MODULE_TVSB7_MF0,      "TVSB7_MF0"},
        {MODULE_TVSB7_DDR_TR_GEN,       "TVSB7_DDR_TR_GEN"},
        {MODULE_TVSB7,  "TVSB7"},

        {MODULE_ANY,    "ANY"},
};

unsigned char key_table_audio[] = {MODULE_TVSB1_AUDIO,MODULE_TVSB1_AUDIO_2};
unsigned char key_table_se[] = {MODULE_TVSB3_SEQ0,MODULE_TVSB3_SEQ1};
unsigned char key_table_dispi[] = {MODULE_TVSB2_DIW,MODULE_TVSB2_DIR,MODULE_TVSB2_VO1_Y,MODULE_TVSB2_VO1_C,
                                                MODULE_TVSB2_I3DDMA_Y,
                                                MODULE_TVSB2_SNR_R};
unsigned char key_table_dispm[] = {MODULE_TVSB2_M_CAP,MODULE_TVSB2_M_DISP};
unsigned char key_table_dispd[] = {MODULE_TVSB4_ODW,MODULE_TVSB4_ODR,MODULE_TVSB4_DEMURA};
unsigned char key_table_gpu[] = {MODULE_GPU};
unsigned char key_table_osd[] = {MODULE_TVSB4_OSD1,MODULE_TVSB4_OSD2,MODULE_TVSB4_OSD3};
unsigned char key_table_vd[] = {MODULE_TVSB1_VD};
unsigned char key_table_vde[] = {MODULE_TVSB7_VDE};
unsigned char key_table_usb[] = {MODULE_SB1_USB};
unsigned char key_table_tp[] = {MODULE_SB1_SSB1_TP_NKC,MODULE_SB1_SSB1_TP_KC,MODULE_SB1_SSB1_TP2_KC,MODULE_SB1_SSB1_TP2_NKC};
unsigned char key_table_uart[] = {MODULE_SB2_UART};
unsigned char key_table_md[] = {MODULE_SB1_MD_KC,MODULE_SB1_MD_NKC_NVC,MODULE_SB1_MD_VC};
unsigned char key_table_etn[] = {MODULE_SB1_ETN};
unsigned char key_table_cpus[] = {MODULE_SB2_VCPU};
unsigned char key_table_emmc[] = {MODULE_SB1_EMMC_KC,MODULE_SB1_EMMC_NKC};
unsigned char key_table_demod[] = {MODULE_TVSB5_DEMOD};
unsigned char key_table_vbi[] = {MODULE_TVSB1_VBI};
unsigned char key_table_hostif[] = {MODULE_SB2_HOST_IF};
unsigned char key_table_me[] = {MODULE_TVSB7_ME};
unsigned char key_table_dma[] = {MODULE_TVSB5_DEBUGDMA,MODULE_TVSB4_DMATO3DLUT};

const MODULE_SEARCH_TABLE key_table[] =
{
        {"AUDIO", key_table_audio, sizeof(key_table_audio)},
        {"SE", key_table_se, sizeof(key_table_se)},
        {"DISP_I", key_table_dispi, sizeof(key_table_dispi)},
        {"DISP_M", key_table_dispm, sizeof(key_table_dispm)},
        {"DISP_D", key_table_dispd, sizeof(key_table_dispd)},
        {"GPU", key_table_gpu, sizeof(key_table_gpu)},
        {"OSD", key_table_osd, sizeof(key_table_osd)},
        {"VD", key_table_vd, sizeof(key_table_vd)},
        {"VDE", key_table_vde, sizeof(key_table_vde)},
        {"USB", key_table_usb, sizeof(key_table_usb)},
        {"TP", key_table_tp, sizeof(key_table_tp)},
        {"UART", key_table_uart, sizeof(key_table_uart)},
        {"MD", key_table_md, sizeof(key_table_md)},
        {"ETN", key_table_etn, sizeof(key_table_etn)},
        {"CPUS", key_table_cpus, sizeof(key_table_cpus)},
        {"EMMC", key_table_emmc, sizeof(key_table_emmc)},
        {"DEMOD", key_table_demod, sizeof(key_table_demod)},
        {"VBI", key_table_vbi, sizeof(key_table_vbi)},
        {"HOST_IF", key_table_hostif, sizeof(key_table_hostif)},
        {"ME", key_table_me, sizeof(key_table_me)},
        {"DMA", key_table_dma, sizeof(key_table_dma)},
};

/*
Str replace function:
param:
1)str:search dst str
2)oldstr:be replaced str
3)newstr:replace new str
return:
the str after replace 
*/
char *strrpc(char *str,char *oldstr,char *newstr)
{
        char bstr[BOOT_PARAM_LEN_MAX];
        memset(bstr,0,sizeof(bstr));
     
        for(int i = 0;i < strlen(str);i++){
            if(!strncmp(str+i,oldstr,strlen(oldstr))){
                if((strlen(newstr)+strlen(bstr))<BOOT_PARAM_LEN_MAX)
                    strncat(bstr,newstr,strlen(newstr));
                else
                    return NULL;
                i += strlen(oldstr) - 1;
            }else{
            	strncat(bstr,str + i,1);
    	    }
        }

        strncpy(str,bstr,BOOT_PARAM_LEN_MAX-1);
        return str;
} 

/**
 * strncasecmp - Case insensitive, length-limited string comparison
 * @s1: One string
 * @s2: The other string
 * @len: the maximum number of characters to compare
 */
#define DIFF_VALUE ('a' - 'A')
#define FALSE ('z' - 'A')
int strncasecmp(const char *s1, const char *s2, size_t n)
{
    int ch1 = 0;
    int ch2 = 0;

    if(NULL == s1 || NULL == s2 ){
        return FALSE;
    }

    if(0 == n){
        return 0;
    }

    do{
        if((ch1 = *(unsigned char *)s1++) >= 'A' && (ch1 <= 'Z')){
            ch1 += DIFF_VALUE;
        }
        if((ch2 = *(unsigned char *)s2++) >= 'A' && (ch2 <= 'Z')){
            ch2 += DIFF_VALUE;
        }
    }while(n-- && ch1 && (ch1 == ch2));

    return ch1 - ch2;
}

int rtk_plat_boot_prep_dcmt(void)
{
        append_bootargs(DCMT_ENV_MEM_UNSET);
        append_bootargs(DCMT_ENV_MEM_SET);
        append_bootargs(DCMT_ENV_KEEP_GO);
        append_bootargs(DCMT_ENV_DBG_LOG);
        append_bootargs(DCMT_ENV_DRIVER_DISABLE);

        return 0;
}

int go_clear_dcmt_set(void)
{
        char * cmdline = NULL;
        printf("\n******** Clear all dcmt set******** \n");
        cmdline = getenv(DCMT_ENV_MEM_SET);
        DBG_YELLOW("\ndcmt set param : %s\n",cmdline);
        clear_boot_param(DCMT_ENV_MEM_SET);
        cmdline = getenv(DCMT_ENV_MEM_SET);
        DBG_YELLOW("\nupdate dcmt set param to : %s\n",cmdline);
        return 0;
}

int go_dcmt_add_set_cmd(void)
{
        char* buf = dcmt_cmd_buf;
        char * cmdline = NULL;
        char setbuf[BOOT_PARAM_LEN_MAX]; 

        memset(setbuf,0,BOOT_PARAM_LEN_MAX);
        memset(buf,0,DCMT_CMD_BUF_LEN);
        cmdline = getenv(DCMT_ENV_MEM_SET);
        printf("\n******** Input command to set dcmt monitor******** \n");
        DBG_YELLOW("dcmt set param: %s\n",cmdline);
        printf("Input example:\n"
                FYELLOW_START "0x108000-0x700000 ip:CPUS(w) mode:1 mem:7\n" FCOLOR_END
                "Can use ';' to split 2 settings as:\n"
                FYELLOW_START "0x108000-0x700000 ip:CPUS(w) mode:1 mem:7;0x800000-0x900000 ip:CPUS(w),GPU(W) mode:1 mem:8\n" FCOLOR_END);
        printf("\nadd set cmd(append): ");
        fgets(buf,DCMT_CMD_BUF_LEN);
        
       if(hwmonitor_dcmt_set_cmd_check(buf) != 0)//check cmd format
       {
            return 0;
       }
       
        strncat(setbuf,"set#",sizeof(setbuf)-1);//Transfer to kernel dcmt set format
        strncat(setbuf, buf , sizeof(setbuf)-1-strlen(setbuf));
        strrpc(setbuf,";0x",",set#0x");
        strrpc(setbuf,"-",",");
        strrpc(setbuf," ","#");
        
        DBG_YELLOW("\nyour input is:%s\n",setbuf);

        add_boot_param(DCMT_ENV_MEM_SET,setbuf);

        cmdline = getenv(DCMT_ENV_MEM_SET);
        DBG_YELLOW("\nupdate dcmt set param to : %s\n",cmdline);

        return 0;

}

void go_show_module_info(void)
{
        int i,j,k;
        
        DBG_YELLOW("\nModule name reference as flow:\nmodule group:\n");

        for (i = 0; i < sizeof(key_table)/sizeof(MODULE_SEARCH_TABLE); i++){
            printf("%s:\t",key_table[i].key);
            for (j = 0; j < key_table[i].table_len; j++){
                for (k = 0; k < sizeof(module_info)/sizeof(MODULE_INFO); k++){
                    if(module_info[k].id == *(key_table[i].table+j))
                        printf("%s,",module_info[k].name);
                }
            }
            printf("\n");
        }

        DBG_YELLOW("\nOr use single module:\n");
        for (i = 0; i < 16; i++){
            for (j = 0, k = 0; j < sizeof(module_info)/sizeof(MODULE_INFO); j++){
                if(BRIDGE_ID(module_info[j].id) == i){
                    printf("%-20s",module_info[j].name);
                    k++;
                    if((k%4) ==0)
                        printf("\n");
                }
            }
            
            if((k!= 0)&&((k%4) !=0))
                printf("\n\n");
        }

        return;
}

int go_show_dcmt_set_format(void)
{
        printf("\n******** dcmt set Format and members information******** \n"
                "Input format(detail info see below):"
                FYELLOW_START "0x108000-0x700000 ip:AUDIO(w) mode:1 mem:7\n" 
                                           "1:0x108000 is Saddr\n"
                                           "2:0x700000 is Eaddr\n"            
                                           "3:AUDIO is module name(module name refer to below)\n"FCOLOR_END); 

        printf(FYELLOW_START"4:(w) is access mode,can use 'r,w,rw'\n"
                                            "5:mode:1 is monitor mode,can use '1,2,3'\n"
                                            "6:mem:7 is mem num,can use '0~15'\n"FCOLOR_END
                                            "mem num 0/3/4/5/6/7 will be used by kernel,better not to input them. \n\n"
                                            "Can use ';' to split 2 settings as:\n"
                  FPINK_START"0x108000-0x700000 ip:AUDIO(w) mode:1 mem:7"FCOLOR_END
                  FGREEN_START";"FCOLOR_END
                  FPINK_START"0x800000-0x900000 ip:CPUS(w),GPU(w) mode:1 mem:8\n" FCOLOR_END);

        go_show_module_info();
        return 0;

}

void delete_mem_num_str(char* mem_str,int delete_num)
{
        char tmpbuf[BOOT_PARAM_LEN_MAX] = {0};
        char* one_setting_buf[16];
        char* tmp_mem_addr = NULL;
        int setting_num = 0;
        int tmp_num = 0;
        int i = 0;

        //init all param
        if(mem_str != NULL)
        {
            if(strlen(mem_str) < BOOT_PARAM_LEN_MAX)
                strncpy(tmpbuf,mem_str,BOOT_PARAM_LEN_MAX-1);
            else
                return;
        }else{
            return;
        } 
        
        for(i = 0;i < 16;i++)
            one_setting_buf[i] = NULL;
        
        //find 
        one_setting_buf[0] = tmpbuf;//first setting addr
        setting_num++;
        
        for(i = 0;tmpbuf[i] != '\0';i++)//find setting total num
        {
                if((tmpbuf[i] == ',')&&(tmpbuf[i+1] == 's'))
                {
                        tmpbuf[i]= '\0';
                        one_setting_buf[setting_num] = tmpbuf+i+1;
                        setting_num++;
                }
        }

        if(setting_num>16)
        {
            DBG_YELLOW("--error -- dcmt set total num more than 16 \n");
            return;
        }
                        
        for(i = 0;i < setting_num;i++)//find mem num and compare
        {                        
                tmp_mem_addr = strstr(one_setting_buf[i],"mem:");
                
                if (tmp_mem_addr != NULL)//find str "mem:" of one setting 
                {
                          if(sscanf(tmp_mem_addr,"mem:%d",&tmp_num)<1)
                          {
                                DBG_YELLOW("find setting mem num error\n");
                          }
                          else
                          {
                                if(tmp_num == delete_num)
                                {
                                        one_setting_buf[i] = NULL;
                                }
                          }
                }
        }
        
        tmp_num = strlen(mem_str);
        memset(mem_str,0,tmp_num);
        
        for(i = 0;i < setting_num;i++)//re-set setting buf
        {
                if (one_setting_buf[i] != NULL)
                {
                        strncat(mem_str,one_setting_buf[i],tmp_num);
                        if(i != setting_num -1)
                            strncat(mem_str, ",", tmp_num);
                }
        }
        
        return;
        
}

int parse_delete_cmd(char* buf,int* input_num)   
{
        char* tmp_buf =NULL;
        char* one_setting_buf[16];
        int total_num = 0;
        int ture_total_num = 0;
        int i = 0;
        
        if(buf != NULL)
        {
                for(i = 0;i < 16;i++)
                    one_setting_buf[i] = NULL;

                tmp_buf = buf;
                
                one_setting_buf[0] = tmp_buf;//first delet str
                total_num++; 

                for(;*tmp_buf !='\0';tmp_buf++)//find every delete num str
                {
                        if(*tmp_buf == ',')
                        {
                               one_setting_buf[total_num] = tmp_buf+1;
                               total_num++; 
                               *tmp_buf = '\0';
                        }
                }      
                
                if(total_num>16)
                {
                    DBG_YELLOW("--error -- dcmt delete total num more than 16 \n");
                }
                
                for(i = 0;i < total_num;i++)
                {
                        if(sscanf(one_setting_buf[i],"%d",input_num)<1)
                        {
                                DBG_YELLOW("\ninput num invalid\n");
                        }
                        else
                        {
                                if((*input_num<0)||(*input_num>16))
                                {
                                        DBG_YELLOW("\ninput num error\n");
                                }
                                else
                                {
                                      input_num++;  
                                      ture_total_num++;
                                }
                        }
                }
  
        }
        
        return ture_total_num;
}           

int go_delete_dcmt_set(void)
{
        char* cmdline = NULL;
        char buf[50] = {0};
        char setbuf[BOOT_PARAM_LEN_MAX] = {0}; 
        int input_num_group[16] = {0};
        int total_num = 0;
        int i = 0;
        
        cmdline = getenv(DCMT_ENV_MEM_SET);
        DBG_YELLOW("Befor delete dcmt set: %s\n",cmdline);
        printf("\n******** Delete dcmt setting mem num******** \n");
        printf("\nplease input delete mem num: ");
        fgets(buf,sizeof(buf));
        
        printf("\n");
        
        total_num = parse_delete_cmd(buf,input_num_group);
        
        if(total_num > 0)
        {   
                if(cmdline != NULL)
                {
                        strncpy(setbuf,cmdline,BOOT_PARAM_LEN_MAX-1);
                        
                        for(i=0;i<total_num;i++)
                            delete_mem_num_str(setbuf,input_num_group[i]);
                        
                        update_boot_param(DCMT_ENV_MEM_SET,setbuf);                
                }
        }
 
        cmdline = getenv(DCMT_ENV_MEM_SET);
        DBG_YELLOW("After Delete dcmt set: %s\n",cmdline);
        return 0;
}


int go_show_dcmt_set(void)
{
        char * cmdline = NULL;
        cmdline = getenv(DCMT_ENV_MEM_SET);
        printf("\n******** show dcmt set information******** \n");
        DBG_YELLOW("dcmt set: %s\n",cmdline);
        return 0;
}

int go_dcmt_mem_set(void)
{
        state.menu = dcmt_mem_set;
        prepare_go_ahead();
        return 0;
}

int go_clear_dcmt_unset(void)
{
        char * cmdline = NULL;
        printf("\n******** Clear all dcmt unset******** \n");
        cmdline = getenv(DCMT_ENV_MEM_UNSET);
        DBG_YELLOW("\ndcmt unset param : %s\n",cmdline);
        clear_boot_param(DCMT_ENV_MEM_UNSET);
        cmdline = getenv(DCMT_ENV_MEM_UNSET);
        DBG_YELLOW("\nupdate dcmt unset param to : %s\n",cmdline);
        return 0;
}

int go_show_dcmt_unset_format(void)
{
        printf("\n******** dcmt unset Format and members information******** \n"
                "input format:\n"
                FYELLOW_START"<number>\n"FCOLOR_END
                "input example:\n"
                FYELLOW_START "0,1\n\n" FCOLOR_END);
        
        printf(FYELLOW_START"1:number is mem num,can use '0~15'\n"FCOLOR_END
                  "number 0/3/4/5/6/7 will be used by kernel,better not to input them. \n");
        
        return 0;

}

int go_dcmt_add_unset_cmd(void)
{
        char* buf = dcmt_cmd_buf;
        char * cmdline = NULL;
        char setbuf[BOOT_PARAM_LEN_MAX] = {0}; 
        int input_num_group[16] = {0};
        int total_num = 0;
        int i = 0;

        memset(buf,0,DCMT_CMD_BUF_LEN);
        cmdline = getenv(DCMT_ENV_MEM_UNSET);
        printf("\n******** Input command to unset dcmt monitor******** \n");
        DBG_YELLOW("dcmt unset param: %s\n",cmdline);
        printf("can input a mem number or monitor-type to unset DCMT monitor.\n"
                "input format : <number>\n"
                "input example: " FYELLOW_START "0,1\n" FCOLOR_END);
        printf("\nupdate unset option(update):");
        fgets(buf,DCMT_CMD_BUF_LEN);

        DBG_YELLOW("\nyour input is:%s\n",buf);

        update_boot_param(DCMT_ENV_MEM_UNSET,buf);

        cmdline = getenv(DCMT_ENV_MEM_UNSET);
        DBG_YELLOW("\nupdate dcmt unset param to : %s\n",cmdline);

        //delete dcmt set param
        cmdline = getenv(DCMT_ENV_MEM_SET);
        total_num = parse_delete_cmd(buf,input_num_group);
        
        if(total_num > 0)
        {   
                if(cmdline != NULL)
                {
                        strncpy(setbuf,cmdline,BOOT_PARAM_LEN_MAX-1);
                        
                        for(i=0;i<total_num;i++)
                            delete_mem_num_str(setbuf,input_num_group[i]);
                        
                        update_boot_param(DCMT_ENV_MEM_SET,setbuf);                
                }
        }

        return 0;

}

int go_show_dcmt_unset(void)
{
        char * cmdline = NULL;
        cmdline = getenv(DCMT_ENV_MEM_UNSET);
        printf("\n******** show dcmt unset information******** \n");
        DBG_YELLOW("dcmt unset: %s\n",cmdline);
        return 0;
}

int go_dcmt_mem_unset(void)
{
        state.menu = dcmt_mem_unset;
        prepare_go_ahead();
        return 0;
}

int go_dcmt_dbg_log(void)
{
        char* buf = dcmt_cmd_buf;
        char * cmdline = NULL;

        printf("\n******** dcmt log control flow ********\n");
        memset(buf,0,DCMT_CMD_BUF_LEN);
        cmdline = getenv(DCMT_ENV_DBG_LOG);
        DBG_YELLOW("now debug log param is : %s\n",cmdline);
        printf("you can input a option to enable some DCMT debug log.\n"
                "input option : 'all','init','set','isr','err','cmd','tip','addr','dbg'.\n"
                "input example: " FYELLOW_START "addr,tip \n" FCOLOR_END
                "if you use key words 'none', param will be cleard!\n");
        printf("\nupdate debug log option(update):");
        fgets(buf,DCMT_CMD_BUF_LEN);
        DBG_YELLOW("\nyour chooice is:%s\n",buf);

        if(strcmp(buf,"none") == 0)
        {
                clear_boot_param(DCMT_ENV_DBG_LOG);
        }
        else
        {
                update_boot_param(DCMT_ENV_DBG_LOG,buf);
        }

        cmdline = getenv(DCMT_ENV_DBG_LOG);
        DBG_YELLOW("\nupdate debug log param to : %s\n",cmdline);

        return 0;
}

int go_dcmt_keep_going(void)
{
        char* buf = dcmt_cmd_buf;
        char * cmdline = NULL;

        printf("\n******** dcmt interrupt keep going flow ********\n");
        memset(buf,0,DCMT_CMD_BUF_LEN);
        cmdline = getenv(DCMT_ENV_KEEP_GO);
        DBG_YELLOW("DCMT interrupt keep going param is : %s\n",cmdline);
        printf("\noption '0':silence mode, '1':simple mode, '2':normal mode, '3':panic mode \n"
               "if you use key words 'none', param will be cleard!\n");
        printf("\ninput keep going option(update):");
        fgets(buf,DCMT_CMD_BUF_LEN);
        DBG_YELLOW("\nyour chooice is:%s\n",buf);

        if(strcmp(buf,"none") == 0)
        {
                clear_boot_param(DCMT_ENV_KEEP_GO);
        }
        else
        {
                update_boot_param(DCMT_ENV_KEEP_GO,buf);
        }

        cmdline = getenv(DCMT_ENV_KEEP_GO);
        DBG_YELLOW("\nupdate interrupt keep going param to : %s\n",cmdline);

        return 0;
}

int go_dcmt_driver_disable(void)
{
        char* buf = dcmt_cmd_buf;
        char * cmdline = NULL;

        printf("\n******** dcmt driver disable flow ********\n");
        memset(buf,0,DCMT_CMD_BUF_LEN);
        cmdline = getenv(DCMT_ENV_DRIVER_DISABLE);
        DBG_YELLOW("DCMT driver disable param is : %s\n",cmdline);
        printf("\ndo you want to disable DCMT driver?(Yes/No):");
        fgets(buf,DCMT_CMD_BUF_LEN);
        DBG_YELLOW("\nyour chooice is:%s\n",buf);

        if((strcmp(buf,"y") == 0)||(strcmp(buf,"Y") == 0)||(strcmp(buf,"yes") == 0)||(strcmp(buf,"YES") == 0)||(strcmp(buf,"Yes") == 0))
        {
                update_boot_param(DCMT_ENV_DRIVER_DISABLE,"y");
        }
        else if((strcmp(buf,"n") == 0)||(strcmp(buf,"N") == 0)||(strcmp(buf,"no") == 0)||(strcmp(buf,"NO") == 0)||(strcmp(buf,"No") == 0))
        {
                clear_boot_param(DCMT_ENV_DRIVER_DISABLE);
        }

        cmdline = getenv(DCMT_ENV_DRIVER_DISABLE);
        DBG_YELLOW("\nupdate disable DCMT driver param to : %s\n",cmdline);

        return 0;
}

int go_dcmt_param_default(void)
{
        char* buf = dcmt_cmd_buf;

        printf("\n******** dcmt default param flow ********\n");
        memset(buf,0,DCMT_CMD_BUF_LEN);
        DBG_YELLOW("dcmt boot param is as below:\n");
        show_boot_param(DCMT_ENV_KEEP_GO);
        show_boot_param(DCMT_ENV_MEM_SET);
        show_boot_param(DCMT_ENV_MEM_UNSET);
        show_boot_param(DCMT_ENV_DBG_LOG);
        show_boot_param(DCMT_ENV_DRIVER_DISABLE);
        printf("\ndo you want to clear them all?(Yes/No):");
        fgets(buf,DCMT_CMD_BUF_LEN);
        DBG_YELLOW("\nyour chooice is:%s\n",buf);

        if((strcmp(buf,"y") == 0)||(strcmp(buf,"Y") == 0)||(strcmp(buf,"yes") == 0)||(strcmp(buf,"YES") == 0)||(strcmp(buf,"Yes") == 0))
        {
                clear_boot_param(DCMT_ENV_KEEP_GO);
                clear_boot_param(DCMT_ENV_MEM_SET);
                clear_boot_param(DCMT_ENV_MEM_UNSET);
                clear_boot_param(DCMT_ENV_DBG_LOG);
                clear_boot_param(DCMT_ENV_DRIVER_DISABLE);

                DBG_YELLOW("\nupdate dcmt boot param is as below:\n");
                show_boot_param(DCMT_ENV_KEEP_GO);
                show_boot_param(DCMT_ENV_MEM_SET);
                show_boot_param(DCMT_ENV_MEM_UNSET);
                show_boot_param(DCMT_ENV_DBG_LOG);
                show_boot_param(DCMT_ENV_DRIVER_DISABLE);
        }
        else
        {
                printf("param not changed...\n");
        }

        return 0;
}

int go_dcmt_param_show(void)
{
        char* buf = dcmt_cmd_buf;

        memset(buf,0,DCMT_CMD_BUF_LEN);
        show_boot_param(DCMT_ENV_KEEP_GO);
        show_boot_param(DCMT_ENV_MEM_SET);
        show_boot_param(DCMT_ENV_MEM_UNSET);
        show_boot_param(DCMT_ENV_DBG_LOG);
        show_boot_param(DCMT_ENV_DRIVER_DISABLE);

        //run_command(cmd,0); //cmd = "env set %s %x"
        printf("\npress 'ENTER' to continue...\n");
        fgets(buf,DCMT_CMD_BUF_LEN);

        return 0;
}

setmenu dcmt_root_menu[] =
{
        {"dcmt", "******** Welcome to dcmt set menu ********", NULL, top_menu},
        {"mem set", "set DCMT to monitor a region", go_dcmt_mem_set, dcmt_root_menu},
        {"mem unset", "unset a region which DCMT monitor by default", go_dcmt_mem_unset, dcmt_root_menu},
        {"keep go", "disable/enable kernel panic when trash happened", go_dcmt_keep_going, dcmt_root_menu},
        {"driver disable", "disable DCMT driver in kernel", go_dcmt_driver_disable, dcmt_root_menu},
        {"dbg log", "enable/disable DCMT debug log", go_dcmt_dbg_log, dcmt_root_menu},
        {"show boot param", "show bootcode param for DCMT driver", go_dcmt_param_show, dcmt_root_menu},
        {"default", "restore bootcode param as default setting", go_dcmt_param_default, dcmt_root_menu},
        {"go back", "go to up menu", go_up_menu, dcmt_root_menu},
        {NULL, NULL, NULL, NULL},
};

setmenu dcmt_mem_set[] =
{
        {"dcmt mem set", "******** Welcome to dcmt mem set menu ********", NULL, dcmt_root_menu}, //title
        {"Show", "show dcmt set param", go_show_dcmt_set, dcmt_mem_set},
        {"Add", "add set cmd(append)", go_dcmt_add_set_cmd, dcmt_mem_set},
        {"Format", "show dcmt set format and member information", go_show_dcmt_set_format, dcmt_mem_set},
        {"Delete", "Delete setting mem num", go_delete_dcmt_set, dcmt_mem_set},
        {"Clear", "clear all dcmt set param", go_clear_dcmt_set, dcmt_mem_set},
        {"Exit", "go exit", go_up_menu, dcmt_mem_set},
        {NULL, NULL, NULL, NULL},
};

setmenu dcmt_mem_unset[] =
{
        {"dcmt mem unset", "******** Welcome to dcmt mem unset menu ********", NULL, dcmt_root_menu}, //title
        {"Show", "show dcmt unset param", go_show_dcmt_unset, dcmt_mem_unset},
        {"Set", "set dcmt unset cmd", go_dcmt_add_unset_cmd, dcmt_mem_unset},
        {"Format", "show dcmt unset format and member information", go_show_dcmt_unset_format, dcmt_mem_unset},
        {"Clear", "clear all dcmt unset param", go_clear_dcmt_unset, dcmt_mem_unset},
        {"Exit", "go exit", go_up_menu, dcmt_mem_unset},
        {NULL, NULL, NULL, NULL},
};

void hwmonitor_dcmt_help(void)
{
        printf("\n******** hwmonitor dcmt help information ********\n"
                  FYELLOW_START"(1)dcmt set format:\n"FCOLOR_END
                  "\thwmonitor dcmt set \"0x108000-0x700000 ip:CPUS(w) mode:1 mem:4;0x800000-0x900000 ip:CPUS(w),GPU(w) mode:1 mem:13\"\n"
                  FYELLOW_START"(2)dcmt set clear format:\n"FCOLOR_END
                  "\thwmonitor dcmt set clear\n"

                  FYELLOW_START"(3)dcmt unset format:\n"FCOLOR_END
                  "\thwmonitor dcmt unset \"4,13\"\n"
                  FYELLOW_START"(4)dcmt unset clear format:\n"FCOLOR_END
                  "\thwmonitor dcmt unset clear\n"

                  FYELLOW_START"(5)dcmt show format:\n"FCOLOR_END
                  "\thwmonitor dcmt show\n");
}

void hwmonitor_dcmt_set_help(void)
{
        printf("\n******** hwmonitor dcmt set help information ********\n"
                  FYELLOW_START"dcmt set format:\n"FCOLOR_END
                  "\thwmonitor dcmt set \"0x108000-0x700000 ip:CPUS(w) mode:1 mem:4;0x800000-0x900000 ip:CPUS(w),GPU(w) mode:1 mem:13\"\n");
}

void hwmonitor_dcmt_unset_help(void)
{
        printf("\n******** hwmonitor dcmt unset help information ********\n"
                   FYELLOW_START"dcmt unset format:\n"FCOLOR_END
                  "\thwmonitor dcmt unset \"4,13\"\n");
}

/*
hwmonitor_dcmt_module_check
return vaule:
-3:format error
-2:access mode error
-1:module str error
0:Ok
*/
int hwmonitor_dcmt_module_check(char *dcmt_module)
{
    char* onemodulebuf[16];
    char module_str[MAX_MODULE_NAME_LEN];
    char access_mode[3];
    int i,j,k;
    char* sep = NULL;
    char* point= NULL;
        
    for(i = 0; i < 16 ; i++){
        onemodulebuf[i] = NULL;
    }

    memset(module_str, 0 ,sizeof(module_str));
    memset(access_mode, 0 ,sizeof(access_mode));
    
    //step1:Split DCMT module cmd
    strrpc(dcmt_module,"("," ");
    strrpc(dcmt_module,")"," ");
    //DBG_YELLOW("dcmt_module:%s\n", dcmt_module);
    i = 0;
    point = dcmt_module;
    do
    {
        onemodulebuf[i] = point;
        sep = strchr(point, ','); 
        if (sep != NULL){
            *sep = '\0'; // changed to null terminated
            point = sep + 1;
        }else{
            //DBG_YELLOW("one module buf[%d]:%s\n",i , onemodulebuf[i]);
            break;
        }
        //DBG_YELLOW("one module buf[%d]:%s\n",i , onemodulebuf[i]);
        i++;
    }while(i < 16);

    //step2:check module str&&access mode
    for(i = 0; i < 16 ; i++){
        if(onemodulebuf[i] != NULL){
            //DBG_YELLOW("group[%d]module_buf:%s\n",i , onemodulebuf[i]);
            if(sscanf(onemodulebuf[i],"%s %s ", module_str, access_mode) != 2)
            {
                return -3;
            }else{
                //DBG_YELLOW("group[%d]module_str:%s\n",i , module_str);
                //DBG_YELLOW("group[%d]access_mode:%s\n",i , access_mode);
                for (j = 0; j < 4; j++)//check access mode
                {
                        if (strncasecmp(access_mode, prot_str[j], strlen(access_mode)) == 0){
                            break;
                        }       
                }
                
                if(j == 4)
                    return  -2;

                for (j = 0; j < sizeof(key_table)/sizeof(MODULE_SEARCH_TABLE); j++)//check module group str
                {
                        if (strncasecmp(module_str, key_table[j].key, strlen(module_str)) == 0){
                            //DBG_YELLOW("search module group str match:%s\n", key_table[j].key);
                            break;
                        }       
                }
                
                if(j == sizeof(key_table)/sizeof(MODULE_SEARCH_TABLE)){
                    for (k = 0; k < sizeof(module_info)/sizeof(MODULE_INFO); k++)//check module str
                    {
                            if (strncasecmp(module_str, module_info[k].name, strlen(module_str)) == 0){
                                //DBG_YELLOW("search module_str match:%s\n", module_info[k].name);
                                break;
                            }       
                    } 

                   if(k == sizeof(module_info)/sizeof(MODULE_INFO))
                        return -1;
                }
                
            }
        }
    }

    return 0;
}

int hwmonitor_dcmt_set_cmd_check(char *dcmt_cmd)
{
        char cmdbuf[BOOT_PARAM_LEN_MAX];
        char* onecmdbuf[16];
        char modulebuf[DCMT_CMD_BUF_LEN];
        unsigned int s_addr = 0;
        unsigned int e_addr = 0;
        unsigned int mode = 0;
        unsigned int mem_num = 0;
        char* sep = NULL;
        char* point= NULL;
        int i = 0;
        int ret = -1;
        int module_ret = -1;
        
        memset(cmdbuf, 0 , BOOT_PARAM_LEN_MAX);
        memset(modulebuf, 0 , DCMT_CMD_BUF_LEN);
        
        for(i = 0; i < 16 ; i++){
            onecmdbuf[i] = NULL;
        }

        if(strlen(dcmt_cmd) >= (BOOT_PARAM_LEN_MAX -1)){
            DBG_RED("\ndcmt cmd too length\n");
            return ret;
        }else{
            strncpy(cmdbuf, dcmt_cmd, BOOT_PARAM_LEN_MAX -1);
        }
        
        //DBG_YELLOW("\ncmdbuf:%s\n", cmdbuf);
        //step1:Split DCMT cmd
        i = 0;
        point = cmdbuf;
        do
        {
            onecmdbuf[i] = point;
            sep = strchr(point, ';'); 
            if (sep != NULL){
                *sep = '\0'; // changed to null terminated
                point = sep + 1;
            }else{
                //DBG_YELLOW("one cmd buf[%d]:%s\n",i , onecmdbuf[i]);
                break;
            }
            //DBG_YELLOW("one cmd buf[%d]:%s\n",i , onecmdbuf[i]);
            i++;
        }while(i < 16);
        
        //step2:check addr/module/access/mode/mem
        for(i = 0; i < 16 ; i++){
            if(onecmdbuf[i] != NULL){
                if(sscanf(onecmdbuf[i],"0x%x-0x%x ip:%s mode:%d mem:%d", &s_addr, &e_addr, modulebuf, &mode, &mem_num) != 5)
                {
                    DBG_RED("input dcmt cmd group[%d] format error,please re-input dcmt cmd\n",i);
                    return ret;
                }else{
                    //DBG_YELLOW("group[%d]s_addr:0x%x\n",i , s_addr);
                    //DBG_YELLOW("group[%d]e_addr:0x%x\n",i , e_addr);
                    //DBG_YELLOW("group[%d]modulebuf:%s\n",i , modulebuf);
                    //DBG_YELLOW("group[%d]mode:%d\n",i , mode);
                    //DBG_YELLOW("group[%d]mem_num:%d\n",i , mem_num);
                    if((int)(e_addr - s_addr) < 0){//check addr 
                        DBG_RED("input dcmt cmd group[%d] addr error,please re-input dcmt cmd(must end_addr > start_addr )\n",i);
                        return ret;
                    }
                    
                    module_ret = hwmonitor_dcmt_module_check(modulebuf);
                    if(module_ret != 0){//check module & access
                        if(module_ret == -1){
                            DBG_RED("input dcmt cmd group[%d] module error,please re-input dcmt cmd(module str as blew)\n",i);
                            go_show_module_info();
                        }else if(module_ret == -2){
                            DBG_RED("input dcmt cmd group[%d] access mode error,please re-input dcmt cmd(access mode option:r/w/rw)\n",i);
                        }else{
                            DBG_RED("input dcmt cmd module format error,please re-input dcmt cmd\n");
                        }
                        return ret;
                    }
                    
                    if((mode > 3)||(mode == 0)){//check mode 
                        DBG_RED("input dcmt cmd group[%d] mode error,please re-input dcmt cmd(mode option:1/2/3)\n",i);
                        return ret;
                    }
                    
                    if(mem_num > 15){//check mem_num 
                        DBG_RED("input dcmt cmd group[%d] error,please re-input dcmt cmd(mem option:0~15)\n",i);
                        return ret;
                    }

                }
            }
        }
        
        return 0;
}

/****QUICK SETTING AREA****/
int hwmonitor_dcmt_quick_set(char *dcmt_mode, char *dcmt_cmd)
{
        char setbuf[BOOT_PARAM_LEN_MAX] = {0}; 
        char* cmdline = NULL;
        int input_num_group[16] = {0};
        int total_num = 0;
        int i = 0;
        
        if(dcmt_mode == NULL)
        {
            hwmonitor_dcmt_help();
            return -1;
        }
        
        if(dcmt_cmd != NULL){
            if(!strcmp("set", dcmt_mode))//Transfer to kernel dcmt set format
            {
                if(!strcmp("clear", dcmt_cmd))
                {
                    go_clear_dcmt_set();
                    return 0;
                }

               if(hwmonitor_dcmt_set_cmd_check(dcmt_cmd) != 0)//check cmd format
               {
                    hwmonitor_dcmt_set_help();
                    return -1;
               }
               
                snprintf(setbuf,sizeof(setbuf),"%s","set#");//Transfer to kernel dcmt set format
                if(strlen(dcmt_cmd)+4<BOOT_PARAM_LEN_MAX){
                    strncat(setbuf,dcmt_cmd,strlen(dcmt_cmd));
                }else{
                    return -1;
                }
                strrpc(setbuf,";0x",",set#0x");
                strrpc(setbuf,"-",",");
                strrpc(setbuf," ","#");
                
                update_boot_param(DCMT_ENV_MEM_SET,setbuf);
            }
            else if(!strcmp("unset", dcmt_mode))
            {
                if(!strcmp("clear", dcmt_cmd))
                {
                    go_clear_dcmt_unset();
                    return 0;
                }
                             
                update_boot_param(DCMT_ENV_MEM_UNSET,dcmt_cmd);
                
                //delete dcmt set param
                memset(setbuf,0,sizeof(setbuf));
                cmdline = getenv(DCMT_ENV_MEM_SET);
                total_num = parse_delete_cmd(dcmt_cmd,input_num_group);

                if(total_num > 0)
                {   
                        if(cmdline != NULL)
                        {
                                strncpy(setbuf,cmdline,BOOT_PARAM_LEN_MAX-1);
                                
                                for(i=0;i<total_num;i++)
                                    delete_mem_num_str(setbuf,input_num_group[i]);
                                
                                update_boot_param(DCMT_ENV_MEM_SET,setbuf);                
                        }
                }
            }else{
                hwmonitor_dcmt_help();
                return -1;
            }
        }else{
            if(!strcmp("show", dcmt_mode))
            {
                go_show_dcmt_set();
                go_show_dcmt_unset();
            }
            else
            {
                hwmonitor_dcmt_help();
                return -1;
            }
        }
        return 0;
}

