#include <command.h>
#include <common.h>

typedef enum CONSOLE_LEVEL_
{
	KERN_EMERG=0,	// system is unusable
	KERN_ALERT,		// action must be taken immediately
	KERN_CRIT,		// critical conditions
	KERN_ERR,		// error conditions
	KERN_WARNING,	// warning conditions
	KERN_NOTICE,	// normal but significant condition
	KERN_INFO,		// informational
	KERN_DEBUG		// debug-level messages
}CONSOLE_LEVEL;

#ifdef  OPTION_DEFAULT_LOGLEVEL
#define LOGLEVEL_DEFAULT (OPTION_DEFAULT_LOGLEVEL)
#else
#define LOGLEVEL_DEFAULT KERN_WARNING
#endif

int get_current_loglevel (void)
{
	int loglevel=LOGLEVEL_DEFAULT;
    char *cmdline = NULL;

    cmdline=getenv("loglevel");
	if (cmdline != NULL)
	{
        loglevel = simple_strtoul(cmdline, NULL, 16);
        //printf("loglevel = %d\n",loglevel);

        if (0==loglevel)
            loglevel=KERN_INFO;
        if (loglevel>KERN_DEBUG)
            loglevel=KERN_DEBUG;
	}
    //printf("loglevel = %d\n",loglevel);
    return loglevel;
}
int do_loglevel (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    char cmd[128]={0};
    int loglevel;
    char * str[]={
        "KERN_EMERG",
        "KERN_ALERT",
        "KERN_CRIT",
        "KERN_ERR",
        "KERN_WARNING",
        "KERN_NOTICE",
        "KERN_INFO",
        "KERN_DEBUG"
    };

    if (argc==2)
    {
        loglevel = simple_strtoul(argv[1], NULL, 16);
        if (0==loglevel || loglevel>KERN_DEBUG)
        {
            printf("Invalid loglevel value, the level value must between %d and %d!\n",KERN_ALERT,KERN_DEBUG);
            return CMD_RET_FAILURE;
        }

        sprintf(cmd,"env set loglevel %x",loglevel);
        if (run_command(cmd,0)!=0)
        {
            printf("%s failed.  env set loglevel failed.\n",__func__);
            return CMD_RET_FAILURE;
        }

        if (run_command("saveenv",0)!=0)
        {
            printf("%s failed.  saveenv failed.\n",__func__);
            return CMD_RET_FAILURE;
        }
    }
    else
    {
        loglevel=get_current_loglevel();
        printf("current console loglevel is %d. \e[33m%s\e[0m\n\n",loglevel,str[loglevel]);
        return CMD_RET_USAGE;
    }
    return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
    loglevel,  2,  0,  do_loglevel,
    "set console loglevel to env and bootcode will pass this value to kernel",
    "value  - switch specifyied value. ex: loglevel 5  (default loglevel is 6, range is between 1 and 7)\n"
);

