#include "cmd_hwmonitor.h"

#define SB2_ENV_MEM_SET                 "sb2_set"
#define SB2_ENV_MEM_BIT_SET          "sb2_bit_set"

extern char *getenv(const char *name);
extern int setenv(const char *varname, const char *varvalue);

int rtk_plat_boot_prep_sb2(void)
{
        append_bootargs(SB2_ENV_MEM_SET);
        append_bootargs(SB2_ENV_MEM_BIT_SET);

        return 0;
}

int split_str(char * str, char split_flag, char out_str[][16], int max_cnt)
{
        int num = 0;
        int i;
        int start;
        int end;
        if((str == NULL) || (out_str == NULL)) {
                return -1;
        }
        //get strings
        for(start = 0, end = 0; str[end] != '\0'; ++end) {
                if(str[end] == split_flag) {
                        if(start < end) { // if any char between start-end range;
                                for(i = 0; start < end; start++, i++) { // copy string
                                        out_str[num][i] = str[start];
                                }
                                start = end + 1; //ignore the split str;
                                num++;
                                if(num >= max_cnt) {
                                        return num;
                                }
                        } else {
                                start = end + 1; //ignore the split str;
                        }
                }
        }
        //get last string
        if(start < end) { // if any char between start-end range;
                for(i = 0; start < end; start++, i++) {
                        out_str[num][i] = str[start];
                }
                num++;
        }

        return num;
}

int get_cputype_id(const char * cpu)
{
        int return_id = -1;
        if (strcmp(cpu, "all") == 0) {
                return_id = 0;
        } else if (strcmp(cpu, "vcpu") == 0) {
                return_id = 1;
        } else if (strcmp(cpu, "scpu") == 0) {
                return_id = 2;
        } else {
                printf("invalid cpu type - %s\n", cpu);
                return -1;
        }

        return return_id;
}

int parse_cpu_field(char * field_str)
{
        char cpu_str[7][16] = {{0}};
        int cpu_num = 0;
        int i =0;
        
        memset(cpu_str, '\0', sizeof(cpu_str));

        if(field_str == NULL) {
                printf("cpu_field is NULL\n");
                return 1;
        }

        cpu_num = split_str(field_str, ',', cpu_str, 7);
        if((cpu_num <= 0) || (cpu_num > 7)) { //no cpu or too many cpus
                printf("unexpected cpu type num,expected between [1-7],actual:%d!\n", cpu_num);
                return 2;
        }

        for(i = 0; i < cpu_num; ++i) 
        { //parse cpu id
                cpu_str[i][15] = '\0';
                if(get_cputype_id(&cpu_str[i][0])<0)
                {
                    printf("cpu_set is invalid\n");
                    return 3;
                }
        }
        
        return 0;
}
int parse_range_field(char * field_str)
{
        char addr_str[2][16] = {{0}};
        int start=0;
        int end=0;
        
        memset(addr_str, '\0', sizeof(addr_str));

        //check input
        if(field_str == NULL){
                printf("range_field is NULL\n");
                return 1;
        }

        if(split_str(field_str, '-', addr_str, 2) != 2) {
                printf("unexpected addr param,excepted start_addr-end_addr!\n");
                return 2;
        }

        addr_str[0][15] = '\0';
        if(sscanf(&addr_str[0][0], "%x", &start) < 1) {
                printf("parse start range fail\n");
                return 3;
        }

        addr_str[1][15] = '\0';
        if(sscanf(&addr_str[1][0], "%x", &end) < 1) {
                printf("parse end range fail\n");
                return 4;
        }

        if(start > end)
        {
                printf("sb2 set fail, MT start range:0x%08x > end range:0x%08x\n",start,end);
                return 5;
        }
        return 0;
}

int parse_bit_cpu_field(char * field_str)
{
        char cpu_str[4][16] = {{0}};
        int cpu_num = 0;
        int i =0;
        
        memset(cpu_str, '\0', sizeof(cpu_str));

        if(field_str == NULL){
                printf("cpu_field is NULL\n");
                return 1;
        }

        cpu_num = split_str(field_str, ',', cpu_str, 4);
        if((cpu_num <= 0) || (cpu_num > 4)) 
        { //no cpu or too many cpus
                printf("unexpected cpu type num,expected between [1-4],actual:%d!\n", cpu_num);
                return 2;
        }

        for(i = 0; i < cpu_num; ++i) 
        { //parse cpu id
                cpu_str[i][15] = '\0';
                if(get_cputype_id(&cpu_str[i][0])<0)
                {
                    printf("cpu_set is invalid\n");
                    return 3;
                }
        }
        
        return 0;
}

int parse_bit_range_field(char * field_str)
{
        unsigned int check;
        int addr=0;
        //check input
        if(field_str == NULL) {
                printf("range_field is NULL\n");
                return -1;
        }

        if (sscanf(field_str, "%x", &addr) < 1) {
                printf("sb2 set fail, addr :0x%08x \n",addr);
                return -1;
        }

        if((addr < (unsigned int)0x18000000)||(addr > (unsigned int)0x1817FFFFF))
        {
            printf("sb2 set fail, addr:0x%08x ,addr range:( 0x18000000->0x1817ffff )\n",addr);
            return -1;
        }
        
        check = (addr -0x18000000)%4;
        if(check != 0)
        {
            printf("sb2 set fail, addr remainder=%d , need to set 4byte (example:0x18000004)\n",check);
            return -1;
        }
        
        return 0;
}
void hwmonitor_sb2_help(void)
{
        printf("\n******** hwmonitor sb2 help information ********\n"
                  FYELLOW_START"(1)sb2 set format:\n"FCOLOR_END
                  "\thwmonitor sb2 monitor \"id:9 scpu 18061900-18061920 rw str\"\n"
                  FYELLOW_START"(2)sb2 set format:\n"FCOLOR_END
                  "\thwmonitor sb2 clear \"id:9\"\n"
                  FYELLOW_START"(3)sb2 set format:\n"FCOLOR_END
                  "\thwmonitor sb2 bit_monitor \"id:1 18061500 0x8000 0x8000 acpu str match\"\n"
                  "\thwmonitor sb2 bit_monitor \"id:1 18061500 0x8000 0x8000 acpu str nmatch\"\n"
                  FYELLOW_START"(4)sb2 set format:\n"FCOLOR_END
                  "\thwmonitor sb2 bit_clear \"id:1\"\n");
}

/****QUICK SETTING AREA****/
int hwmonitor_sb2_quick_set(char *sb2_mode, char *sb2_cmd)
{
        char num[5]={0};
        char cpu[20]={0};
        char range[20]={0};
        char rw[5]={0};
        char str[5]={0};
        char match[7]={0};
        int Num=0;
        int dbg3_1=0;
        int dbg3_2=0;
        
        if(sb2_cmd == NULL)
        {
            hwmonitor_sb2_help();
            return -1;
        }

        if(sb2_cmd != NULL)
        {
            if(strcmp("monitor", sb2_mode)==0)
            {
                sscanf(sb2_cmd,"%s %s %s %s %s",num,cpu,range,rw,str);
                sscanf(&num[3],"%d",&Num);
                if((Num>16)||(Num<1))
                {   
                       return -2;
                }
                if(parse_cpu_field(cpu))
                {
                       return -3;
                }
                if(parse_range_field(range))
                {
                       return -4;
                }
                if((strcmp("r", rw))&&(strcmp("w", rw))&&(strcmp("rw", rw)))
                {
                       return -5;
                }
                if((strcmp("str", str))&&(strcmp("nstr", str)))
                {
                       return -6;
                }         
                update_boot_param(SB2_ENV_MEM_SET,sb2_cmd);

                
            }
            else if(strcmp("clear",sb2_mode)==0)
            {
                sscanf(&sb2_cmd[3],"%d",&Num);
                if((Num>16)||(Num<1))
                {   
                       return -2;
                }
                clear_boot_param(SB2_ENV_MEM_SET);
            }
            else if(strcmp("bit_monitor", sb2_mode)==0)
            {
                sscanf(sb2_cmd,"%s %s %x %x %s %s %s",num,range,&dbg3_1,&dbg3_2,cpu,str,match);
                sscanf(&num[3],"%d",&Num);
                if((Num>4)||(Num<1))
                {   
                       return -2;
                }
                if(parse_bit_range_field(range))
                {   
                       return -3;
                }
                if((dbg3_1<1)||(dbg3_2<1))
                {
                       return -4;
                }
                if(parse_bit_cpu_field(cpu))
                {   
                       return -5;
                }      
                if((strcmp("str", str))&&(strcmp("nstr", str)))
                {
                       return -6;
                } 
                if((strcmp("match", match))&&(strcmp("nmatch", match)))
                {
                       return -7;
                }                 
                update_boot_param(SB2_ENV_MEM_BIT_SET,sb2_cmd);
                
            }
            else if(strcmp("bit_clear", sb2_mode)==0)
            {
                sscanf(&sb2_cmd[3],"%d",&Num);
                if((Num>4)||(Num<1))
                {   
                       return -2;
                }
                clear_boot_param(SB2_ENV_MEM_BIT_SET);
            }
            else
            {
               hwmonitor_sb2_help();
               return -1;
            }
        }
        else
        {
               hwmonitor_sb2_help();
               return -1;
        }

        return 0;
}
