#include <common.h>
#include <parse_h.h>
#include <asm/arch/bootparam.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>
#include <exports.h>
#include <factorylib.h>
#include <platform_lib/board/pcb.h>

extern uchar boot_ac_on;
extern int boot_power_on_mode;
extern int prev_power_on_status;

#ifdef CONFIG_CUSTOM_SN_PANEL
#define BOOT_PARAM_FILE_NAME	FACTORY_HEADER_FILE_NAME"bin_panel/000BootParam.h"

extern void enable_console(void);
extern void disable_console(void);
#ifdef CONFIG_CUSTOM_PQ_SELECT
extern bool CusApi_env_writeEnvFile(char *pItemName, char *pParameter);
#endif

#ifdef CONFIG_FSW
#define BLOCK_SHIFT 9
#define BLOCK_SIZE (1 << BLOCK_SHIFT)
extern unsigned int rtk_eMMC_read( unsigned int address, unsigned int data_size, unsigned int * buffer);
#endif

#define DEBUG_MODE 0
#define NORMAL_MODE 1

#endif

void CusApi_get_bootparam(char *dst_addr, int dst_length)
{
}


char CusApi_get_bootparam_PowerMode(void)
{

        char *dst_addr;
        char *retAddr;
        int dst_length;
        uint retVal;

        if(factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length))
            return 1;

        if(parse_h("POWER_ON_MODE", dst_addr, (uint)dst_length, &retVal, &retAddr))
            return 1;

        printf("POWER_ON_MODE = %d--\n",retVal);
        if(1 == retVal)
            return 1;
        else
            return 0;
}


char CusApi_get_bootparam_SHUTDOWN(void)
{

        char *dst_addr;
        char *retAddr;
        int dst_length;
        uint retVal;

        if(factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length))
            return 0;

        if(parse_h("SHUTDOWN_DISABLE", dst_addr, (uint)dst_length, &retVal, &retAddr))
            return 0;

        printf("SHUTDOWN_DISABLE = %d--\n",retVal);
        if(1 == retVal)
            return 1;
        else
            return 0;
}


char CusApi_bootparam_PowerMode_Clear(void)
{

        char *dst_addr;
        char *retAddr;
        int dst_length;
        uint retVal;

        if(factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length))
            return 0;

        if(parse_h("POWER_ON_MODE", dst_addr, (uint)dst_length, &retVal, &retAddr))
            return 0;


        *retAddr = 0x30; //ASCII char '0'

        if(factory_write(BOOT_PARAM_FILE_NAME, dst_addr, dst_length)!=0) {
            printf("[DEBUG]%s factory_write error\n",__FUNCTION__);
            return 0;
        }

        if(factory_save()!=0) {
            printf("[DEBUG]%s factory_save error\n",__FUNCTION__);
            return 0;
        }

        printf("PowerMode_Clear: success");
        return 1;

}



int CusApi_Bootparam_STDWN_RTRY_Reset(bool enable){
    char *dst_addr;
    char *retAddr;
    int dst_length;
    int mRTRY_CNT;
    uint retVal;

    if(factory_read(BOOT_PARAM_FILE_NAME, &dst_addr, &dst_length))
        return 0;

    if(parse_h("BOOT_STDWN_RTRY_CNT", dst_addr, (uint)dst_length, &retVal, &retAddr))
        return 0;

    if(enable == true){
        if(retVal == 0)
            return 0;
        mRTRY_CNT = 0;
        *retAddr = 0x30; //ASCII char '0','
    }else{
        if(retVal > 5)
            return 0;
        mRTRY_CNT = retVal + 1;
        *retAddr = 0x30 + retVal + 1; //ASCII char '0','1'...'6'
    }
    if(factory_write(BOOT_PARAM_FILE_NAME, dst_addr, dst_length)!=0) {
        printf("[DEBUG]%s factory_write error\n",__FUNCTION__);
        return 0;
    }

    if(factory_save()!=0) {
        printf("[DEBUG]%s factory_save error\n",__FUNCTION__);
        return 0;
    }

    printf("STDWN_RTRY_Reset = %d, (%d)\n",enable,mRTRY_CNT);

    return mRTRY_CNT;
}



void CusApi_decide_power_on_mode(void)
{
	// RTK config don't care prev_power_state and always power on
	boot_power_on_mode = POWER_ON;
	prev_power_on_status = bootparam.prev_power_state;

#if defined(CONFIG_BOOTUP_BY_PREV_POWER_STATUS) || defined(ENABLE_BOOTUP_POLICY)
	if(boot_ac_on)
		boot_power_on_mode = bootparam.prev_power_state;
#endif

#ifdef ENABLE_BOOTUP_POLICY
	printf("bootup_policy=%d\n", bootparam.bootup_policy);
	if(bootparam.bootup_policy != 2)	//0: always powerOff, 1: always powerOn, 2:auto by prev_status
	{
		if(boot_ac_on)
			boot_power_on_mode = bootparam.bootup_policy;
		else
			boot_power_on_mode = POWER_ON;
	}
#endif

}

int CusApi_get_bootparam_recovery_status(void)
{
    int ret = 0;
#ifdef CONFIG_SYS_RTK_EMMC_FLASH
#ifdef CONFIG_FSW
    extern layout_part_desc misc_part_info;
    unsigned int *u32dst_addr = NULL;
    unsigned int block_no = 0;
    block_no =  misc_part_info.offset/BLOCK_SIZE;
    u32dst_addr = malloc(sizeof(bootloader_message));
    printf("boot_power_on_mode = %d\n", boot_power_on_mode);
    if(u32dst_addr == NULL)
    {
        printf("malloc fail not enough memory!!!!\n");
    }

    rtk_eMMC_read(block_no, sizeof(bootloader_message), u32dst_addr);

    bootloader_message* pBootMsg = (bootloader_message*)u32dst_addr;
    if (strcmp("boot-recovery", pBootMsg->command) == 0) {
        printf("\n------------We will Enter Recovery Rescue\n");
        ret =1;
    }
#endif
#endif
return ret;
}
