#include <qc_verify_common.h>
#include "bisr_qc.h"

extern void enable_audio_clock(void);

#define DEBUG_MODE 0

int ACPU_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = (BIT15|BIT14|BIT13);
	UINT32 done_bit = (BIT15|BIT14|BIT13);
	UINT32 remap_bit = (BIT15|BIT14|BIT13);
	UINT32 fail_bit = (BIT3|BIT2|BIT1);
	UINT32 repair_bit = (BIT31|BIT30|BIT29);		

	if((rtd_inl(ACPU_BISR_DONE)&done_bit) == done_bit) //did in boot code
	{
		if((rtd_inl(ACPU_BISR_FAIL)&fail_bit) == 0) //no fail signal[2:0]
		{
			if((rtd_inl(ACPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("\n->(V)ACPU BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(ACPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(ACPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("\n->(V)ACPU BISR:Pass(redo hold remap in QC)\n");
				else
					printf("\n->(X)ACPU BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("\n->(X)ACPU BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//ACPU clk enable
		enable_audio_clock();

		rtd_maskl(ACPU_BISR_RSTN, 0xFFFdFFFF, 0x20000); //[17]ACPU PWR RSTN
		
		//BISR reset		
		rtd_maskl(ACPU_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(ACPU_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(ACPU_BISR_FAIL)&fail_bit) == 0) //no fail signal
				{
					rtd_maskl(ACPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("\n->(X)ACPU BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("\n->(X)ACPU BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(ACPU_BISR_REMAP)&remap_bit) == remap_bit)
			printf("\n->(V)ACPU BISR:Pass(do in QC)\n");
		else
		{
			printk("\n->(X)ACPU BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(ACPU_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit[31:29]
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int DISPD_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT0;
	UINT32 done_bit = BIT0;
	UINT32 remap_bit = BIT0;
	UINT32 fail_bit = BIT0; 
	UINT32 repair_bit = BIT31; 	
	
	if((rtd_inl(DISPD_BISR_DONE)&done_bit) == done_bit) //done
	{	
		if((rtd_inl(DISPD_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(DISPD_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("->(V)DISPD BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(DISPD_BISR_REMAP, ~remap_bit, remap_bit);	//hold remap
				if((rtd_inl(DISPD_BISR_REMAP)&remap_bit) == remap_bit)
					printf("->(V)DISPD BISR:Pass(redo hold remap in QC)\n");
				else
					printf("->(X)DISPD BISR:no hold remap!\n");				
			}			
		}
		else
		{
			printf("->(X)DISPD BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//BISR reset
		rtd_maskl(DISPD_BISR_RSTN, ~rstn_bit, rstn_bit);
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(DISPD_BISR_DONE)&done_bit) == done_bit) //done[0]
			{
				if((rtd_inl(DISPD_BISR_FAIL)&fail_bit) == 0) //no fail signal
				{
					rtd_maskl(DISPD_BISR_REMAP, ~remap_bit, remap_bit); //hold remap[0]				
					break;
				}
				else
				{
					printf("->(X)DISPD BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("->(X)DISPD BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}			
		if((rtd_inl(DISPD_BISR_REMAP)&remap_bit) == remap_bit)
			printf("->(V)DISPD BISR:Pass(do in QC)\n");
		else
		{
			printk("->(X)DISPD BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(DISPD_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int GPU_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT0;
	UINT32 done_bit = BIT0;
	UINT32 remap_bit = BIT0;
	UINT32 fail_bit = (BIT3|BIT2|BIT1|BIT0);
	UINT32 repair_bit = BIT0;		

	if((rtd_inl(GPU_BISR_DONE)&done_bit) == done_bit) //did in boot code
	{
		if((rtd_inl(GPU_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(GPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("\n->(V)GPU BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(GPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(GPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("\n->(V)GPU BISR:Pass(redo hold remap in QC)\n");
				else
					printf("\n->(X)GPU BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("\n->(X)GPU BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//BISR reset
		rtd_maskl(GPU_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(GPU_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(GPU_BISR_FAIL)&fail_bit) == 0) //no fail signal
				{
					rtd_maskl(GPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("\n->(X)GPU BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("\n->(X)GPU BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(GPU_BISR_REMAP)&remap_bit) == remap_bit)
			printf("\n->(V)GPU BISR:Pass(do in QC)\n");
		else
		{
			printk("\n->(X)GPU BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(GPU_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int MCU_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT0;
	UINT32 done_bit = BIT0;
	UINT32 remap_bit = BIT0;
	UINT32 fail_bit = BIT0;
	UINT32 repair_bit = BIT0;	

	if((rtd_inl(MCU_BISR_DONE)&done_bit) == done_bit)
	{
		if((rtd_inl(MCU_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(MCU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("->(V)MCU BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(MCU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(MCU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("->(V)MCU BISR:Pass(redo hold remap in QC)\n");
				else
					printf("->(X)MCU BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("->(X)MCU BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//BISR reset
		rtd_maskl(MCU_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(MCU_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(MCU_BISR_FAIL)&fail_bit) == 0) //no fail signal[1]
				{
					rtd_maskl(MCU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("->(X)MCU BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("->(X)MCU BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(MCU_BISR_REMAP)&remap_bit) == remap_bit)
			printf("->(V)MCU BISR:Pass(do in QC)\n");
		else
		{
			printk("->(X)MCU BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(MCU_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int MEMC_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT0;
	UINT32 done_bit = BIT0;
	UINT32 remap_bit = BIT0;
	UINT32 fail_bit = BIT0;
	UINT32 repair_bit = BIT0;	

	if((rtd_inl(MEMC_BISR_DONE)&done_bit) == done_bit)
	{
		if((rtd_inl(MEMC_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(MEMC_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("->(V)MEMC BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(MEMC_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(MEMC_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("->(V)MEMC BISR:Pass(redo hold remap in QC)\n");
				else
					printf("->(X)MEMC BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("->(X)MEMC BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//BISR reset
		rtd_maskl(MEMC_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(MEMC_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(MEMC_BISR_FAIL)&fail_bit) == 0) //no fail signal[1]
				{
					rtd_maskl(MEMC_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("->(X)MEMC BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("->(X)MEMC BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(MEMC_BISR_REMAP)&remap_bit) == remap_bit)
			printf("->(V)MEMC BISR:Pass(do in QC)\n");
		else
		{
			printk("->(X)MEMC BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(MEMC_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int SCPU_BISR(void)
{
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT2;
	UINT32 done_bit = BIT2;
	UINT32 remap_bit = BIT2;
	UINT32 fail_bit = 0xFF; //[7:0]
	UINT32 repair_bit = BIT0;		

	if((rtd_inl(SCPU_BISR_DONE)&done_bit) == done_bit) //did in boot code
	{
		if((rtd_inl(SCPU_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(SCPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("\n->(V)SCPU BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(SCPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(SCPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("\n->(V)SCPU BISR:Pass(redo hold remap in QC)\n");
				else
					printf("\n->(X)SCPU BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("\n->(X)SCPU BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}

		if((rtd_inl(SCPU_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
			printf("              =>needs to repair!\n");		
	}
	else	
	{
		printk("\n->(X)SCPU BISR:boot code didn't do!\n");
		bStatus = FAILED;		
	}
	
	return bStatus;
}

int SFG_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT6;
	UINT32 done_bit = BIT6;
	UINT32 remap_bit = BIT6;
	UINT32 fail_bit = BIT0;
	UINT32 repair_bit = BIT31;		

	if((rtd_inl(SFG_BISR_DONE)&done_bit) == done_bit)
	{
		if((rtd_inl(SFG_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(SFG_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("->(V)SFG BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(SFG_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(SFG_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("->(V)SFG BISR:Pass(redo hold remap in QC)\n");
				else
					printf("->(X)SFG BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("->(X)SFG BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{        
		//BISR reset
		rtd_maskl(SFG_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(SFG_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(SFG_BISR_FAIL)&fail_bit) == 0) //no fail signal[0]
				{
					rtd_maskl(SFG_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("->(X)SFG BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("->(X)SFG BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(SFG_BISR_REMAP)&remap_bit) == remap_bit)
			printf("->(V)SFG BISR:Pass(do in QC)\n");
		else
		{
			printk("->(X)SFG BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(SFG_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit[31]
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int DISPI_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT0;
	UINT32 done_bit = BIT0;
	UINT32 remap_bit = BIT0;
	UINT32 fail_bit = BIT0; 
	UINT32 repair_bit = BIT31; 	
	
	if((rtd_inl(DISPI_BISR_DONE)&done_bit) == done_bit) //done
	{	
		if((rtd_inl(DISPI_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(DISPI_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("->(V)DISPI BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(DISPI_BISR_REMAP, ~remap_bit, remap_bit);	//hold remap
				if((rtd_inl(DISPI_BISR_REMAP)&remap_bit) == remap_bit)
					printf("->(V)DISPI BISR:Pass(redo hold remap in QC)\n");
				else
					printf("->(X)DISPI BISR:no hold remap!\n");				
			}			
		}
		else
		{
			printf("->(X)DISPI BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//BISR reset
		rtd_maskl(DISPI_BISR_RSTN, ~rstn_bit, rstn_bit);
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(DISPI_BISR_DONE)&done_bit) == done_bit) //done
			{
				if((rtd_inl(DISPI_BISR_FAIL)&fail_bit) == 0) //no fail signal
				{
					rtd_maskl(DISPI_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("->(X)DISPI BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("->(X)DISPI BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}			
		if((rtd_inl(DISPI_BISR_REMAP)&remap_bit) == remap_bit)
			printf("->(V)DISPI BISR:Pass(do in QC)\n");
		else
		{
			printk("->(X)DISPI BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(DISPI_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int VCPU_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT1;
	UINT32 done_bit = BIT1;
	UINT32 remap_bit = BIT1;
	UINT32 fail_bit = BIT0;
	UINT32 repair_bit = BIT31;			

	if((rtd_inl(VCPU_BISR_DONE)&done_bit) == done_bit) //did in boot code
	{
		if((rtd_inl(VCPU_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(VCPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("\n->(V)VCPU BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(VCPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(VCPU_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("\n->(V)VCPU BISR:Pass(redo hold remap in QC)\n");
				else
					printf("\n->(X)VCPU BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("\n->(X)VCPU BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{
		//VCPU clk enable
		enable_video_clock();

		rtd_maskl(VCPU_BISR_RSTN, 0xFFFeFFFF, 0x10000); //[16]VCPU PWR RSTN
		
		//BISR reset
		rtd_maskl(VCPU_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(VCPU_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(VCPU_BISR_FAIL)&fail_bit) == 0) //no fail signal
				{
					rtd_maskl(VCPU_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("\n->(X)VCPU BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("\n->(X)VCPU BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(VCPU_BISR_REMAP)&remap_bit) == remap_bit)
			printf("\n->(V)VCPU BISR:Pass(do in QC)\n");
		else
		{
			printk("\n->(X)VCPU BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(VCPU_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

int VE_BISR(void)
{
	UINT16 i;
	UINT16 bScan = 1000;
	int   bStatus = SUCCESS;
	UINT32 rstn_bit = BIT0;
	UINT32 done_bit = BIT0;
	UINT32 remap_bit = BIT0;
	UINT32 fail_bit = BIT0;
	UINT32 repair_bit = BIT31;			

	if((rtd_inl(VE_BISR_DONE)&done_bit) == done_bit) //did in boot code
	{
		if((rtd_inl(VE_BISR_FAIL)&fail_bit) == 0) //no fail signal
		{
			if((rtd_inl(VE_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
				printf("\n->(V)VE BISR:Pass(did in boot code)\n");
			else
			{
				rtd_maskl(VE_BISR_REMAP, ~remap_bit, remap_bit); //hold remap
				if((rtd_inl(VE_BISR_REMAP)&remap_bit) == remap_bit) //hold remap
					printf("\n->(V)VE BISR:Pass(redo hold remap in QC)\n");
				else
					printf("\n->(X)VE BISR:no hold remap!\n");				
			}
		}
		else
		{
			printf("\n->(X)VE BISR:has fail flag(did in boot code)!\n");
			bStatus = FAILED;
		}
	}
	else	
	{	    
		//VCPU clk enable
		rtd_maskl(0xb8000204, 0xeFFFFFFF, BIT28);
	
		//BISR reset
		rtd_maskl(VE_BISR_RSTN, ~rstn_bit, rstn_bit);
		
		for(i=0;i<=bScan;i++)
		{
			udelay(100);
			if((rtd_inl(VE_BISR_DONE)&done_bit) == done_bit)
			{
				if((rtd_inl(VE_BISR_FAIL)&fail_bit) == 0) //no fail signal
				{
					rtd_maskl(VE_BISR_REMAP, ~remap_bit, remap_bit); //hold remap				
					break;
				}
				else
				{
					printf("\n->(X)VE BISR:has fail flag(do in QC)!\n");
					bStatus = FAILED;
				}
			}
			else if(i == bScan)
			{
				printf("\n->(X)VE BISR:can't wait done bit(do in QC)!\n");
				bStatus = FAILED;
			}
		}
		if((rtd_inl(VE_BISR_REMAP)&remap_bit) == remap_bit)
			printf("\n->(V)VE BISR:Pass(do in QC)\n");
		else
		{
			printk("\n->(X)VE BISR:no hold remap(do in QC)!\n");
			bStatus = FAILED;
		}		
	}

	if((rtd_inl(VE_BISR_REPAIR)&repair_bit) != 0) //check BISR repair bit
		printf("              =>needs to repair!\n");
	
	return bStatus;
}

BISR_QC bisr_fun_list[] = 
{
	{"ACPU_BISR",ACPU_BISR},
	{"DISPD_BISR",DISPD_BISR},
	{"GPU_BISR",GPU_BISR},	
	{"MCU_BISR",MCU_BISR},		
	{"MEMC_BISR",MEMC_BISR},
	{"SCPU_BISR",SCPU_BISR},
	{"SFG_BISR",SFG_BISR},	
	{"DISPI_BISR",DISPI_BISR},
	{"VCPU_BISR",VCPU_BISR},	
	{"VE_BISR",VE_BISR},		
};

int QC_Verifier_BISR(void)
{
	int bStatus[10];
	UINT8 i, item;

	#if DEBUG_MODE
	BISR_QC bisr;
	INT32	argc = 0;
	INT8	**argv = NULL, tNum;	
	UINT8	abyCommandBuffer[128];	

	while(1)
	{
		printk("test BISR no:");
		memset(abyCommandBuffer, 0, 128);
		GetLine(abyCommandBuffer, 128, 1);
		printk("\n");

		//Parse Argument Count by Input Data
		argc = GetArgc((const INT8 *) abyCommandBuffer);

		//Parse Argument String by Input Data
		argv = (INT8 **) GetArgv((const INT8 *) abyCommandBuffer);

		//Convert Argument String to Upper Case
		StrUpr(argv[0]);
		tNum = strtoul((const INT8 *) (argv[0]), (INT8 **) NULL, 10);

		if(tNum == 100)
			break;
		else
		{
			printf("%s\n",bisr_fun_list[tNum].BISR_flag);
			bisr_fun_list[tNum].bisr_function();	
		}
	}
	#else

	item = 10;
	
	bStatus[0] = ACPU_BISR();
	bStatus[1] = DISPD_BISR();
	bStatus[2] = GPU_BISR();
	bStatus[3] = MCU_BISR();
	bStatus[4] = MEMC_BISR();
	bStatus[5] = SCPU_BISR();	
	bStatus[6] = SFG_BISR();
	bStatus[7] = DISPI_BISR();
	bStatus[8] = VCPU_BISR();
	bStatus[9] = VE_BISR();	

	for(i = 0; i < item; i++)
	{
		if(bStatus[i] == FAILED)
			return FAILED;
	}
	#endif	
	return SUCCESS;		
} 

