/*----------------------------------------------------------------------------
*
*The confidential and proprietary information contained in this file may
*only be used by a person authorised under and to the extent permitted
*by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
*This entire notice must be reproduced on all copies of this file
*and copies of this file may only be made by a person if such person is
*permitted to do so under the terms of a subsisting license agreement
*from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------
* 
*-----------------------------------------------------------------------------
* Abstract : System Specific Functions - including interrupts
*----------------------------------------------------------------------------*/

#include "MaliFns.h"
//#include <stdio.h>
#include "../bootcode/common.h"
#include "IntHandler.h"

#define MALI_T720_JOB_IRQ 64
#define MALI_T720_MMU_IRQ 65
#define MALI_T720_GPU_IRQ 66

//#define rtd_outl(offset,val)            (*(volatile unsigned long *)(offset) = val)


void CPU_EnableInterrupts(void) {
	#ifdef RT_MALI_DV
  enable_irq();
	#endif//
};
void CPU_DisableInterrupts(void) {
	#ifdef RT_MALI_DV
  disable_irq();
	#endif//
};
void CPU_InitialiseInterrupts(void) {
  init_and_install_gic();
};

void Mali_WaitForInterrupt(void) {
 // call_wfi();
  int test ;  
  rtd_outl (0xB804e060,0x00000123) ;
  test = read32(0xB804e000);
  while (test == 0x00000000) { test = read32(0xB804e000);
                               rtd_outl (0xB804e060,test) ; 
                                                        }  //  wait posedge irq # start_job
  rtd_outl (0xB804e060,0x00000321) ; //change 'h1a to 1
 
};

void Mali_WaitForClk(int cyc)
{ 
    int count = cyc *4;
    printf ("Mali_WaitForClk: cyc=%d, count=%d\n", cyc, count);
    while (count > 0)
        count--;
};

void Mali_InstallIntHandlers(void (*callback_mali_gpu)(int, int), 
                            void (*callback_mali_mmu)(int, int), 
                            void (*callback_mali_job)(int, int)) 
{
#if MALI == t720
  printf("t720\n");
  interrupt_handler_set( MALI_T720_GPU_IRQ, callback_mali_gpu );
  interrupt_handler_set( MALI_T720_MMU_IRQ, callback_mali_mmu );
  interrupt_handler_set( MALI_T720_JOB_IRQ, callback_mali_job );

  int_enable( MALI_T720_GPU_IRQ );
  int_enable( MALI_T720_MMU_IRQ );
  int_enable( MALI_T720_JOB_IRQ );
#else
	#error "flag fail"
	
#endif
};

/* For future use. Called just before a PP job starts */
void Mali_SetupOutputFrame (void) {
 #ifdef RT_DEBUG
	printf("enter %s",__FUNCTION__);
 #endif//RT_DEBUG
};

/* For future used. Called once a job part has recieved its required interrupts */
/* It is not currently used; but would be a suitable place to position CPU cache cleaning */
/* routines - if it was required to clean/invalidate the cpu caches before performing */
/* self checking */
void Mali_JobPartDone (void) {
	#ifdef RT_DEBUG
	printf("enter %s",__FUNCTION__);
	#endif// RT_DEBUG

};

void Mali_Message(char *s) {
  printf("%s",s);
};
