/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines-t720.h"
#include "adv3_t720_mem_0.h"

/* int Check_adv3_t720_check_0_a (struct t_adv3_t720_mem_0 *p); */

int RunMaliTest_adv3_t720_part0 (void);
int RunMaliTest_adv3_t720_part1 (void);

static volatile struct t_adv3_t720_mem_0 *malidata_0;
static int mali_step, t_adv3_t720__int_count;

int RunMaliTest_adv3_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_adv3_t720_part0();
    case 1  : return RunMaliTest_adv3_t720_part1();
    default : return 0;
  };
};

int RunMaliTest_adv3_t720_part0 (void) {
  int res = 0;

  int gpuid, num_cores, l2_size, axi_width, i, as_present, js_present, core_bitmap;

  printf("RUNNING TEST: adv3\n");
  printf("  Purpose: Checks the correct operation of the JOB interrupt signal\n");
  printf("  A simple test which executes a NULL job loaded into memory for job slot 0 which then completes with a JOB interrupt.\n");

  // Get current configuration to allow testing all registers
  gpuid         = Mali_RdReg(0x20, 0, 0x0000);
  core_bitmap   = Mali_RdReg(0x20, 0, 0x0100);
  l2_size       = ((Mali_RdReg(0x20, 0, 0x0004) >> 16) & 0xFF);
  axi_width     = (1 <<((Mali_RdReg(0x20, 0, 0x0004) >> 24) & 0xFF));
  as_present    = Mali_RdReg(0x20, 0, 0x0018);
  js_present    = Mali_RdReg(0x20, 0, 0x001c);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i >>= 1;
  }
  printf("Selected configuration:\n");
  printf("  Bus width: %d\n", axi_width);
  printf("  Number of shader cores: %d\n", num_cores);
  printf("  L2 cache size: %d kB\n", ((1<<l2_size)/1024));

  Mali_Reset();
  
  /* ********************************** */
  printf ("Power on the shader cores\n");
  Mali_WrReg(0x20,0x0, SHADER_PWRON_LO     , 0xFFFFFFFF);
  printf ("Power on the Tiler\n");
  Mali_WrReg(0x20,0x0, TILER_PWRON_LO      , 0xFFFFFFFF);
  printf ("Poll the power_changed_all flag until it is set\n");
  (void) Mali_PollReg (0x20, 0x0, GPU_IRQ_RAWSTAT, 10, 1, "GPU_IRQ_RAWSTAT[POWER_CHANGED_ALL]");
  /* ********************************** */
  
  
  /* ********************************** */
  printf ("---> Setup Job Slot 0 to point to the NULL job descriptor in memory\n");
  Mali_WrReg(0x20,0x0, JS0_CONFIG_NEXT     , 0x00000000);
  Mali_WrReg(0x20,0x0, JS0_HEAD_NEXT_LO    , 0x00001000); /* the logical address of the job descriptor for job slot 0*/
  Mali_WrReg(0x20,0x0, JS0_AFFINITY_NEXT_LO, 0xFFFFFFFF); /* set the affinity to ALL cores */

  Mali_WrReg(0x20,0x0, JS1_CONFIG_NEXT     , 0x00000000);
  Mali_WrReg(0x20,0x0, JS1_HEAD_NEXT_LO    , 0x00001020); /* the logical address of the job descriptor for job slot 1*/
  Mali_WrReg(0x20,0x0, JS1_AFFINITY_NEXT_LO, 0xFFFFFFFF); /* set the affinity to ALL cores */

  Mali_WrReg(0x20,0x0, JS2_CONFIG_NEXT     , 0x00000000);
  Mali_WrReg(0x20,0x0, JS2_HEAD_NEXT_LO    , 0x00001040); /* the logical address of the job descriptor for job slot 2*/
  Mali_WrReg(0x20,0x0, JS2_AFFINITY_NEXT_LO, 0xFFFFFFFF); /* set the affinity to ALL cores */

  /* Mali_WrReg(0x20,0x0, JOB_IRQ_THROTTLE    , 0x00000FFF); */

  /* Setup the MMU with the right base address of the existing page table in memory */
  (struct t_adv3_t720_mem_0 *)malidata_0 = Mali_LdMem(&adv3_t720_mem_0,sizeof(struct t_adv3_t720_mem_0),(int)adv3_t720_mem_0.ttb);
  
  printf ("Clear all job IRQs and set the masks to 1\n");
  Mali_WrReg(0x20,0x0, JOB_IRQ_CLEAR       , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, JOB_IRQ_MASK        , 0xFFFFFFFF);
  /* ********************************** */

  t_adv3_t720__int_count = 0;
  
  printf ("Execute the jobs and wait for the done interrupt\n");
  Mali_WrReg(0x20,0x0,JS0_COMMAND_NEXT,0x00000001);
  Mali_WrReg(0x20,0x0,JS1_COMMAND_NEXT,0x00000001);
  Mali_WrReg(0x20,0x0,JS2_COMMAND_NEXT,0x00000001);
  mali_step++;
  if (res == 0) {
   return 255;
  }
  return res;
}

int RunMaliTest_adv3_t720_part1 (void) {
  int res = 0;
  int job_irq_rawstat = Mali_RdReg(0x20,0, JOB_IRQ_RAWSTAT);
  int job_irq_status  = Mali_RdReg(0x20,0, JOB_IRQ_STATUS ); 

  t_adv3_t720__int_count ++;

  if (t_adv3_t720__int_count >= 3)
  {

    /* Check that the JOB_DONE flag is set for JOB slots 0, 1 and 2 in the JOB_IRQ_RAWSTAT register */
    if (job_irq_rawstat == 0x7) 
    {
     printf ("The JOB interrupt was set correctly to %x\n", job_irq_rawstat);
    }
    else
    {
     printf ("The JOB interrupt was NOT set correctly, expected 0x00000007 got job_irq_rawstat=%x  job_irq_status=%x\n", job_irq_rawstat, job_irq_status );
     res = 1;
    }

    /* Check that the status of the jobs in Job slots 0, 1 and 2 is DONE */
    Mali_CheckReg(32, 0, JOB_IRQ_STATUS, 0x00000007);

    Mali_MaskAllInterrupts();

    mali_step++;
  
  }
  else
  {
    if (res==0)
    {
      return 255;
    }
  }

  return res;
}

