/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines-t720.h"
#include "prf1_t720_mem_0.h"
#include "prf1_t720_check_0_00.h"
#include "../bootcode/common.h"


int Check_prf1_t720_check_0_00 (struct t_prf1_t720_mem_0 *p);
int RunMaliTest_prf1_t720_part0 (void);
int RunMaliTest_prf1_t720_part1 (void);
int RunMaliTest_prf1_t720_part2 (void);

#if 0
//#define write32(offset,val)            (*(volatile unsigned long *)(offset) = val)
//#define rtd_outl(offset,val)            (*(volatile unsigned long *)(offset) = val)
//#define read32(addr) (*(volatile unsigned int *)(addr))
#endif//0

static volatile struct t_prf1_t720_mem_0 *malidata_0;
static int mali_step;

int RunMaliTest_prf1_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_prf1_t720_part0();
    case 1  : return RunMaliTest_prf1_t720_part1();
    case 2  : return RunMaliTest_prf1_t720_part2();
    default : return 0;
  };
};

int RunMaliTest_prf1_t720_part0 (void) {
  int res = 0;
  printf("RUNNING TEST: prf1\n");
  write32(0xB804e060,0x20000001);

  Mali_Reset();                                      /* reset */
  /* IGNORED */                                      /* timeout 50000 */
  /* will check file dump_color.hex */               /* check_mem ./dump_color.hex */
  /* SEE BELOW */                                    /* load_mem memory.hex */
  /* Parsed hex file. Will pagetables will be setup when frame render start detected */

  /* Filtering MMU command */;                       /* writereg 20002400 0000001d  */
  /* Filtering MMU command */;                       /* writereg 20002404 00000000  */
  /* Filtering MMU command */;                       /* writereg 20002418 00000001  */
  Mali_WrReg(0x20,0x0,0x180,0xffffffff);             /* writereg 20000180 ffffffff  */
  Mali_WrReg(0x20,0x0,0x184,0xffffffff);             /* writereg 20000184 ffffffff  */
  Mali_WrReg(0x20,0x0,0x190,0xffffffff);             /* writereg 20000190 ffffffff  */
  Mali_WrReg(0x20,0x0,0x194,0xffffffff);             /* writereg 20000194 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1a0,0xffffffff);             /* writereg 200001a0 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1a4,0xffffffff);             /* writereg 200001a4 ffffffff  */
  Mali_WrReg(0x20,0x0,0x28,0x00000081);              /* writereg 20000028 00000081  */
  Mali_WrReg(0x20,0x0,0x1008,0xffffffff);            /* writereg 20001008 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1004,0xffffffff);            /* writereg 20001004 ffffffff  */
  Mali_WrReg(0x20,0x0,0x18c0,0x00001000);            /* writereg 200018c0 00001000  */
  Mali_WrReg(0x20,0x0,0x18c4,0x00000000);            /* writereg 200018c4 00000000  */
  Mali_WrReg(0x20,0x0,0x18d0,0xffffffff);            /* writereg 200018d0 ffffffff  */
  Mali_WrReg(0x20,0x0,0x18d4,0xffffffff);            /* writereg 200018d4 ffffffff  */
  
  /* Detected Frame Start. Initialising memory*/     /* writereg 200018e0 00000001  */
  //(struct t_prf1_t720_mem_0 *)malidata_0 = Mali_LdMem(&prf1_t720_mem_0,sizeof(struct t_prf1_t720_mem_0),(int)prf1_t720_mem_0.ttb);
  malidata_0 = (struct t_prf1_t720_mem_0 *)Mali_LdMem(&prf1_t720_mem_0,sizeof(struct t_prf1_t720_mem_0),(int)prf1_t720_mem_0.ttb);
    
  /* Enable optional performance counting */
  Mali_InitPerfCounters();
  Mali_WrReg(0x20,0x0,0x18e0,0x00000001);
  mali_step++;        
                      /* wait posedge irq  */
  Mali_WaitForInterrupt();
  /* WAIT FOR INTERRUPT - unless an error - return with WFI code */
  write32(0xB804e060,0x20000002);
  if (res == 0) {
   return 255;
  }
  return res;
}

int RunMaliTest_prf1_t720_part1 (void) {
  int res = 0;
  int have_interrupts = 0;
  /* Check for any interrupt */
  write32(0xB804e060,0x30000001);
  have_interrupts = Mali_InterruptCheck(0xFFFFFFFF,0xFFFFFFFF);
  if (!have_interrupts) {
   write32(0xB804e060,0x44444444);
   jump_to_addr(0xa0003020);
   return 255;
  }

  Mali_ReadPerfCounters();
  Mali_JobPartDone();
  /* Filtering MMU command */;                       /* writereg 20002400 0000001d  */
  /* Filtering MMU command */;                       /* writereg 20002404 00000000  */
  /* Filtering MMU command */;                       /* writereg 20002418 00000001  */
  Mali_WrReg(0x20,0x0,0x180,0xffffffff);             /* writereg 20000180 ffffffff  */
  Mali_WrReg(0x20,0x0,0x184,0xffffffff);             /* writereg 20000184 ffffffff  */
  Mali_WrReg(0x20,0x0,0x190,0xffffffff);             /* writereg 20000190 ffffffff  */
  Mali_WrReg(0x20,0x0,0x194,0xffffffff);             /* writereg 20000194 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1a0,0xffffffff);             /* writereg 200001a0 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1a4,0xffffffff);             /* writereg 200001a4 ffffffff  */
  Mali_WrReg(0x20,0x0,0x28,0x00000081);              /* writereg 20000028 00000081  */
  Mali_WrReg(0x20,0x0,0x1008,0xffffffff);            /* writereg 20001008 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1004,0xffffffff);            /* writereg 20001004 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1840,0x000010c0);            /* writereg 20001840 000010c0  */
  Mali_WrReg(0x20,0x0,0x1844,0x00000000);            /* writereg 20001844 00000000  */
  Mali_WrReg(0x20,0x0,0x1850,0xffffffff);            /* writereg 20001850 ffffffff  */
  Mali_WrReg(0x20,0x0,0x1854,0xffffffff);            /* writereg 20001854 ffffffff  */
  Mali_SetupOutputFrame();                           /* writereg 20001860 00000001  */
  
  /* Enable optional performance counting */
  Mali_InitPerfCounters();
  Mali_WrReg(0x20,0x0,0x1860,0x00000001);
  mali_step++;                                       /* wait posedge irq  */
  /* WAIT FOR INTERRUPT - unless an error - return with WFI code */
  Mali_WaitForInterrupt();
  write32(0xB804e060,0x30000002);
  if (res == 0) {
   return 255;
  }
  return res;
}

int RunMaliTest_prf1_t720_part2 (void) {
  int res = 0;
  int have_interrupts = 0;
  /* Check for any interrupt */
  write32(0xB804e060,0x40000001);
  have_interrupts = Mali_InterruptCheck(0xFFFFFFFF,0xFFFFFFFF);
  if (!have_interrupts) {
   write32(0xB804e060,0x44444444);
   jump_to_addr(0xa0003020);
   return 255;
  }

  Mali_ReadPerfCounters();
  Mali_JobPartDone();
  res |= Check_prf1_t720_check_0_00(malidata_0); /* dump_mem 1340 41340 > dump_color.hex */
  Mali_MaskAllInterrupts();
  mali_step++;
  write32(0xB804e060,0x40000002);
  return res;
}

int Check_prf1_t720_check_0_00 (struct t_prf1_t720_mem_0 *p) {
  struct t_CheckData_prf1_t720_check_0_00 *r = &CheckData_prf1_t720_check_0_00;
  int res = 0;
  write32(0xB804e060,0x50000001);
  Mali_Message("Performing data check of 262144 bytes\n");
  res |= Mali_MemCmp(p->data_00001000, 0x00001340, r->data_00001340, 0, 0x330);
  res |= Mali_MemCmp(p->data_00002000, 0x00002000, r->data_00002000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00003000, 0x00003000, r->data_00003000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00004000, 0x00004000, r->data_00004000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00005000, 0x00005000, r->data_00005000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00006000, 0x00006000, r->data_00006000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00007000, 0x00007000, r->data_00007000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00008000, 0x00008000, r->data_00008000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00009000, 0x00009000, r->data_00009000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0000a000, 0x0000a000, r->data_0000a000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0000b000, 0x0000b000, r->data_0000b000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0000c000, 0x0000c000, r->data_0000c000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0000d000, 0x0000d000, r->data_0000d000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0000e000, 0x0000e000, r->data_0000e000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0000f000, 0x0000f000, r->data_0000f000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00010000, 0x00010000, r->data_00010000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00011000, 0x00011000, r->data_00011000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00012000, 0x00012000, r->data_00012000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00013000, 0x00013000, r->data_00013000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00014000, 0x00014000, r->data_00014000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00015000, 0x00015000, r->data_00015000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00016000, 0x00016000, r->data_00016000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00017000, 0x00017000, r->data_00017000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00018000, 0x00018000, r->data_00018000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00019000, 0x00019000, r->data_00019000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0001a000, 0x0001a000, r->data_0001a000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0001b000, 0x0001b000, r->data_0001b000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0001c000, 0x0001c000, r->data_0001c000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0001d000, 0x0001d000, r->data_0001d000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0001e000, 0x0001e000, r->data_0001e000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0001f000, 0x0001f000, r->data_0001f000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00020000, 0x00020000, r->data_00020000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00021000, 0x00021000, r->data_00021000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00022000, 0x00022000, r->data_00022000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00023000, 0x00023000, r->data_00023000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00024000, 0x00024000, r->data_00024000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00025000, 0x00025000, r->data_00025000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00026000, 0x00026000, r->data_00026000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00027000, 0x00027000, r->data_00027000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00028000, 0x00028000, r->data_00028000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00029000, 0x00029000, r->data_00029000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0002a000, 0x0002a000, r->data_0002a000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0002b000, 0x0002b000, r->data_0002b000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0002c000, 0x0002c000, r->data_0002c000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0002d000, 0x0002d000, r->data_0002d000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0002e000, 0x0002e000, r->data_0002e000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0002f000, 0x0002f000, r->data_0002f000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00030000, 0x00030000, r->data_00030000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00031000, 0x00031000, r->data_00031000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00032000, 0x00032000, r->data_00032000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00033000, 0x00033000, r->data_00033000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00034000, 0x00034000, r->data_00034000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00035000, 0x00035000, r->data_00035000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00036000, 0x00036000, r->data_00036000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00037000, 0x00037000, r->data_00037000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00038000, 0x00038000, r->data_00038000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00039000, 0x00039000, r->data_00039000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0003a000, 0x0003a000, r->data_0003a000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0003b000, 0x0003b000, r->data_0003b000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0003c000, 0x0003c000, r->data_0003c000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0003d000, 0x0003d000, r->data_0003d000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0003e000, 0x0003e000, r->data_0003e000, 0, 0x400);
  res |= Mali_MemCmp(p->data_0003f000, 0x0003f000, r->data_0003f000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00040000, 0x00040000, r->data_00040000, 0, 0x400);
  res |= Mali_MemCmp(p->data_00041000, 0x00041000, r->data_00041000, 0, 0x0d0);
  /* Return -1 if a compare fail */
  write32(0xB804e060,0x50000002);
  if (res) { 
   write32(0xB804e060,0x44444444);
   jump_to_addr(0xa0003020);
   return -1; 
  };
  
  Mali_Message("Check okay \n");
  return 0;
};
