/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited or its affiliates.
*
*        (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited or its affiliates.
*
* Release   : r0p0-00eac0
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines_DVALIN.h"
#include "non_coherent_DVALIN_Dump.h"
#include "non_coherent_DVALIN_Load.h"

//#define VERBOSE_CHECK 1
#define MALI_CHECKMEM 1
#define MALI_CHECK_LASTMEM 1
#define ENABLE_CCI_COUNTERS 1

#define CCI_CNT0 "[S4_0_GPU_read_any]"
#define CCI_CNT1 "[S4_A_GPU_write_any]"
#define CCI_CNT2 "[S4_3_GPU_Read_Once]"
#define CCI_CNT3 "[S4_4_GPU_ReadClean_ReadShared_ReadNotSharedDirty_ReadUnique]"
#define CCI_CNT4 "[S4_9_GPU_read_snoop_hits]"
#define CCI_CNT5 "[S4_F_GPU_WriteLineUnique]"
#define CCI_CNT6 "[S4_E_GPU_WriteUnique]"
#define CCI_EVENT0 0x080
#define CCI_EVENT1 0x08A
#define CCI_EVENT2 0x083
#define CCI_EVENT3 0x084
#define CCI_EVENT4 0x089
#define CCI_EVENT5 0x08E
#define CCI_EVENT6 0x08F

int RunMaliTest_non_coherent_DVALIN_part0 (void);
int RunMaliTest_non_coherent_DVALIN_part1 (void);

static int mali_step;
static long last_cycle_count;
static long initial_cycle_count;
int *reg0_ptr;
int *reg1_ptr;
int *reg2_ptr;
int *reg3_ptr;
int *reg4_ptr;
int *reg5_ptr;
int *reg6_ptr;
int *reg_ptr;
int *reg_ptr_1;


int RunMaliTest_non_coherent_DVALIN (int init) {
  int i = 0;

  if (init) { 
	  mali_step = 0; last_cycle_count = 0; 

#ifdef ENABLE_CCI_COUNTERS
		/*###########################################################################*/
		/*# Enable counter */
		reg0_ptr = (int *)0x60210008; *reg0_ptr = 0x1;
		reg1_ptr = (int *)0x60220008; *reg1_ptr = 0x1;
		reg2_ptr = (int *)0x60230008; *reg2_ptr = 0x1;
		reg3_ptr = (int *)0x60240008; *reg3_ptr = 0x1;
		reg4_ptr = (int *)0x60250008; *reg4_ptr = 0x1;
		reg5_ptr = (int *)0x60260008; *reg5_ptr = 0x1;
		reg6_ptr = (int *)0x60270008; *reg6_ptr = 0x1;
		
		/*# Set up CCI event output */
		reg0_ptr = (int *)0x60210000; *reg0_ptr = CCI_EVENT0;
		reg1_ptr = (int *)0x60220000; *reg1_ptr = CCI_EVENT1;
		reg2_ptr = (int *)0x60230000; *reg2_ptr = CCI_EVENT2;
		reg3_ptr = (int *)0x60240000; *reg3_ptr = CCI_EVENT3;
		reg4_ptr = (int *)0x60250000; *reg4_ptr = CCI_EVENT4;
		reg5_ptr = (int *)0x60260000; *reg5_ptr = CCI_EVENT5;
		reg6_ptr = (int *)0x60270000; *reg6_ptr = CCI_EVENT6;
		
		/*# Enable and reset PMU counters */
		reg_ptr = (int *)0x60200100; *reg_ptr = 0x40013;
		/*# Enable all interface monitors */
		reg_ptr_1 = (int *)0x60200104; *reg_ptr_1 = 0x1;
		/*###########################################################################*/
#endif
  };

  switch (mali_step) {
    case 0  : return RunMaliTest_non_coherent_DVALIN_part0();
    case 1  : return RunMaliTest_non_coherent_DVALIN_part1();
    default : return 0;
  };

};

int RunMaliTest_non_coherent_DVALIN_part0 (void) {
  int res = 0;
  unsigned long cycle_count, timestamp;
  int reg_300, reg_304;
  printf("RUNNING TEST: non_coherent_DVALIN\n");

  Mali_Reset();                                                          /* reset */
  /*###########################################################################*/
  /* ACE-Lite mode enabled */
  printf("ACE-Lite\n");
  reg_300 = Mali_RdReg(0x300);
  printf("COHERENCY_FEATURES is %08x\n", reg_300);
  reg_304 = Mali_RdReg(0x304);
  printf("COHERENCY_ENABLE is %08x\n", reg_304);
  /*###########################################################################*/

  Mali_WrReg(0x180,0xffffffff);                                 /* writereg 20000180 ffffffff  */
  Mali_WrReg(0x184,0xffffffff);                                 /* writereg 20000184 ffffffff  */
  Mali_WrReg(0x190,0xffffffff);                                 /* writereg 20000190 ffffffff  */
  Mali_WrReg(0x194,0xffffffff);                                 /* writereg 20000194 ffffffff  */
  Mali_WrReg(0x1a0,0xffffffff);                                 /* writereg 200001a0 ffffffff  */
  Mali_WrReg(0x1a4,0xffffffff);                                 /* writereg 200001a4 ffffffff  */
  Load_fpgahex();                                                        /* load_mem fpgahex.hex */
  Mali_WrReg(0x2400,0xD6000003);                                /* writereg 20002400 D6000003 */
  Mali_WrReg(0x2404,0x00000000);                                /* writereg 20002404 00000000 */
  Mali_WrReg(0x2408,0x48484848);                                /* writereg 20002408 48484848 */
  Mali_WrReg(0x240c,0x48484848);                                /* writereg 2000240c 48484848 */
  Mali_WrReg(0x2418,0x00000001);                                /* writereg 20002418 00000001  */
  Mali_WrReg(0x2440,0xD6000003);                                /* writereg 20002440 D6000003 */
  Mali_WrReg(0x2444,0x00000000);                                /* writereg 20002444 00000000 */
  Mali_WrReg(0x2448,0x48484848);                                /* writereg 20002448 48484848 */
  Mali_WrReg(0x244c,0x48484848);                                /* writereg 2000244c 48484848 */
  Mali_WrReg(0x2458,0x00000001);                                /* writereg 20002458 00000001 */
  Load_input_000001_remapped();                                          /* load_mem input_000001.remapped.hex */
  Load_input_000002_remapped();                                          /* load_mem input_000002.remapped.hex */
  Load_input_000003_remapped();                                          /* load_mem input_000003.remapped.hex */
  Load_input_000004_remapped();                                          /* load_mem input_000004.remapped.hex */
  memset((void *)0xd6036000,0,64);                                       /* clear_mem d6036000 d6036040  */
  memset((void *)0xd600b000,0,131072);                                   /* clear_mem d600b000 d602b000  */
  #ifdef MALI_CHECKMEM                                                   /* dump_mem d600b000 d602b000 > output_dmp_000001.remapped_out.hex */
  res += Check_output_dmp_000001_remapped(res);
  #endif
  
  Load_input_000007_remapped();                                          /* load_mem input_000007.remapped.hex */
  Load_input_000008_remapped();                                          /* load_mem input_000008.remapped.hex */
  Load_input_000009_remapped();                                          /* load_mem input_000009.remapped.hex */
  Load_input_000010_remapped();                                          /* load_mem input_000010.remapped.hex */
  Load_input_000011_remapped();                                          /* load_mem input_000011.remapped.hex */
  Load_input_000012_remapped();                                          /* load_mem input_000012.remapped.hex */
  Load_input_000013_remapped();                                          /* load_mem input_000013.remapped.hex */
  #ifdef MALI_CHECKMEM                                                   /* dump_mem d602b000 d602f000 > output_dmp_000002.remapped_out.hex */
  res += Check_output_dmp_000002_remapped(res);
  #endif
  
  memset((void *)0xd602f000,0,16384);                                    /* clear_mem d602f000 d6033000  */
  #ifdef MALI_CHECKMEM                                                   /* dump_mem d602f000 d6033000 > output_dmp_000003.remapped_out.hex */
  res += Check_output_dmp_000003_remapped(res);
  #endif
  
  Load_input_000015_remapped();                                          /* load_mem input_000015.remapped.hex */
  Mali_WrReg(0x18c0,0x415e2080);                                /* writereg 200018c0 415e2080 */
  Mali_WrReg(0x18c4,0x00000000);                                /* writereg 200018c4 00000000 */
  Mali_WrReg(0x18d0,0x0000000f);                                /* writereg 200018d0 0000000f */
  Mali_WrReg(0x18d4,0x00000000);                                /* writereg 200018d4 00000000 */
  Mali_WrReg(0x18d8,0x00087b00);                                /* writereg 200018d8 00087b00 */
  Mali_WrReg(0x28,0x00000081);                                  /* writereg 20000028 00000081  */
  Mali_WrReg(0x1008,0xffffffff);                                /* writereg 20001008 ffffffff  */
  Mali_WrReg(0x1004,0xffffffff);                                /* writereg 20001004 ffffffff  */
  /* Ensure all memory updates are visible to Mali */                    /* writereg 200018e0 00000001 */
  
  /* Enable optional performance counting */
  Mali_InitPerfCounters();
  printf("First Job\n");
  Mali_WrReg(0x18e0,0x00000001);
  mali_step++;                                                           /* wait posedge irq */
  /* Next step - if error - then stop now - otherwise - return with go to next step code */
  if (res) {
    Mali_MaskAllInterrupts();
    return res;
  }
  return 254;
}

int RunMaliTest_non_coherent_DVALIN_part1 (void) {
  int res = 0;
  int have_interrupts = 0;
  unsigned long cycle_count, timestamp;
  /* Check for any interrupt */
  have_interrupts = Mali_InterruptCheck(0xFFFFFFFF,0xFFFFFFFF);
  if (!have_interrupts) {
   return 255;
  }
  Mali_WrReg(0x1004,0x00000002);                                /* writereg 20001004 00000002 */
  #ifdef MALI_CHECKMEM                                          /* dump_mem d602f000 d6033000 > output_dmp_000004.remapped_out.hex */
  res += Check_output_dmp_000004_remapped(res);
  #endif
  
  //memset((void *)0xd602f000,0,16384);                                    /* clear_mem d602f000 d6033000  */
  Mali_WrReg(0x30,8); // Clean and invalidate all caches
  Mali_MaskAllInterrupts();
  mali_step++;

  /*# Set up CCI event output */
#ifdef ENABLE_CCI_COUNTERS
	reg0_ptr = (int *)0x60210004; printf("%s\t\t= %u\n", CCI_CNT0, *reg0_ptr);
    reg1_ptr = (int *)0x60220004; printf("%s\t\t= %u\n", CCI_CNT1, *reg1_ptr);
    reg2_ptr = (int *)0x60230004; printf("%s\t\t= %u\n", CCI_CNT2, *reg2_ptr);
    reg3_ptr = (int *)0x60240004; printf("%s\t\t= %u\n", CCI_CNT3, *reg3_ptr);
    reg4_ptr = (int *)0x60250004; printf("%s\t\t= %u\n", CCI_CNT4, *reg4_ptr);
    reg5_ptr = (int *)0x60260004; printf("%s\t\t= %u\n", CCI_CNT5, *reg5_ptr);
    reg6_ptr = (int *)0x60270004; printf("%s\t\t= %u\n", CCI_CNT6, *reg6_ptr);
#endif

  return res;
}

