#include <peripheral_common.h>

/*
Sophia:
For different IC, Please update these below first:
1. irda_tx_init  ---change
2. irda_ps  ---change
3. regsiter define -
4. irda_crt
5. irda_clear
*/

//-----------test define ---------------
#define Debug_irda    (1)
#define TEST_MORE    (0)

#define REPEAT_TIMES		1  //  1=>  + 1 rpt code / +1 normal key 

//-----------protocol define ---------------
typedef enum
{
	IRDA_NEC,
	IRDA_C03,			//sky
	IRDA_SONY,			//12 bit
	IRDA_SONY_15,		
	IRDA_SONY_20,		
	IRDA_SHARP,
	IRDA_RC5,
	IRDA_RC6,
	IRDA_RCA,        	
	IRDA_KANKA,
	IRDA_PANASONIC, 	//no check
	IRDA_UNKNOWN,

}IRDA_PROTOCOL_TYPE;

char* irda_protocol_name[] =
{
/*IRDA_NEC*/	"NEC",
/*IRDA_C03*/      "C03",
/*IRDA_SONY*/		"SONY",
/*IRDA_SONY_15*/	"SONY_15",
/*IRDA_SONY_20*/	"SONY_20",
/*IRDA_SHARP*/	"SHARP",
/*IRDA_RC5*/	"RC5",
/*IRDA_RC6*/	"RC6",
/*IRDA_RCA*/	"RCA",
/*IRDA_KANKA*/	"KANKA",
/*IRDA_PANASONIC*/"PANASONIC",
/*IRDA_UNKNOWN*/"UNKNOWN",
};

static UINT32 IRDA_DATA1[][5]=		//TEST KEY: Power most
{
/*IRDA_NEC*/	{0x04, 0x5F, 0xA05FFB04, 0x00000000, 0x11100111},	//NEC: addr_up=0xff; cmd_up=0xff
/*IRDA_C03*/      {0x0e, 0x0c, 0xf30c0e0e, 0x00000000, 0x11100111},  //C03: addr=0x0e, cmd=0x0c
/*IRDA_SONY*/		{0x01, 0x15, 0x09500000, 0x00000000, 0x11101001},	//sony: addr_up=0x1f; cmd_up=0xef
/*IRDA_SONY_15*/	{0x01, 0x15, 0x012a0000, 0x00000000, 0x11110001},	//sony: addr_up=0x1f; cmd_up=0xef   to be updated
/*IRDA_SONY_20*/	{0x01, 0x15, 0x00095000, 0x00000000, 0x11100101},	//sony: addr_up=0x1f; cmd_up=0xef   to be updated
/*IRDA_SHARP*/	{0x01, 0x16, 0x45820000, 0x00000000, 0x01100101},	//sharp: addr_up=0x1f; cmd_up=0xff
/*IRDA_RC5*/	{0x00, 0x0c, 0x30080000, 0x00000000, 0x01100101},   //RC5: addr_up=0x1f; cmd_up=0x3f;  other key: 0x05, 0x35, power: 0x00, 0x18, 0x30180000, 
/*IRDA_RC6*/	{0x00, 0x0c, 0x30008000, 0x00000000, 0x11100101},   //RC6: tr=0 => 0x30000000
/*IRDA_RCA*/	{0x0f, 0x2a, 0x54fab000, 0x00000000, 0x11100101},
/*IRDA_KANKA*/  {0x02, 0x0b, 0xd0400000, 0x00000000, 0x11100101},
/*IRDA_PANASONIC*/ {0x00,0x3dbd, 0xbd3d0080,0x20020000, 0x11100101},  //0x00: k0~k7;  0x3dbd: d0~d7(data code) + p0~p7(Parity Bit) 
/*IRDA_UNKNOWN*/
};

typedef enum
{
	ADDR_PART,
	CMD_PART,
	IR_RP_PART,
	IR_RP2_PART,
	IR_SR_PART,
		
}IRDA_DATA_FORMAT;

//-----------regsiter define:  misc: irda, hw ---------------
typedef enum
{
	IR_PSR,   
	IR_PER, 
	IR_PSR_UP,
	IR_PER_UP,
	IR_DATA_NUM_MAIN,
	IR_CR,
	IR_DPIR,
	IR_RPT_CTRL,
	IR_SR,
	IR_RP_LOW,
	IR_RP_HIGH,
	IR_MAIN,
		
}IRDA_ADDR_INFO;

static UINT32 IRDA_ADDR[][2]=		
{
 {0xb8061400, 0xb8061430},  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us     
 {0xb8061404, 0xb8061434},  //IR_PER: de-bounce len=1                                        
 {0xb8061408, 0xb8061438},  //IR_PSR_UP up bound                                             
 {0xb806140C, 0xb806143C},  //IR_PER_UP up bound 0x80250080, no work                              
 {0xb8061410, 0xb8061440},  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0                             
 {0xb8061414, 0xb8061444},  //IR_CR: +irrc unit enable                                       
 {0xb8061418, 0xb8061448},  //IR_DPIR: def:00a303ff                                          
 {0xb806141c, 0xb806144c},  //IR_RPT_CTRL                                                    
 {0xb8061420, 0xb8061450},  //IR_SR                                                                                       
 {0xb8061424, 0xb8061454},  //IR_RP_LOW
 {0xb8061428, 0xb8061458},  //IR_RP_HIGH 
 {0xb806142c, 0xb806145c},  //IR_MAIN: IRSBC=50Khz/20us

};

//---------regsiter define:   iso_misc_off: irda, raw -----------
#define ADDR_IR1_SF   (0xb8062408)
#define ADDR_IR1_CR    (0xb8062410)
#define ADDR_IR1_SR   (0xb8062418)
#define ADDR_IR1_RAW_CTRL  		 (0xb806241c)
#define ADDR_IR1_RAW_FF   			(0xb8062420)
#define ADDR_IR1_RAW_SAMPLE_TIME   (0xb8062424)
#define ADDR_IR1_RAW_WL  			 (0xb8062428)
#define ADDR_IR1_RAW_DEB   			(0xb806242c)
#define ADDR_IR1_RAW_SF  			 (0xb8062440)

#define  ADDR_ISR_ISO_MISC 			0xb8061000 //BIT5
#define  ADDR_ISR_ISO_MISC_OFF		0xb8062100 //BIT3
#define  BIT_IRDA_ISO_MISC_INT           (5)
#define  BIT_IRDA_ISO_MISC_OFF_INT   (3)

//--------------------irda tx----------------------------------------




void print_bin(int n)// 
{
    int l = sizeof(n)*8;//λ
    int i;
 
    for(i = l-1;i>=0; i --)
        printf("%d", (n&(1<<i)) != 0);
}




int irda_tx_init()
{
	 
	//SetGPIOOut(39, GPIO_ISO, 0xb8060224,31,28,0xf,1);	 //gpio_ps_as_irtx: ST_GPIO_10 =>bit12   
	//SetGPIOOut(40, GPIO_ISO, 0xb8060224,23,20,0xf,1);	 //gpio_ps_as_irtx: ST_GPIO_40>bit13  
	//SetGPIOOut(21, GPIO_ISO, 0xb8060204,19,16,0xf,1);	 //gpio_ps_as_irtx: ST_GPIO_21      
#if defined(CONFIG_RTK_B27_QC)
                SetGPIOOut(21, GPIO_ISO, 0xb8060204,19,16,0xf,1);	 //gpio_ps_as_irtx: ST_GPIO_21
#elif defined(CONFIG_RTK_B22_QC)
                SetGPIOOut(39, GPIO_ISO, 0xb8060224,31,28,0xf,1);	 //gpio_ps_as_irtx: ST_GPIO_10 =>bit12   
#endif 	

}

int irda_tx(char* protocol, UINT32 irAddr, UINT32 irCommand, UINT8 nRepeatTimes)
{
	//for test nec
	/*
	SetGPOHL(1);//gpo -->high	
	udelay(100000); 
	SetGPOHL(0);//gpo -->high	
	udelay(9000); 
	SetGPOHL(1);//gpo -->high	
	udelay(50000); 
	*/
	
	SetGPOHL(1);//gpo -->high	
	udelay(100000); 
	//udelay(50000); 

 	Irda_Out(protocol, irAddr, irCommand,nRepeatTimes,0);		//contain log
}

//--------------------irda rx init----------------------------------------

void irda_crt()
{
	//clk = 0  rst = 0  clk = 1  clk = 0  rst = 1  clk = 1
	rtd_outl(0xb8060044,0x4); 		//clken_irda:0
	rtd_outl(0xb8060040,0x10); 	//clken_irda_off:0	

	rtd_outl(0xb8060034,0x4); 		//RSTN_irda:0
	rtd_outl(0xb8060030,0x10); 	//RSTN_irda_off:0      

	rtd_outl(0xb8060044,0x5); 		//clken_irda:1
	rtd_outl(0xb8060040,0x11); 	//clken_irda_off:1

	rtd_outl(0xb8060044,0x4); 		//clken_irda:0
	rtd_outl(0xb8060040,0x10); 	//clken_irda_off:0		

	rtd_outl(0xb8060034,0x5); 		//RSTN_irda:1
	rtd_outl(0xb8060030,0x11); 	//RSTN_irda_off:1    
	
	rtd_outl(0xb8060044,0x5); 		//clken_irda:1
	rtd_outl(0xb8060040, 0x11);       //clken_irda_off:1	

}

void irda_ps()
{
	rtd_part_outl(0xb806024c,21,20,0x0);//src0:input mux
	rtd_part_outl(0xb8060210,15,12,0x8);//(null):R659  Ps_Reg:st_gpio_10_ps 
}

//--------------------------------------------------------------------

void irda_hw_clear(int mainID)
{
	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];

	//clear fifo
	rtd_part_outl(ADDR_IR_MAIN, 20, 20, 0x1);

	//clear status
	rtd_maskl(ADDR_IR_SR,0xffffffff,0x00000000);	//IR_SR(wrclr):leader_wrong;data_still_toggle;rmt_big;raw_fifo_ov;raw_fifo_val;IRRF;IRDVF
	rtd_inl(ADDR_IR_SR);				// --> if 1?
	rtd_inl(ADDR_ISR_ISO_MISC);				//ISR:iso_misc isr ->misc irda

}

void irda_hw_status(int mainID)
{
	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];


	//=>state & read buffer0....
	printk("[INT] ADDR_ISR_ISO_MISC:0x%08X \n", rtd_inl(ADDR_ISR_ISO_MISC));
 	printk("[Status] ADDR_IR_SR:0x%08X \n", rtd_inl(ADDR_IR_SR));
	printk("Clear IRDVF \n");
	rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
	printk("[FIFO][Buffer0] ADDR_IR_RP_LOW:0x%08X; ADDR_IR_RP_HIGH\n", rtd_inl(ADDR_IR_RP_LOW),rtd_inl(ADDR_IR_RP_HIGH));
	
	
	//=>state & read buffer1....
	printk("[INT] ADDR_ISR_ISO_MISC:0x%08X \n", rtd_inl(ADDR_ISR_ISO_MISC));
 	printk("[Status] ADDR_IR_SR:0x%08X \n", rtd_inl(ADDR_IR_SR));
	printk("Clear IRDVF \n");
	rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
	printk("[FIFO][Buffer1] ADDR_IR_RP_LOW:0x%08X; ADDR_IR_RP_HIGH\n", rtd_inl(ADDR_IR_RP_LOW),rtd_inl(ADDR_IR_RP_HIGH));

}


//--------------------irda rx hw----------------------------------------

void irda_rx_hw_prepare_new(int  iProtocol, int mainID)
{
	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];

	irda_hw_clear(mainID);

	switch(iProtocol)
	{
		case IRDA_NEC:
			//protocol
			rtd_outl(ADDR_IR_PSR,0x4E13133B);  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x014e1327);  //IR_PER: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x80303060);  //IR_PSR_UP up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x802500ff);  //IR_PER_UP up bound 0x80250080, no work                                           
			rtd_outl(ADDR_IR_DPIR,0x000003ff);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x00000020);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00004a01);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		case IRDA_C03:
			rtd_outl(ADDR_IR_PSR,0x27101034);  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x00252525);  //IR_PER: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x40302060);  //IR_PSR_UP up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x404000FF);  //IR_PER_UP up bound: 0x40400040, no work                                           
			rtd_outl(ADDR_IR_DPIR,0x000003ff);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x00000020);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00004a01);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL

			rtd_part_outl(ADDR_IR_PER_UP,15,8,0x80);	//IR_PER_UP_RC5_RC6:End_pattern_rc56(*4), ml<0.56 80=>10.24ms<high; but>longest high in wave SL/RP_SL
		break;
		
		case IRDA_SONY:
		case IRDA_SONY_15:
		case IRDA_SONY_20:
			rtd_outl(ADDR_IR_PSR,0x18141430);  //IR_PSR_RC5_RC6: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x01000006);  //IR_PER_RC5_RC6: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x20282850);  //IR_PSR_UP_RC5_RC6 up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x000000ff);  //IR_PER_UP_RC5_RC6 up bound, 0x0000000f, no work                                           
			rtd_outl(ADDR_IR_DPIR,0x00000110);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x000f0c14);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00001a01);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		case IRDA_SHARP:
			rtd_outl(ADDR_IR_PSR,0x000c173f);  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x01000000);  //IR_PER: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x00203070);  //IR_PSR_UP up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x00000000);  //IR_PER_UP up bound                                           
			rtd_outl(ADDR_IR_DPIR,0x000003ff);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x0000000f);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00000801);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		case IRDA_RC5:
			rtd_outl(ADDR_IR_PSR,0x00242424);  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x01000000);  //IR_PER: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x00303030);  //IR_PSR_UP up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x00000000);  //IR_PER_UP up bound                                           
			rtd_outl(ADDR_IR_DPIR,0x00000d00);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x0000000d);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00000401);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		case IRDA_RC6:
			rtd_outl(ADDR_IR_PSR,0x1a101010);  //IR_PSR_RC5_RC6: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x02000008);  //IR_PER_RC5_RC6: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x25232020);  //IR_PSR_UP_RC5_RC6 up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x000000ff);  //IR_PER_UP_RC5_RC6 up bound, 0x00000010, no work                                           
			rtd_outl(ADDR_IR_DPIR,0x00000d00);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x00000014);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x0000a601);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		case IRDA_RCA:
			rtd_outl(ADDR_IR_PSR,0x25154721);  //IR_PSR_RC5_RC6: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x01000025);  //IR_PER_RC5_RC6: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x3c227270);  //IR_PSR_UP_RC5_RC6 up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x000000ff);  //IR_PER_UP_RC5_RC6 up bound                                           
			rtd_outl(ADDR_IR_DPIR,0x000000fa);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x00000018);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00000a01);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		case IRDA_KANKA:
			rtd_outl(ADDR_IR_PSR,0x1e143c64);  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x0300001a);  //IR_PER: de-bounce len=1 0x0300001a                                  
			rtd_outl(ADDR_IR_PSR_UP,0x2d245a96);  //IR_PSR_UP up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x000000ff);  //IR_PER_UP up bound, 0x0000002D,OK?                                           
			rtd_outl(ADDR_IR_DPIR,0x0000030C);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x00000010);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00000a01);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
				
		case IRDA_PANASONIC:
			rtd_outl(ADDR_IR_PSR,0x24100e30);  //IR_PSR: burst=6.24ms, module=380us, zero=380us, one=1180us               
			rtd_outl(ADDR_IR_PER,0x01000010);  //IR_PER: de-bounce len=1                                   
			rtd_outl(ADDR_IR_PSR_UP,0x30202050);  //IR_PSR_UP up bound                                           
			rtd_outl(ADDR_IR_PER_UP,0x000000ff);  //IR_PER_UP up bound, 0x00000020, no work                                           
			rtd_outl(ADDR_IR_DPIR,0x00000900);  //IR_DPIR: def:00a303ff                           
			rtd_outl(ADDR_IR_MAIN,0x0001021b);  //IR_MAIN: IRSBC=50Khz/20us, idle time=110ms, one command time=0? 
			rtd_outl(ADDR_IR_DATA_NUM_MAIN,0x00000030);  //IR_DATA_NUM_MAIN: 21:16; 13:8; 5:0
			rtd_outl(ADDR_IR_CR,0x00000a01);  //IR_CR: +irrc unit enable  
			rtd_outl(ADDR_IR_RPT_CTRL,0x00000000);  //IR_RPT_CTRL
		break;
		
		default:
		break;

	};
	
}

void irda_rx_hw_prepare_rpt(int  iProtocol, int mainID, int rptNum)
{
	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];

	switch(iProtocol)
	{
		case IRDA_NEC:
		case IRDA_C03:
			rtd_part_outl(ADDR_IR_RPT_CTRL, 15, 0, 0x1ff);
			rtd_part_outl(ADDR_IR_RPT_CTRL, 19, 16, rptNum);
		break;
			
		default:
		break;
	};
	
}

//------------------------------ irda_rx_raw------------------------------------------------
void irda_raw_clear()
{
	//clear FIFO
	rtd_part_outl(ADDR_IR1_RAW_CTRL,24,24,1);		//IR_RAW_CTRL: stop sampling=1 to clear fifo
	rtd_part_outl(ADDR_IR1_RAW_CTRL,24,24,0);		//IR_RAW_CTRL: stop sampling=0;
	rtd_outl(ADDR_IR1_RAW_FF,0x00000000);		  	//IR_RAW_FF; no effect 
	rtd_outl(ADDR_IR1_RAW_SAMPLE_TIME,0x00000000);	//IR_RAW_SAMPLE_TIME, no effect
	//clear status
	rtd_outl(ADDR_IR1_SR,0xffffffff);			//IR_SR:raw_fifo_ov;raw_fifo_val; wr_clr after stop_sampling
	//clear INT
	rtd_inl(ADDR_ISR_ISO_MISC_OFF);	//ISR:iso_misc off ->misc off irda; clear with local clear IR_SR	  
}

void irda_raw_status()
{
	//=>org state........
#if 0 
	//fifo
	//rtd_inl(ADDR_IR1_RAW_FF);	//IR_RAW_FF:RAW mode RXFIFO read-port
	rtd_inl(ADDR_IR1_RAW_SAMPLE_TIME);	//IR_RAW_SAMPLE_TIME
	rtd_inl(ADDR_IR1_RAW_WL);  //IR_RAW_WL         
	//status
	rtd_inl(ADDR_IR1_SR);	//IR_SR:raw_fifo_ov;raw_fifo_val;
	//INT
	rtd_inl(ADDR_ISR_ISO_MISC_OFF);	//ISR:iso_misc off ->misc off irda
#endif 
	
	
 	printk("[Status][Ori]ADDR_IR1_RAW_SAMPLE_TIME:0x%08X , ADDR_IR1_RAW_WL:0x%08X ,\
 	ADDR_IR1_SR:0x%08x INT:0x%08X \n", rtd_inl(ADDR_IR1_RAW_SAMPLE_TIME), rtd_inl(ADDR_IR1_RAW_WL),\
 	rtd_inl(ADDR_IR1_SR),rtd_inl(ADDR_ISR_ISO_MISC_OFF));

#if 0 
	//=>state after read....
	//fifo
	rtd_inl(ADDR_IR1_RAW_FF);	//IR_RAW_FF:RAW mode RXFIFO read-port
	rtd_inl(ADDR_IR1_RAW_SAMPLE_TIME);	//IR_RAW_SAMPLE_TIME
	rtd_inl(ADDR_IR1_RAW_WL);  //IR_RAW_WL         
	//status
	rtd_inl(ADDR_IR1_SR);	//IR_SR:raw_fifo_ov;raw_fifo_val;
	//INT
	rtd_inl(ADDR_ISR_ISO_MISC_OFF);	//ISR:iso_misc off ->misc off irda	 
#endif

	
 	printk("[Status][2nd]ADDR_IR1_RAW_SAMPLE_TIME:0x%08X , ADDR_IR1_RAW_WL:0x%08X ,\
 	ADDR_IR1_SR:0x%08x INT:0x%08X \n", rtd_inl(ADDR_IR1_RAW_SAMPLE_TIME), rtd_inl(ADDR_IR1_RAW_WL),\
 	rtd_inl(ADDR_IR1_SR),rtd_inl(ADDR_ISR_ISO_MISC_OFF));


}

UINT32  irda_check_raw_int()
{
	return rtd_part_inl(ADDR_ISR_ISO_MISC_OFF,3,3);
}


void irda_rx_raw_prepare(int  iProtocol, UINT32 thresholdVal)
{
	irda_raw_clear();
	irda_raw_status();

	switch(iProtocol)
	{
		case IRDA_NEC:
		default:
			//rtd_outl(ADDR_IR1_RAW_CTRL,0x00000010);	//*IR_RAW_CTRL: stop sample & stop time & fifo_thr 
			rtd_outl(ADDR_IR1_RAW_CTRL,thresholdVal);
			rtd_outl(ADDR_IR1_RAW_DEB,0x00000a8b);  //*IR_RAW_DEB:de-bounce len=0;CLK =10khz,100us                                                                   
			rtd_outl(ADDR_IR1_RAW_SF,0x00001517);	//*IR_RAW_SF: use 5000 Hz sampling rate
			rtd_outl(ADDR_IR1_CR,0x00007000); 	//IR_CR:raw_fifo_ov_en;raw_fifo_val_en;RAW_EN
		break;
	};
}

//------------------------------------------------------------------------

int Verify_Irda_RAW_Test(int iProtocol)
{
	UINT32 i;
	UINT32 getVal;
	UINT32 Ir_err=0;
	UINT32 IR_RP_FIFO;
	int timeout = 0;
	UINT32 threshold = 0x10;
	int fifoLen = 32;
	bool bGetInt = 0;
	UINT32 Fifo[32];

	char* protocol =  irda_protocol_name[iProtocol]; 
	UINT32 IR_TX_addr = IRDA_DATA1[iProtocol][ADDR_PART];
	UINT32 IR_TX_cmd = IRDA_DATA1[iProtocol][CMD_PART];

	printk(" \x1b[33m\t[BEGIN]Irda Verify: %s protocol \x1b[0m\n", protocol);

	//init irda_crt
	irda_crt();
	
	//init irrx_ps
	irda_ps();	
	
	//===irda settings=========
	irda_rx_raw_prepare(iProtocol,threshold);//13*32*200us => 83.2ms <110ms  for do not want get repeat key.

    	//====input irrx data==========
	//gpio_ps_as_irtx: st_gpio_7, PIN105
	irda_tx(protocol, IR_TX_addr, IR_TX_cmd, 0);// do not reapeat
	//irda_tx(protocol, IR_TX_addr, IR_TX_cmd, REPEAT_TIMES);// reapeat 1 time

      	timeout = 5000;
	while(timeout)
	{
		if(irda_check_raw_int())//get int
		{
			bGetInt = true;
			for(fifoLen = 0;fifoLen<32;fifoLen++)				
			{
			       getVal = rtd_inl(ADDR_IR1_RAW_FF);
				printk("[%d] FIFO: %08x \n", fifoLen,getVal);
				printk("[%d] WaterLevel: %08x \n", fifoLen,rtd_inl(ADDR_IR1_RAW_WL));
				printk("[%d] SampleCnt: %08x \n", fifoLen,rtd_inl(ADDR_IR1_RAW_SAMPLE_TIME));
				printk("[%d] Status: %08x\n\n", fifoLen,rtd_inl(ADDR_IR1_SR));
				Fifo[fifoLen] = getVal;
		
			}
			//print fifo with binary
			printk(" %s 's FIFO:\n",protocol);
			for(fifoLen = 0;fifoLen<32;fifoLen++)
			{
				print_bin(Fifo[fifoLen]);	
			}

			//Clear FIfo 
			printk("\t[FIFO & Status Clear]:\n");
			irda_raw_clear();
			//Check Status Again
			printk("\t[Check Status after clear]:\n");
			irda_raw_status();
			
			printk("\n");
			printk("[END][SUCCESS]Irda Verify: Raw Mode : %s\n\n", protocol);

			return 0;

		}	
	udelay(100);
	timeout--;
	}

	if(!bGetInt)
	{
		printk("\x1b[31m\t[INT NOT Raised!] Irda-Verify Raw Failed!\x1b[0m\n\n");
		printk("FIFO: %08x \n", rtd_inl(ADDR_IR1_RAW_FF));
		printk("WaterLevel: %08x \n", rtd_inl(ADDR_IR1_RAW_WL));
		printk("SampleCnt: %08x \n", rtd_inl(ADDR_IR1_RAW_SAMPLE_TIME));
		printk("Status: %08x \n\n", rtd_inl(ADDR_IR1_SR));
		return 1;
	}

}

//--------------------------------------------------------------------------------

int Verify_Irda_Single(int iProtocol,  int  mainID)
{
	UINT32 i;
	int timeout = 0;
	UINT32 getVal = 0;
	UINT32 getVal1 = 0;
	UINT32 Ir_err=0;
	UINT32 flag = 0;
	UINT32 rptflag = 0;
	UINT32 rptcnt = 0;
	UINT32 intRaise = 0;
	UINT32 intClear = 0;

	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];
	
	char* protocol =  irda_protocol_name[iProtocol]; 
	UINT32 IR_TX_addr = IRDA_DATA1[iProtocol][ADDR_PART];
	UINT32 IR_TX_cmd = IRDA_DATA1[iProtocol][CMD_PART];
	UINT32 IR_RP_data = IRDA_DATA1[iProtocol][IR_RP_PART];
	UINT32 IR_RP2_data = IRDA_DATA1[iProtocol][IR_RP2_PART];
	UINT32 IR_SR_data = IRDA_DATA1[iProtocol][IR_SR_PART];	

	printk("\n");
	printk("[BEGIN]Irda Verify: %s protocol -> Check [IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X]? IR_SR = 0x%08x \n", protocol, IR_RP_data, IR_RP2_data, IR_SR_data);

	//init irda_crt
	 irda_crt();

	//init irrx_ps
	irda_ps();

	//===irda settings=========
	irda_rx_hw_prepare_new(iProtocol, mainID);	//clear & settting
	
    	//====input irrx data==========
	irda_tx(protocol, IR_TX_addr, IR_TX_cmd, REPEAT_TIMES);	//REPEAT_TIMES
	udelay(5000);
	
	//======check irrx received data=========
	Ir_err=1;
#if 1
	timeout = 500;
	do
	{
		flag = rtd_inl(ADDR_IR_SR); 
		if(flag&0x1)	//IRDVF
		{
		       //check INT:
		       intRaise = rtd_part_inl(ADDR_ISR_ISO_MISC,BIT_IRDA_ISO_MISC_INT,BIT_IRDA_ISO_MISC_INT);
			printk("[INT] ADDR_ISR_ISO_MISC:0x%08X \n", rtd_inl(ADDR_ISR_ISO_MISC));
			   
			rptflag = (flag&0x10)>>4; 	//IRRF
			udelay(100);
			
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data && flag == IR_SR_data){		
				rptcnt++;

			}
			flag = rtd_inl(ADDR_IR_SR); 
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(iProtocol == IRDA_SHARP)
				IR_RP_data = 0xba420000;		
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data && flag == 0){		
				rptcnt++;	
			}
			
		      //check INT:
		      intClear = rtd_part_inl(ADDR_ISR_ISO_MISC,BIT_IRDA_ISO_MISC_INT,BIT_IRDA_ISO_MISC_INT);		    
			printk("[INT] ADDR_ISR_ISO_MISC:0x%08X \n", rtd_inl(ADDR_ISR_ISO_MISC));
			if(rptcnt>1)	
			{
			      irda_hw_clear(mainID);
				if(!(intClear==0)&&(intRaise==1))//Check INT
				{				       
					Ir_err=1;		
					printk("\x1b[31m\t[INT Check Failed!\x1b[0m\n");				
				}
				else
				{
					Ir_err=0;	
					break;
				}
			}
			
		}
		timeout--;
		udelay(1000);
		
	}while(timeout>0);		
#endif


	if(Ir_err==0)
	{		
		printk("[END][SUCCESS]Irda Verify: %s protocol \n", protocol);
		return 0	;
	}
	else
	{
		printk("\x1b[31m--->?[END][FAIL]Irda Verify: %s protocol -> timeout: %d, IR_SR =0x%08x \x1b[0m\n", protocol, timeout, flag);
		//printk("[END][FAIL]Irda Verify: %s protocol,  flag =%X \n", protocol, flag);
		return 1;
	}

}

int Verify_Irda_Main0_Main1(int iProtocol,  int  iProtocol1)
{
	UINT32 i;
	int timeout = 0;
	UINT32 getVal = 0;
	UINT32 getVal1 = 0;
	UINT32 Ir_err=0;
	UINT32 Ir_err1=0;
	UINT32 flag = 0;
	UINT32 rptflag = 0;
	UINT32 rptcnt = 0;

	int mainID = 0;
	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];
	
	char* protocol =  irda_protocol_name[iProtocol]; 
	UINT32 IR_TX_addr = IRDA_DATA1[iProtocol][ADDR_PART];
	UINT32 IR_TX_cmd = IRDA_DATA1[iProtocol][CMD_PART];
	UINT32 IR_RP_data = IRDA_DATA1[iProtocol][IR_RP_PART];
	UINT32 IR_RP2_data = IRDA_DATA1[iProtocol][IR_RP2_PART];
	UINT32 IR_SR_data = IRDA_DATA1[iProtocol][IR_SR_PART];	

	mainID = 1;
	UINT32 ADDR_IR_PSR1 = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER1 = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP1 = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP1 = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN1 = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR1 = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR1 = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL1 = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR1 = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW1 = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH1 = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN1 = IRDA_ADDR[IR_MAIN][mainID];
	
	char* protocol1 =  irda_protocol_name[iProtocol1]; 
	UINT32 IR_TX_addr1 = IRDA_DATA1[iProtocol1][ADDR_PART];
	UINT32 IR_TX_cmd1 = IRDA_DATA1[iProtocol1][CMD_PART];
	UINT32 IR_RP_data1 = IRDA_DATA1[iProtocol1][IR_RP_PART];
	UINT32 IR_RP2_data1 = IRDA_DATA1[iProtocol1][IR_RP2_PART];
	UINT32 IR_SR_data1 = IRDA_DATA1[iProtocol1][IR_SR_PART];	
	
	printk("\n");
	printk(" ============= Verify main0 & main1: %s  +  %s =============\n",  protocol, protocol1);

	//init irda_crt
	 irda_crt();

	//init irrx_ps
	irda_ps();

	//===irda settings=========
	irda_rx_hw_prepare_new(iProtocol, 0);	//clear & settting
	irda_rx_hw_prepare_new(iProtocol1, 1);	//clear & settting

    	//====input irrx data==========
	irda_tx(protocol, IR_TX_addr, IR_TX_cmd, REPEAT_TIMES);		//REPEAT_TIMES
	irda_tx(protocol1, IR_TX_addr1, IR_TX_cmd1, REPEAT_TIMES);	//REPEAT_TIMES
	udelay(5000);
	
#if 1
	//======check irrx received data for main0=========
	printk("[BEGIN]main0 Irda Verify: %s protocol -> Check [IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X]?  IR_SR=0x%08x \n", protocol, IR_RP_data, IR_RP2_data, IR_SR_data);
	Ir_err=1;
	rptcnt = 0;
	timeout = 500;
	do
	{
		flag = rtd_inl(ADDR_IR_SR); 
		if(flag&0x1)	//IRDVF
		{
			rptflag = (flag&0x10)>>4; 	//IRRF
			udelay(100);
			
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data && flag == IR_SR_data){		
				rptcnt++;

			}
			flag = rtd_inl(ADDR_IR_SR); 
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data && flag == 0){		
				rptcnt++;	
			}
			if(rptcnt>0)	
			{
				Ir_err=0;
				irda_hw_clear(0);				
				break;
			}
			
		}
		timeout--;
		udelay(1000);
		
	}while(timeout>0);		
	if(Ir_err !=0)
	{
		printk("\x1b[31m--->? [END][FAIL]Irda Verify: %s protocol -> timeout: %d, IR_SR =0x%08x \x1b[0m\n", protocol, timeout, flag);
	}
#endif

#if 1
	//======check irrx received data for main1=========
	printk("[BEGIN]main1 Irda Verify: %s protocol -> Check [IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X]?  IR_SR=0x%08x \n", protocol1, IR_RP_data1, IR_RP2_data1, IR_SR_data1);
	Ir_err1=1;
	rptcnt = 0;
	timeout = 500;
	do
	{
		flag = rtd_inl(ADDR_IR_SR1); 
		if(flag&0x1)	//IRDVF
		{
			rptflag = (flag&0x10)>>4; 	//IRRF
			udelay(100);
			
			//fifo
			rtd_part_outl(ADDR_IR_SR1, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH1);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW1);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(getVal==IR_RP_data1  && getVal1 == IR_RP2_data1 && flag == IR_SR_data1){		
				rptcnt++;
			}
			flag = rtd_inl(ADDR_IR_SR1); 
			//fifo
			rtd_part_outl(ADDR_IR_SR1, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH1);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW1);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(getVal==IR_RP_data1  && getVal1 == IR_RP2_data1 && flag == 0){		
				rptcnt++;	
			}
			if(rptcnt>0)	
			{
				Ir_err1=0;
				irda_hw_clear(1);				
				break;
			}
			
		}
		timeout--;
		udelay(1000);
		
	}while(timeout>0);		
	if(Ir_err1 !=0)
	{
		printk("\x1b[31m--->? [END][FAIL]Irda Verify: %s protocol -> timeout: %d, IR_SR =0x%08x \x1b[0m\n", protocol1, timeout, flag);
	}
#endif

	if((Ir_err |Ir_err1) ==0)
	{		
		printk("[END][SUCCESS]Irda Verify: main0: %s  &  main1: %s \n", protocol, protocol1);
		return 0	;
	}
	else
	{
		return 1;
	}

}

int Verify_Irda_EndPattern(int iProtocol,  int  mainID,  bool bVarLenEn,  UINT32 endPattern, bool bGetNull)
{
	UINT32 i;
	int timeout = 0;
	UINT32 getVal = 0;
	UINT32 getVal1 = 0;
	UINT32 Ir_err=0;
	UINT32 flag = 0;
	UINT32 rptflag = 0;
	UINT32 rptcnt = 0;
	UINT32 varLen = 0;

	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];
	
	char* protocol =  irda_protocol_name[iProtocol]; 
	UINT32 IR_TX_addr = IRDA_DATA1[iProtocol][ADDR_PART];
	UINT32 IR_TX_cmd = IRDA_DATA1[iProtocol][CMD_PART];
	UINT32 IR_RP_data = IRDA_DATA1[iProtocol][IR_RP_PART];
	UINT32 IR_RP2_data = IRDA_DATA1[iProtocol][IR_RP2_PART];
	UINT32 IR_SR_data = IRDA_DATA1[iProtocol][IR_SR_PART];
	
	if(bGetNull)
	{
		IR_RP_data = 0;
		IR_RP2_data = 0;
		//IR_SR_data [5] set 1
		if(GetFieldValue(IR_SR_data,4,4))// check if have repeat key
			IR_SR_data = SetFieldValue(IR_SR_data, 5,5,1);// repeat invalid would raise!

		if(iProtocol == IRDA_C03) //C03's data detect not raise! :JIIR:RL6583-3446
			IR_SR_data = SetFieldValue(IR_SR_data, 8,8,0);// repeat invalid would raise!
	}

	printk("\n");
	printk("[BEGIN]Irda Verify: %s_%d_0x%x -> Check [IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X]? IR_SR = 0x%08x\n", protocol, bVarLenEn, endPattern, IR_RP_data, IR_RP2_data, IR_SR_data);

	//init irda_crt
	 irda_crt();

	//init irrx_ps
	irda_ps();

	//===irda settings=========
	irda_rx_hw_prepare_new(iProtocol, mainID);	//clear & settting
	rtd_part_outl(ADDR_IR_PER_UP, 15, 8, endPattern);
	rtd_part_outl(ADDR_IR_CR, 23, 23, bVarLenEn);
	
    	//====input irrx data==========
	irda_tx(protocol, IR_TX_addr, IR_TX_cmd, REPEAT_TIMES);	//REPEAT_TIMES
	udelay(5000);
	
	//======check irrx received data=========
	Ir_err=1;
#if 1
	timeout = 500;
	do
	{
		flag = rtd_inl(ADDR_IR_SR); 
		if(flag&0x1)	//IRDVF
		{
			rptflag = (flag&0x10)>>4; 	//IRRF
			varLen = rtd_part_inl(ADDR_IR_CR, 29, 24);
			udelay(100);
			
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  varlen=0x%x \n", getVal, getVal1, flag, varLen);
			
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data && flag == IR_SR_data){		
				rptcnt++;

			}

			flag = rtd_inl(ADDR_IR_SR); 
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  varlen=0x%x \n", getVal, getVal1, flag, varLen);
			if(iProtocol == IRDA_SHARP && !bGetNull)
				IR_RP_data = 0xba420000;		
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data && flag == 0){		
				rptcnt++;	
			}
			if(rptcnt>1)	
			{
				Ir_err=0;
				irda_hw_clear(mainID);				
				break;
			}
			
		}
		timeout--;
		udelay(1000);
		
	}while(timeout>0);		
#endif


	if(Ir_err==0)
	{		
		printk("[END][SUCCESS]Irda Verify: %s protocol \n", protocol);
		return 0	;
	}
	else
	{
		printk("\x1b[31m--->? [END][FAIL]Irda Verify: %s protocol -> timeout: %d, IR_SR =0x%08x  \x1b[0m\n", protocol, timeout, flag);
		return 1;
	}

}


int Verify_Irda_Repeat(int iProtocol,  int  mainID,  int rptNum, int rptTimes, bool bRaised)
{
	UINT32 i;
	int timeout = 0;
	UINT32 getVal, getVal1;
	UINT32 Ir_err=0;
	UINT32 Ir_err2= 0;
	UINT32 flag = 0;
	UINT32 rptflag = 0;
	UINT32 rptcnt = 0;
	
	UINT32 ADDR_IR_PSR = IRDA_ADDR[IR_PSR][mainID];
	UINT32 ADDR_IR_PER = IRDA_ADDR[IR_PER][mainID];
	UINT32 ADDR_IR_PSR_UP = IRDA_ADDR[IR_PSR_UP][mainID];
	UINT32 ADDR_IR_PER_UP = IRDA_ADDR[IR_PER_UP][mainID];
	UINT32 ADDR_IR_DATA_NUM_MAIN = IRDA_ADDR[IR_DATA_NUM_MAIN][mainID];
	UINT32 ADDR_IR_CR = IRDA_ADDR[IR_CR][mainID];
	UINT32 ADDR_IR_DPIR = IRDA_ADDR[IR_DPIR][mainID];
	UINT32 ADDR_IR_RPT_CTRL = IRDA_ADDR[IR_RPT_CTRL][mainID];
	UINT32 ADDR_IR_SR = IRDA_ADDR[IR_SR][mainID];
	UINT32 ADDR_IR_RP_LOW = IRDA_ADDR[IR_RP_LOW][mainID];
	UINT32 ADDR_IR_RP_HIGH = IRDA_ADDR[IR_RP_HIGH][mainID];
	UINT32 ADDR_IR_MAIN = IRDA_ADDR[IR_MAIN][mainID];
	
	char* protocol =  irda_protocol_name[iProtocol]; 
	UINT32 IR_TX_addr = IRDA_DATA1[iProtocol][ADDR_PART];
	UINT32 IR_TX_cmd = IRDA_DATA1[iProtocol][CMD_PART];
	UINT32 IR_RP_data = IRDA_DATA1[iProtocol][IR_RP_PART];
	UINT32 IR_RP2_data = IRDA_DATA1[iProtocol][IR_RP2_PART];
	UINT32 IR_SR_data = IRDA_DATA1[iProtocol][IR_SR_PART];	
	
	printk("\n");
	printk("[BEGIN] Verify_Irda: %s protocol -> Check [IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X]? \n", protocol, IR_RP_data, IR_RP2_data);
	printk(" Verify_Irda_Repeat:  rpt valied num = %d,  signal rptTimes = %d,  IRRF=%d \n", rptNum, rptTimes, bRaised);

	//init irda_crt
	 irda_crt();

	//init irrx_ps
	irda_ps();

	//===irda settings=========
	irda_rx_hw_prepare_new(iProtocol, mainID);	//clear & settting
	irda_rx_hw_prepare_rpt(iProtocol, mainID, rptNum-1);
	
    	//====input irrx data==========
	irda_tx(protocol, IR_TX_addr, IR_TX_cmd, rptTimes);	//REPEAT_TIMES
	udelay(5000);
	
	//======check irrx received data=========
	Ir_err=1;
#if 1
	timeout = 500;
	do
	{
		flag = rtd_inl(ADDR_IR_SR); 
		if(flag&0x1)	//IRDVF
		{
			rptflag = (flag&0x10)>>4; 	//IRRF
			udelay(100);
			
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data){		
				rptcnt++;

			}
			flag = rtd_inl(ADDR_IR_SR); 
			//fifo
			rtd_part_outl(ADDR_IR_SR, 0,0, 0x1);	//clr IRDVF -> update rd pt=0;
			getVal =rtd_inl(ADDR_IR_RP_HIGH);		//IR_RP_HIGH:Read port of Fifo(write 0 to fifo??)
			getVal1 =rtd_inl(ADDR_IR_RP_LOW);		//IR_RP_LOW:Read port of Fifo(write 0 to fifo??)  ==> check value here
			printk("[IR_RP_HIGH, IR_RP_LOW]=[0x%08X, 0x%08X], IR_SR=0x%08X  \n", getVal, getVal1, flag);
			if(iProtocol == IRDA_SHARP)
				IR_RP_data = 0xba420000;		
			if(getVal==IR_RP_data  && getVal1 == IR_RP2_data){		
				rptcnt++;	
			}
			if(rptflag == bRaised)	
			{
				if((rptflag+1) == rptcnt)		
				{
					Ir_err=0;
					irda_hw_clear(mainID);				
					break;
				}		
			}
			
		}
		timeout--;
		udelay(1000);
		
	}while(timeout>0);		
#endif


	if(Ir_err==0)
	{		
		printk("[END][SUCCESS]Irda Repeat Verify: %s protocol  \n", protocol);
		return 0	;
	}
	else
	{
		printk("--->?[END][FAIL]Irda Verify: %s protocol -> timeout: %d, IR_SR =0x%08x \n", protocol, timeout, flag);
		return 1;
	}
}


int Verify_Irda(void)
{
	UINT32 i,j;
	UINT32 t_temp;
	UINT32 Ir_err=0;		//for basic function
	UINT32 Ir_err2=0;	//for rpt num
	UINT32 Ir_err3=0;	//for end pattern
	UINT32 Ir_err4=0;	//for main0 & main1
	UINT32 Ir_err5 = 0;    //for raw mode
	char strResult[500] = {""};
	char tmpResult[30] = {""}; 
	int mainID = 0;
	memset(strResult, 0, sizeof(strResult));
	memset(tmpResult, 0, sizeof(tmpResult));
	

	printk("Irda Verifing...\n");

	irda_tx_init();	//gpio as irda_tx pin
#if 1
	//verify start
	printk("\x1b[33m\t========== main0 moudle ==========\x1b[0m\n");
	for(i=IRDA_NEC; i<IRDA_PANASONIC+1; i++)	
	{		
		Ir_err += Verify_Irda_Single(i, 0);
	}

	printk("\x1b[33m\t========== main1 moudle ===========\x1b[0m\n");
	for(i=IRDA_NEC; i<IRDA_PANASONIC+1; i++)
	{		
		Ir_err += Verify_Irda_Single(i, 1);
	}
	//conclude results
	if(Ir_err>0)
	{
		printk("\x1b[31m\t[FAIL] Irda-Verify finished!: Fail to verify some protocol!\x1b[0m\n");//show error data.
	}
#endif

#if 0  //open when test more
	#if 1
	printk(" \x1b[33m\t============Verify_Irda_Repeat!==========\x1b[0m\n");
	Ir_err2 += Verify_Irda_Repeat(IRDA_NEC, 0, 2, 1, false);   // 2 rpt Valid, send 1 rpt, IRRF should be false
	Ir_err2 += Verify_Irda_Repeat(IRDA_NEC, 0, 2, 2, true);	 // 2 rpt Valid, send 2 rpt, IRRF should be true
	Ir_err2 += Verify_Irda_Repeat(IRDA_NEC, 0, 2, 11, true);
	Ir_err2 += Verify_Irda_Repeat(IRDA_NEC, 0, 11, 11, true);
	Ir_err2 += Verify_Irda_Repeat(IRDA_NEC, 0, 13, 13, false);
	
	Ir_err2 += Verify_Irda_Repeat(IRDA_C03, 0, 2, 1, false);
	Ir_err2 += Verify_Irda_Repeat(IRDA_C03, 0, 2, 2, true);
	Ir_err2 += Verify_Irda_Repeat(IRDA_C03, 0, 2, 12, true);
	Ir_err2 += Verify_Irda_Repeat(IRDA_C03, 0, 12, 12, true);
	Ir_err2 += Verify_Irda_Repeat(IRDA_C03, 0, 13, 13, false);

	if(Ir_err2>0)
	{
		Ir_err++;
		printk("\x1b[31m\t[FAIL] Irda-Verify finished!: Fail to Verify_Irda_Repeat \x1b[0m\n");//show error data.
	}
	#endif

	#if 1
	printk(" \x1b[33m\t============Verify_Irda_EndPattern!==========\x1b[0m\n");	
	//for lead & ml(L)
	printk(" \x1b[33m\t============Main0 EndPattern!==========\x1b[0m\n");	
	//RPT protocol & has end pattern
	 Verify_Irda_EndPattern(IRDA_C03, mainID, 0, 0x10, true);		//IR_RP OK?  IR_SR = OK? 0x111000111 != 0x11100011, effect->detect num    
	Ir_err3 += Verify_Irda_EndPattern(IRDA_C03, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK?  
	 Verify_Irda_EndPattern(IRDA_C03, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK? 0x111000111 != 0x11100011, effect->detect num
	Ir_err3 += Verify_Irda_EndPattern(IRDA_C03, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK?

	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 	

	//None-RPT protocol but has end pattern
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 

	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 

	//None-RPT protocol & has no end-pattern
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 1, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 

	printk(" \x1b[33m\t============Main1 EndPattern!==========\x1b[0m\n");

	mainID = 1;
	//RPT protocol & has end pattern
	 Verify_Irda_EndPattern(IRDA_C03, mainID, 0, 0x10, true);		//IR_RP OK?  IR_SR = OK? 0x111000111 != 0x11100011, effect->detect num    
	Ir_err3 += Verify_Irda_EndPattern(IRDA_C03, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK?  
	 Verify_Irda_EndPattern(IRDA_C03, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK? 0x111000111 != 0x11100011, effect->detect num
	Ir_err3 += Verify_Irda_EndPattern(IRDA_C03, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK?

	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_NEC, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 	

	//None-RPT protocol but has end pattern
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SHARP, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 

	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 1, 0x10, true);		//IR_RP =0?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_KANKA, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 

	//None-RPT protocol & has no end-pattern
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 0, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 0, 0x80, false);		//IR_RP OK?  IR_SR = OK? 
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 1, 0x10, false);		//IR_RP OK?  IR_SR = OK?
	Ir_err3 += Verify_Irda_EndPattern(IRDA_SONY, mainID, 1, 0x80, false);		//IR_RP OK?  IR_SR = OK? 

	

	if(Ir_err3>0)
	{
		Ir_err++;
		printk("\x1b[31m\t[FAIL] Irda-Verify finished!: Fail to Verify_Irda_EndPattern \x1b[0m\n");//show error data.
	}
	#endif

	#if 1
	printk(" \x1b[33m\t============Verify_Irda_Main0_Main1!==========\x1b[0m\n");
	for(i=IRDA_NEC; i<IRDA_PANASONIC+1; i++)	
	{		
		if(i ==IRDA_SONY || i ==IRDA_SONY_15)
			continue;
		for(j=i+1; j<IRDA_PANASONIC+1; j++)	
		{				
			if(Verify_Irda_Main0_Main1(i, j))
			{
				snprintf(tmpResult, sizeof(tmpResult), "[ %s, %s ] ", irda_protocol_name[i], irda_protocol_name[j]);
				strcat(strResult, tmpResult);
				Ir_err4++;
			}
		}
	}	
	if(Ir_err4>0)
	{
		//Ir_err++;
		printk("\x1b[31m\t[FAIL] Irda-Verify finished!: Fail to Verify_Irda_Main0_Main1 \x1b[0m\n"); 
		printk("==>Result: %s\n", strResult); 
	}
	#endif

	#if 0
	printk(" \x1b[33m\t============Verify_Irda_Raw_Mode!==========\x1b[0m\n");	

	for(i=IRDA_NEC; i<IRDA_PANASONIC+1; i++)	
	{		
		Ir_err5 += Verify_Irda_RAW_Test(i);
	}	
		 
	if(Ir_err5>0)
	{
		printk("\x1b[31m\t[FAIL] Irda-Verify finished!: Fail to Verify_Irda_Raw Mode!\x1b[0m\n"); //show error data.
	}
	#endif

	

#endif

	if(Ir_err>0)
	{
		return 1;
	}
	
	printk("\x1b[32m\t[SUCCESS] Irda-Verify finished!: pass all protocol & functions!\x1b[0m\n");
	return 0;
}
