#include <qc_verify_common.h>
#include <common.h>
#include <asm/arch/io.h>
#include <rbusCRTReg.h>
#include <SeStretchUtils.h>
#include <qc_verify_se.h>
#include <se_reg.h>
//=======================================================
//

//#define abs(x) ((x<0)?-(x):(x))
#define absdiff(a,b) ((a>b)?(a-b):(b-a))
#define get_mask(value, shift)  ((value>>shift)&0xff)
#define BIT(x)               (1 <<x)
#define TOLERANCE		10

//#define SE_MEASURE
#undef SE_MEASURE
#define SPLIT_SIZE_MIR (64)
#define SPLIT_SIZE_ROT (256)

#define printf_info(fmt, arg...) //printf(fmt, ##arg)

static bool se_dcu1_write_test(void);

const short VO_coef_2t8p_qc[8] =
{
  0x203, 0x2e8, 0x4a5, 0x6d6, 0x92a, 0xb5b, 0xd18, 0xdfd
};


void qc_se_WriteRegister(SEINFO_REGMODE  bRegisterModeSelect,
	SEINFO_WRITEREG wRegisterAddress,
	uint32_t        dwRegisterValue);

void SetVideoScalingCoeffs_qc (
	unsigned short *coeff,
	int delta_phase,
	int init_phase,
	int taps,
	int swap)
{
	int i;

	if ( abs(delta_phase - 0x4000) < 0x20 && init_phase == 0 ){
		memset (coeff, 0, taps << 4);
		for(i = 1; i <= 8; i++)
			coeff[(taps << 3) - i] = 0x1000;
	}else {
		 //if ( taps == 2 ){
		for (i = 0; i < 16; i+=2)
		{
			unsigned short c = VO_coef_2t8p_qc[i>>1];
			coeff[i  ] = c;
			coeff[i+1] = c;
		}
	}
}
//===================================================================================================
//static bool _gSEState = false;  // false:Not Ready,   true:Ready,
static uint32_t* _gCmdBuffer = (unsigned int*)SE_CMD_BUFFER_START_ADDR;
static uint32_t  _gCmdLength=0;
static uint32_t  _gInstCount=0;
static uint32_t _g_SE_CMD_BUFFER_START_ADDR = (unsigned int)SE_CMD_BUFFER_START_ADDR;


void *_gpSurface[2];
#define swapColor(c) ((c >> 24) | (c << 24) | ((c >> 8) & 0x0000ff00) | ((c << 8) & 0x00ff0000))

#if !TEST_IN_AP
typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  write_data:1;
        RBus_UInt32  go:1;
        RBus_UInt32  endiswap:1;
        RBus_UInt32  res1:29;
    };
}ctrl_RBUS;
#endif // #if !TEST_IN_AP


void qc_se_power(void)
{
	printf("%s.....\n", __func__);

	rtd_outl(SYS_REG_SYS_SRST1_reg, BIT(SYS_REG_SYS_SRST1_rstn_se_shift));
	udelay(10);

	rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_se_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	//rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_se2_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	udelay(10);

	rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_se_shift));
	//rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_se2_shift));
	udelay(10);

	rtd_outl(SYS_REG_SYS_SRST1_reg, BIT(SYS_REG_SYS_SRST1_rstn_se_shift));
	udelay(10);

	rtd_outl(SYS_REG_SYS_SRST1_reg, BIT(SYS_REG_SYS_SRST1_rstn_se_shift) | SYS_REG_SYS_SRST1_write_data_mask);
	udelay(10);

	rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_se_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	//rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_se2_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);

}

void qc_se_hw_reg_init(void)
{
	ctrl_RBUS se_ctrl;
	rtd_outl(SE_CMDBASE_0_reg, _g_SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDLMT_0_reg, _g_SE_CMD_BUFFER_START_ADDR+SE_CMD_SIZE);
	rtd_outl(SE_CMDRPTR_0_reg, _g_SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDWPTR_0_reg, _g_SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_INSTCNT_L_0_reg, 0x0);
	rtd_outl(SE_INSTCNT_H_0_reg, 0x0);
	se_ctrl.write_data = 1;
	se_ctrl.endiswap = 1;
	rtd_outl(SE_CTRL_0_reg, se_ctrl.regValue);
}

int qc_se_checkfinish(void)
{
    int delaycounter=0;
    int ret = -1;

    while(ret == -1)
    {
        if((rtd_inl(SE_IDLE_0_reg) & 0x1) &&
           !(rtd_inl(SE_CTRL_0_reg) & 0x2) &&
           (rtd_inl(SE_CMDRPTR_0_reg) == rtd_inl(SE_CMDWPTR_0_reg)))
        {
            ret = 0;
            break;
        }
        mdelay(SE_CMD_DELAY_MS_ATOMIC);
        delaycounter++;
        if(delaycounter >=500)
        {
            printf("0x%08x, 0x%08x, 0x%08x, 0x%08x\n", rtd_inl(SE_IDLE_0_reg), rtd_inl(SE_CTRL_0_reg), rtd_inl(SE_CMDRPTR_0_reg), rtd_inl(SE_CMDWPTR_0_reg));
            printf("Error! %s timtout! call baker.cheng.\n", __func__);
            ret = 0;
            break;
        }
    }
    if(delaycounter >= 500)
	ret = -1;
    return ret;
}

void qc_se_writecmd(void)
{
    uint32_t dwDataCounter = 0;
    uint32_t wrptr = 0;
    uint32_t u32Alignment = 0;
    ctrl_RBUS se_ctrl;
    int32_t lCommandLength=_gCmdLength;

    if(lCommandLength ==0) return;
    _gCmdLength = 0;
    _gInstCount = 0;
    lCommandLength = lCommandLength*4;   //bytes base
    wrptr = rtd_inl(SE_CMDWPTR_0_reg) + lCommandLength;   //bytes base

//    if(se_state_Guard() == false)
//    {
//        printf("%s! \n",__func__);
//        return;
//    }

    if(lCommandLength%16 != 0)
        u32Alignment = ((lCommandLength^0xf)&0xf) + 1;

    //printf("%s %d wrptr = %08x, lCommandLength =%d bytes, u32Alignment = %d bytes\n", __func__, __LINE__,wrptr, lCommandLength,u32Alignment);
    for(dwDataCounter = 0; dwDataCounter < (uint32_t)u32Alignment; dwDataCounter += sizeof(uint32_t))
    {
        *(uint32_t*)wrptr = 0xf;
        wrptr += sizeof(uint32_t);
        if(wrptr >= (_g_SE_CMD_BUFFER_START_ADDR + SE_CMD_SIZE))
            wrptr = _g_SE_CMD_BUFFER_START_ADDR;
    }

    flush_dcache_all();
//    flush_dcache_range(rtd_inl(SE_CMDRPTR_0_reg), wrptr);
    //printf("%s -  flush from %08x to %08x \n", __func__,rtd_inl(SE_CMDRPTR_0_reg), wrptr);
    rtd_outl(SE_CMDWPTR_0_reg, wrptr);
    se_ctrl.write_data = 1;
    se_ctrl.go = 1;
    se_ctrl.endiswap = 1;
    rtd_outl(SE_CTRL_0_reg, se_ctrl.regValue);
}

void qc_se_writecmd_busy(void)
{
    uint32_t dwDataCounter = 0;
    uint32_t wrptr = 0;
    uint32_t u32Alignment = 0;
    ctrl_RBUS se_ctrl;
    int32_t lCommandLength=_gCmdLength;

    if(lCommandLength ==0) return;
    _gCmdLength = 0;
    _gInstCount = 0;
    lCommandLength = lCommandLength*4;   //bytes base
    wrptr = rtd_inl(SE_CMDWPTR_0_reg) + lCommandLength;   //bytes base

//    if(se_state_Guard() == false)
//    {
//        printf("%s! \n",__func__);
//        return;
//    }

    if(lCommandLength%16 != 0)
        u32Alignment = ((lCommandLength^0xf)&0xf) + 1;

    //printf("%s %d wrptr = %08x, lCommandLength =%d bytes, u32Alignment = %d bytes\n", __func__, __LINE__,wrptr, lCommandLength,u32Alignment);
    for(dwDataCounter = 0; dwDataCounter < (uint32_t)u32Alignment; dwDataCounter += sizeof(uint32_t))
    {
        *(uint32_t*)wrptr = 0xf;
        wrptr += sizeof(uint32_t);
        if(wrptr >= (SE_CMD_BUFFER_START_ADDR + SE_CMD_SIZE*1024))
            wrptr = SE_CMD_BUFFER_START_ADDR;
    }

    flush_dcache_all();
//    flush_dcache_range(rtd_inl(SE_CMDRPTR_0_reg), wrptr);
    //printf("%s -  flush from %08x to %08x \n", __func__,rtd_inl(SE_CMDRPTR_0_reg), wrptr);
    rtd_outl(SE_CMDWPTR_0_reg, wrptr);
    se_ctrl.write_data = 1;
    se_ctrl.go = 1;
    se_ctrl.endiswap = 1;
    rtd_outl(SE_CTRL_0_reg, se_ctrl.regValue);
}

int qc_se_module_init(void)
{
    uint32_t u32Ctrl=0, u32Idle=0;

    _gpSurface[0] = (void*)SE_INSTALL_UI_SURFACE1_ADDR;
    _gpSurface[1] = (void*)SE_INSTALL_UI_SURFACE2_ADDR;
    qc_se_power();
    qc_se_hw_reg_init();
    u32Ctrl = rtd_inl(SE_CTRL_0_reg);
    u32Idle = rtd_inl(SE_IDLE_0_reg);
    if(u32Ctrl == 0x4 && u32Idle == 0x1)
    {
//        _gSEState = true;
        // Set the DMA info.
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_BASE_ADDR1, (uint32_t)_gpSurface[0]);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH1, SE_SURFACE_PITCH);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_BASE_ADDR1+0x4, (uint32_t)_gpSurface[1]);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH1+0x4, SE_SURFACE_PITCH);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_BASE_ADDR0, (uint32_t)_gpSurface[0]);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH0, SE_SURFACE_PITCH);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_BASE_ADDR1, (uint32_t)_gpSurface[1]);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH1, SE_SURFACE_PITCH);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_BASE_ADDR2, (uint32_t)_gpSurface[1]);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH2, SE_SURFACE_PITCH);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_BASE_ADDR3, (uint32_t)_gpSurface[1]);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH3, SE_SURFACE_PITCH);

        qc_se_writecmd();
        qc_se_checkfinish();
        return 0;
    }
    printf("%s...Fail!", __func__);
    return -1;

};

//----------------------------------------------------------------------
void qc_reset_cmd_buffer(void)
{
    _gCmdBuffer = (unsigned int*)_g_SE_CMD_BUFFER_START_ADDR;
    _gCmdLength = 0;
    _gInstCount = 0;

//    Initialize the command buffer.
//    qc_se_hw_reg_init();
    rtd_outl(SE_CMDRPTR_0_reg, _g_SE_CMD_BUFFER_START_ADDR);
    rtd_outl(SE_CMDWPTR_0_reg, _g_SE_CMD_BUFFER_START_ADDR);
}

void qc_se_write(uint8_t *pbyCommandBuffer, int dwCommandLength, int inst_count)
{
    flush_dcache_all();
    memcpy((uint32_t*)_gCmdBuffer+_gCmdLength, pbyCommandBuffer, dwCommandLength);
    _gCmdLength += dwCommandLength/4;
    _gInstCount += inst_count;

}

int ceil(float num)
{
    int inum = (int)num;
    if (num == (float)inum)
    {
        return inum;
    }
    return inum + 1;
}

int floor(float num)
{
    int inum = (int)num;
    return inum;
}

void qc_se_WriteRegister(SEINFO_REGMODE  bRegisterModeSelect,
                      SEINFO_WRITEREG wRegisterAddress,
                      uint32_t        dwRegisterValue)
{
    SECMD_WRITE_REGISTER SeCmdWriteRegister;

    memset((void *)&SeCmdWriteRegister, 0, sizeof(SECMD_WRITE_REGISTER));
    SeCmdWriteRegister.Word1.Reserved = 0;
    SeCmdWriteRegister.Word1.OpCode = SEINFO_OPCODE_WRITE_REGISTER;
    SeCmdWriteRegister.Word1.Clut = bRegisterModeSelect;
    if(bRegisterModeSelect)
        SeCmdWriteRegister.Word1.RegisterAddress = wRegisterAddress & 0xff;
    else
        SeCmdWriteRegister.Word1.RegisterAddress = wRegisterAddress;
    SeCmdWriteRegister.Word2.RegisterValue = dwRegisterValue;

    SeCmdWriteRegister.Word3.Reserved = SEINFO_OPCODE_NOP;
    SeCmdWriteRegister.Word4.Reserved = SEINFO_OPCODE_NOP;
    SeCmdWriteRegister.Word5.Reserved = SEINFO_OPCODE_NOP;
    SeCmdWriteRegister.Word6.Reserved = SEINFO_OPCODE_NOP;
    SeCmdWriteRegister.Word7.Reserved = SEINFO_OPCODE_NOP;
    SeCmdWriteRegister.Word8.Reserved = SEINFO_OPCODE_NOP;

    qc_se_write((uint8_t *) &SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER), 7);
}
#if 1
void qc_se_WriteRegister2(SEINFO_WRITEREG wRegisterAddress1,
	uint32_t dwRegisterValue1,
	SEINFO_WRITEREG wRegisterAddress2,
        uint32_t dwRegisterValue2,
	SEINFO_WRITEREG wRegisterAddress3,
        uint32_t dwRegisterValue3,
	SEINFO_WRITEREG wRegisterAddress4,
        uint32_t dwRegisterValue4)
{
    SECMD_WRITE_REGISTER2 SeCmdWriteRegister;

    memset((void *)&SeCmdWriteRegister, 0, sizeof(SECMD_WRITE_REGISTER));
    SeCmdWriteRegister.Word1.OpCode = SEINFO_OPCODE_WRITE_REGISTER;
    SeCmdWriteRegister.Word1.RegisterAddress = wRegisterAddress1;
    SeCmdWriteRegister.Word2.RegisterValue = dwRegisterValue1;
    SeCmdWriteRegister.Word3.OpCode = SEINFO_OPCODE_WRITE_REGISTER;
    SeCmdWriteRegister.Word3.RegisterAddress = wRegisterAddress2;
    SeCmdWriteRegister.Word4.RegisterValue = dwRegisterValue2;
    SeCmdWriteRegister.Word5.OpCode = SEINFO_OPCODE_WRITE_REGISTER;
    SeCmdWriteRegister.Word5.RegisterAddress = wRegisterAddress3;
    SeCmdWriteRegister.Word6.RegisterValue = dwRegisterValue3;
    SeCmdWriteRegister.Word7.OpCode = SEINFO_OPCODE_WRITE_REGISTER;
    SeCmdWriteRegister.Word7.RegisterAddress = wRegisterAddress4;
    SeCmdWriteRegister.Word8.RegisterValue = dwRegisterValue4;


    qc_se_write((uint8_t *) &SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER), 4);
}
#endif

void SE_SetColorFormat(SEINFO_COLOR_FORMAT		byColorFormat,
			SEINFO_ARGB_ALPHA_LOCATION	byArgbAlphaLocation,
			SEINFO_ARGB_ALPHA_LOCATION	byOutputArgbAlphaLocation,
			void*				pExtraParam
)
{
	SEREG_CLR_FMT SeRegColorFormat;
	memset((void *)&SeRegColorFormat, 0, sizeof(SEREG_CLR_FMT));
	SeRegColorFormat.Value = 0;
	SeRegColorFormat.Fields.Format = byColorFormat;
	SeRegColorFormat.Fields.write_enable1 = true;

	if(byArgbAlphaLocation != SEINFO_ARGB_ALPHA_NONE) {
		SeRegColorFormat.Fields.alpha_loc = byArgbAlphaLocation;
		SeRegColorFormat.Fields.write_enable2 = true;
	}

	if(byOutputArgbAlphaLocation != SEINFO_ARGB_ALPHA_NONE) {
		SeRegColorFormat.Fields.alpha_loc2 = byOutputArgbAlphaLocation;
		SeRegColorFormat.Fields.write_enable5 = true;
	}

	if (pExtraParam) {
		SE_COLOR_FORMAT_EX* pClrFormatEx = (SE_COLOR_FORMAT_EX*) pExtraParam;
		SeRegColorFormat.Fields.big_endian_i1  = pClrFormatEx->bInputSrcColorFormatBigEndian;
		SeRegColorFormat.Fields.write_enable6  = true;
		SeRegColorFormat.Fields.big_endian_i2  = pClrFormatEx->bInputDestColorFormatBigEndian;
		SeRegColorFormat.Fields.write_enable7  = true;
		SeRegColorFormat.Fields.big_endian_o   = pClrFormatEx->bOutputColorFormatBigEndian;
		SeRegColorFormat.Fields.write_enable8  = true;
		SeRegColorFormat.Fields.rounding_en    = pClrFormatEx->bEnableRoundingOption;
		SeRegColorFormat.Fields.write_enable10 = true;
	}
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_COLOR_FORMAT, SeRegColorFormat.Value);
}

bool UpdateFormatConvertWithBitblit(SECMD_FORMAT_CONVERSION* pSeCmdFormatConversion,
                                    SE_FORMAT_CONVERT_EX*    pFormatConvEx)
{
    if (!pFormatConvEx) return false;

    if (!pFormatConvEx->bEnableFormatConvertWithBitblit)
        return false;
    //Magellan
//    CheckFormatConvsionBitblitConditionsOfOutput(pFormatConvEx->bEnableDirectFB, pFormatConvEx->ropMode,
//            (SEINFO_SRC_FORMAT)pSeCmdFormatConversion->Word2.SourceFormat,
//            (SEINFO_DEST_FORMAT)pSeCmdFormatConversion->Word3.DestinationFormat);
    //Magellan End

    pSeCmdFormatConversion->Word5.sync_vo = pFormatConvEx->bSyncVO;

    pSeCmdFormatConversion->Word2.FormatConversionBitblit = pFormatConvEx->bEnableFormatConvertWithBitblit;
    pSeCmdFormatConversion->Word2.directFB = pFormatConvEx->bEnableDirectFB;

//    CheckSelectOutputAlpha(pFormatConvEx->outputAlphaModeSelect,
//                           (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)cmdQue].Fields.Format);

    if(!pFormatConvEx->bEnableDirectFB)
    {
        if(pFormatConvEx->ropMode == SEINFO_ROP_MODE)
        {
            pSeCmdFormatConversion->Word5.rop_mode = SEINFO_ROP_MODE;
            pSeCmdFormatConversion->Word5.rop_code = pFormatConvEx->ropCode;
        }
        else //alpha blending mode without direct fb
        {
            pSeCmdFormatConversion->Word5.sel_src_alpha = pFormatConvEx->srcAlphaModeSelect;
            pSeCmdFormatConversion->Word5.sel_dest_alpha = pFormatConvEx->destAlphaModeSelect;
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_SRC_ALPHA,
                             pFormatConvEx->srcAplhaValue);
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_DEST_ALPHA,
                             pFormatConvEx->destAplhaValue);
            pSeCmdFormatConversion->Word5.bitblit_color_key = pFormatConvEx->colorKeySelect;
        }
        return true;
    }

//    CheckSelectAlphaDfbFeature(pFormatConvEx->bEnableDirectFB, (SEINFO_ALPHAMODE)pSeCmdFormatConversion->Word5.sel_dest_alpha);
//    CheckSelectAlphaDfbFeature(pFormatConvEx->bEnableDirectFB, (SEINFO_ALPHAMODE)pSeCmdFormatConversion->Word5.sel_src_alpha);
//    CheckDfbColorKey(pFormatConvEx->bEnableDirectFB, (SEINFO_COLORKEY)pSeCmdFormatConversion->Word5.bitblit_color_key);
//
//    CheckIfBitBlitForbidColorFormat(pFormatConvEx->destAlphaModeSelect,
//                                    (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)cmdQue].Fields.Format);
//    CheckIfBitBlitForbidColorFormat(pFormatConvEx->srcAlphaModeSelect,
//                                    (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)cmdQue].Fields.Format);
    pSeCmdFormatConversion->Word5.rop_mode          = SEINFO_ALPHA_BLENDING;  //dont output temperal data to DDR
    pSeCmdFormatConversion->Word5.sel_src_alpha     = pFormatConvEx->srcAlphaModeSelect;
    pSeCmdFormatConversion->Word5.sel_dest_alpha    = pFormatConvEx->destAlphaModeSelect;
    pSeCmdFormatConversion->Word5.bitblit_color_key = pFormatConvEx->colorKeySelect;
    if (pFormatConvEx->dfbOperation != SEINFO_DFB_OP_DISABLED)
    {
        pSeCmdFormatConversion->Word5.rop_code = pFormatConvEx->dfbOperation;
    }
    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                     SEINFO_REG_SRC_ALPHA,
                     pFormatConvEx->srcAplhaValue);
    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                     SEINFO_REG_DEST_ALPHA,
                     pFormatConvEx->destAplhaValue);

    pSeCmdFormatConversion->Word5.src_clr_transform     = pFormatConvEx->srcClrTranform;
    pSeCmdFormatConversion->Word5.src_clr_transparency  = pFormatConvEx->bEnableSrcClrTransparency;
    pSeCmdFormatConversion->Word5.fb_dst_premultiply    = pFormatConvEx->bEnableDfbDstPremultiply;
    pSeCmdFormatConversion->Word5.fb_src_premulcolor    = pFormatConvEx->bEnableDfbSrcPremulcolor;
    pSeCmdFormatConversion->Word5.fb_src_premultiply    = pFormatConvEx->bEnableDfbSrcPremultiply;
    pSeCmdFormatConversion->Word5.fb_xor                = pFormatConvEx->bEnableDfbXor;
    pSeCmdFormatConversion->Word5.fb_colorize           = pFormatConvEx->bEnableDfbColorize;
    pSeCmdFormatConversion->Word5.fb_blend_coloralpha   = pFormatConvEx->bEnableDfbReplaceSrcAlphaWithConstAlpha;
    pSeCmdFormatConversion->Word5.fb_blend_alphachannel = pFormatConvEx->bEnableDfbModulateSrcAlphaWithConstAlpha;
    pSeCmdFormatConversion->Word5.fb_demultiply         = pFormatConvEx->bEnableDfbDemultiply;
    pSeCmdFormatConversion->Word5.fb_sel_srcclr_blend   = pFormatConvEx->selSrcClrBlend;
    pSeCmdFormatConversion->Word5.fb_sel_destclr_blend  = pFormatConvEx->selDestClrBlend;

    if (pFormatConvEx->bEnableDfbSrcPremulcolor ||
        pFormatConvEx->bEnableDfbReplaceSrcAlphaWithConstAlpha ||
        pFormatConvEx->bEnableDfbModulateSrcAlphaWithConstAlpha)
    {
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_ALPHA,
                         pFormatConvEx->constAlphaValue);
    }

    if (pFormatConvEx->bEnableDfbColorize)
    {
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_COLOR,
                         pFormatConvEx->constColorValue);
    }

    if (pFormatConvEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_ENABLE_FOR_ALPHA_ONLY ||
        pFormatConvEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_ENABLE_FOR_ALL)
    {
//        CheckSourceColorComponentCoeff(pFormatConvEx->srcColorRed,
//                                       pFormatConvEx->srcColorGreen,
//                                       pFormatConvEx->srcColorBlue,
//                                       pFormatConvEx->srcColorAlpha);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_RED,
                         pFormatConvEx->srcColorRed.Value);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_GREEN,
                         pFormatConvEx->srcColorGreen.Value);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_BLUE,
                         pFormatConvEx->srcColorBlue.Value);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_ALPHA,
                         pFormatConvEx->srcColorAlpha.Value);
    }
    return true;
}

bool UpdateBitblitDfbFunctions(SECMD_BITBLT *pSeCmdBitblt, SE_BITBLIT_EX* pBlitEx)
{
    if (!pBlitEx) return false;

    pSeCmdBitblt->Word4.alpha_stream_mode = pBlitEx->bEnableAlphaStreamMode;
    if(pBlitEx->bEnableLinearGraident)
    {
        pSeCmdBitblt->Word4.directFB = 0;
        pSeCmdBitblt->Word3.RopMode = 0;
        pSeCmdBitblt->Word4.linear_gradient_en = 1;
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_COLOR,
                         pBlitEx->constColorValue);
        //steps
        if(pBlitEx->GradientDir == SEINFO_GRADIENT_LEFT_UP || pBlitEx->GradientDir == SEINFO_GRADIENT_LEFT_DOWN ||
            pBlitEx->GradientDir == SEINFO_GRADIENT_LEFT) {
        //ABS(0,0);
        //PASS
        }
    }

    if(pBlitEx->srcClrTranform != SEINFO_SRC_CLR_TRANSFORM_DISABLE)
    {
        pSeCmdBitblt->Word4.directFB = 0;
        pSeCmdBitblt->Word4.src_clr_transform = pBlitEx->srcClrTranform;
        pSeCmdBitblt->Word4.src_clr_transparency = pBlitEx->bEnableSrcClrTransparency;
        pSeCmdBitblt->Word1.OpCode = SEINFO_OPCODE_BIBLIT_EX;
        pSeCmdBitblt->Word2.sync_vo = pBlitEx->bSyncVO;

        if (pBlitEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_ENABLE_FOR_ALPHA_ONLY ||
                pBlitEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_ENABLE_FOR_ALL)
        {
//            CheckSourceColorComponentCoeff(pBlitEx->srcColorRed,
//                                           pBlitEx->srcColorGreen,
//                                           pBlitEx->srcColorBlue,
//                                           pBlitEx->srcColorAlpha);
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_SRC_CLR_RED,
                             pBlitEx->srcColorRed.Value);
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_SRC_CLR_GREEN,
                             pBlitEx->srcColorGreen.Value);
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_SRC_CLR_BLUE,
                             pBlitEx->srcColorBlue.Value);
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_SRC_CLR_ALPHA,
                             pBlitEx->srcColorAlpha.Value);
        }
        return true;
    }

    if(!pBlitEx->bEnableDirectFB)
    {
        return false;
    }

//    CheckSelectAlphaDfbFeature(pBlitEx->bEnableDirectFB, (SEINFO_ALPHAMODE)pSeCmdBitblt->Word1.SelectDestAlpha);
//    CheckSelectAlphaDfbFeature(pBlitEx->bEnableDirectFB, (SEINFO_ALPHAMODE)pSeCmdBitblt->Word3.SelectSrcAlpha);
//    CheckDfbColorKey(pBlitEx->bEnableDirectFB, (SEINFO_COLORKEY)pSeCmdBitblt->Word3.ColorKey);

    pSeCmdBitblt->Word1.OpCode  = SEINFO_OPCODE_BIBLIT_EX;
    pSeCmdBitblt->Word2.sync_vo = pBlitEx->bSyncVO;
    if (pBlitEx->bEnableDirectFB && pBlitEx->dfbOperation != SEINFO_DFB_OP_DISABLED)
    {
        pSeCmdBitblt->Word2.RopCode = pBlitEx->dfbOperation;
    }
    pSeCmdBitblt->Word4.directFB              = pBlitEx->bEnableDirectFB;
    pSeCmdBitblt->Word4.fb_dst_premultiply    = pBlitEx->bEnableDfbDstPremultiply;
    printf("dst_premultiply = %d\n", pSeCmdBitblt->Word4.fb_dst_premultiply);
    pSeCmdBitblt->Word4.fb_src_premulcolor    = pBlitEx->bEnableDfbSrcPremulcolor;
    pSeCmdBitblt->Word4.fb_src_premultiply    = pBlitEx->bEnableDfbSrcPremultiply;
    pSeCmdBitblt->Word4.fb_xor                = pBlitEx->bEnableDfbXor;
    pSeCmdBitblt->Word4.fb_colorize           = pBlitEx->bEnableDfbColorize;
    pSeCmdBitblt->Word4.fb_blend_coloralpha   = pBlitEx->bEnableDfbReplaceSrcAlphaWithConstAlpha;
    pSeCmdBitblt->Word4.fb_blend_alphachannel = pBlitEx->bEnableDfbModulateSrcAlphaWithConstAlpha;
    pSeCmdBitblt->Word4.fb_demultiply         = pBlitEx->bEnableDfbDemultiply;
    pSeCmdBitblt->Word4.fb_sel_srcclr_blend   = pBlitEx->selSrcClrBlend;
    pSeCmdBitblt->Word4.fb_sel_destclr_blend  = pBlitEx->selDestClrBlend;

    if (pBlitEx->bEnableDfbSrcPremulcolor                ||
        pBlitEx->bEnableDfbReplaceSrcAlphaWithConstAlpha ||
        pBlitEx->bEnableDfbModulateSrcAlphaWithConstAlpha)
    {
        printf("Update const alpha as %d\n", pBlitEx->constAlphaValue);

        //+JT - FIXME? Looks like we need another field to store alpha for replacing purpose.
        if(pBlitEx->bEnableDfbReplaceSrcAlphaWithConstAlpha)
        {
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_DFB_COLORALPHA,
                             pBlitEx->constAlphaValue);
        }
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_ALPHA,
                         pBlitEx->constAlphaValue);
    }


    if (pBlitEx->bEnableDfbColorize)
    {
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_COLOR,
                         pBlitEx->constColorValue);
    }
    return true;
}

void SE_BitBltEx(SE_PROCINFO            *pSeDestinationProcInfo,
                 SE_PROCINFO            *pSeSourceProcInfo,
                 SEINFO_ALPHAMODE        bySrcAlphaModeSelect,
                 uint8_t                 bySrcAplhaValue,
                 SEINFO_ALPHAMODE        byDestAlphaModeSelect,
                 uint8_t                 byDestAplhaValue,
                 SEINFO_OUTPUT_ALPHAMODE byOutputAlphaModeSelect,
                 uint8_t                 byOutputAlphaValue,
                 SEINFO_SETUPMODE        bSetupModeSelect,
                 SEINFO_CORNER           byStartingCorner,
                 SEINFO_GRADIENT         byGradientModeSelect,
                 SEINFO_COLORKEY         byColorKeySelect,
                 SEINFO_ROPCODE          byRopCode,
                 SE_BITBLIT_EX*          pBlitEx,
                 uint32_t                vo_sync)
{
    SECMD_BITBLT SeCmdBitblt;

    memset(&SeCmdBitblt, 0, sizeof(SeCmdBitblt));
    SeCmdBitblt.Word1.InputSelect = (uint32_t) bSetupModeSelect;
    SeCmdBitblt.Word1.Height      = pSeDestinationProcInfo->wDataHeight;
    SeCmdBitblt.Word1.Width       = pSeDestinationProcInfo->wDataWidth;
    SeCmdBitblt.Word1.OpCode      = SEINFO_OPCODE_BITBLT;

    if(vo_sync) //+JT
    {
        SeCmdBitblt.Word2.sync_vo = 1;
    }
    SeCmdBitblt.Word2.DestAddressIndex = (uint32_t) pSeDestinationProcInfo->byBaseAddressIndex;
    SeCmdBitblt.Word2.SelectOutputAlpha = byOutputAlphaModeSelect;
//    CheckSelectOutputAlpha(byOutputAlphaModeSelect,
//                           (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)byCommandQueue].Fields.Format);
    SeCmdBitblt.Word2.OutputX = pSeDestinationProcInfo->wDataX;
    SeCmdBitblt.Word2.OutputY = pSeDestinationProcInfo->wDataY;

    SeCmdBitblt.Word3.InputX = pSeSourceProcInfo->wDataX;
    SeCmdBitblt.Word3.InputY = pSeSourceProcInfo->wDataY;

    if(bySrcAlphaModeSelect != SEINFO_BITBLT_ALPHA_DISABLE &&
       byDestAlphaModeSelect != SEINFO_BITBLT_ALPHA_DISABLE)
    {
//        CheckIfBitBlitForbidColorFormat(byDestAlphaModeSelect,
//                                        (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)byCommandQueue].Fields.Format);
//        CheckIfBitBlitForbidColorFormat(bySrcAlphaModeSelect,
//                                        (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)byCommandQueue].Fields.Format);
        SeCmdBitblt.Word3.RopMode = SEINFO_ALPHA_BLENDING;
        SeCmdBitblt.Word3.SelectSrcAlpha = bySrcAlphaModeSelect;
        SeCmdBitblt.Word1.SelectDestAlpha = byDestAlphaModeSelect;
        SeCmdBitblt.Word3.ColorKey = byColorKeySelect;
//        printf("Set color key as %d\n", SeCmdBitblt.Word3.ColorKey);
//        printf("Set const alpha as %d\n", bySrcAplhaValue);
    }
    else
    {
        SeCmdBitblt.Word3.RopMode = SEINFO_ROP_MODE;
        SeCmdBitblt.Word2.RopCode = byRopCode;
    }

    if(byOutputAlphaModeSelect == SEINFO_OUTPUT_ALPHA_CONST)
    {
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_RESULT_ALPHA,
                         byOutputAlphaValue);
    }

    if (bSetupModeSelect == SEINFO_BITBLT_SETUP_MANUAL)
    {
//        if(g_VFlip_Enable)
//            SeCmdBitblt.Word2.StartingCorner = SEINFO_BITBLT_CORNER_BOTTOM_LEFT;//byStartingCorner;
//        else
        SeCmdBitblt.Word2.StartingCorner = byStartingCorner;
        SeCmdBitblt.Word3.SrcAddressIndex = (uint32_t) pSeSourceProcInfo->byBaseAddressIndex;
    }

    int iInstructCnt = 5;
    if (!UpdateBitblitDfbFunctions(&SeCmdBitblt, pBlitEx))
    {
        SeCmdBitblt.Word4.Reserved = 0x3;
        SeCmdBitblt.Word4.linear_gradient_en = 0x1;
        SeCmdBitblt.Word4.opengl = 0x1;
        iInstructCnt++;
    }
    SeCmdBitblt.Word5.gl_sel_destclr_blend = SEINFO_OPCODE_NOP;
    SeCmdBitblt.Word6.Reserved = SEINFO_OPCODE_NOP;
    SeCmdBitblt.Word7.Reserved = SEINFO_OPCODE_NOP;
    SeCmdBitblt.Word8.Reserved = SEINFO_OPCODE_NOP;

//    printf("\r\n\r\n----------- SE_BitBltEx -------------------\r\n");
//    printf("0x%08x\r\n", *(uint32_t*)&SeCmdBitblt.Word1);
//    printf("0x%08x\r\n", *(uint32_t*)&SeCmdBitblt.Word2);
//    printf("0x%08x\r\n", *(uint32_t*)&SeCmdBitblt.Word3);
//    printf("0x%08x\r\n", *(uint32_t*)&SeCmdBitblt.Word4);
//    printf("0x%08x\r\n", *(uint32_t*)&SeCmdBitblt.Word5);
//    printf("\r\n\r\n");

    qc_se_write((uint8_t *) &SeCmdBitblt, sizeof(SECMD_BITBLT), iInstructCnt);
}

void SE_FormatConversion(SEINFO_CMDQUEUE         byCommandQueue,
                         SE_PROCINFO            *pSeDestinationProcInfo,
                         SE_PROCINFO            *pSeSourceProcInfo,
                         SEINFO_CONVERSION       byConversionModeSelect,
                         SEINFO_DEST_FORMAT      byDestinationFormatSelect,
                         SEINFO_SRC_FORMAT       bySourceFormatSelect,
                         SEINFO_COLORKEY         byColorKeySelect,
                         SEINFO_OUTPUT_ALPHAMODE byOutputAlphaModeSelect,
                         uint8_t                 byOutputAlphaValue,
                         SE_FORMAT_CONVERT_EX*   pExtraParam)
{
    SECMD_FORMAT_CONVERSION    SeCmdFormatConversion;

    memset((void *)&SeCmdFormatConversion, 0, sizeof(SECMD_FORMAT_CONVERSION));
    SeCmdFormatConversion.Word1.ConversionMode    = byConversionModeSelect;
    SeCmdFormatConversion.Word1.ColorKey          = byColorKeySelect;
    SeCmdFormatConversion.Word1.SelectOutputAlpha = byOutputAlphaModeSelect;
//    CheckSelectOutputAlpha(byOutputAlphaModeSelect,
//                           (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)byCommandQueue].Fields.Format);
    SeCmdFormatConversion.Word1.Width = pSeDestinationProcInfo->wDataWidth;
    SeCmdFormatConversion.Word1.Height = pSeDestinationProcInfo->wDataHeight;
//    CheckFormatConvertWidthAndHeight(SeCmdFormatConversion);
    SeCmdFormatConversion.Word1.OpCode = SEINFO_OPCODE_FORMAT_CONVERSION;

    SeCmdFormatConversion.Word2.SourceFormat = bySourceFormatSelect;
    SeCmdFormatConversion.Word2.InputX = pSeSourceProcInfo->wDataX;
    SeCmdFormatConversion.Word2.InputY = pSeSourceProcInfo->wDataY;
    SeCmdFormatConversion.Word2.FormatConversionBitblit = 0;
    SeCmdFormatConversion.Word2.directFB = 0;
#if 0//defined(CONFIG_SE_COPY_WITH_FMTCONV)
    if(g_booleanFastCopy == true) {
        SeCmdFormatConversion.Word2.ARGB8888MatrixEnable=1;
        g_booleanFastCopy=false;
    }
#endif
    SeCmdFormatConversion.Word3.DestinationFormat = byDestinationFormatSelect;
    SeCmdFormatConversion.Word3.OutputX           = pSeDestinationProcInfo->wDataX;
    SeCmdFormatConversion.Word3.OutputY           = pSeDestinationProcInfo->wDataY;

    SeCmdFormatConversion.Word4.SrcAddressIndex  = pSeSourceProcInfo->byBaseAddressIndex;
    SeCmdFormatConversion.Word4.DestAddressIndex = pSeDestinationProcInfo->byBaseAddressIndex;
    if(byConversionModeSelect == SEINFO_CONVERSION_ARGB_YUV)
    {
        SeCmdFormatConversion.Word4.ChromaAddressIndex = pSeDestinationProcInfo->byChronmaAddressIndex;
    }
    else
    {
        SeCmdFormatConversion.Word4.ChromaAddressIndex = pSeSourceProcInfo->byChronmaAddressIndex;
    }

    if(byOutputAlphaModeSelect == SEINFO_OUTPUT_ALPHA_CONST)
    {
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_RESULT_ALPHA, byOutputAlphaValue);
    }

    int iInstructCnt = 4;
    if (!UpdateFormatConvertWithBitblit(&SeCmdFormatConversion, pExtraParam))
    {
        //Magellan
        if (SeCmdFormatConversion.Word3.DestinationFormat == SEINFO_DEST_FORMAT_ARGB8565)
        {
            printf("FormatConversion error: Destination format ARGB8565 is only for format conversion + bitblit mode!!\n");
            assert(0);
        }
        //Magellan End
        // Workaround for NOP. Word5.fb_sel_destclr_blend[3:0] is 4 bits.
        SeCmdFormatConversion.Word5.fb_sel_destclr_blend = SEINFO_OPCODE_NOP;
        iInstructCnt++;
    }

    SeCmdFormatConversion.Word6.gl_sel_destclr_blend = SEINFO_OPCODE_NOP;
    SeCmdFormatConversion.Word7.Reserved = SEINFO_OPCODE_NOP;
    SeCmdFormatConversion.Word8.Reserved = SEINFO_OPCODE_NOP;
    printf("\r\n");

    qc_se_write((uint8_t *) &SeCmdFormatConversion, sizeof(SECMD_FORMAT_CONVERSION), iInstructCnt);
}

bool UpdateStretchWithBitblit(SECMD_STRETCH* pSeCmdStretch,
                              void* pExtraParam)
{
    if (!pExtraParam) return false;

    SE_STRETCH_EX* pStretchEx = (SE_STRETCH_EX*)pExtraParam;
    if (!pStretchEx->bEnableStretchWithBitblit &&
        (pStretchEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_DISABLE &&
         !pStretchEx->bEnableDirectFB))
    {
        return false;
    }

    pSeCmdStretch->Word2.StretchBitblit = pStretchEx->bEnableStretchWithBitblit;

//    CheckSelectAlphaDfbFeature(pStretchEx->bEnableDirectFB, (SEINFO_ALPHAMODE)pStretchEx->destAlphaModeSelect);
//    CheckSelectAlphaDfbFeature(pStretchEx->bEnableDirectFB, (SEINFO_ALPHAMODE)pStretchEx->srcAlphaModeSelect);
//    CheckDfbColorKey(pStretchEx->bEnableDirectFB, (SEINFO_COLORKEY)pStretchEx->colorKeySelect);

//    CheckSelectOutputAlpha(pStretchEx->outputAlphaModeSelect,
//                           (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)cmdQue].Fields.Format);

    if(pStretchEx->srcAlphaModeSelect != SEINFO_BITBLT_ALPHA_DISABLE &&
       pStretchEx->destAlphaModeSelect != SEINFO_BITBLT_ALPHA_DISABLE /*&&
       pStretchEx->bEnableDirectFB*/)
    {
        printf("Use blend mode\n");
//        CheckIfBitBlitForbidColorFormat(pStretchEx->destAlphaModeSelect,
//                                        (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)cmdQue].Fields.Format);
//        CheckIfBitBlitForbidColorFormat(pStretchEx->srcAlphaModeSelect,
//                                        (SEINFO_COLOR_FORMAT)SeRegInfo->SeColorFormat[(int)cmdQue].Fields.Format)//;

        pSeCmdStretch->Word5.rop_mode          = SEINFO_ALPHA_BLENDING;
        pSeCmdStretch->Word5.sel_src_alpha     = pStretchEx->srcAlphaModeSelect;
        pSeCmdStretch->Word5.sel_dest_alpha    = pStretchEx->destAlphaModeSelect;
        pSeCmdStretch->Word5.bitblit_color_key = pStretchEx->colorKeySelect;
        pSeCmdStretch->Word5.sel_output_alpha  = pStretchEx->outputAlphaModeSelect == SEINFO_OUTPUT_ALPHA_CONST ? 1 : 0;
        //20111101 peggy: already set in DG api, some cases should not set, ex. CONST_ALPHA
        //SE_SetSourceAlpha1(cmdQue, pStretchEx->srcAplhaValue);
        //SE_SetDestAlpha(cmdQue, pStretchEx->destAplhaValue);

        if (pStretchEx->dfbOperation != SEINFO_DFB_OP_DISABLED)
        {
            pSeCmdStretch->Word5.rop_code = pStretchEx->dfbOperation;
        }
    }
    else
    {
        printf("ROP mode\n");
        pSeCmdStretch->Word5.rop_mode = SEINFO_ROP_MODE;
        pSeCmdStretch->Word5.rop_code = pStretchEx->ropCode;
    }

    pSeCmdStretch->Word5.sync_vo              = pStretchEx->bSyncVO;
    pSeCmdStretch->Word5.src_clr_transform    = pStretchEx->srcClrTranform;
    pSeCmdStretch->Word5.src_clr_transparency = pStretchEx->bEnableSrcClrTransparency;
    pSeCmdStretch->Word2.directFB             = pStretchEx->bEnableDirectFB;
    pSeCmdStretch->Word5.fb_dst_premultiply   = pStretchEx->bEnableDfbDstPremultiply;
    pSeCmdStretch->Word5.fb_src_premulcolor   = pStretchEx->bEnableDfbSrcPremulcolor;
    pSeCmdStretch->Word5.fb_src_premultiply   = pStretchEx->bEnableDfbSrcPremultiply;

    printf("premultiply = %d\n", pSeCmdStretch->Word5.fb_src_premultiply);
    pSeCmdStretch->Word5.fb_xor                = pStretchEx->bEnableDfbXor;
    pSeCmdStretch->Word5.fb_colorize           = pStretchEx->bEnableDfbColorize;
    pSeCmdStretch->Word5.fb_blend_coloralpha   = pStretchEx->bEnableDfbReplaceSrcAlphaWithConstAlpha;
    pSeCmdStretch->Word5.fb_blend_alphachannel = pStretchEx->bEnableDfbModulateSrcAlphaWithConstAlpha;

    printf("color alpha = %d\n", pSeCmdStretch->Word5.fb_blend_alphachannel);
    pSeCmdStretch->Word5.fb_demultiply        = pStretchEx->bEnableDfbDemultiply;
    pSeCmdStretch->Word5.fb_sel_srcclr_blend  = pStretchEx->selSrcClrBlend;
    pSeCmdStretch->Word5.fb_sel_destclr_blend = pStretchEx->selDestClrBlend;
    pSeCmdStretch->Word5.sel_output_alpha     = pStretchEx->outputAlphaModeSelect == SEINFO_OUTPUT_ALPHA_CONST ? 1 : 0;

    if (pStretchEx->bEnableDfbSrcPremulcolor ||
        pStretchEx->bEnableDfbReplaceSrcAlphaWithConstAlpha ||
        pStretchEx->bEnableDfbModulateSrcAlphaWithConstAlpha)
    {
#if 1// defined(IS_ARCH_ARM_COMMON) || defined(IS_MACARTHUR3) || defined(IS_MAGELLAN22)
            //+JT - FIXME? Looks like we need another field to store alpha for replacing purpose.
        if(pStretchEx->bEnableDfbReplaceSrcAlphaWithConstAlpha)
        {
            qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                             SEINFO_REG_DFB_COLORALPHA,
                             pStretchEx->constAlphaValue);
        }
#endif
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_ALPHA,
                         pStretchEx->constAlphaValue);
    }


    if (pStretchEx->bEnableDfbColorize)
    {
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_COLOR,
                         pStretchEx->constColorValue);
    }

    if (pStretchEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_ENABLE_FOR_ALPHA_ONLY ||
        pStretchEx->srcClrTranform == SEINFO_SRC_CLR_TRANSFORM_ENABLE_FOR_ALL)
    {
//        CheckSourceColorComponentCoeff(pStretchEx->srcColorRed,
//                                       pStretchEx->srcColorGreen,
//                                       pStretchEx->srcColorBlue,
//                                       pStretchEx->srcColorAlpha);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_RED,
                         pStretchEx->srcColorRed.Value);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_GREEN,
                         pStretchEx->srcColorGreen.Value);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_BLUE,
                         pStretchEx->srcColorBlue.Value);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
                         SEINFO_REG_SRC_CLR_ALPHA,
                         pStretchEx->srcColorAlpha.Value);
    }

    return true;
}

void SE_Stretch(
	SE_PROCINFO		*pSeDestinationProcInfo,
	SE_PROCINFO		*pSeSourceProcInfo,
	SEINFO_SACLING_TYPE	byVerticalScalingType,
	uint32_t		dwVerticalScalingRatio,
	SEINFO_STRETCH_VTAP	byVerticalTapNumber,
	SEINFO_SACLING_TYPE	byHorizontalScalingType,
	uint32_t		dwHorizontalScalingRatio,
	SEINFO_STRETCH_HTAP	byHorizontalTapNumber,
	SEINFO_STRETCH		byScalingAlgorithmSelect,
	SEINFO_COLORKEY		byColorKeySelect,
	SEINFO_ALPHAPROC_EN	bAlphaEnable,
	uint8_t			iVerticalScalingOffset,
	void*			pExtraParam)
{
	SECMD_STRETCH SeCmdStretch;

	memset(&SeCmdStretch, 0x00, sizeof(SECMD_STRETCH));
	uint32_t dwDataCounter = 0;
	uint32_t dwScalingSetting = 0;
	bool bHEnabled = (byHorizontalScalingType != SEINFO_SCALING_DISABLE ? true : false);
	bool bVEnabled = (byVerticalScalingType   != SEINFO_SCALING_DISABLE ? true : false);

	dwScalingSetting = ((uint32_t) byHorizontalTapNumber << 3) |
                        ((uint32_t) byVerticalTapNumber << 2);
	//Magellan End
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_STRETCH, dwScalingSetting);

	if(bVEnabled) {
		dwScalingSetting = (dwVerticalScalingRatio & 0x0003FFFF);
		qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_VDHS, dwScalingSetting);
		unsigned short tmp_coef_4t16p[32];
		SetVideoScalingCoeffs_qc((unsigned short*)tmp_coef_4t16p,
			dwVerticalScalingRatio,
			iVerticalScalingOffset,
			1 << (byVerticalTapNumber+1),
			0) ;
		for(dwDataCounter = 0; dwDataCounter < SEINFO_STRETCH_VSCALING_COEFF_MAX; dwDataCounter ++) {
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32 + (dwDataCounter << 2)),
				tmp_coef_4t16p[dwDataCounter]);
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32_2 + (dwDataCounter << 2)),
				tmp_coef_4t16p[dwDataCounter+16]);
			}
	}

	if(bHEnabled) {
		dwScalingSetting = (dwHorizontalScalingRatio & 0x0003FFFF);
		qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_HDHS, dwScalingSetting);
		unsigned short tmp_coef_8t16p[64];
		SetVideoScalingCoeffs_qc((unsigned short*)tmp_coef_8t16p,
			dwHorizontalScalingRatio,
			0,
			1 << (byHorizontalTapNumber+1),
			0) ;
		for(dwDataCounter = 0; dwDataCounter < SEINFO_STRETCH_HSCALING_COEFF_MAX; dwDataCounter ++) {
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32 + (dwDataCounter << 2)),
				tmp_coef_8t16p[dwDataCounter]);
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER,
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32_2 + (dwDataCounter << 2)),
				tmp_coef_8t16p[dwDataCounter+32]);
		}
	}

	SeCmdStretch.Word1.SrcInterleaving = pSeSourceProcInfo->bInterleavingModeEnable;
	SeCmdStretch.Word1.VScalingEnable  = bVEnabled;
	SeCmdStretch.Word1.HScalingEnable  = bHEnabled;
	SeCmdStretch.Word1.ColorKey        = (byColorKeySelect == SEINFO_COLORKEY_SOURCE) ? SEINFO_COLORKEY_SOURCE : SEINFO_COLORKEY_NONE;
	SeCmdStretch.Word1.OutputY         = pSeDestinationProcInfo->wDataY;
	SeCmdStretch.Word1.OutputX         = pSeDestinationProcInfo->wDataX;

	SeCmdStretch.Word1.OpCode          = SEINFO_OPCODE_STRETCH;

	SeCmdStretch.Word2.ScalingAlgorithm = byScalingAlgorithmSelect;
	SeCmdStretch.Word2.Reserved         = 0;
	SeCmdStretch.Word2.StretchBitblit   = 0;
	SeCmdStretch.Word2.directFB         = 0;

	SeCmdStretch.Word2.OutputHeight     = pSeDestinationProcInfo->wDataHeight;
	SeCmdStretch.Word2.OutputWidth      = pSeDestinationProcInfo->wDataWidth;

	SeCmdStretch.Word3.SrcAddressIndex  = pSeSourceProcInfo->byBaseAddressIndex;
	SeCmdStretch.Word3.DestInterleaving = pSeDestinationProcInfo->bInterleavingModeEnable;;
	SeCmdStretch.Word3.InputY           = pSeSourceProcInfo->wDataY;
	SeCmdStretch.Word3.InputX           = pSeSourceProcInfo->wDataX;

	SeCmdStretch.Word4.DestAddressIndex = pSeDestinationProcInfo->byBaseAddressIndex;
	SeCmdStretch.Word4.InputHeight      = pSeSourceProcInfo->wDataHeight;
	SeCmdStretch.Word4.InputWidth       = pSeSourceProcInfo->wDataWidth;


	int iInstructCnt = 4;
	//if (!UpdateStretchWithBitblit(&SeCmdStretch, pExtraParam)) {
        // Workaround for NOP. Word5.fb_sel_destclr_blend[3:0] is 4 bits.
	SeCmdStretch.Word5.fb_sel_destclr_blend = SEINFO_OPCODE_NOP;
	iInstructCnt++;
	//}

	SeCmdStretch.Word6.gl_sel_destclr_blend = SEINFO_OPCODE_NOP;
	SeCmdStretch.Word7.Reserved = SEINFO_OPCODE_NOP;
	SeCmdStretch.Word8.Reserved = SEINFO_OPCODE_NOP;
	qc_se_write((uint8_t *) &SeCmdStretch, sizeof(SECMD_STRETCH), iInstructCnt);

}

void DG_Stretch(int			iXDes,
		int			iYDes,
		int			iWDes,
		int			iHDes,
		int			iXSrc,
		int			iYSrc,
		int			iWSrc,
		int			iHSrc,
		COLORKEY_MODE		byColorKeySelect,
		unsigned long		iColorKey,
		bool			bAlphaEnable,
		STRETCH_COEFF_MODE	StretchMode,
		uint8_t			iVerticalScalingOffset,
		int			iVerticalTapNumber)
{
	SE_PROCINFO		hSrcProcInfo;
	SE_PROCINFO		hDestProcInfo;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;
	SEINFO_COLORKEY		colorKeySelect;

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	colorKeySelect = SEINFO_COLORKEY_NONE;

	// alpha setting
	SEINFO_ALPHAPROC_EN alphaEnable;
	alphaEnable = SEINFO_ALPHAPROC_DISABLE;

	iVerticalTapNumber = 2;

	SEINFO_SACLING_TYPE scalingTypeH, scalingTypeW;
	uint32_t fRatioW, fRatioH;

	fRatioW = (iWSrc << 14) / iWDes;
	if (iWDes > iWSrc) {
		scalingTypeW = SEINFO_SCALING_UP;
	} else if (iWDes < iWSrc) {
		scalingTypeW = SEINFO_SCALING_DOWN;
	} else {
		scalingTypeW = SEINFO_SCALING_DISABLE;
		fRatioW = 0x4000;
	}

	fRatioH = (iHSrc << 14) / iWDes;

	if (iHDes > iHSrc) {
		scalingTypeH = SEINFO_SCALING_UP;
	} else if (iHDes < iHSrc) {
		scalingTypeH = SEINFO_SCALING_DOWN;
	} else {
		scalingTypeH = SEINFO_SCALING_DISABLE;
		fRatioH = 0x4000;
	}

	hSrcProcInfo.wDataX = iXSrc;
	hSrcProcInfo.wDataY = iYSrc;
	hSrcProcInfo.wDataWidth = iWSrc;
	hSrcProcInfo.wDataHeight = iHSrc;
	hSrcProcInfo.byBaseAddressIndex = 0;

	hDestProcInfo.wDataX = iXDes;
	hDestProcInfo.wDataY = iYDes;
	hDestProcInfo.wDataWidth = iWDes;
	hDestProcInfo.wDataHeight = iHDes;
	hDestProcInfo.byBaseAddressIndex = 1;

	SE_Stretch(&hDestProcInfo,
		&hSrcProcInfo,
		scalingTypeH,
		fRatioH,
		SEINFO_VSCALING_2TAP,
		scalingTypeW,
		fRatioW,
		SEINFO_HSCALING_2TAP,
		SEINFO_STRETCH_FIR,
		colorKeySelect,
		alphaEnable,
		iVerticalScalingOffset,
		NULL);

	qc_se_writecmd();
	qc_se_checkfinish();

}

void DrawRectangle(int nDstBaseIdx, int nXDst, int nYDst, int nWidth, int nHeight, uint32_t srcColor)
{
    SE_PROCINFO srcInfo, dstInfo;
    SE_COLOR_FORMAT_EX SeColorFormatEx;

    qc_reset_cmd_buffer();

    memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
    SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
    SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
    SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
    SeColorFormatEx.bEnableRoundingOption          = 0;
    SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
                      SEINFO_ARGB_ALPHA_LEFTMOST,
                      SEINFO_ARGB_ALPHA_LEFTMOST,
                      &SeColorFormatEx);

    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, (srcColor)>>24);
    srcColor = swapColor(srcColor);
    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_COLOR, srcColor);

    srcInfo.wDataX             = nXDst;
    srcInfo.wDataY             = nYDst;
    srcInfo.wDataWidth         = nWidth;
    srcInfo.wDataHeight        = nHeight;
    srcInfo.byBaseAddressIndex = 0;

    dstInfo.wDataX             = nXDst;
    dstInfo.wDataY             = nYDst;
    dstInfo.wDataWidth         = nWidth;
    dstInfo.wDataHeight        = nHeight;
    dstInfo.byBaseAddressIndex = nDstBaseIdx;
    SE_BitBltEx(&dstInfo,
                &srcInfo,
                SEINFO_BITBLT_ALPHA_DISABLE,
                0,
                SEINFO_BITBLT_ALPHA_DISABLE,
                0,
                SEINFO_OUTPUT_ALPHA_NORMAL,
                0,
                SEINFO_BITBLT_SETUP_CONST,
                SEINFO_BITBLT_CORNER_NONE,
                SEINFO_GRADIENT_DISABLE,
                SEINFO_COLORKEY_NONE,
                SEINFO_ROPCODE_COPY,
                NULL,
                0);

    qc_se_writecmd();
    qc_se_checkfinish();
}

void EXTDrawRectangle(int nDstBaseIdx, int nXDst, int nYDst, int nWidth, int nHeight, uint32_t srcColor)
{
	SE_COLOR_FORMAT_EX SeColorFormatEx;

	qc_reset_cmd_buffer();

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);

	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, (srcColor)>>24);
	//srcColor = swapColor(srcColor);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_COLOR, srcColor);

	SECMD_EXT_FMT_CONV se_ext_conv;
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));


	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.lay_ctl = 0;
	se_ext_conv.Word1.lay_num = 0;
	se_ext_conv.Word1.rslt_argb_order = 0;

	se_ext_conv.Word2.rslt_out_x = nXDst;
	se_ext_conv.Word2.rslt_out_y = nYDst;

	se_ext_conv.Word3.rslt_addr_index = nDstBaseIdx;

	se_ext_conv.Word4.src1_inpt_w = nWidth;
	se_ext_conv.Word4.src1_inpt_h = nHeight;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = nXDst;
	se_ext_conv.Word5.src1_inpt_y = nYDst;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = nDstBaseIdx;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

	qc_se_writecmd();
	qc_se_checkfinish();
}

void CopyRectangle(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{
    SE_PROCINFO srcInfo, dstInfo;
    SE_COLOR_FORMAT_EX SeColorFormatEx;

    qc_reset_cmd_buffer();

    memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
    SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
    SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
    SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
    SeColorFormatEx.bEnableRoundingOption          = 0;
    SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
                      SEINFO_ARGB_ALPHA_LEFTMOST,
                      SEINFO_ARGB_ALPHA_LEFTMOST,
                      &SeColorFormatEx);

    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, 0xff);

    srcInfo.wDataX             = prcSrc->x;
    srcInfo.wDataY             = prcSrc->y;
    srcInfo.wDataWidth         = prcSrc->w;
    srcInfo.wDataHeight        = prcSrc->h;
    srcInfo.byBaseAddressIndex = nSrcBaseIdx;

    dstInfo.wDataX             = prcDst->x;
    dstInfo.wDataY             = prcDst->y;
    dstInfo.wDataWidth         = prcDst->w;
    dstInfo.wDataHeight        = prcDst->h;
    dstInfo.byBaseAddressIndex = nDstBaseIdx;

    SE_BitBltEx(&dstInfo,
                &srcInfo,
                SEINFO_BITBLT_ALPHA_DISABLE,
                0,
                SEINFO_BITBLT_ALPHA_DISABLE,
                0,
                SEINFO_OUTPUT_ALPHA_NORMAL,
                0,
                SEINFO_BITBLT_SETUP_MANUAL,
                SEINFO_BITBLT_CORNER_NONE,
                SEINFO_GRADIENT_DISABLE,
                SEINFO_COLORKEY_NONE,
                SEINFO_ROPCODE_COPY,
                NULL,
                0);

    qc_se_writecmd();
    qc_se_checkfinish();
}

int EXTCopyRectangle(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{

	SE_COLOR_FORMAT_EX SeColorFormatEx;
	SECMD_EXT_FMT_CONV_BLIT se_ext_conv;
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV_BLIT));
	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;

	qc_reset_cmd_buffer();
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);

	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, 0xff);




	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;

	se_ext_conv.Word1.lay_num = 0;
	se_ext_conv.Word1.lay_ctl = 0;
	se_ext_conv.Word1.two_dc_mode = 0;
	se_ext_conv.Word2.rslt_out_x = prcDst->x;
	se_ext_conv.Word2.rslt_out_y = prcDst->y;

	se_ext_conv.Word3.rslt_addr_index = nDstBaseIdx;


	se_ext_conv.Word4.src1_inpt_w = prcSrc->w;
	se_ext_conv.Word4.src1_inpt_h = prcSrc->h;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = prcSrc->x;
	se_ext_conv.Word5.src1_inpt_y = prcSrc->y;

	se_ext_conv.Word6.src1_addr_index = nSrcBaseIdx;

	se_ext_conv.Word7.src2_inpt_x = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.src2_addr_index = SEINFO_OPCODE_NOP;
	se_ext_conv.Word9.sel_s2_alpha = 0x7;
	se_ext_conv.Word9.reserved0 = 0x1;
	se_ext_conv.Word10.rop_code_1 = SEINFO_OPCODE_NOP;
	se_ext_conv.Word11.gl_sel_destclr_1 = SEINFO_OPCODE_NOP;

	se_ext_conv.Word12.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word13.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word14.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word15.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word16.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV_BLIT), 11);
	qc_se_writecmd();
	return qc_se_checkfinish();
}

void EXTIndex8(int nDstBaseIdx, int nXDst, int nYDst, int nWidth, int nHeight, uint32_t srcColor)
{
	SE_COLOR_FORMAT_EX SeColorFormatEx;
	int i=0;

	qc_reset_cmd_buffer();

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);

	for(i=0;i<256;i++) {
		qc_se_WriteRegister(SEINFO_REGMODE_CLUT,i,srcColor);
	}

	SECMD_EXT_FMT_CONV se_ext_conv;
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.lay_ctl = 0;
	se_ext_conv.Word1.lay_num = 0;
	se_ext_conv.Word1.rslt_argb_order = 0;

	se_ext_conv.Word2.rslt_out_x = nXDst;
	se_ext_conv.Word2.rslt_out_y = nYDst;

	se_ext_conv.Word3.rslt_addr_index = nDstBaseIdx;

	se_ext_conv.Word4.src1_inpt_w = nWidth;
	se_ext_conv.Word4.src1_inpt_h = nHeight;
	se_ext_conv.Word4.src1_fmt = 0xe;

	se_ext_conv.Word5.src1_inpt_x = nXDst;
	se_ext_conv.Word5.src1_inpt_y = nYDst;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x0; //if 0x1d, swap!!
	se_ext_conv.Word1.src2_argb_order = 0x0;

	se_ext_conv.Word6.src1_addr_index = 0; //don't need to care data of this buffer
	se_ext_conv.Word6.src1_inpt_sel = 0;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

	qc_se_writecmd();
	qc_se_checkfinish();
}

void EXTAlpha8(int nDstBaseIdx, int nXDst, int nYDst, int nWidth, int nHeight, uint32_t srcColor)
{

#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif
	SE_COLOR_FORMAT_EX SeColorFormatEx;
	qc_reset_cmd_buffer();

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);



	SECMD_EXT_FMT_CONV_BLIT se_ext_conva8;
	memset((void *)&se_ext_conva8, 0, sizeof(SECMD_EXT_FMT_CONV));
	se_ext_conva8.Word1.opcode = 0xb;
	se_ext_conva8.Word1.rslt_fmt = 0xc;

	se_ext_conva8.Word1.lay_num = 1;
	se_ext_conva8.Word1.lay_ctl = 1;
	se_ext_conva8.Word1.two_dc_mode = 0;
	se_ext_conva8.Word2.rslt_out_x = nXDst;
	se_ext_conva8.Word2.rslt_out_y = nYDst;

	se_ext_conva8.Word3.rslt_addr_index = 1;


	se_ext_conva8.Word4.src1_inpt_w = nWidth;
	se_ext_conva8.Word4.src1_inpt_h = nHeight;
	se_ext_conva8.Word4.src1_fmt = 0xc;

	se_ext_conva8.Word5.src1_inpt_x = nXDst;
	se_ext_conva8.Word5.src1_inpt_y = nYDst;

	se_ext_conva8.Word6.src1_addr_index = 1;

	se_ext_conva8.Word7.src2_inpt_x = nXDst;
	se_ext_conva8.Word7.src2_inpt_y = nYDst;
	se_ext_conva8.Word7.src2_fmt = 0xd;

	se_ext_conva8.Word8.src2_addr_index = 0;

	se_ext_conva8.Word9.sel_s2_alpha = 0x7;
	se_ext_conva8.Word9.sel_s1_alpha = 0x7;

	se_ext_conva8.Word9.lay_num_ext = 1;
	//se_ext_coa8nv.Word10.rop_code_1 = 0x3;

	se_ext_conva8.Word10.fb_sel_destclr_blend_1 = 0x1;
	se_ext_conva8.Word10.fb_sel_srcclr_blend_1 = 0x1;

	se_ext_conva8.Word10.blend_sel_1 = 0x4;

	se_ext_conva8.Word12.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conva8.Word13.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conva8.Word14.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conva8.Word15.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conva8.Word16.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_conva8, sizeof(SECMD_EXT_FMT_CONV_BLIT), 6);


#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif
	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] elapsed=%d us\n", __func__, nWidth, nHeight,(t2-t1)*1000000/90090);
#endif
	qc_se_checkfinish();
}

void EXTLinearG(int nDstBaseIdx, int nXDst, int nYDst, int nWidth, int nHeight, uint32_t srcColor)
{

#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif
	SE_COLOR_FORMAT_EX SeColorFormatEx;
	qc_reset_cmd_buffer();

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);


	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, (srcColor)>>24);
	//srcColor = swapColor(srcColor);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_COLOR, srcColor);


	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, 0xA90, 0x01000100);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, 0xAA0, 0x01000100);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, 0xAB0, 0x01000100);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, 0xAC0, 0x01000100);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, 0xAD0, 0);

	SECMD_EXT_FMT_CONV_BLIT se_ext_conv;
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));
	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;

	se_ext_conv.Word1.lay_num = 1;
	se_ext_conv.Word1.lay_ctl = 1;
	se_ext_conv.Word1.two_dc_mode = 0;
	se_ext_conv.Word2.rslt_out_x = nXDst;
	se_ext_conv.Word2.rslt_out_y = nYDst;

	se_ext_conv.Word3.rslt_addr_index = 1;


	se_ext_conv.Word4.src1_inpt_w = nWidth;
	se_ext_conv.Word4.src1_inpt_h = nHeight;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = nXDst;
	se_ext_conv.Word5.src1_inpt_y = nYDst;

	se_ext_conv.Word6.src1_addr_index = 1;

	se_ext_conv.Word7.src2_inpt_x = nXDst;
	se_ext_conv.Word7.src2_inpt_y = nYDst;
	se_ext_conv.Word7.src2_fmt = 0xc;

	se_ext_conv.Word8.src2_lg = 1;
	se_ext_conv.Word8.src2_addr_index = 0;

	se_ext_conv.Word9.sel_s2_alpha = 0x7;
	se_ext_conv.Word9.sel_s1_alpha = 0x7;

	se_ext_conv.Word9.lay_num_ext = 1;
	//se_ext_conv.Word10.rop_code_1 = 0x3;

	se_ext_conv.Word10.fb_sel_destclr_blend_1 = 0x1;
	se_ext_conv.Word10.fb_sel_srcclr_blend_1 = 0x1;

	se_ext_conv.Word10.blend_sel_1 = 0x4;

	se_ext_conv.Word12.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word13.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word14.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word15.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word16.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV_BLIT), 6);


#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif
	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] elapsed=%d us\n", __func__, nWidth, nHeight,(t2-t1)*1000000/90090);
#endif
	qc_se_checkfinish();
}

int BlendRectangle(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{
    SE_PROCINFO srcInfo, dstInfo;
    SE_COLOR_FORMAT_EX SeColorFormatEx;
    SEINFO_OUTPUT_ALPHAMODE sel_output_alpha;
    SEINFO_COLORKEY colorkey_mode = SEINFO_COLORKEY_NONE;
    SEINFO_ALPHAMODE sel_dest_alpha;
    SEINFO_ALPHAMODE sel_src_alpha;
    uint8_t srcA=0xff, desA=0xff, forcedDesA=0xff;
    SE_BITBLIT_EX bltEx;

    qc_reset_cmd_buffer();

    memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
    SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
    SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
    SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
    SeColorFormatEx.bEnableRoundingOption          = 0;
    SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
                      SEINFO_ARGB_ALPHA_LEFTMOST,
                      SEINFO_ARGB_ALPHA_LEFTMOST,
                      &SeColorFormatEx);

    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, 0xff);

    srcInfo.wDataX             = prcSrc->x;
    srcInfo.wDataY             = prcSrc->y;
    srcInfo.wDataWidth         = prcSrc->w;
    srcInfo.wDataHeight        = prcSrc->h;
    srcInfo.byBaseAddressIndex = nSrcBaseIdx;

    dstInfo.wDataX             = prcDst->x;
    dstInfo.wDataY             = prcDst->y;
    dstInfo.wDataWidth         = prcDst->w;
    dstInfo.wDataHeight        = prcDst->h;
    dstInfo.byBaseAddressIndex = nDstBaseIdx;

    colorkey_mode    = SEINFO_COLORKEY_NONE;
    sel_output_alpha = SEINFO_OUTPUT_ALPHA_NORMAL;
    sel_dest_alpha   = SEINFO_BITBLT_ALPHA_1_SRC;
    sel_src_alpha    = SEINFO_BITBLT_ALPHA_SRC;

    memset(&bltEx, 0, sizeof(bltEx));

    SE_BitBltEx(&dstInfo,
                &srcInfo,
                sel_src_alpha,
                srcA,
                sel_dest_alpha,
                desA,
                sel_output_alpha,
                forcedDesA,
                SEINFO_BITBLT_SETUP_MANUAL,
                SEINFO_BITBLT_CORNER_TOP_LEFT,
                SEINFO_GRADIENT_DISABLE,
                colorkey_mode,
                SEINFO_ROPCODE_COPY,
                NULL,
                0);

    qc_se_writecmd();
    return qc_se_checkfinish();
}

int EXTBlendRectangle(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{
	qc_reset_cmd_buffer();

	SE_COLOR_FORMAT_EX SeColorFormatEx;
	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);

	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, 0xff);

	SECMD_EXT_FMT_CONV_BLIT se_ext_conv;
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV_BLIT));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;

	se_ext_conv.Word1.lay_num = 1;
	se_ext_conv.Word1.lay_ctl = 1;
	se_ext_conv.Word1.two_dc_mode = 0;
	se_ext_conv.Word2.rslt_out_x = prcDst->x;
	se_ext_conv.Word2.rslt_out_y = prcDst->y;

	se_ext_conv.Word3.rslt_addr_index = 1;


	se_ext_conv.Word4.src1_inpt_w = prcDst->w;
	se_ext_conv.Word4.src1_inpt_h = prcDst->h;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = prcDst->x;
	se_ext_conv.Word5.src1_inpt_y = prcDst->y;

	se_ext_conv.Word6.src1_addr_index = 1;

	se_ext_conv.Word7.src2_inpt_x = prcSrc->x;
	se_ext_conv.Word7.src2_inpt_y = prcSrc->y;
	se_ext_conv.Word7.src2_fmt = 0xc;

	se_ext_conv.Word8.src2_addr_index = 0;

	se_ext_conv.Word9.sel_s2_alpha = 0x4;
	se_ext_conv.Word9.sel_s1_alpha = 0x5;

	se_ext_conv.Word9.lay_num_ext = 1;
	//se_ext_conv.Word10.rop_code_1 = 0x3;

	se_ext_conv.Word10.fb_sel_destclr_blend_1 = 0x5;
	se_ext_conv.Word10.fb_sel_srcclr_blend_1 = 0x4;

	se_ext_conv.Word10.blend_sel_1 = 0x2;

	se_ext_conv.Word12.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word13.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word14.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word15.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word16.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV_BLIT), 6);
	qc_se_writecmd();
	return qc_se_checkfinish();
}

int EXTBlendRectangle_busy(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{
	int cnt=0;
	qc_reset_cmd_buffer();

	SE_COLOR_FORMAT_EX SeColorFormatEx;
	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		SEINFO_ARGB_ALPHA_LEFTMOST,
		&SeColorFormatEx);

	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, 0xff);

	SECMD_EXT_FMT_CONV_BLIT se_ext_conv;
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV_BLIT));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;

	se_ext_conv.Word1.lay_num = 1;
	se_ext_conv.Word1.lay_ctl = 1;
	se_ext_conv.Word1.two_dc_mode = 0;
	se_ext_conv.Word2.rslt_out_x = prcDst->x;
	se_ext_conv.Word2.rslt_out_y = prcDst->y;

	se_ext_conv.Word3.rslt_addr_index = 1;


	se_ext_conv.Word4.src1_inpt_w = prcDst->w;
	se_ext_conv.Word4.src1_inpt_h = prcDst->h;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = prcDst->x;
	se_ext_conv.Word5.src1_inpt_y = prcDst->y;

	se_ext_conv.Word6.src1_addr_index = 1;

	se_ext_conv.Word7.src2_inpt_x = prcSrc->x;
	se_ext_conv.Word7.src2_inpt_y = prcSrc->y;
	se_ext_conv.Word7.src2_fmt = 0xc;

	se_ext_conv.Word8.src2_addr_index = 0;

	se_ext_conv.Word9.sel_s2_alpha = 0x4;
	se_ext_conv.Word9.sel_s1_alpha = 0x5;

	se_ext_conv.Word9.lay_num_ext = 1;
	//se_ext_conv.Word10.rop_code_1 = 0x3;

	se_ext_conv.Word10.fb_sel_destclr_blend_1 = 0x5;
	se_ext_conv.Word10.fb_sel_srcclr_blend_1 = 0x4;

	se_ext_conv.Word10.blend_sel_1 = 0x2;

	se_ext_conv.Word12.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word13.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word14.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word15.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word16.Reserved = SEINFO_OPCODE_NOP;

	for(cnt=0;cnt<3000;cnt++) {
		qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV_BLIT), 6);
	}
	qc_se_writecmd_busy();
	return true;
}


int StretchRectangle(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{
    qc_reset_cmd_buffer();

//    qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_SRC_ALPHA, 0xff);

    DG_Stretch(prcDst->x, prcDst->y, prcDst->w, prcDst->h,
               prcSrc->x, prcSrc->y, prcSrc->w, prcSrc->h,
               ColorKey_None, 0,
               false,
               SCALING_MODE_AUTO,
               0,
               4);

	return qc_se_checkfinish();
}

int EXTStretchRectangle(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc)
{
	qc_reset_cmd_buffer();

	SECMD_EXT_STCH_BLIT	se_ext_stch;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	uint32_t fRatioW, fRatioH;
#if 1
	bool bHEnabled = 0, bVEnabled=0;
	fRatioW = (prcSrc->w << 14) / prcDst->w;
	if (prcDst->w > prcSrc->w) {
		bHEnabled = 1;
	} else if (prcDst->w < prcSrc->w) {
		bHEnabled = 1;
	} else {
		fRatioW = 0x4000;
		bHEnabled = 0;
	}

	fRatioH = (prcSrc->h << 14) / prcDst->h;
	if (prcSrc->h > prcDst->h) {
		bVEnabled = 1;
	} else if (prcSrc->h < prcDst->h) {
		bVEnabled = 1;
	} else {
		fRatioH = 0x4000;
		bVEnabled = 0;
	}
#endif
	uint32_t dwScalingSetting = 0;
	uint32_t dwDataCounter=0;

	dwScalingSetting = 0;
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_STRETCH, dwScalingSetting);

	//if(bVEnabled) {
	if(1) {
		dwScalingSetting = (fRatioH  & 0x0003FFFF);
		qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_VDHS, dwScalingSetting);
		unsigned short tmp_coef_4t16p[32];
		SetVideoScalingCoeffs_qc((unsigned short*)tmp_coef_4t16p,
			fRatioH , 0, 2, 0);
		for(dwDataCounter = 0; dwDataCounter < SEINFO_STRETCH_VSCALING_COEFF_MAX; dwDataCounter+=4) {
#if 1
			qc_se_WriteRegister2(
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32 + (dwDataCounter << 4)),
				tmp_coef_4t16p[dwDataCounter],
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32 + ((dwDataCounter+1) << 4)),
				tmp_coef_4t16p[dwDataCounter+1],
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32 + ((dwDataCounter+2) << 4)),
				tmp_coef_4t16p[dwDataCounter+2],
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32 + ((dwDataCounter+3) << 4)),
				tmp_coef_4t16p[dwDataCounter+3]);

			qc_se_WriteRegister2(
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32_2 + (dwDataCounter << 4)),
				tmp_coef_4t16p[dwDataCounter+16],
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32_2 + ((dwDataCounter+1) << 4)),
				tmp_coef_4t16p[dwDataCounter+16+1],
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32_2 + ((dwDataCounter+2) << 4)),
				tmp_coef_4t16p[dwDataCounter+16+2],
				(SEINFO_WRITEREG) (SE20INFO_REG_VCOEF1_32_2 + ((dwDataCounter+3) << 4)),
				tmp_coef_4t16p[dwDataCounter+16+3]);
		}
#endif
	}

	//if(bHEnabled) {
	if(1) {
		dwScalingSetting = (fRatioW & 0x0003FFFF);
		qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_HDHS, dwScalingSetting);
		unsigned short tmp_coef_8t16p[64];
		SetVideoScalingCoeffs_qc((unsigned short*)tmp_coef_8t16p,
			fRatioW, 0, 2, 0);
		for(dwDataCounter = 0; dwDataCounter < SEINFO_STRETCH_HSCALING_COEFF_MAX; dwDataCounter+=4) {
#if 1
			qc_se_WriteRegister2(
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32 + (dwDataCounter << 4)),
				tmp_coef_8t16p[dwDataCounter],
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32 + ((dwDataCounter+1) << 4)),
				tmp_coef_8t16p[dwDataCounter+1],
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32 + ((dwDataCounter+2) << 4)),
				tmp_coef_8t16p[dwDataCounter+2],
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32 + ((dwDataCounter+3) << 4)),
				tmp_coef_8t16p[dwDataCounter+3]);
			qc_se_WriteRegister2(
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32_2 + (dwDataCounter << 4)),
				tmp_coef_8t16p[dwDataCounter+16],
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32_2 + ((dwDataCounter+1) << 4)),
				tmp_coef_8t16p[dwDataCounter+16+1],
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32_2 + ((dwDataCounter+2) << 4)),
				tmp_coef_8t16p[dwDataCounter+16+2],
				(SEINFO_WRITEREG) (SE20INFO_REG_HCOEF1_32_2 + ((dwDataCounter+3) << 4)),
				tmp_coef_8t16p[dwDataCounter+16+3]);
#endif
		}
	}


	memset(&se_ext_stch, 0x00, sizeof(SECMD_EXT_STCH_BLIT));

	se_ext_stch.Word1.opcode = 0xb;
	se_ext_stch.Word1.rslt_fmt = 0xc;
	//se_ext_stch.Word1.lay_num = 1;
	//se_ext_stch.Word1.lay_ctl = 1;
	se_ext_stch.Word2.rslt_out_x = prcDst->x;
	se_ext_stch.Word2.rslt_out_y = prcDst->y;
	se_ext_stch.Word3.rslt_addr_index = nDstBaseIdx;

	se_ext_stch.Word4.src1_inpt_w = prcSrc->w;
	se_ext_stch.Word4.src1_inpt_h = prcSrc->h;
	se_ext_stch.Word4.src1_fmt = 0xc;

	se_ext_stch.Word5.src1_inpt_x = prcSrc->x;
	se_ext_stch.Word5.src1_inpt_y = prcSrc->y;
	se_ext_stch.Word5.src1_fcv_stch = 1;


	se_ext_stch.Word6.src1_addr_index = nSrcBaseIdx;

	se_ext_stch.Word9.lay_num_ext = 2;
	se_ext_stch.Word10.blend_sel_1 = 0x0;
	se_ext_stch.Word10.rop_code_1 = SEINFO_ROPCODE_COPY;
	se_ext_stch.Word12.stch1_out_w = prcDst->w;
	se_ext_stch.Word12.stch1_out_h = prcDst->h;
	se_ext_stch.Word12.stch1_src = 0x1;
	se_ext_stch.Word12.stch1_algo = 0;//0:FIR, 1:repeat
	se_ext_stch.Word12.stch1_h_en = bHEnabled;
	se_ext_stch.Word12.stch1_v_en = bVEnabled;
	se_ext_stch.Word13.src2_inpt_w = SEINFO_OPCODE_NOP;
	se_ext_stch.Word14.src2_out_x = SEINFO_OPCODE_NOP;
	se_ext_stch.Word15.stch2_out_w = SEINFO_OPCODE_NOP;
	se_ext_stch.Word16.Reserved = SEINFO_OPCODE_NOP;

	qc_se_write((uint8_t *) &se_ext_stch, sizeof(SECMD_EXT_STCH_BLIT), 5);

	qc_se_writecmd();
	return qc_se_checkfinish();

}

void SE_EXT_MIRROR(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc, int dir)
{

#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif
	unsigned int split_cnt=0;
	qc_reset_cmd_buffer();

	SECMD_EXT_FMT_CONV se_ext_conv;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.lay_ctl = 0;
	se_ext_conv.Word1.lay_num = 0;
	se_ext_conv.Word1.rslt_argb_order = 0;

	se_ext_conv.Word2.rslt_out_x = 0;
	se_ext_conv.Word2.rslt_out_y = 0;
	se_ext_conv.Word2.rslt_mirror = dir;

	se_ext_conv.Word3.rslt_addr_index = nDstBaseIdx;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W;
	se_ext_conv.Word4.src1_inpt_h = SE_SURFACE_H-8;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word6.src1_addr_index = nSrcBaseIdx;
	se_ext_conv.Word6.src1_inpt_sel = 0;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	//=============================
	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W;
	se_ext_conv.Word4.src1_inpt_h = SPLIT_SIZE_MIR;
	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word2.rslt_out_x = 0;


	for(split_cnt=0;split_cnt<SE_SURFACE_H/SPLIT_SIZE_MIR;split_cnt++) {
		//X_MIRROR
		if(dir == SEINFO_MIRROR_X) {
			se_ext_conv.Word2.rslt_out_y = SPLIT_SIZE_MIR*split_cnt;
		}else if(dir == SEINFO_MIRROR_Y || dir == SEINFO_MIRROR_XY){
			se_ext_conv.Word2.rslt_out_y = SE_SURFACE_H - SPLIT_SIZE_MIR*(split_cnt+1);
		}

		se_ext_conv.Word5.src1_inpt_y = SPLIT_SIZE_MIR*split_cnt;
		qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);
#if 0
		printf("DIR=%d, cnt = %d ; [%d,%d](%d*%d) go [%d,%d]\n",
			dir, split_cnt, se_ext_conv.Word5.src1_inpt_x, se_ext_conv.Word5.src1_inpt_y ,
			se_ext_conv.Word4.src1_inpt_w, se_ext_conv.Word4.src1_inpt_h,
			se_ext_conv.Word2.rslt_out_x, se_ext_conv.Word2.rslt_out_y
			);
#endif
	}


	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W;
	se_ext_conv.Word4.src1_inpt_h = 64;
	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word2.rslt_out_x = 0;

	if(dir == SEINFO_MIRROR_X) {
		se_ext_conv.Word2.rslt_out_y = SE_SURFACE_H - 64;
	}else if(dir == SEINFO_MIRROR_Y || dir == SEINFO_MIRROR_XY){
		se_ext_conv.Word2.rslt_out_y = 0;
	}

	se_ext_conv.Word5.src1_inpt_y = SE_SURFACE_H - 64;
#if 0
	printf("DIR=%d, cnt = %d ; [%d,%d](%d*%d) go [%d,%d]\n",
		dir, split_cnt, se_ext_conv.Word5.src1_inpt_x, se_ext_conv.Word5.src1_inpt_y ,
		se_ext_conv.Word4.src1_inpt_w, se_ext_conv.Word4.src1_inpt_h,
		se_ext_conv.Word2.rslt_out_x, se_ext_conv.Word2.rslt_out_y
		);
#endif
	qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);


#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif

	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] mode=%d, elapsed=%d us\n", __func__,SE_SURFACE_W, SE_SURFACE_H,dir,(t2-t1)*1000000/90090);
#endif

}

void SE_MIRROR(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc, int dir)
{

#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif
	qc_reset_cmd_buffer();

	SECMD_MIRROR	se_mirror;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	memset(&se_mirror, 0x00, sizeof(SECMD_MIRROR));

	se_mirror.Word1.opcode = 0x7;
	se_mirror.Word1.mirrordirection = dir;
	se_mirror.Word1.outputx = 0;
	se_mirror.Word1.outputy = 0;
	se_mirror.Word1.destaddressindex = nDstBaseIdx;

	se_mirror.Word2.width = SE_SURFACE_W;
	se_mirror.Word2.height = SE_SURFACE_H;


	se_mirror.Word3.inputx = 0;
	se_mirror.Word3.inputy = 0;
	se_mirror.Word3.srcaddressindex = nSrcBaseIdx;
	se_mirror.Word4.reserved = 0xf;

	qc_se_write((uint8_t *) &se_mirror, sizeof(SECMD_MIRROR), 4);

#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif
	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] mode=%d, elapsed=%d us\n", __func__,SE_SURFACE_W, SE_SURFACE_H,dir,(t2-t1)*1000000/90090);
#endif

}

void SE_EXT_ROTATE_ALL(int nDstBaseIdx, int nSrcBaseIdx, int dir, int rot270)
{

#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif

	unsigned int rslt_x = 0;
	unsigned int rslt_y = 0;
	SECMD_EXT_FMT_CONV se_ext_conv;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;
	qc_reset_cmd_buffer();

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.lay_ctl = 0;
	se_ext_conv.Word1.lay_num = 0;
	se_ext_conv.Word1.rslt_argb_order = 0;
	
	se_ext_conv.Word2.rslt_rotate = (dir+1);

	se_ext_conv.Word3.rslt_addr_index = nDstBaseIdx;

	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word6.src1_addr_index = nSrcBaseIdx;
	se_ext_conv.Word6.src1_inpt_sel = 0;
	
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	//=================================================
	if ((dir+1)%2 == 1) {
		if (rot270) {
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH1, SE_SURFACE_ROTATE_PITCH);
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH0, SE_SURFACE_ROTATE_PITCH);
		} else {
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH2, SE_SURFACE_ROTATE_PITCH);
			qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH1, SE_SURFACE_ROTATE_PITCH);
		}
	} else {
		qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH2, SE_SURFACE_PITCH);
		qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH1, SE_SURFACE_PITCH);
	}

	// merlin4 ext cmd has limit that both rslt.x and rslt.y should be multiple of 16
	// so rotate is divided into two part
	// 1. the max retangle whitch w, h is multiple of 16
	// 2. the remained retangle

	if (dir == (uint32_t)SEINFO_ROTATE_90) {
		se_ext_conv.Word4.src1_inpt_w = 1920;
		se_ext_conv.Word4.src1_inpt_h = 1072;
		se_ext_conv.Word5.src1_inpt_x = 0;
		se_ext_conv.Word5.src1_inpt_y = 0;
		se_ext_conv.Word2.rslt_out_x = 0;
		se_ext_conv.Word2.rslt_out_y = 0;
		qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

		se_ext_conv.Word4.src1_inpt_w = 1920;
		se_ext_conv.Word4.src1_inpt_h = 8;
		se_ext_conv.Word5.src1_inpt_x = 0;
		se_ext_conv.Word5.src1_inpt_y = 1072;
		se_ext_conv.Word2.rslt_out_x = 1072;
		se_ext_conv.Word2.rslt_out_y = 0;
		qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);
	} else if (dir == (uint32_t)SEINFO_ROTATE_180) {
		se_ext_conv.Word4.src1_inpt_w = 1920;
		se_ext_conv.Word4.src1_inpt_h = 1072;
		se_ext_conv.Word5.src1_inpt_x = 0;
		se_ext_conv.Word5.src1_inpt_y = 0;
		se_ext_conv.Word2.rslt_out_x = 0;
		se_ext_conv.Word2.rslt_out_y = 8;
		qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

		se_ext_conv.Word4.src1_inpt_w = 1920;
		se_ext_conv.Word4.src1_inpt_h = 8;
		se_ext_conv.Word5.src1_inpt_x = 0;
		se_ext_conv.Word5.src1_inpt_y = 1072;
		se_ext_conv.Word2.rslt_out_x = 0;
		se_ext_conv.Word2.rslt_out_y = 0;
		qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);
	} else if (dir == (uint32_t)SEINFO_ROTATE_270) {
		; // Rotate 270 will be done by rotate 180 then rotate 90
	} else {
		printf("input dir error!\n");
	}

#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif
	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] mode=%d, elapsed=%d us\n", __func__,SE_SURFACE_W, SE_SURFACE_H,dir,(t2-t1)*1000000/90090);
#endif

}


void SE_EXT_ROTATE(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc, int dir)
{

#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif

	unsigned int split_cnt_w=0;
	unsigned int split_cnt_h=0;
	unsigned int final_w=0;
	unsigned int final_h=0;
	SECMD_EXT_FMT_CONV se_ext_conv;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;
	qc_reset_cmd_buffer();

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.lay_ctl = 0;
	se_ext_conv.Word1.lay_num = 0;
	se_ext_conv.Word1.rslt_argb_order = 0;

	se_ext_conv.Word2.rslt_out_x = 0;
	se_ext_conv.Word2.rslt_out_y = 0;
	se_ext_conv.Word2.rslt_rotate = (dir+1);

	se_ext_conv.Word3.rslt_addr_index = nDstBaseIdx;


	if(dir == (uint32_t)SEINFO_ROTATE_180) {
		final_w=SE_SURFACE_W;
		final_h=SE_SURFACE_H;
	}else{
		final_w=SE_SURFACE_H;
		final_h=SE_SURFACE_H;
	}
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word6.src1_addr_index = nSrcBaseIdx;
	se_ext_conv.Word6.src1_inpt_sel = 0;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	//=================================================

	se_ext_conv.Word4.src1_inpt_w = SPLIT_SIZE_ROT;
	se_ext_conv.Word4.src1_inpt_h = SPLIT_SIZE_ROT;

	for(split_cnt_h=0;split_cnt_h<(final_h/SPLIT_SIZE_ROT);split_cnt_h++) {
		for(split_cnt_w=0;split_cnt_w<(final_w/SPLIT_SIZE_ROT);split_cnt_w++) {

			se_ext_conv.Word4.src1_inpt_w = SPLIT_SIZE_ROT;
			se_ext_conv.Word4.src1_inpt_h = SPLIT_SIZE_ROT;

			se_ext_conv.Word5.src1_inpt_x = split_cnt_w*SPLIT_SIZE_ROT;
			se_ext_conv.Word5.src1_inpt_y = split_cnt_h*SPLIT_SIZE_ROT;

			if(dir == (uint32_t)SEINFO_ROTATE_90) {
				se_ext_conv.Word2.rslt_out_x = split_cnt_h*SPLIT_SIZE_ROT;
				se_ext_conv.Word2.rslt_out_y = final_h-(split_cnt_w+1)*SPLIT_SIZE_ROT;
			}

			qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

#if 1
			printf("DIR=%d, w_cnt = %d h_cnt=%d; [%d,%d](%d*%d) go [%d,%d]\n",
				dir, split_cnt_w, split_cnt_h, se_ext_conv.Word5.src1_inpt_x, se_ext_conv.Word5.src1_inpt_y ,
				se_ext_conv.Word4.src1_inpt_w, se_ext_conv.Word4.src1_inpt_h,
				se_ext_conv.Word2.rslt_out_x, se_ext_conv.Word2.rslt_out_y
				);
#endif

		}
		if(final_w > (final_w/SPLIT_SIZE_ROT)*SPLIT_SIZE_ROT) {
			se_ext_conv.Word5.src1_inpt_x = final_w - SPLIT_SIZE_ROT;
			se_ext_conv.Word5.src1_inpt_y = split_cnt_h*SPLIT_SIZE_ROT;

			if(dir == (uint32_t)SEINFO_ROTATE_90) {
				se_ext_conv.Word2.rslt_out_x = split_cnt_h*SPLIT_SIZE_ROT;
				se_ext_conv.Word2.rslt_out_y = 0;

			}

			qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

#if 1
			printf("margin_w DIR=%d, w_cnt = %d h_cnt=%d; [%d,%d](%d*%d) go [%d,%d]\n",
				dir, split_cnt_w, split_cnt_h, se_ext_conv.Word5.src1_inpt_x, se_ext_conv.Word5.src1_inpt_y ,
				se_ext_conv.Word4.src1_inpt_w, se_ext_conv.Word4.src1_inpt_h,
				se_ext_conv.Word2.rslt_out_x, se_ext_conv.Word2.rslt_out_y
				);
#endif
		}
	}
//final
#if 1
	if(final_h > (final_h/SPLIT_SIZE_ROT)*SPLIT_SIZE_ROT) {
		for(split_cnt_w=0;split_cnt_w<(final_w/SPLIT_SIZE_ROT);split_cnt_w++) {
			se_ext_conv.Word4.src1_inpt_w = SPLIT_SIZE_ROT;
			se_ext_conv.Word4.src1_inpt_h = SPLIT_SIZE_ROT;

			se_ext_conv.Word5.src1_inpt_x = split_cnt_w*SPLIT_SIZE_ROT;
			se_ext_conv.Word5.src1_inpt_y = final_h - SPLIT_SIZE_ROT;

			if(dir == (uint32_t)SEINFO_ROTATE_90) {
				se_ext_conv.Word2.rslt_out_x = final_w - SPLIT_SIZE_ROT;
				se_ext_conv.Word2.rslt_out_y = final_h - (split_cnt_w+1)*SPLIT_SIZE_ROT;
			}

			qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

#if 0
			printf("margin_h DIR=%d, w_cnt = %d h_cnt=%d; [%d,%d](%d*%d) go [%d,%d]\n",
				dir, split_cnt_w, split_cnt_h, se_ext_conv.Word5.src1_inpt_x, se_ext_conv.Word5.src1_inpt_y ,
				se_ext_conv.Word4.src1_inpt_w, se_ext_conv.Word4.src1_inpt_h,
				se_ext_conv.Word2.rslt_out_x, se_ext_conv.Word2.rslt_out_y
				);
#endif

		}
		if(final_w > (final_w/SPLIT_SIZE_ROT)*SPLIT_SIZE_ROT) {
			se_ext_conv.Word5.src1_inpt_x = final_w - SPLIT_SIZE_ROT;
			se_ext_conv.Word5.src1_inpt_y = final_h - SPLIT_SIZE_ROT;

			if(dir == (uint32_t)SEINFO_ROTATE_90) {
				se_ext_conv.Word2.rslt_out_x = final_w - SPLIT_SIZE_ROT;
				se_ext_conv.Word2.rslt_out_y = 0;
			}
			qc_se_write((uint8_t *) &se_ext_conv, sizeof(SECMD_EXT_FMT_CONV), 3);

#if 0
			printf("margin_h DIR=%d, w_cnt = %d h_cnt=%d; [%d,%d](%d*%d) go [%d,%d]\n",
				dir, split_cnt_w, split_cnt_h, se_ext_conv.Word5.src1_inpt_x, se_ext_conv.Word5.src1_inpt_y ,
				se_ext_conv.Word4.src1_inpt_w, se_ext_conv.Word4.src1_inpt_h,
				se_ext_conv.Word2.rslt_out_x, se_ext_conv.Word2.rslt_out_y
				);
#endif
		}
	}
#endif

#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif
	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] mode=%d, elapsed=%d us\n", __func__,SE_SURFACE_W, SE_SURFACE_H,dir,(t2-t1)*1000000/90090);
#endif

}

void SE_ROTATE(int nDstBaseIdx, RECT_SE *prcDst, int nSrcBaseIdx, RECT_SE *prcSrc, int dir)
{
#if defined(SE_MEASURE)
	unsigned int t1 = rtd_inl(0xb801b690);
	unsigned int t2 =0;
#endif
	qc_reset_cmd_buffer();

	SECMD_ROTATE	se_rotate;
	SE_COLOR_FORMAT_EX	SeColorFormatEx;

	memset(&SeColorFormatEx, 0, sizeof(SeColorFormatEx));
	SeColorFormatEx.bInputSrcColorFormatBigEndian  = 0;
	SeColorFormatEx.bInputDestColorFormatBigEndian = 0;
	SeColorFormatEx.bOutputColorFormatBigEndian    = 0;
	SeColorFormatEx.bEnableRoundingOption          = 0;
	SE_SetColorFormat(SEINFO_COLOR_FORMAT_ARGB8888,
		SEINFO_ARGB_ALPHA_LEFTMOST, SEINFO_ARGB_ALPHA_LEFTMOST, &SeColorFormatEx);

	memset(&se_rotate, 0x00, sizeof(SECMD_ROTATE));

	se_rotate.Word1.opcode = 0xa;
	se_rotate.Word1.rotatemode = dir;
	se_rotate.Word2.inputx = 0;
	se_rotate.Word2.inputy = 0;
	se_rotate.Word2.srcaddressindex = nSrcBaseIdx;

	se_rotate.Word3.outputx = 0;
	se_rotate.Word3.outputy = 0;
	se_rotate.Word3.destaddressindex = nDstBaseIdx;

	if(dir == (uint32_t)SEINFO_ROTATE_180) {
		se_rotate.Word4.inputwidth = SE_SURFACE_W;
		se_rotate.Word4.inputheight = SE_SURFACE_H;
	}else{
		se_rotate.Word4.inputwidth = SE_SURFACE_H;
		se_rotate.Word4.inputheight = SE_SURFACE_H;
	}


	qc_se_write((uint8_t *) &se_rotate, sizeof(SECMD_ROTATE), 4);

#if defined(SE_MEASURE)
	t1 = rtd_inl(0xb801b690);
#endif

	qc_se_writecmd();
	qc_se_checkfinish();

#if defined(SE_MEASURE)
	t2 = rtd_inl(0xb801b690);
	printf("%s test [%d,%d] mode=%d, elapsed=%d us\n", __func__,SE_SURFACE_W, SE_SURFACE_H,dir,(t2-t1)*1000000/90090);
#endif

}

bool ExamineRotate(int nBaseIdx, RECT_SE *prcSrc, RECT_SE *prcDst, uint32_t u32Color, int dir, char *pMsg)
{
    bool bRet = true;
    uint32_t *pDst;
    int x, y;

    flush_dcache_all();
    if(nBaseIdx == 0) {
        pDst = (uint32_t*)_gpSurface[0];
    } else {
        pDst = (uint32_t*)_gpSurface[1];
    }

	if (dir == SEINFO_ROTATE_90) {
		if (pDst[SE_SURFACE_H*(SE_SURFACE_W-(prcSrc->x+prcSrc->w))] != u32Color ||





			pDst[SE_SURFACE_H*(SE_SURFACE_W-(prcSrc->x+prcSrc->w))+prcSrc->h-1] != u32Color ||
			pDst[SE_SURFACE_H*(prcSrc->x)+1] != u32Color ||
			pDst[SE_SURFACE_H*(prcSrc->x)+prcSrc->h-1] != u32Color) {
			printf("90_color = (0x%x, 0x%x, 0x%x, 0x%x)\n", 
				pDst[SE_SURFACE_H*(SE_SURFACE_W-(prcSrc->x+prcSrc->w))],
				pDst[SE_SURFACE_H*(SE_SURFACE_W-(prcSrc->x+prcSrc->w))+prcSrc->h-1],
				pDst[SE_SURFACE_H*(prcSrc->x)+1],
				pDst[SE_SURFACE_H*(prcSrc->x)+prcSrc->h-1]);
			bRet = false;
			goto ExamineRotate_Exit;
		}
	} else if (dir == SEINFO_ROTATE_180) {
		if (pDst[SE_SURFACE_W*(SE_SURFACE_H-prcSrc->h)+(SE_SURFACE_W-(prcSrc->x+prcSrc->w))] != u32Color ||
			pDst[SE_SURFACE_W*(SE_SURFACE_H-prcSrc->h)+(SE_SURFACE_W-prcSrc->x-1)] != u32Color ||
			pDst[SE_SURFACE_W*SE_SURFACE_H-(prcSrc->x+prcSrc->w-1)] != u32Color ||
			pDst[SE_SURFACE_W*SE_SURFACE_H-(SE_SURFACE_W-prcSrc->x-1)] != u32Color) {
			printf("180_color = (0x%x, 0x%x, 0x%x, 0x%x)\n", 
				pDst[SE_SURFACE_W*(SE_SURFACE_H-prcSrc->h)+(SE_SURFACE_W-(prcSrc->x+prcSrc->w))],
				pDst[SE_SURFACE_W*(SE_SURFACE_H-prcSrc->h)+(SE_SURFACE_W-prcSrc->x-1)],
				pDst[SE_SURFACE_W*SE_SURFACE_H-(prcSrc->x+prcSrc->w-1)],
				pDst[SE_SURFACE_W*SE_SURFACE_H-(SE_SURFACE_W-prcSrc->x-1)]);
			bRet = false;
			goto ExamineRotate_Exit;
		}
	} else if (dir == SEINFO_ROTATE_270) {
		if (pDst[SE_SURFACE_H*(prcSrc->x)+SE_SURFACE_H-prcSrc->h] != u32Color ||
			pDst[SE_SURFACE_H*(prcSrc->x+1)-1] != u32Color ||
			pDst[SE_SURFACE_H*(prcSrc->x+prcSrc->w-1)+SE_SURFACE_H-prcSrc->h] != u32Color ||
			pDst[SE_SURFACE_H*(prcSrc->x+prcSrc->w)-1] != u32Color) {
			printf("270_color = (0x%x, 0x%x, 0x%x, 0x%x)\n", 
				pDst[SE_SURFACE_H*(prcSrc->x)+SE_SURFACE_H-prcSrc->h],
				pDst[SE_SURFACE_H*(prcSrc->x+1)-1],
				pDst[SE_SURFACE_H*(prcSrc->x+prcSrc->w-1)+SE_SURFACE_H-prcSrc->h],
				pDst[SE_SURFACE_H*(prcSrc->x+prcSrc->w)-1]);
			bRet = false;
			goto ExamineRotate_Exit;
		}
	} else {
		printf("input data error!\n");
	}

ExamineRotate_Exit:

    return bRet;
}


bool ExamineRectangle(int nBaseIdx, RECT_SE *prcSrc, uint32_t u32Color, char *pMsg)
{
    bool bRet=true;
    uint32_t *pDst;
    int x, y;

    flush_dcache_all();
    if(nBaseIdx == 0)
    {
        pDst=(uint32_t*)_gpSurface[0];
    }
    else
    {
        pDst=(uint32_t*)_gpSurface[1];
    }

    for(y = prcSrc->y; y < prcSrc->y+prcSrc->h; y++)
    {
        for(x = prcSrc->x; x < prcSrc->x+prcSrc->w; x++)
        {
//            printf("%s (%d, %d) = 0x%08x : 0x%08x\r\n\r\n", pMsg, x, y, pDst[y*SE_SURFACE_W+x], u32Color);
            if(pDst[y*SE_SURFACE_W+x] != u32Color)
            {

                printf("%s \nGet error pixel at coordinate (%d, %d) value = 0x%08X.\n\tRight value is 0x%08X\r\n",
			pMsg, x, y, pDst[y*SE_SURFACE_W+x], u32Color);

                printf("\tPhysicall address 0x%p = %08X\n", &pDst[y*SE_SURFACE_W+x-2], pDst[y*SE_SURFACE_W+x-2]);
                printf("\tPhysicall address 0x%p = %08X\n", &pDst[y*SE_SURFACE_W+x-1], pDst[y*SE_SURFACE_W+x-1]);
                printf("\tPhysicall address 0x%p = %08X(Error pixel is here!)\n", &pDst[y*SE_SURFACE_W+x], pDst[y*SE_SURFACE_W+x]);
                printf("\tPhysicall address 0x%p = %08X\n", &pDst[y*SE_SURFACE_W+x+1], pDst[y*SE_SURFACE_W+x+1]);
                printf("\tPhysicall address 0x%p = %08X\n", &pDst[y*SE_SURFACE_W+x+2], pDst[y*SE_SURFACE_W+x+2]);
                bRet = false;
                printf("\t----------------------\n");
                goto ExamineRectangle_Exit;

            }
        }
    }

ExamineRectangle_Exit:

    return bRet;
}

#define TEST_DRAW_RECTANGLE_NUM     4
typedef struct _TestItemDrawRectangle
{
    RECT_SE rc;
    uint32_t u32Color;
} TestItemDrawRectangle;
TestItemDrawRectangle _gDrawRectangleTestItem[TEST_DRAW_RECTANGLE_NUM] =
{
    {{0,   0,   400, 400}, 0xFFFF0000},
    {{200, 160, 800, 256}, 0xFF00FFFF},
    {{160, 200, 960, 480}, 0x0000FFFF},
    {{256, 80,  720, 760}, 0x00FFFF00}
};

bool Test_EXTDrawRectangle(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemDrawRectangle *pTest=&_gDrawRectangleTestItem[0];
	printf_info("SE: EXT Rectangle Fill Test...\r\n");
	for(test=0; test<TEST_DRAW_RECTANGLE_NUM; test++) {
		printf_info("Test %d: EXT Fill (%d, %d, %d, %d) 0x%08x rectangle.\r\n",
			test,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		// Fill the destination background color to black.
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000040);


		EXTDrawRectangle(1,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		printf_info(szMsg, "EXT Rectangle Fill Test %d: ", test);

		snprintf(szMsg,256, "EXT Rectangle Fill Test %d: ", test);
		bRet = ExamineRectangle(1, &pTest[test].rc, pTest[test].u32Color, szMsg);
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT Rectangle Fill Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

#define TEST_COPY_NUM           4
typedef struct _TestItemCopy
{
    RECT_SE rcSrc;
    RECT_SE rcDst;
    uint32_t u32Color;
} TestItemCopy;
TestItemCopy _gTestItemCopy[TEST_COPY_NUM]=
{
    {{0,   0,   400, 400}, {400, 400, 400, 400}, 0xFFFF0000},
    {{200, 160, 800, 256}, {320, 320, 800, 256}, 0xFF00FFFF},
    {{160, 200, 960, 480}, {880, 160, 960, 480}, 0x0000FFFF},
    {{256, 80,  720, 760}, {512, 200, 720, 760}, 0x00FFFF00}
};

bool Test_EXTCopy(void)
{
    bool bRet=true;
    int test;
    char szMsg[256];
    TestItemCopy *pTest = &_gTestItemCopy[0];

    printf_info("SE: EXT Copy Test...\r\n");
    for(test=0; test<TEST_COPY_NUM; test++)
    {
        printf_info("Test %d: EXT Copy (%d, %d, %d, %d) to (%d, %d).\r\n",
               test,
               pTest[test].rcSrc.x, pTest[test].rcSrc.y,
               pTest[test].rcSrc.w, pTest[test].rcSrc.h,
               pTest[test].rcDst.x, pTest[test].rcDst.y);

        // step 1: Fill the destination background color to black.
        EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);

        // step 2: Fill source rectangle.
        EXTDrawRectangle(0,
                      pTest[test].rcSrc.x,
                      pTest[test].rcSrc.y,
                      pTest[test].rcSrc.w,
                      pTest[test].rcSrc.h,
                      pTest[test].u32Color);

        // step 3: copy from source to destination.
        EXTCopyRectangle(1, &pTest[test].rcDst, 0, &pTest[test].rcSrc);

        printf_info(szMsg, "EXT Copy Test %d: ", test);

	snprintf(szMsg,256, "EXT Copy Test %d: ", test);
        bRet = ExamineRectangle(1, &pTest[test].rcDst, pTest[test].u32Color, szMsg);
        if(bRet == false)
        {
            break;
        }
        printf_info("\r\nEXT Copy Test %d: OK\r\n\r\n", test);
    }

    return bRet;
}

bool Test_EXTCopy_2(TestItemCopy *pTest)
{
	bool bRet=true;
	int test = 0;
	char szMsg[256];
#if 0
	RECT_SE fillrect;
	fillrect.x=0;
	fillrect.y=0;
	fillrect.w=SE_SURFACE_W;
	fillrect.h=SE_SURFACE_H;
#endif

	printf_info("SE: EXT Copy Test...\r\n");

		printf_info("Test %d: EXT Copy (%d, %d, %d, %d) to (%d, %d).\r\n",
		test,
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);


        // step 1: Fill the destination background color to black.
	EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);

#if 0
	bRet = ExamineRectangle(1, &fillrect, 0, szMsg);
	if(bRet == false) {
		printf("\r\nEXT COPY Test FILL dst %d: NG\r\n\r\n", test);
		printf("Test %d: EXT Copy (%d, %d, %d, %d) to (%d, %d).\r\n",
		test,
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);
		return bRet;
	}
#endif
	EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);

#if 0
	bRet = ExamineRectangle(0, &fillrect, 0, szMsg);
	if(bRet == false) {
		printf("\r\nEXT COPY Test FILL src %d: NG\r\n\r\n", test);
		printf("Test %d: EXT Copy (%d, %d, %d, %d) to (%d, %d).\r\n",
		test,
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);
		return bRet;
	}
#endif
        // step 2: Fill source rectangle.
        EXTDrawRectangle(0,
                      pTest[test].rcSrc.x,
                      pTest[test].rcSrc.y,
                      pTest[test].rcSrc.w,
                      pTest[test].rcSrc.h,
                      pTest[test].u32Color);
#if 0
	bRet = ExamineRectangle(0, &pTest[test].rcSrc, pTest[test].u32Color, szMsg);
	if(bRet == false) {
		printf("\r\nEXT COPY Test FILL src %d: NG\r\n\r\n", test);
		printf("Test %d: EXT Copy (%d, %d, %d, %d) to (%d, %d).\r\n",
		test,
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);
		return bRet;
	}
#endif
        // step 3: copy from source to destination.
	EXTCopyRectangle(1, &pTest[test].rcDst, 0, &pTest[test].rcSrc);

	printf_info(szMsg, "EXT Copy Test %d: ", test);
	snprintf(szMsg,256, "EXT Copy Test %d: ", test);
	bRet = ExamineRectangle(1, &pTest[test].rcDst, pTest[test].u32Color, szMsg);
	//bRet = ExamineRectangle(0, &pTest[test].rcDst, 0, szMsg);
	if(bRet == false) {
		printf("\r\nEXT COPY Test final cpy %d: NG\r\n\r\n", test);
		printf("Test %d: EXT Copy (%d, %d, %d, %d) to (%d, %d).\r\n",
		test,
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);
		return bRet;
	}else {
	printf_info("\r\nEXT Copy Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}




#define TEST_BLEND_NUM           4
typedef struct _TestItemBlend
{
    RECT_SE rcSrc;
    RECT_SE rcDst;
    uint32_t u32ColorSrc;
    uint32_t u32ColorDst;
    uint32_t u32ColorResult;
} TestItemBlend;
TestItemBlend _gTestItemBlend[TEST_BLEND_NUM]=
{
#if 0
    {{0,   0,   100, 100}, { 50,  50, 100, 100}, 0x80800000, 0x20490000, 0x50650000},
    {{200, 160, 100, 156}, {200, 200, 100, 156}, 0xC000C700, 0x80008000, 0xb000b600},
    {{160, 200, 160, 120}, {200, 160, 160, 120}, 0x63000085, 0x400000D3, 0x4e0000b5},
    {{156, 80,  120, 100}, {800, 900, 120, 100}, 0x20987654, 0x00234567, 0x04324b65}
#else
    //    source RECT    ,   destination RECT  , src color , dst color , result color
    {{0,   0,   400, 400}, {400, 400, 400, 400}, 0x80800000, 0x20490000, 0x50650000},
    {{200, 160, 800, 256}, {200, 200, 800, 256}, 0xC000C700, 0x80008000, 0xb000b600},
    {{160, 200, 960, 480}, {200, 160, 960, 480}, 0x63000085, 0x400000D3, 0x4e0000b5},
    {{256, 80,  720, 760}, {200, 200, 720, 760}, 0x20987654, 0x00234567, 0x04324b65},
#endif
};

bool Test_EXTBlend(void) {
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemBlend *pTest=&_gTestItemBlend[0];

	printf_info("SE: EXT Blend Test...\r\n");
	for(test=0; test<TEST_BLEND_NUM; test++) {
		printf_info("Test %d: EXT Blend (%d, %d, %d, %d) to (%d, %d).\r\n",
			test,
			pTest[test].rcSrc.x, pTest[test].rcSrc.y,
			pTest[test].rcSrc.w, pTest[test].rcSrc.h,
			pTest[test].rcDst.x, pTest[test].rcDst.y);

		// step 1: Fill the destination background color to black.
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);
		EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);

		// step 2: Fill source rectangle.
		EXTDrawRectangle(0,
			pTest[test].rcSrc.x,
			pTest[test].rcSrc.y,
			pTest[test].rcSrc.w,
			pTest[test].rcSrc.h,
			pTest[test].u32ColorSrc);

		// step 3: Fill destination rectangle.
		EXTDrawRectangle(1,
			pTest[test].rcDst.x,
			pTest[test].rcDst.y,
			pTest[test].rcDst.w,
			pTest[test].rcDst.h,
			pTest[test].u32ColorDst);

		// step 4: Blend from source to destination.
		EXTBlendRectangle(1, &pTest[test].rcDst, 0, &pTest[test].rcSrc);


		printf_info(szMsg, "EXT Blend Test %d: ", test);

		snprintf(szMsg,256, "EXT Blend Test %d: ", test);
		bRet = ExamineRectangle(1, &pTest[test].rcDst, pTest[test].u32ColorResult, szMsg);
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT Blend Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

bool Test_EXTBlend_2(TestItemBlend *pTest) {
	bool bRet=true;
	char szMsg[256];
	int test =0;

#if 0
	RECT_SE fillrect;
	fillrect.x=0;
	fillrect.y=0;
	fillrect.w=SE_SURFACE_W;
	fillrect.h=SE_SURFACE_H;
#endif


	printf(szMsg, "EXT Blend Test : ");


	// step 1: Fill the destination background color to black.
	EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);
#if 0
	printf(szMsg, "EXT Blend Test FILL 1: ");

	bRet = ExamineRectangle(1, &fillrect, 0, szMsg);
	if(bRet == false) {
		printf("\r\nEXT Blend Test FILL %d: NG\r\n\r\n", test);

	printf("Test EXT Blend (%d, %d, %d, %d) to (%d, %d).\r\n",
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);

		return bRet;
	}

	printf(szMsg, "EXT Blend Test FILL 2: ");
#endif
	EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);
#if 0
	bRet = ExamineRectangle(0, &fillrect, 0, szMsg);
	if(bRet == false) {
		printf("\r\nEXT Blend Test FILL %d: NG\r\n\r\n", test);
	printf("Test EXT Blend (%d, %d, %d, %d) to (%d, %d).\r\n",
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);

		return bRet;
	}
#endif

	// step 2: Fill source rectangle.
	EXTDrawRectangle(0,
		pTest[test].rcSrc.x,
		pTest[test].rcSrc.y,
		pTest[test].rcSrc.w,
		pTest[test].rcSrc.h,
		pTest[test].u32ColorSrc);
#if 0
	bRet = ExamineRectangle(0, &pTest[test].rcSrc, pTest[test].u32ColorSrc, szMsg);
	if(bRet == false) {
		printf("\r\nEXT Blend Test write src %d: NG\r\n\r\n", test);
	printf("Test EXT Blend (%d, %d, %d, %d) to (%d, %d).\r\n",
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);

		return bRet;
	}
#endif
	// step 3: Fill destination rectangle.
	EXTDrawRectangle(1,
		pTest[test].rcDst.x,
		pTest[test].rcDst.y,
		pTest[test].rcDst.w,
		pTest[test].rcDst.h,
		pTest[test].u32ColorDst);
#if 0
	bRet = ExamineRectangle(1, &pTest[test].rcDst, pTest[test].u32ColorDst, szMsg);
	if(bRet == false) {
		printf("\r\nEXT Blend Test write dst %d: NG\r\n\r\n", test);
	printf("Test EXT Blend (%d, %d, %d, %d) to (%d, %d).\r\n",
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);

		return bRet;
	}
#endif
	// step 4: Blend from source to destination.
	EXTBlendRectangle(1, &pTest[test].rcDst, 0, &pTest[test].rcSrc);


	printf(szMsg, "EXT Blend Test blend: ");


	snprintf(szMsg,256, "EXT Blend Test %d: ", test);
	bRet = ExamineRectangle(1, &pTest[test].rcDst, pTest[test].u32ColorResult, szMsg);
	if(bRet == false) {

	printf("Test EXT Blend (%d, %d, %d, %d) to (%d, %d).\r\n",
		pTest[test].rcSrc.x, pTest[test].rcSrc.y,
		pTest[test].rcSrc.w, pTest[test].rcSrc.h,
		pTest[test].rcDst.x, pTest[test].rcDst.y);

		return bRet;
	}
	printf("\r\nEXT Blend Test final %d: OK\r\n\r\n", test);

	return bRet;
}

void PrepareStretchBitmap(uint32_t *pBitmap,
                          int nBmpWidth,
                          int nBmpHeight,
                          uint32_t dstAddr,
                          int nX,
                          int nY,
                          int nDstPitch)
{
    uint8_t *pDst=(uint8_t*)dstAddr;
    int i;

    for(i=0;i<nBmpHeight;i++)
    {
        memcpy(&pDst[nDstPitch*(nY+i)+nX*4], &pBitmap[i*nBmpWidth], sizeof(uint32_t)*nBmpWidth);
    }

	flush_dcache_all();
}

/*return false  --> fail*/
bool cmp_argb_channel(unsigned int x, unsigned int y) {

	int tol_channel = 0;

	if( absdiff(get_mask(x,24), get_mask(y,24)) >= TOLERANCE) {
		//printf("(%d, %d) \n",get_mask(x,24) ,get_mask(y,24));
		tol_channel++;
	}

	if( absdiff(get_mask(x,16), get_mask(y,16)) >= TOLERANCE) {
		//printf("(%d, %d) \n",get_mask(x,16) ,get_mask(y,16));
		tol_channel++;
	}

	if( absdiff(get_mask(x,8), get_mask(y,8)) >= TOLERANCE) {
		//printf("(%d, %d) \n",get_mask(x,8) ,get_mask(y,8));
		tol_channel++;
	}

	if( absdiff(get_mask(x,0), get_mask(y,0)) >= TOLERANCE) {
		//printf("(%d, %d) \n",get_mask(x,0) ,get_mask(y,0));
		tol_channel++;
	}

	if(tol_channel > 3) {
		printf("(%08X, %08X) \n",x, y);
		return false;
	}

	return true;
}

bool ExamineStretchRectangle(
	uint32_t srcAddr,
	int nX,
	int nY,
	int nW,
	int nH,
	int nSrcPitch,
	uint32_t *pResult,
	char *pMsg)
{
	uint32_t *pSrc, *pPat;
	bool bRet = true;
	int x, y;

	flush_dcache_all();
	pPat = pResult;
	for(y=0;y<nH;y++) {
		pSrc = (uint32_t*)(srcAddr + (nY + y) * nSrcPitch + nX * 4);
		for(x=0;x<nW;x++) {
			if(cmp_argb_channel(pSrc[x],pPat[x]) == false) {
				bRet = false;
				printf("%s (%d, %d) = 0x%08x : 0x%08x\r\n", pMsg, x, y, pSrc[x], pPat[x]);
				goto ExamineStretchRectangle_Exit;
			}
		}
		pPat += nW;
	}
ExamineStretchRectangle_Exit:
	return bRet;
}

#if CONFIG_FT
#define TEST_STRETCH_NUM        2
#else
#define TEST_STRETCH_NUM        6
#endif
typedef struct _TestItemStretch
{
    RECT_SE rcSrc;
    RECT_SE rcDst;
    uint32_t *pResult;
} TestItemStretch;
TestItemStretch _gTestItemStretch[TEST_STRETCH_NUM]=
{
    //    source RECT    ,   destination RECT  ,
    {{0, 0, BITMAP_WIDTH, BITMAP_HEIGHT}, {400, 400, 180, 60},  _gStretchBmp1},
    {{0, 0, BITMAP_WIDTH, BITMAP_HEIGHT}, {320, 320,  80, 160}, _gStretchBmp2},
#if !CONFIG_FT
    {{0, 0, BITMAP_WIDTH, BITMAP_HEIGHT}, {880, 160, 180, 180}, _gStretchBmp3},
    {{0, 0, BITMAP_WIDTH, BITMAP_HEIGHT}, {512, 200,  60, 60},  _gStretchBmp4},
    {{0, 0, BITMAP_WIDTH, BITMAP_HEIGHT}, {512, 200,  80, 32},  _gStretchBmp5},
    {{0, 0, BITMAP_WIDTH, BITMAP_HEIGHT}, {512, 200,  60, 32},  _gStretchBmp6},
#endif

};


bool Test_EXTStretch(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemStretch *pTest=&_gTestItemStretch[0];

	printf_info("SE: EXT Stretch Test...\r\n");
	for(test=0; test<TEST_STRETCH_NUM; test++) {
		printf_info("Test %d: Stretch (%d, %d, %d, %d) to (%d, %d, %d, %d).\r\n",
			test,
			pTest[test].rcSrc.x, pTest[test].rcSrc.y,
			pTest[test].rcSrc.w, pTest[test].rcSrc.h,
			pTest[test].rcDst.x, pTest[test].rcDst.y,
			pTest[test].rcDst.w, pTest[test].rcDst.h);

		// step 1: Fill the destination background color to black.
		EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);

		// step 2: Prepare source bitmap.
		PrepareStretchBitmap(_gStretchBmp,
			BITMAP_WIDTH,
			BITMAP_HEIGHT,
			(uint32_t)_gpSurface[0],
			pTest[test].rcSrc.x,
			pTest[test].rcSrc.y,
			SE_SURFACE_PITCH);

		// step 3: Stretch from source to destination.

		EXTStretchRectangle(1, &pTest[test].rcDst, 0, &pTest[test].rcSrc);

#if 1
		// step 4: examine the result of stretch
		printf_info(szMsg, "EXT Stretch Test %d: ", test);
		bRet = ExamineStretchRectangle((uint32_t)_gpSurface[1],
			pTest[test].rcDst.x,
			pTest[test].rcDst.y,
			pTest[test].rcDst.w,
			pTest[test].rcDst.h,
			SE_SURFACE_PITCH,
			pTest[test].pResult,
			szMsg);
#endif
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT Stretch Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

#if !CONFIG_FT
#define TEST_MIR_NUM           3
typedef struct _TestItemMirror
{
    RECT_SE rcSrc;
    RECT_SE rcDst;
    uint32_t u32Color;
    uint32_t u32dir;
} TestItemMir;
TestItemMir _gTestItemMir[TEST_MIR_NUM]=
{
    //    source RECT    ,   destination RECT  , color , direction(X,Y,XY)
    {{0,   0,   200, 100}, {SE_SURFACE_W-200,                    0, 200, 100}, 0xff112233, (uint32_t)SEINFO_MIRROR_X},
    {{200, 200, 100, 100}, {             200, SE_SURFACE_H-200-100, 100, 100}, 0xff112233, (uint32_t)SEINFO_MIRROR_Y},
    {{0,   0,   280, 120}, {SE_SURFACE_W-280,     SE_SURFACE_H-120, 280, 120}, 0xff112233, (uint32_t)SEINFO_MIRROR_XY}
};

bool Test_Mirror(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemMir *pTest=&_gTestItemMir[0];

	printf_info("SE: Mirror Test...\r\n");
	for(test=0; test<TEST_MIR_NUM; test++) {
		printf_info("Test %d: Mirror (%d, %d, %d, %d) to (%d, %d, %d, %d). type = %d\r\n",
			test,
			pTest[test].rcSrc.x, pTest[test].rcSrc.y,
			pTest[test].rcSrc.w, pTest[test].rcSrc.h,
			pTest[test].rcDst.x, pTest[test].rcDst.y,
			pTest[test].rcDst.w, pTest[test].rcDst.h,
			pTest[test].u32dir);

		// step 1: Fill the destination background color to black.
		EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);

		// step 2: Prepare rect.
		EXTDrawRectangle(0,
			pTest[test].rcSrc.x, pTest[test].rcSrc.y,
			pTest[test].rcSrc.w, pTest[test].rcSrc.h,
			pTest[test].u32Color);

		// step 3: Stretch from source to destination.
		SE_EXT_MIRROR(1, &pTest[test].rcDst, 0, &pTest[test].rcSrc , pTest[test].u32dir);

		// step 4: examine the result of stretch
		printf_info(szMsg, "Mirror Test %d: ", test);

		snprintf(szMsg,256, "Mirror Test %d: ", test);
		bRet = ExamineRectangle(1, &pTest[test].rcDst, pTest[test].u32Color, szMsg);
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT MIRROR Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

#define TEST_ROT_NUM           3
typedef struct _TestItemRotate
{
    RECT_SE rcSrc;
    RECT_SE rcDst;
    uint32_t u32Color;
    uint32_t u32dir;
} TestItemRot;

TestItemRot _gTestItemRot[TEST_ROT_NUM]=
{
    //    source RECT    ,   destination RECT  , src color , dst color , result color
    {{SE_SURFACE_H-200, 0, 200, 100}, {0,   0,   100, 200}, 0xff112233, (uint32_t)SEINFO_ROTATE_90},
    {{SE_SURFACE_H-200, 0, 200, 100}, {0,   0,   100, 200}, 0xff112233, (uint32_t)SEINFO_ROTATE_180},
    {{SE_SURFACE_H-200, 0, 200, 100}, {0,   0,   100, 200}, 0xff112233, (uint32_t)SEINFO_ROTATE_270}
};

bool Test_Rotate(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemRot *pTest=&_gTestItemRot[0];

	printf_info("SE: Rotate Test...\r\n");
	for(test=0; test<TEST_ROT_NUM; test++) {
		printf_info("Test %d: Rotate (%d, %d, %d, %d) to (%d, %d, %d, %d). type = %d\r\n",
			test,
			pTest[test].rcSrc.x, pTest[test].rcSrc.y,
			pTest[test].rcSrc.w, pTest[test].rcSrc.h,
			pTest[test].rcDst.x, pTest[test].rcDst.y,
			pTest[test].rcDst.w, pTest[test].rcDst.h,
			pTest[test].u32dir);

		// step 1: Fill the destination background color to black.
		EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000000);
#if 0
		EXTDrawRectangle(0,    0,  0, 256, 128, 0x11111111);
		EXTDrawRectangle(0,  256,  0, 256, 128, 0x22222222);
		EXTDrawRectangle(0,  512,  0, 256, 128, 0x33333333);
		EXTDrawRectangle(0,  768,  0, 256, 128, 0x44444444);
		EXTDrawRectangle(0, 1024,  0,  56, 128, 0x55555555);

		EXTDrawRectangle(0,    0,  128, 256, 128, 0x66666666);
		EXTDrawRectangle(0,  256,  128, 256, 128, 0x77777777);
		EXTDrawRectangle(0,  512,  128, 256, 128, 0x88888888);
		EXTDrawRectangle(0,  768,  128, 256, 128, 0x99999999);
		EXTDrawRectangle(0, 1024,  128,  56, 128, 0xaaaaaaaa);

		EXTDrawRectangle(0,    0,  256, 256, 128, 0xbbbbbbbb);
		EXTDrawRectangle(0,  256,  256, 256, 128, 0xcccccccc);
		EXTDrawRectangle(0,  512,  256, 256, 128, 0xdddddddd);
		EXTDrawRectangle(0,  768,  256, 256, 128, 0xeeeeeeee);
		EXTDrawRectangle(0, 1024,  256,  56, 128, 0xffffffff);
#endif


		// step 2: Prepare rect.
		EXTDrawRectangle(0,
			pTest[test].rcSrc.x, pTest[test].rcSrc.y,
			pTest[test].rcSrc.w, pTest[test].rcSrc.h,
			pTest[test].u32Color);

		// step 3: Stretch from source to destination.
		// Note: Rotate 270 will be done by rotate 180 then rotate 90
		if(pTest[test].u32dir == (uint32_t)SEINFO_ROTATE_270) {
			SE_EXT_ROTATE_ALL(1, 0, SEINFO_ROTATE_180, 0);
			SE_EXT_ROTATE_ALL(0, 1, SEINFO_ROTATE_90, 1);
		} else {
			SE_EXT_ROTATE_ALL(1, 0, pTest[test].u32dir, 0);
		}
		
		// step 4: examine the result of stretch
		printf_info(szMsg, "Rotate Test %d: ", test);

		snprintf(szMsg,256, "Rotate Test %d: ", test);
		if(pTest[test].u32dir == (uint32_t)SEINFO_ROTATE_270) {
			bRet = ExamineRotate(0, &pTest[test].rcSrc, &pTest[test].rcDst, pTest[test].u32Color, pTest[test].u32dir, szMsg);
		} else {
			bRet = ExamineRotate(1, &pTest[test].rcSrc, &pTest[test].rcDst, pTest[test].u32Color, pTest[test].u32dir, szMsg);
		}
		if(bRet == false) {
			break;
		}

		printf_info("\r\nEXT Rotate Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}
#endif

#define TEST_I8_NUM     4
typedef struct _TestItemI8
{
    RECT_SE rc;
    uint32_t u32Color;
} TestItemI8;
TestItemI8 _gI8TestItem[TEST_I8_NUM] =
{
    {{0,   0,   400, 400}, 0xFFFF0000},
    {{200, 160, 800, 256}, 0xFF00FFFF},
    {{160, 200, 960, 480}, 0x0000FFFF},
    {{256, 80,  720, 760}, 0x00FFFF00}
};

bool Test_Index8(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemI8 *pTest=&_gI8TestItem[0];
	printf_info("SE: Index8 Fill Test...\r\n");
	for(test=0; test<TEST_I8_NUM; test++) {
		printf_info("Test %d: EXT I8 (%d, %d, %d, %d) 0x%08x rectangle.\r\n",
			test,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		// Fill the destination background color to black.
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0x00000040);


		EXTIndex8(1,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		printf_info(szMsg, "EXT I8 Test %d: ", test);

		snprintf(szMsg,256, "EXT I8 Test %d: ", test);
		bRet = ExamineRectangle(1, &pTest[test].rc, pTest[test].u32Color, szMsg);
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT I8 Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

#define TEST_A8_NUM     4
typedef struct _TestItemA8
{
    RECT_SE rc;
    uint32_t u32Color;
    uint32_t u32alpha;
} TestItemA8;
TestItemA8 _gA8TestItem[TEST_A8_NUM] =
{
    {{0,   0,   1920, 1080}, 0xFFFF0000, 0xFFFFFFFF},
    {{200, 160, 800, 256}, 0x0000FFFF, 0x00000000},
    {{160, 200, 960, 480}, 0x1A00FFFF, 0x1A1A1A1A},
    {{256, 80,  720, 760}, 0x2BFFFF00, 0x2B2B2B2B}
};

bool Test_Alpha8(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemA8 *pTest=&_gA8TestItem[0];
	printf_info("SE: Alpha8 Fill Test...\r\n");
	for(test=0; test<TEST_A8_NUM; test++) {
		printf_info("Test %d: EXT A8 (%d, %d, %d, %d) 0x%08x rectangle.\r\n",
			test,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		// Fill the destination background color to black.
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, pTest[test].u32Color&0x11ffffff);
		EXTDrawRectangle(0, 0, 0, SE_SURFACE_W, SE_SURFACE_H, pTest[test].u32alpha);


		EXTAlpha8(1,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		printf_info(szMsg, "EXT A8 Test %d: ", test);

		snprintf(szMsg,256, "EXT A8 Test %d: ", test);
		bRet = ExamineRectangle(1, &pTest[test].rc, pTest[test].u32Color, szMsg);
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT A8 Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

#define TEST_LG_NUM     4
typedef struct _TestItemLG
{
    RECT_SE rc;
    uint32_t u32Color;
} TestItemLG;
TestItemLG _gLGTestItem[TEST_LG_NUM] =
{
    {{0,   0,   1920, 1080}, 0xFF00FF00},
    {{200, 160, 800, 256}, 0x0000FFFF },
    {{160, 200, 960, 480}, 0x1A00FFFF },
    {{256, 80,  720, 760}, 0x2BFFFF00 }
};

bool Test_LinearG(void)
{
	bool bRet=true;
	int test;
	char szMsg[256];
	TestItemLG *pTest=&_gLGTestItem[0];
	printf_info("SE: LinearGradient Fill Test...\r\n");
	for(test=0; test<TEST_LG_NUM; test++) {
		printf_info("Test %d: EXT LinearG (%d, %d, %d, %d) 0x%08x rectangle.\r\n",
			test,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		// Fill the destination background color to black.
		EXTDrawRectangle(1, 0, 0, SE_SURFACE_W, SE_SURFACE_H, 0);


		EXTLinearG(1,
			pTest[test].rc.x, pTest[test].rc.y,
			pTest[test].rc.w, pTest[test].rc.h,
			pTest[test].u32Color);

		printf_info(szMsg, "EXT LinearG Test %d: ", test);

		snprintf(szMsg,256, "EXT LinearG Test %d: ", test);
		//bRet = ExamineRectangle(1, &pTest[test].rc, pTest[test].u32Color, szMsg);
		if(bRet == false) {
			break;
		}
		printf_info("\r\nEXT LinearG Test %d: OK\r\n\r\n", test);
	}

	return bRet;
}

#if CONFIG_FT
#define TEST_ITEM_NUM       2
#else
#define TEST_ITEM_NUM       9

#endif
typedef struct _TestItem
{
    bool (*func)(void);
    char szTestName[64];
} TestItem;
TestItem _gTestItem[TEST_ITEM_NUM]=
{
#if !CONFIG_FT
    {Test_EXTDrawRectangle,	"EXT Draw rectangle test"},
    {Test_EXTCopy,		"EXT Copy test"},
#endif

    {Test_EXTBlend,		"EXT blit test"},
    {Test_EXTStretch,		"EXT blit stch test"},
#if !CONFIG_FT
    {Test_Mirror,		"Mirror test"},
    {Test_Index8,		"INDEX8 test"},
    {Test_Alpha8,		"ALPHA8 test"},
    {Test_LinearG,		"Linear Gradient test"},
    {Test_Rotate,		"Rotate test"},
#endif
};

//=====================================================================================only for merlin2
bool se_dcu1_write_check(uint32_t u32Color, uint32_t width)
{
	bool bRet=true;
	int x;
	int errcnt=0;
	uint32_t *pDst = (uint32_t*)0x16401000;
	flush_dcache_all();

	for(x = 0; x < width; x++) {
		if(pDst[x] != u32Color) {
			errcnt++;
		}
	}
	printf("[SE][DCU1 test] addr = %p, errcnt = %d\n",pDst, errcnt);
	if(errcnt >=16) {
		bRet = false;
	}
	return bRet;
}

static bool se_dcu1_write_test(void){
	//write 1*256, value=0, only for merlin2;

	bool bRet=true;
	int cnt=0;
	_g_SE_CMD_BUFFER_START_ADDR = 0x16400000;
	qc_reset_cmd_buffer();
#if 0
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_BASE_ADDR1, (uint32_t)0x16400000);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH1, SE_SURFACE_PITCH);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_BASE_ADDR1+0x4, (uint32_t)0x16400000);
	qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SEINFO_REG_PITCH1+0x4, SE_SURFACE_PITCH);
#endif

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_BASE_ADDR0, (uint32_t)0x16401000);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH0, SE_SURFACE_PITCH);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_BASE_ADDR1, (uint32_t)0x16401000);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH1, SE_SURFACE_PITCH);

	qc_se_writecmd();
	qc_se_checkfinish();

	for(cnt=0;cnt<1;cnt++ ) {
		EXTDrawRectangle(0, 0, 0, 1920, 1, 0);
		EXTDrawRectangle(0, 0, 0, 1920, 1, 0xfff0f0f0);
		//DrawRectangle(0, 0, 0, 768, 1, 0);
		//DrawRectangle(0, 0, 0, 768, 1, 0xfff0f0f0);
		bRet = se_dcu1_write_check(0xfff0f0f0, 1920);
	}

	_g_SE_CMD_BUFFER_START_ADDR = SE_CMD_BUFFER_START_ADDR;
	qc_reset_cmd_buffer();
	return bRet;


}

int QC_Verifier_SE(void)
{
	int i=0,j=0;
	bool bPass[TEST_ITEM_NUM] = {false};
	int nRet = SUCCESS;

#if defined(SE_MEASURE)
	rtd_outl(0xb801b688, 1); // timer enable for measure performance
#endif
	qc_se_module_init();
	qc_reset_cmd_buffer();

	for(i=0;i<TEST_ITEM_NUM;i++) {
		bPass[i] = _gTestItem[i].func();
	}


	printf("\r\n\r\n");
	printf("===========================================\r\n");
	printf("              SE QC Conclusion             \r\n");
	printf("===========================================\r\n");
	for(i=0;i<TEST_ITEM_NUM;i++) {
		printf(_gTestItem[i].szTestName);
		if(bPass[i] == true) {
			printf(": Passed!! \r\n");
		}else{
			nRet = FAILED;
			printf(": FAILED!! \r\n");
		}
	}
#if 0

	TestItemCopy t2[3]={
		{{0,   0,   1920, 1}, { 0,  0, 1920, 1}, 0x12345678},
		{{160, 200, 960, 480}, {880, 160, 960, 480}, 0x0000FFFF},
		{{160,   200,   100, 100}, { 700,  400, 100, 100}, 0x12345678},
		//{{0,   0,   100, 100}, { 50,  50, 100, 100}, 0x80800000}
	};

	for(i=0;i<256;i++) {
		qc_se_WriteRegister(SEINFO_REGMODE_CLUT,i,0);
	}

	t2[0].rcSrc.h=17;
	t2[0].rcDst.h=17;
	for(j=0;j<6;j++) {
		if(Test_EXTCopy_2(t2) == false) {
			return FAILED;
		}
	}

	t2[0].rcSrc.h=39;
	t2[0].rcDst.h=39;
	for(j=0;j<6;j++) {
		if(Test_EXTCopy_2(t2) == false) {
			return FAILED;
		}
	}

	t2[0].rcSrc.h=42;
	t2[0].rcDst.h=42;
	for(j=0;j<6;j++) {
		if(Test_EXTCopy_2(t2) == false) {
			return FAILED;
		}
	}

	t2[0].rcSrc.h=44;
	t2[0].rcDst.h=44;
	for(j=0;j<6;j++) {
		if(Test_EXTCopy_2(t2) == false) {
			return FAILED;
		}
	}
	if(se_dcu1_write_test() == false)
		return FAILED;
#endif
	return nRet;
}
#if 1
int se_hang_test_blend(int ext){
	int dx=0,dy=0;
	int dw=0,dh=0;
	char str[16];
	RECT_SE rcSrc = {0,0,0,0};
	RECT_SE rcDst = {0,0,0,0};

	if(ext == 1) {
		snprintf(str, "%s", "EXT");
	}else if(ext == 0) {
		snprintf(str, "%s", "OLD");
	}else{
		printf("\r\n\t- %s parameters Fail!..\n", __func__);
		return -1;
	}

	printf("\r\n\t- %s %s start..\n", str, __func__);


	for(dw=16;dw<=64;dw++) {
		for(dh=16;dh<=64;dh++){
			for(dy=0;dy<=64;dy++) {
				for(dx=0;dx<=64;dx++) {
					rcSrc.x=0;
					rcSrc.y=0;
					rcDst.x=dx;
					rcDst.y=dy;
					rcSrc.w = rcDst.w = dw;
					rcSrc.h = rcDst.h = dh;

					if(ext == 1) {
						if(EXTBlendRectangle(1, &rcDst, 0, &rcSrc)) {
							printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
								rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h,
								rcDst.x, rcDst.y, rcDst.w, rcDst.h);
							return -1;
						}
					}else {
						if(BlendRectangle(1, &rcDst, 0, &rcSrc)) {
							printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
								rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h,
								rcDst.x, rcDst.y, rcDst.w, rcDst.h);
							return -1;
						}
					}

					if(ext == 1) {
						if(EXTBlendRectangle(1, &rcSrc, 0, &rcDst)) {
							printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
								rcDst.x, rcDst.y, rcDst.w, rcDst.h,
								rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h);
							return -1;
						}
					}else {
						if(BlendRectangle(1, &rcSrc, 0, &rcDst)) {
							printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
								rcDst.x, rcDst.y, rcDst.w, rcDst.h,
								rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h);
							return -1;
						}
					}
					return 0;
				}
			}
			printf(".");

		}
	}

	return 0;
}

int se_hang_test_stretch(int ext){
	int dx=0,dy=0;
	int dw=16,dh=16;
	int fold_w = 1, fold_h=1;
	char str[16];
	RECT_SE rcSrc = {0,0,0,0};
	RECT_SE rcDst = {0,0,0,0};

	if(ext == 1) {
		snprintf(str, "%s", "EXT");
	}else if(ext == 0) {
		snprintf(str, "%s", "OLD");
	}else{
		printf("\r\n\t- %s parameters Fail!..\n", __func__);
		return -1;
	}

	printf("\r\n\t- %s %s start..\n", str, __func__);


	for(dw=16;dw<=64;dw++) {
		for(dh=16;dh<=64;dh++){
			for(dy=0;dy<=64;dy++) {
				for(dx=0;dx<=64;dx++) {
					rcSrc.x=0;
					rcSrc.y=0;
					rcDst.x=dx;
					rcDst.y=dy;
					rcSrc.w = rcDst.w = dw;
					rcSrc.h = rcDst.h = dh;
					for(fold_w =2;fold_w <=8;fold_w++) {
						for(fold_h =2;fold_h <=8;fold_h++) {
							rcDst.w = rcSrc.w*fold_w;
							rcDst.h = rcSrc.h*fold_h;

							if(ext == 1) {
								if(EXTStretchRectangle(1, &rcDst, 0, &rcSrc)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h,
										rcDst.x, rcDst.y, rcDst.w, rcDst.h);
									return -1;
								}
							}else {
								if(StretchRectangle(1, &rcDst, 0, &rcSrc)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h,
										rcDst.x, rcDst.y, rcDst.w, rcDst.h);
									return -1;
								}
							}

							if(ext == 1) {
								if(EXTStretchRectangle(1, &rcSrc, 0, &rcDst)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcDst.x, rcDst.y, rcDst.w, rcDst.h,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h);
									return -1;
								}
							}else {
								if(StretchRectangle(1, &rcSrc, 0, &rcDst)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcDst.x, rcDst.y, rcDst.w, rcDst.h,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h);
									return -1;
								}
							}
						}
					}
					printf(".");
				}
			}

		}
	}

	return 0;
}

int se_hang_test_stretch_bruteforce(int ext){
	int dx=0,dy=0;
	int dw=16,dh=16;
	int fold_w = 1, fold_h=1;
	int stepw = 1, steph=1;
	char str[16];
	RECT_SE rcSrc = {0,0,0,0};
	RECT_SE rcDst = {0,0,0,0};
	RECT_SE rcstepDst = {0,0,0,0};

	if(ext == 1) {
		snprintf(str, "%s", "EXT");
	}else if(ext == 0) {
		snprintf(str, "%s", "OLD");
	}else{
		printf("\r\n\t- %s parameters Fail!..\n", __func__);
		return -1;
	}

	printf("\r\n\t- %s %s start..\n", str, __func__);


	for(dw=17;dw<=256;dw++) {
		for(dh=17;dh<=256;dh++){
			for(dy=0;dy<=256;dy++) {
				for(dx=0;dx<=256;dx++) {
					rcSrc.x=0;
					rcSrc.y=0;
					rcDst.x=dx;
					rcDst.y=dy;
					rcSrc.w = rcDst.w = dw;
					rcSrc.h = rcDst.h = dh;
					for(fold_w =1;fold_w <=8;fold_w++) {
						for(fold_h =1;fold_h <=8;fold_h++) {
							rcDst.w = rcSrc.w*fold_w;
							rcDst.h = rcSrc.h*fold_h;

							for(stepw=rcSrc.w;stepw <=rcDst.w;stepw++) {
							for(steph=rcSrc.h;steph <=rcDst.h;steph++) {
							rcstepDst.w= stepw;
							rcstepDst.h= steph;

							if(ext == 1) {
								if(EXTStretchRectangle(1, &rcstepDst, 0, &rcSrc)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h,
										rcstepDst.x, rcstepDst.y, rcstepDst.w, rcstepDst.h);
									return -1;
								}
							}else {
								if(StretchRectangle(1, &rcDst, 0, &rcSrc)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h,
										rcstepDst.x, rcstepDst.y, rcstepDst.w, rcstepDst.h);
									return -1;
								}
							}

							if(ext == 1) {
								if(EXTStretchRectangle(1, &rcSrc, 0, &rcstepDst)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcstepDst.x, rcstepDst.y, rcstepDst.w, rcstepDst.h,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h);
									return -1;
								}
							}else {
								if(StretchRectangle(1, &rcSrc, 0, &rcstepDst)) {
									printf("\r\t  * %s src[%d,%d,%d,%d] to dst[%d,%d,%d,%d]\n", str,
										rcstepDst.x, rcstepDst.y, rcstepDst.w, rcstepDst.h,
										rcSrc.x, rcSrc.y, rcSrc.w, rcSrc.h);
									return -1;
								}
							}

							}
							}
						}
					}
					printf(".");
				}
			}

		}
	}

	return 0;
}
#
int se_hang_test(void) {
	printf("\r\n[%s] start..\n", __func__);
	qc_se_module_init();
	qc_reset_cmd_buffer();
#if 1
	if(se_hang_test_blend(0)) {
		printf("\r\n[%s] se_hang_test_blend OLD test fail!..\n", __func__);
		return -1;
	}else{
		printf("\r\n[%s] se_hang_test_blend OLD test PASS!..\n", __func__);
	}


	if(se_hang_test_blend(1)) {
		printf("\r\n[%s] se_hang_test_blend EXT test fail!..\n", __func__);
		return -1;
	}else{
		printf("\r\n[%s] se_hang_test_blend EXT test PASS!..\n", __func__);
	}
#endif
#if 1
	if(se_hang_test_stretch(0)) {
		printf("\r\n[%s] se_hang_test_stretch OLD test fail!..\n", __func__);
		return -1;
	}else{
		printf("\r\n[%s] se_hang_test_stretch OLD test PASS!..\n", __func__);
	}


	if(se_hang_test_stretch(1)) {
		printf("\r\n[%s] se_hang_test_stretch EXT test fail!..\n", __func__);
		return -1;
	}else{
		printf("\r\n[%s] se_hang_test_stretch EXT test PASS!..\n", __func__);
	}
#endif
#if 1
	if(se_hang_test_stretch_bruteforce(0)) {
		printf("\r\n[%s] se_hang_test_bruteforce OLD test fail!..\n", __func__);
		return -1;
	}else{
		printf("\r\n[%s] se_hang_test_bruteforce OLD test PASS!..\n", __func__);
	}


	if(se_hang_test_stretch_bruteforce(1)) {
		printf("\r\n[%s] se_hang_test_bruteforce EXT test fail!..\n", __func__);
		return -1;
	}else{
		printf("\r\n[%s] se_hang_test_bruteforce EXT test PASS!..\n", __func__);
	}
#endif
	return 0;

}

int se_busy_test(void) {
	printf("\r\n[%s] start..\n", __func__);
	qc_se_module_init();
	qc_reset_cmd_buffer();

	rtd_outl(SE_CMDLMT_0_reg, SE_CMD_BUFFER_START_ADDR+SE_CMD_SIZE*1024);

        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH0, 8000*4);
        qc_se_WriteRegister(SEINFO_REGMODE_REGISTER, SE20INFO_REG_PITCH1, 8000*4);

	RECT_SE rcSrc = {0,0,8000,8000};
	RECT_SE rcDst = {0,0,8000,8000};
	EXTBlendRectangle_busy(1, &rcSrc, 0, &rcDst);


	return 0;

}

int do_sehang(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret;
	//se_hang_test();
	//se_hang_test_stretch_bruteforce(1);
	ret = se_busy_test();
	return ret;
}

U_BOOT_CMD(
	callsehangtest,	4,	1,	do_sehang,
	"se hang testcase",
	"se hang testcase"
);

#endif
