#include "vp9_common.h"
#include "vp9_plat.h"
#include "auto_vp9_cfg.h"

extern unsigned int _e_v_entry_qc_img;
extern unsigned int _f_v_entry_qc_img;

#ifdef ENABLE_VP9V1_TRUE
#include VP9V1_PATTERN_HEADER
#endif

#ifdef ENABLE_VP9V2_TRUE
#include VP9V2_PATTERN_HEADER
#endif

#define VP9_TIMEOUT 0x00010000

void print_time(void)
{
	unsigned int time = 0;
	time = rtd_inl(0xb801b63c);
	prints("\ntime now:0x");
	print_val(time,8);
	prints(" ticks ,0x");
	time /= 27000;
	print_val(time,8);
	prints(" ms\n");
}

int prepare_vp9_pattern()
{
	int i;
	int len;
#ifdef ENABLE_VP9V1_TRUE
	struct hex_data * pPattern1 = QC_VP9Pattern_V1;
	len = sizeof(QC_VP9Pattern_V1)/sizeof(struct hex_data);
	prints("len:0x");
	print_val(len,8);
	prints("\n");
	for (i = 0;i < len; i++)
	{
		rtd_outl(pPattern1[i].addr, pPattern1[i].data);
	}

#endif

	return 0;
}

int prepare_vp9_vari()
{
#ifdef ENABLE_VP9V1_TRUE
	rtd_outl(VP9V1_STATUS_VAL_ADDR,0x0); //test result
	rtd_outl(VP9V1_TEST_RESULT_ADDR, SWAPEND32(0xbabebabe));
	rtd_outl(VP9V1_TEST_MODE_ADDR, 0x01000000);//vcpu-only test
	rtd_outl(VP9V1_FINISH_RELOAD_VAL_ADDR, SWAPEND32(MIPS_KSEG1BASE|VFW_ENTRY_ADDR)); //when finish vcpu will jump to this entry again
	rtd_outl(VP9V1_RUN_CMD_ADDR, VP9_STATUS_BUSY); //start run
#endif

	return 0;
}

int prepare_vp9_entry()
{
	unsigned char *a,*b;

	a = (unsigned char *)& _f_v_entry_qc_img;
	b = (unsigned char *)& _e_v_entry_qc_img;
	// copy v_entry_qc
	//debug("copy v_entry_qc  from addr = 0x%x, to 0x%x, size = 0x%x\n", a, VFW_ENTRY_ADDR, b-a);
	memcpy((unsigned char *)VFW_ENTRY_ADDR, a, b-a);
#ifdef ENABLE_VP9V1_TRUE
	rtd_outl(VCPU_JUMP_ADDR_reg, MIPS_KSEG1BASE|VFW_ENTRY_ADDR);
	rtd_outl(VFW_ENTRY_PTR_V1_ADDR,SWAPEND32(MIPS_KSEG1BASE|VFW_ENTRY_VP9V1_PARTTEN));
	//rtd_outl(VFW_ENTRY_PTR_V1_ADDR,SWAPEND32(MIPS_KSEG1BASE|VFW_ENTRY_ADDR));
#endif

	return 0;
}

int vp9_ddr_debug (void)
{
	volatile unsigned int check_status_vp9v1 = 0;
	volatile unsigned int check_end_vp9v1 = 0; /* VE1 end flag */
	volatile unsigned int check_end_vp9v2 = 0; /* VE2 end flag */
	unsigned int timeout = VP9_TIMEOUT;
	unsigned int error = 0;

#ifndef ENABLE_VP9V1_TRUE
	check_end_vp9v1 = 1;
#endif

#ifndef ENABLE_VP9V2_TRUE
	check_end_vp9v2 = 1;
#endif

	prints("loop waiting\n");
	while(--timeout)
	{
#ifdef ENABLE_VP9V1_TRUE
		invalidate_dcache_range(VP9V1_STATUS_VAL_ADDR, VP9V1_STATUS_VAL_ADDR+4);
		check_status_vp9v1 = rtd_inl(VP9V1_STATUS_VAL_ADDR); /* VE1 run status addr */
		if ((check_status_vp9v1 == VP9_STATUS_PASS))
		{
			//debug("ve1_done?       %x\n", check_status_ve1);
			prints("\nvp9_done\n");
			check_end_vp9v1 = 1;
		}
		else if ((check_status_vp9v1 == VP9_STATUS_FAIL))
		{
			//debug("ve1_crc_err?       %x    %x\n", check_status_ve1);
			prints("\nvp9_fail\n");
			check_end_vp9v1 = 1;
			error = 1;
		}
		else
		{
			if(timeout % 1000 == 0)
			{
				prints(".");
			}
			cnt_udelay(200);
		}
#endif /* CONFIG_QC_VE1 */

		if((check_end_vp9v1==1) && (check_end_vp9v2==1))
		{
			break;
		}

	} /* while(--timeout) */
	prints("\n");


	if (timeout)
	{
		prints("no-timeout\n");
		//show_debug_info(check_status_ve1,check_status_ve2,"test over!");
		return 0;
	}
	else if (error)
	{
		prints("error\n");
		//show_debug_info(check_status_ve1,check_status_ve2,"crc error!");
		return -1;
	}
	else
	{
		prints("unknown result\n");
		//show_debug_info(check_status_ve1,check_status_ve2,"timeout!");
		return -2;
	}
}

int vp9_uniform_entry(void)
{
	int debug_ret = 0;
	int ret = 0;

	prepare_vp9_pattern();
	prints("\n [prepare_vp9_pattern] finish \n");

	prepare_vp9_vari();
	prints("\n [prepare_vp9_vari] finish \n");

	prepare_vp9_entry();
	prints("\n [prepare_vp9_entry] finish \n");

	flush_dcache_all();
	prints("\n [flush_dcache_all] finish \n");

	enable_vcpus();
	prints("\n [enable_vcpus] finish \n");

	print_time();
	debug_ret = vp9_ddr_debug();
	prints("\n [vp9_ddr_debug] finish \n");
	print_time();

	disable_vcpus();
	prints("\n [disable_vcpus] finish \n");
	ret = (debug_ret == 0)?SUCCESS:FAILED;

	return ret;
}