#include <common.h>
#include <command.h>
#include <linux/ctype.h>
#include <linux/string.h>

#include "rtd_verify.h"

int gets_s(char *s, unsigned int size)
{
    int cnt = 0;
    char    c;

    while((c = getc()) != 0x0D){
        if(c != 0x08){
            cnt++;
            *s++ = c;
            printf("%c",c );
        }else{
            if(cnt > 0){
                cnt--;
                *s-- = ' ';
                printf("\b \b");
            }
        }
        if(cnt >= size-1)
            break;
    }
    *s = 0;

    return(cnt);
}


/*======================================================
 * Func : do_help
 *
 * Desc : do help function
 *
 * Para : NA
 *
 * Retn : 0 : success, others : failed
 ======================================================*/
static int do_help_pv(cmd_tbl_t *cmdtp, int flag,
               int argc, char * const argv[])
{
    int i;

    printf("\nAll supported function below:\n");

    //--------------- print modules ---------------
    printf("==========================================\n");
    for (i=0;  i<VERIFY_FUNC_COUNT; i++)
    {
        printf("       %s : %s \n",
            g_verify_func_list[i].name,
            g_verify_func_list[i].description);
    }
    printf("==========================================\n");

    return 0;
}

/*======================================================
 * Func : do_cmd_loop
 *
 * Desc :
 *
 * Para :
 *
 *
 * Rmmc : 0 : success, others : failed
 ======================================================*/
int do_cmd_loop(int t_mode, int v_cnt,SUB_FUNC* func_list)
{
    int i;
    int c;
    int in_val = 0;
    char buf[128];

    printf("mode=%s; total item=%d \n",(t_mode)?"auto":"manual",v_cnt);

REENTRY:
    printf( "\n######################################################\n");

    for (i=0; i<v_cnt; i++){
        if(t_mode){
            if( i < (v_cnt-1)){
                printf( "%d) %s verify item start---\n",
                    i+1,func_list[i].name);
                func_list[i].verify_func(0,NULL);
            }
        }else{
            if(i==0)
                printf( "\n       q) exit.\n");
            printf("       %d) %s : %s \n",
                i+1,
                func_list[i].name,
                func_list[i].description);
        }
    }
    if(t_mode)
        return 0;

    printf( "\n======================================================");
    printf( "\nplease input test item :");
    gets_s(buf,sizeof(buf));

    c = buf[0];
    if (isalpha(c)){
        if(( c == 'q') | ( c == 'Q')){
            printf( "\n");
            return 0;
        }
        goto REENTRY;
    }

    in_val = strtoul(buf, NULL, 10);

    if((!in_val) || (in_val > v_cnt)){
        printf( "\nunsupported cmd item(%d)",in_val);
        goto REENTRY;
    }

    printf( "\nYou select function:%d) %s",
        in_val,func_list[in_val-1].name);
    printf( "\n======================================================\n");

    func_list[in_val-1].verify_func(0,NULL);

    goto REENTRY;

    return 0;
}


/*======================================================
 * Func : main
 *
 * Desc : main function
 *
 * Para : NA
 *
 * Retn : 0 : success, others : failed
 ======================================================*/
static int do_menu_pv(cmd_tbl_t *cmdtp, int flag,
               int argc, char * const argv[])
{
    int i;
    int c;
    char in_item;
    char in_mode;
    char buf[128];


    printf("\n\n   Realtek Plateform Verify (Ver 0.1)\n\n");
REENTRY:
    printf( "\n######################################################\n");
    printf("       q) exit.\n");
    for (i=0; i<VERIFY_FUNC_COUNT; i++)
    {
        printf("       %d) %s : %s \n",
            i+1,
            g_verify_func_list[i].name,
            g_verify_func_list[i].description);
    }

    printf( "\n======================================================");
    printf( "\nplease select verify module:");

    gets_s(buf,sizeof(buf));

    c = buf[0];
    if (isalpha(c)){
        if(( c == 'q') | ( c == 'Q')){
            printf( "\n");
            return 0;
        }
        goto REENTRY;
    }

    in_item = strtoul(buf, NULL, 10);

    if( !in_item || (in_item>VERIFY_FUNC_COUNT)){
        goto REENTRY;
    }

    printf( "\nYou select module:%d) %s",
        in_item,g_verify_func_list[in_item-1].name);

CHK_MODE:
    printf( "\n0: manual / 1: Auto / q: exit \n");

    gets_s(buf,sizeof(buf));
    c = buf[0];
    if (isalpha(c)){
        if(( c == 'q') | ( c == 'Q')){
            printf( "\n");
            return 0;
        }
        goto CHK_MODE;
    }

    in_mode = strtoul(buf, NULL, 10);
    if(in_mode > 1)
        goto CHK_MODE;

    printf( "\n======================================================\n");

    g_verify_func_list[in_item-1].verify_func(in_mode,NULL);

    goto REENTRY;

}

static cmd_tbl_t cmd_verify[] = {
    U_BOOT_CMD_MKENT(menu, CONFIG_SYS_MAXARGS, 1, do_menu_pv, "", ""),
    U_BOOT_CMD_MKENT(phelp, CONFIG_SYS_MAXARGS, 1, do_help_pv, "", "" ),
};


static int do_verifyops(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    cmd_tbl_t *cp;

    cp = find_cmd_tbl(argv[1], cmd_verify, ARRAY_SIZE(cmd_verify));

    argc--;
    argv++;

    if (cp == NULL || argc > cp->maxargs){
        return CMD_RET_USAGE;
    }
    if (flag == CMD_FLAG_REPEAT && !cp->repeatable){
        return CMD_RET_SUCCESS;
    }

    return cp->cmd(cmdtp, flag, argc, argv);
}

U_BOOT_CMD(
    pv, CONFIG_SYS_MAXARGS, 1, do_verifyops,
    "plateform verify",
    "\n"
    "pv menu - into menu of plateform verify.\n"
    "pv help - usage information.\n"
);


/* end of file */