#include <config.h>
#include <common.h>
#include <command.h>
#include <linux/ctype.h>
#include <linux/string.h>
#include <usb.h>
#include "usb_pv.h"

extern int gets_s(char *s, unsigned int size);
extern int do_cmd_loop(int t_mode, int v_cnt,SUB_FUNC* func_list);

/*======================================================
 *	Local Define
 *======================================================*/

/************************************************************************
 * Func : usb_pv_IP_start
 *
 * Desc : function of USB HC initialize flow test.
 *
 * Para : argc : number of arguments (unused)
 *        argv : input Paraeters (unused)
 *
 * Ret  : 0 : success, others : failed
 ************************************************************************/
int usb_pv_IP_start(int argc, char *argv[])
{
    int ret = 0;

    /* Stop Host controller first */
    usb_stop();
    ret = usb_init();

    USB_PV_SHOW_RESULT(ret);

    return ret;
}


/************************************************************************
 * Func : usb_pv_storage_RW_test
 *
 * Desc : function of USB external storage device R/W test.
 *
 * Para : argc : number of arguments (unused)
 *        argv : input Paraeters (unused)
 *
 * Ret  : 0 : success, others : failed
 ************************************************************************/

#if defined(Config_Module_Test_Boot_TRUE)
extern int usb_storage_rw_test(int dev);
#endif

int usb_pv_storgae_RW_test(int argc, char *argv[])
{
#ifndef CONFIG_USB_STORAGE
    printf(KRED "############ %sNot Support USB Storage%s ############ \n" KNRM);
    return 0;
#else
    int ret = 0, usb_stor_curr_dev = 0;
    char buf[128];

    printf(KCYN "Please plug a USB storage device to one of connectors, enter y to continue [y/N] \n" KNRM);
    gets_s(buf, sizeof(buf));
    if (strncmp(buf, "y", 1) && strncmp(buf, "Y", 1))
        goto NOT_TEST;

    /* Stop Host controller first */
    usb_stop();

    ret = usb_init();
    if (ret)
        goto EXIT;

    /* try to recognize storage devices immediately. usb_stor_scan always return 0 if success */
    usb_stor_curr_dev = usb_stor_scan(1);
    if (usb_stor_curr_dev < 0) {
        ret = -1;
        goto EXIT;
    }

#if defined(Config_Module_Test_Boot_TRUE)
    ret = usb_storage_rw_test(usb_stor_curr_dev); // index0 storage device
#else
    printf(KCYN "Please enable \"Config_Module_Test_Boot_TRUE\" first!\n" KNRM);
#endif

EXIT:
    USB_PV_SHOW_RESULT(ret);

NOT_TEST:
    return ret;
#endif /* !CONFIG_USB_STORAGE */
}



/************************************************************************
 * Func : usb_pv_usb2serial_test
 *
 * Desc : function to instruct user how to test USB2Serial.
 *
 * Para : argc : number of arguments (unused)
 *        argv : input Paraeters (unused)
 *
 * Ret  : 0 : success, others : failed (unused)
 ************************************************************************/
int usb_pv_usb2serial_test(int argc, char *argv[])
{
    int ret = 0;
#if defined(CONFIG_USB_PL2303) && defined(CONFIG_USB_PL2303_CONSOLE)
    char buf[128];
    extern int usb_check_pl2303(void);


    printf(KCYN "Please plug a USB2Serial(PL2303) device to one of connectors, enter y to continue [y/N]. \n" KNRM);
    gets_s(buf, sizeof(buf));
    if (strncmp(buf, "y", 1) && strncmp(buf, "Y", 1))
        goto NOT_TEST;

    /* Stop Host controller first */
    usb_stop();

    ret = usb_init();
    if (ret)
        goto EXIT;

    ret = usb_check_pl2303();
    if (ret)
        goto EXIT;

EXIT:
    USB_PV_SHOW_RESULT(ret);

NOT_TEST:

#else
    printf(KRED "Please enable CONFIG_USB_PL2303 and CONFIG_USB_PL2303_CONSOLE in bootcode configs. and re-burn new bootcode again for this test.. \n" KNRM);
#endif

    return ret;
}

/************************************************************************
 * Always keep help func at last. It's going to be skip at auto testing mode
 ************************************************************************/
int usb_pv_help(int argc, char *argv[])
{
    printf( "%s(%d)display more information what you want user to know\n",__func__,__LINE__);

	return 0;
}


/*======================================================
 * Func : module_name plateform verify
 *
 * Desc : main Function of verify func
 *
 * Para : argc : number of arguments
 *        argv : input Paraeters
 *
 * Ret  : 0 : success, others : failed
 ======================================================*/
int usb_verify(int argc, char *argv[])
{
    return do_cmd_loop(argc, USB_PV_FUNC_COUNT,usb_verify_func_list);
}

