#include <linux/string.h>
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <linux/compiler.h>
#include <asm/io.h>
#include "alc1312.h"
#include <drivers/i2c/rtk_i2c_lib.h>

//*************************************************************
// PUBLIC FUNCTIONS
//*************************************************************


static ALC1312_REG ALC1312_InitTbl[] =
{
	/////////////////initial
	{ ALC_1312_MX0000_CLOCK0            , {0xFF,0xFF}, 0},

	// enable RC clock
	{ ALC_1312_MX0004_CLOCK2            , {0xC0,0x00}, 0},
	{ ALC_1312_MX0006_CLOCK3            , {0x35,0x70}, 0},
	{ ALC_1312_MX0008_CLOCK4            , {0x30,0x00}, 0},
	{ ALC_1312_MX000A_PLL1              , {0x10,0x5E}, 0},
	{ ALC_1312_MX000C_PLL2              , {0x00,0x00}, 0},
	{ ALC_1312_MX0322_POWER1            , {0x40,0x0F}, 0},
	{ ALC_1312_MX0326_POWER3            , {0x00,0x0F}, 0},
	{ ALC_1312_MX0350_POWER_STATUS      , {0x01,0x88}, 0},
	{ ALC_1312_MX0324_POWER2            , {0xC0,0xA0}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x05,0x10}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x25,0x00}, 0},
	{ ALC_1312_MX0103_MASTER_VOLUME_L   , {0x03,0x3E}, 0},//Master Volume L = 0dB
	{ ALC_1312_MX0104_MASTER_VOLUME_R   , {0x03,0x3E}, 0},//Master Volume L = 0dB
	{ ALC_1312_MX0300_CLASS_D           , {0xB8,0x03}, 0},//400K HV32dB
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x00,0x02}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x65,0x05}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x03,0x00}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x01,0x45}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x03,0x02}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x28,0x02}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x03,0x0C}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x0F,0xC0}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x01,0x0C}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0xC0,0x00}, 0},

	// Turn on Efuse & Auto Readback mode
	{ ALC_1312_MX0326_POWER3            , {0x00,0xAF}, 0},
	{ ALC_1312_MX0326_POWER3            , {0x00,0xFF}, 0},
	{ {0x00 ,0x80}                    , {0x80,0x00}, 0},
	{ {0x00 ,0x80}                    , {0x80,0x01}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x02,0x20}, 0},
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x00,0x00}, 0},


	// Turn off Efuse
	{ ALC_1312_MX0326_POWER3            , {0x00,0xAF}, 0},
	{ ALC_1312_MX0326_POWER3            , {0x00,0x0F}, 500}, // need to Delay 0.5sec


	// enable PLL
	{ ALC_1312_MX0004_CLOCK2            , {0x10,0x00}, 0},

	// Turn on PWM
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x02,0x16}, 0},// DC detect H_threshold
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x04,0x70}, 0},
	{ ALC_1312_MX006A_PRIVATE_REG_INDEX , {0x02,0x17}, 0},// DC detect L_threshold
	{ ALC_1312_MX006C_PRIVATE_REG_DATA  , {0x04,0x00}, 0},
	{ {0x02 ,0x14}                    , {0x80,0x00}, 0}, // DC detect enable
	{ ALC_1312_MX0350_POWER_STATUS      , {0x01,0x08}, 0},
	{ {0xFF ,0xFF}                    , {0xFF,0xFF}, 0},
};

#define ALC1312_REG_INIT_SIZE (sizeof(ALC1312_InitTbl)/sizeof(ALC1312_REG))

void alc1312_amp_mute(int enable)
{
    IO_Set(PIN_AMP_RESET, enable);
}

void alc1312_func(int bus_id, unsigned short addr)
{
    unsigned char data[8];
    int i = 0;

    for(i = 0; i < ALC1312_REG_INIT_SIZE ; i++ )
    {

        data[0]    = ALC1312_InitTbl[i].bAddr[0];
        data[1]    = ALC1312_InitTbl[i].bAddr[1];

        if( ( data[0] == END_FLAG ) && ( data[0] == END_FLAG) )
        {
            break;
        }
        data[2] = ALC1312_InitTbl[i].bArray[0];
        data[3] = ALC1312_InitTbl[i].bArray[1];

#if ALC1312_DEBUG
        printf("\n%s i = %d , write addr[0]:0x%02X----addr[1]=0x%02X----data[0]=0x%02X----data[1]=0x%02X\n", __func__ , i ,  data[0],data[1],data[2],data[3]);

#endif
        if (I2C_Write_EX(bus_id, addr,  4, data ) < 0)
            printf("program ACL1312 failed\n");


        #if ALC1312_SUPPORT_BOOT_MUSIC
            mdelay(ALC1312_InitTbl[i].delayTime);
        #endif


#if ALC1312_DEBUG
        unsigned char readData[2] = {0,0};
        if (I2C_Read_EX(bus_id, addr, 2,  data, 2, readData)!= S_OK)
        {
            printf("\n i2c read fail \n"  );
        }
        printf("%s i = %d , read  addr[0]:0x%02X----addr[1]=0x%02X----data[0]=0x%02X----data[1]=0x%02X\n", __func__ , i ,data[0],data[1],readData[0],readData[1]);
#endif
    }
    //After 10msec, Relase PD Pin
    mdelay(10);
    //alc1312_amp_mute(0);
}

