#include <common.h>
#include <drivers/gpio/rtk_gpio.h>
#include <platform_lib/board/gpio.h>
#include <asm/arch/io.h>

RT_GPIO_REGINFO MiscGpioRegInfo[] =
{
    {MIS_GP0DIR, MIS_GP0DATI, MIS_GP0DATO},
    {MIS_GP1DIR, MIS_GP1DATI, MIS_GP1DATO},
    {MIS_GP2DIR, MIS_GP2DATI, MIS_GP2DATO},
    {MIS_GP3DIR, MIS_GP3DATI, MIS_GP3DATO},
    {MIS_GP4DIR, MIS_GP4DATI, MIS_GP4DATO},
    {MIS_GP5DIR, MIS_GP5DATI, MIS_GP5DATO},
};
RT_GPIO_REGINFO IsoGpioRegInfo[] =
{
    {ISO_GPDIR, ISO_GPDATI, ISO_GPDATO},
    {ISO_GPDIR_1, ISO_GPDATI_1, ISO_GPDATO_1},
};

#define MISC_GPIO_REG_GROUP_NUM  sizeof(MiscGpioRegInfo)/sizeof(RT_GPIO_REGINFO)
#define ISO_GPIO_REG_GROUP_NUM   sizeof(IsoGpioRegInfo)/sizeof(RT_GPIO_REGINFO)

static inline void iowrite_reg_bit(volatile unsigned int regAddr, int bit, int value)
{
	value  = (value)? 1 : 0;

    if ((rtd_inl(regAddr) & (0x1 << bit)) != (value << bit))
    {
        rtd_outl( regAddr , value | (0x1 << bit));
    }
}


static int ioread_reg_bit(volatile unsigned int regAddr, int bit)
{
    int regValue;
    regValue = rtd_inl(regAddr);
    if(regValue & (0x01 << bit)){
        return 1;
    }
    else{
        return 0;
    }
}

int rt_gpio_get(int gpioNum, RT_GPIO_TYPE rtGpioType)
{
    int bitOffset;
    int regInfoIndex;
    int retValue = -1;

    regInfoIndex = gpioNum/31;
    bitOffset    = (gpioNum%31)+1;

    gpioDebug("get.pin:%d,I:%d,bit:%d.Type:%d\n", gpioNum,regInfoIndex,bitOffset,rtGpioType);

    switch(rtGpioType)
    {
        case MISC_GPIO:
            if(regInfoIndex >= MISC_GPIO_REG_GROUP_NUM || gpioNum >= MISC_GPIO_MAX_NUM){
                gpioErr("%s misc num:%d invalid,max misc num=%d\n",__func__, gpioNum, MISC_GPIO_MAX_NUM);
                return -1;
            }
            iowrite_reg_bit(MiscGpioRegInfo[regInfoIndex].gpioDir, bitOffset, GPIO_DIR_IN);
            retValue = ioread_reg_bit(MiscGpioRegInfo[regInfoIndex].gpioDatI, bitOffset);
            break;
        case ISO_GPIO:
            if(regInfoIndex >= ISO_GPIO_REG_GROUP_NUM || gpioNum >= ISO_GPIO_MAX_NUM){
                gpioErr("%s iso num:%d invalid,max iso num=%d\n",__func__, gpioNum, ISO_GPIO_MAX_NUM);
                return -1;
            }
            iowrite_reg_bit(IsoGpioRegInfo[regInfoIndex].gpioDir, bitOffset, GPIO_DIR_IN);
            retValue = ioread_reg_bit(IsoGpioRegInfo[regInfoIndex].gpioDatI, bitOffset);
            break;
        default:
            break;
    }
    return retValue;
}

int rt_gpio_set(int gpioNum, int value, RT_GPIO_TYPE rtGpioType)
{
    int bitOffset;
    int regInfoIndex;

    regInfoIndex = (gpioNum/31);
    bitOffset    = (gpioNum%31)+1;

    gpioDebug("set.pin:%d,I:%d,bit:%d,value:%d.Type:%d\n", gpioNum,regInfoIndex,bitOffset,value,rtGpioType);

    switch(rtGpioType)
    {
        case MISC_GPIO:
            if(regInfoIndex >= MISC_GPIO_REG_GROUP_NUM || gpioNum >= MISC_GPIO_MAX_NUM){
                gpioErr("%s misc num:%d invalid,max misc num=%d\n",__func__, gpioNum, MISC_GPIO_MAX_NUM);
                return -1;
            }
            iowrite_reg_bit(MiscGpioRegInfo[regInfoIndex].gpioDatO, bitOffset, value);
            iowrite_reg_bit(MiscGpioRegInfo[regInfoIndex].gpioDir, bitOffset, GPIO_DIR_OUT);
            break;
        case ISO_GPIO:
            if(regInfoIndex >= ISO_GPIO_REG_GROUP_NUM || gpioNum >= ISO_GPIO_MAX_NUM){
                gpioErr("%s iso num:%d invalid,max iso num=%d\n",__func__, gpioNum, ISO_GPIO_MAX_NUM);
                return -1;
            }
            iowrite_reg_bit(IsoGpioRegInfo[regInfoIndex].gpioDatO, bitOffset, value);
            iowrite_reg_bit(IsoGpioRegInfo[regInfoIndex].gpioDir, bitOffset, GPIO_DIR_OUT);
            break;
        default:
            break;
    }
    return 0;
}


int rt_gpo_set(int gpoNum, int value, RT_GPIO_TYPE rtGpioType)
{
#if defined ( MISC_GPO_SUPPORT) || defined(ISO_GPO_SUPPORT )
    int bitOffset = (gpoNum%31)+1;
#endif

    switch(rtGpioType)
    {
    #ifdef MISC_GPO_SUPPORT
        case MISC_GPO:
            if(gpoNum >= MISC_GPO_MAX_NUM) {
                gpioErr("%s misc num:%d invalid,max misc num=%d\n",__func__, gpoNum, MISC_GPO_MAX_NUM);
                return -1;
            }
            iowrite_reg_bit(MIS_GPO_DATO, bitOffset, value);
            break;
    #endif
    #ifdef ISO_GPO_SUPPORT
        case ISO_GPO:
            if(gpoNum >= ISO_GPO_MAX_NUM) {
                gpioErr("%s iso num:%d invalid,max iso num=%d\n",__func__, gpoNum, ISO_GPO_MAX_NUM);
                return -1;
            }
            iowrite_reg_bit(ISO_GPO_DATO, bitOffset, value);
            break;
    #endif
        default:
            break;
    }
    return 0;
}

int rt_gpo_get(int gpoNum, RT_GPIO_TYPE rtGpioType)
{
    int retValue = -1;

#if defined ( MISC_GPO_SUPPORT) || defined(ISO_GPO_SUPPORT )
    int bitOffset = (gpoNum%31)+1;
#endif

    switch(rtGpioType)
    {
    #ifdef MISC_GPO_SUPPORT
        case MISC_GPO:
            if(gpoNum >= MISC_GPO_MAX_NUM) {
                gpioErr("%s misc num:%d invalid,max misc num=%d\n",__func__, gpoNum, MISC_GPO_MAX_NUM);
                return -1;
            }
            retValue = ioread_reg_bit(MIS_GPO_DATO, bitOffset);
            break;
    #endif
    #ifdef ISO_GPO_SUPPORT
        case ISO_GPO:
            if(gpoNum >= ISO_GPO_MAX_NUM) {
                gpioErr("%s iso num:%d invalid,max iso num=%d\n",__func__, gpoNum, ISO_GPO_MAX_NUM);
                return -1;
            }
            retValue = ioread_reg_bit(ISO_GPO_DATO, bitOffset);
            break;
    #endif
        default:
            break;
    }
    return retValue;
}

int rt_gpi_get(int gpiNum, RT_GPIO_TYPE rtGpioType)
{
    int retValue = -1;

#if defined ( MISC_GPI_SUPPORT) || defined(ISO_GPI_SUPPORT )
    int bitOffset = (gpiNum%31)+1;
#endif

    switch(rtGpioType)
    {
    #ifdef MISC_GPI_SUPPORT
        case MISC_GPI:
            if(gpiNum >= MISC_GPI_MAX_NUM) {
                gpioErr("%s iso num:%d invalid,max iso num=%d\n",__func__, gpiNum, MISC_GPI_MAX_NUM);
                return -1;
            }
            retValue = ioread_reg_bit(MIS_GPI_DATI, bitOffset);
            break;
    #endif
    #ifdef ISO_GPI_SUPPORT
        case ISO_GPI:
            if(gpiNum >= ISO_GPI_MAX_NUM) {
                gpioErr("%s iso num:%d invalid,max iso num=%d\n",__func__, gpiNum, ISO_GPI_MAX_NUM);
                return -1;
            }
            retValue = ioread_reg_bit(ISO_GPI_DATI, bitOffset);
            break;
    #endif
        default:
            break;
    }
    return retValue;
}

RTK_GPIO_ID rtk_gpio_get_gid_by_pintype(PCB_PIN_TYPE_T pin_type, unsigned char pin_index)
{
    switch (pin_type) {
        case PCB_PIN_TYPE_GPIO:
            return rtk_gpio_id(MISC_GPIO, pin_index);
        case PCB_PIN_TYPE_ISO_GPIO:
            return rtk_gpio_id(ISO_GPIO, pin_index);
        case PCB_PIN_TYPE_GPO:
            return rtk_gpio_id(MISC_GPO, pin_index);
        case PCB_PIN_TYPE_ISO_GPO:
            return rtk_gpio_id(ISO_GPO, pin_index);
        case PCB_PIN_TYPE_GPI:
            return rtk_gpio_id(MISC_GPI, pin_index);
        case PCB_PIN_TYPE_ISO_GPI:
            return rtk_gpio_id(ISO_GPO, pin_index);
        default:
            gpioErr("pin type incorrect!!!\n");
            return 0xFFFFFFFF;
      }     
}

