
/*
 *
 *  mcp.c
 *
 *  Implementation of hardware CP module
 *
 *
 *
 **/
#include <common.h>
#include <asm/arch/system.h>
//#include <asm/arch/rbus/otp_reg.h>
#include <asm/arch/rsa_key_2048_big.h>
#include <asm/arch/rsa_key_2048_little.h>
#include <asm/arch/error_type.h>
#include <drivers/mcp/rtk_mcp_scpu.h>
#include <drivers/mcp/rtk_mcp_core.h>
#include <malloc.h>
#include <asm/tomcrypt.h>

extern const struct ltc_hash_descriptor sha256_desc ;
extern void rtk_hexdump( const char * str, unsigned char * pcBuf, unsigned int length );
const unsigned int Kh_key_default[4] = { SECURE_KH_KEY0, SECURE_KH_KEY1, SECURE_KH_KEY2, SECURE_KH_KEY3 };
#if 0
static void hexdump( const char * str, unsigned char * pcBuf, unsigned int length )
{
	unsigned int i, j, rows, count;
	printf("======================================================\n");
	printf("%s(base=0x%08x)\n", str, (unsigned int)(pcBuf));
	count = 0;
	rows = (length+((1<<4)-1)) >> 4;
	for( i = 0; ( i < rows ) && (count < length); i++ ) {
		printf("%03x :", i<<4 );
		for( j = 0; ( j < 16 ) && (count < length); j++ ) {
			printf(" %02x",  *pcBuf );
			count++;
			pcBuf++;
		}
		printf("\n");
	}
}
#endif
extern void lowlevel_init(void);
int SW_SHA256_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[8])
{
       unsigned int ret = 0;
 	hash_state md;
	lowlevel_init();
	ret = sha256_init(&md);
	if(ret)
		printf("SW sha256 init fail\n");
	sha256_process(&md,src_addr,length);
	ret = sha256_done(&md, dst_addr);
//	printf("sw sha256 ret %x \n",ret);
	return 0;
}

int Verify_SHA256_hash( unsigned char * src_addr, unsigned int length, unsigned char * ref_sha256, unsigned int do_recovery )
{
    unsigned int ret = 0;
    unsigned char * hash1; // hash value calculated by CP engine
    unsigned char * hash2; // hash value recovery from RSA signature
    unsigned int sys_rsa_key_address;
    unsigned int signature_key_address[256];

#if 0
    sys_rsa_key_address = OTP_DATA;
#else
#if 1 // little endian
    //sys_rsa_key_address = (unsigned int)rsa_key_2048_little;
    sys_rsa_key_address = 0; //use string as input
#else // big endian
    sys_rsa_key_address = (unsigned int)rsa_key_2048_big;
#endif
#endif
    // ref_sha256 may be not 4-byte aligned, so we use local array
    memcpy((void *)signature_key_address, (void *)ref_sha256, sizeof(signature_key_address));
//    display_boottime_log("SHA1");
    hash1 = (unsigned char *)SECURE_IMAGE2HASH_BUF; // temp use this address (use malloc is better?)
    hash2 = (unsigned char *)signature_key_address;
#ifdef CONFIG_RTK_SWSHA256
    SW_SHA256_hash((unsigned char *)src_addr, length, hash1, NULL);
#else



    mcp_dscpt_addr = 0;


    flush_dcache_all();

    ret = CP_SHA256((unsigned char *)src_addr, length, hash1);

//display_boottime_log("SHA2");
    if( ret ) { //[CID-313966]: Logically dead code
        printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, ret );
        return -1;
    }
#endif

    if( do_recovery ) {
        //rtk_hexdump("signature form ref_sha256", ref_sha256, 256 );
        //rtk_hexdump("RSA key", (unsigned char *)sys_rsa_key_address, 256 );
        ret = do_RSA((unsigned int)signature_key_address, sys_rsa_key_address, SECURE_SIGN2HASH_BUF );
        if( ret == 0 ) {
            printf("[ERR] %s: do_RSA return 0\n", __FUNCTION__ );
            return -3;
        }
        hash2 = (unsigned char *)ret;
    }

    ret = memcmp(hash1, hash2, 32);
    if( ret ) {
        printf("[ERR] %s: hash value not match\n", __FUNCTION__ );
        rtk_hexdump("hash 1", hash1, 32 );
        rtk_hexdump("hash 2", hash2, 32 );
        return -2;
    }

    return 0;
}

int verify_image_signature(unsigned char *p_img, unsigned int img_size, unsigned char *p_sig, unsigned char *p_rsa_key, unsigned char *p_rsa_key_r2, unsigned char *p_mcp_sha256)
{
    int ret = 0;
    unsigned char * hash2;
    unsigned char * hash1 = (unsigned char *)p_mcp_sha256;

    if (p_mcp_sha256 == NULL)
        hash1 = (unsigned char *)malloc(32);
    
    if (hash1 == NULL) {
        ret = -1;
        goto err;
    }
    
    ret = CP_SHA256((unsigned char *)p_img, img_size, hash1);
    if (ret) {
        ret = -2;
        goto err;
    }

    if (p_rsa_key_r2 != NULL)
        ret = do_hw_RSA_with_R2((unsigned int)p_sig, (unsigned int)p_rsa_key, (unsigned int)p_rsa_key_r2, SECURE_SIGN2HASH_BUF);
    else
        ret = do_hw_RSA((unsigned int)p_sig, (unsigned int)p_rsa_key, SECURE_SIGN2HASH_BUF);
    
    if( ret == 0 ) {
        printf("[ERR] %s: do_RSA return 0\n", __FUNCTION__ );
        ret = -3;
        goto err;
    }
    hash2 = (unsigned char *)ret;
    
    ret = memcmp(hash1, hash2, 32);
    if(ret) {
        printf("[ERR] %s: hash value not match\n", __FUNCTION__ );
        //hexdump("hash 1", hash1, 32 );
        //hexdump("hash 2", hash2, 32 );
        //hexdump("SECURE_SIGN2HASH_BUF", (unsigned char*)SECURE_SIGN2HASH_BUF, 256);
        ret = -4;
        goto err;
    }
err:
    if (p_mcp_sha256 == NULL && hash1 != NULL)
        free(hash1);
    return ret;
}

void enable_SCPU_RSA (void)
{
    rtd_outl(S_RSA_SEL,1);
    while(1)
    {
        if(rtd_inl(S_RSA_ST)&0x1)
        {
            break;
        }
    }
}

