#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rtk_ecc_algorithm.h"

#define BITS_PER_INT 32
#define BYTES_PER_INT 4
/* Only for 1024bits data
* This function encode 1024bits data to 12bis ECC
* intput parameter:
*              char* orig_data : data bits (1024bits)
*
* output paramter:
*      unsigned int* ecc_data bits (12bits)
*
* return value: 0 success, non-zero if failure.
*/
int EnHammingCode1024(char* orig_data, unsigned int* ecc_data)
{
    //int cnt;
    unsigned int paritybit_idx;
    unsigned int codebit_idx;
    unsigned int databit_idx;
    unsigned int paritybit_pattern;
    unsigned int tmpCheckbit;
    unsigned int lCnt, val, lBitOff;

    if(orig_data == NULL) {
        ECC_FUNC_WARNING("[Error] Empty data ponter!\n");
        return -1;
    }

    if(ecc_data == NULL) {
        ECC_FUNC_WARNING("[Error] Empty ecc data ponter!\n");
        return -2;
    }

    *ecc_data = 0;
    paritybit_idx = 0;
    while(paritybit_idx < THE_NUM_CHECK_1024_BITS) {
        // compare bit by code word
        codebit_idx = 0;
        databit_idx = 0;
        paritybit_pattern = 1;
        tmpCheckbit = 0;
        while(codebit_idx < (THE_NUM_CHECK_1024_BITS + THE_NUM_DATA_1024_BITS)) {
            if(codebit_idx != (paritybit_pattern - 1)) {
                if(((codebit_idx + 1) >> paritybit_idx) & 0x01)  {
                    tmpCheckbit ^= ((*(orig_data + ((databit_idx & 0xfffffff8) >> 3)) >> (databit_idx & 0x7)) & 0x01);
                }

                databit_idx++;
            } else {
                paritybit_pattern = paritybit_pattern << 1;
            }
            codebit_idx++;
        }
        *ecc_data |= ((tmpCheckbit & 0x01) << paritybit_idx);
        paritybit_idx++;
    }

    ECC_FUNC_DBG("ecc = %08x\n", *ecc_data);

    // Calculate the bits number in orig data & ECC data
    val = 0;
    for(lCnt = 0; lCnt < (THE_NUM_DATA_1024_BITS / 8); lCnt++)
        for(lBitOff = 0; lBitOff < 8; lBitOff++)
            if(*(orig_data + lCnt) & (1 << lBitOff))
                val++;

    for(lCnt = 0; lCnt < THE_NUM_CHECK_1024_BITS; lCnt++)
        if(*ecc_data & (1 << lCnt))
            val++;
    *ecc_data |= ((val & 0x01) << THE_NUM_CHECK_1024_BITS);

    return 0;
}



/* Only for 1024bits*/
int DeHammingCode1024(char* ptTrasmit_data, unsigned int ecc_data)
{
    ECC_CHECK_RESULT_TYPE ret = ECC_CHECK_RESULT_SUCC;
    unsigned int syn;
    unsigned int paritybit_pattern;
    unsigned int databit_idx;
    unsigned int codebit_idx;
    unsigned int get_bit_num_idx;

    // Compute syndrome
    syn = 0;
    paritybit_pattern = 0;
    get_bit_num_idx = 0;
    databit_idx = 0;
    for (codebit_idx = 0; codebit_idx < (THE_NUM_CHECK_1024_BITS + THE_NUM_DATA_1024_BITS); codebit_idx++) {
        if(codebit_idx != paritybit_pattern) {
            if(*(ptTrasmit_data + ((databit_idx & 0xfffffff8) >> 3)) & (1 << (databit_idx & 0x07))) {
                syn ^= (codebit_idx + 1);
                get_bit_num_idx++;
            }
            databit_idx++;
        } else {
            if(ecc_data & (paritybit_pattern + 1)) {
                syn ^= (codebit_idx + 1);
                get_bit_num_idx++;
            }

            paritybit_pattern = ((paritybit_pattern + 1) << 1) - 1;
        }
    }
    // printf("syndrome = %d\n", syn);

    // Correct error if needed
    if(syn <= (THE_NUM_CHECK_1024_BITS + THE_NUM_DATA_1024_BITS)) {
        if (syn) {
            paritybit_pattern = 1;
            for(codebit_idx = 0; codebit_idx < THE_NUM_CHECK_1024_BITS; codebit_idx++) {
                if(syn  <=      paritybit_pattern)
                    break;
                paritybit_pattern = paritybit_pattern << 1;
            }

            if((get_bit_num_idx & 0x01) != ((ecc_data >> THE_NUM_CHECK_1024_BITS) & 0x01)) {
                if(paritybit_pattern == syn)
                    ecc_data ^= paritybit_pattern;
                else {
                    *(ptTrasmit_data + (((syn - codebit_idx - 1) & 0xfffffff8) >> 3)) ^= (1 << ((syn - codebit_idx - 1) & 0x07));
                    ret = ECC_CHECK_RESULT_FAIL_DATA_BUT_FIXED;
                }
                ECC_FUNC_DBG("[Info] repaired ok\n");
            } else {
                ECC_FUNC_DBG("[Error] unrepaired\n");
                ret = ECC_CHECK_RESULT_FAIL_BUT_NOT_FIXED;
            }
        } else {
            ECC_FUNC_DBG("[Info] No error for data\n");
            ret = ECC_CHECK_RESULT_SUCC;
        }
    } else {
        ECC_FUNC_DBG("[Error] Get Error syndrome\n");
        ret = ECC_CHECK_RESULT_FAIL_BUT_NOT_FIXED;
    }

    return ret;
}


/*
*Function:get the ecc check bits len
*Input:the check character data array len
*Ret:1-32 is valid
*
*/
static unsigned int get_ecc_check_bits_len(unsigned int char_array_len)
{
    int ecc_check_bits_len = 0;
    for(ecc_check_bits_len = 0; ecc_check_bits_len <= INTEGER_BITS_SIZE; ecc_check_bits_len++) {
        if(((1 << ecc_check_bits_len) - ecc_check_bits_len) >= (CHAR_BITS_SIZE * char_array_len + 1))
            break;
    }
    if(ecc_check_bits_len > INTEGER_BITS_SIZE)
        ecc_check_bits_len = 0;
    printf("%d, %d\n", char_array_len, ecc_check_bits_len);
    return ecc_check_bits_len;
}


/*
* This function encode any bits data to specialbits ECC
* intput parameter:
*              orig_data : data bits
*              orig_data_len:
* output paramter:
*      unsigned int* ecc_data bits (12bits)
*
* return value: 0 success, non-zero if failure.
*/
int EnHammingCode(unsigned char* orig_data, unsigned int orig_data_len,
                  unsigned int* ecc_data, unsigned int *ecc_data_bits_len)
{
    //int cnt;
    unsigned int paritybit_idx;
    unsigned int codebit_idx;
    unsigned int databit_idx;
    unsigned int paritybit_pattern;
    unsigned int tmpCheckbit;
    unsigned int lCnt, val, lBitOff;

    if(orig_data == NULL ||
       orig_data_len == 0) {
        ECC_FUNC_WARNING("[Error] Empty data ponter!\n");
        return -1;
    }

    if(ecc_data == NULL
       || ecc_data_bits_len == NULL) {
        ECC_FUNC_WARNING("[Error] Empty ecc data ponter!\n");
        return -1;
    }
    *ecc_data_bits_len = get_ecc_check_bits_len(orig_data_len);
    if(*ecc_data_bits_len == 0) {
        ECC_FUNC_WARNING("[Error] Can't get valid Ecc check bits len\n");
        return -1;
    }

    *ecc_data = 0;
    paritybit_idx = 0;
    while(paritybit_idx < *ecc_data_bits_len) {
        // compare bit by code word
        codebit_idx = 0;
        databit_idx = 0;
        paritybit_pattern = 1;
        tmpCheckbit = 0;
        while(codebit_idx < (*ecc_data_bits_len + orig_data_len * CHAR_BITS_SIZE)) {
            if(codebit_idx != (paritybit_pattern - 1)) {
                if(((codebit_idx + 1) >> paritybit_idx) & 0x01)  {
                    tmpCheckbit ^= ((*(orig_data + ((databit_idx & 0xfffffff8) >> 3)) >> (databit_idx & 0x7)) & 0x01);
                }

                databit_idx++;
            } else {
                paritybit_pattern = paritybit_pattern << 1;
            }
            codebit_idx++;
        }
        *ecc_data |= ((tmpCheckbit & 0x01) << paritybit_idx);
        paritybit_idx++;
    }

    ECC_FUNC_DBG("ecc = %08x\n", *ecc_data);

    // Calculate the bits number in orig data & ECC data
    val = 0;
    for(lCnt = 0; lCnt < orig_data_len; lCnt++)
        for(lBitOff = 0; lBitOff < 8; lBitOff++)
            if(*(orig_data + lCnt) & (1 << lBitOff))
                val++;

    for(lCnt = 0; lCnt < *ecc_data_bits_len; lCnt++)
        if(*ecc_data & (1 << lCnt))
            val++;
    ECC_FUNC_DBG("val = %08x\n", val);
    *ecc_data |= ((val & 0x01) << (*ecc_data_bits_len));
    /*Need to contain the whole Ecc check bits ,Even it
    *don't belong to the hamming code check bits range.
    */
    *ecc_data_bits_len = *ecc_data_bits_len + 1;
    return 0;
}


/* for any bits len*/
ECC_CHECK_RESULT_TYPE DeHammingCode(unsigned char* ptTrasmit_data, unsigned int ptTrasmit_data_len,
                                    unsigned int *p_ecc_data, unsigned int *p_ecc_check_error_bit_index)
{
    ECC_CHECK_RESULT_TYPE ret = ECC_CHECK_RESULT_SUCC;
    unsigned int syn;
    unsigned int paritybit_pattern;
    unsigned int databit_idx;
    unsigned int codebit_idx;
    unsigned int get_bit_num_idx;
    unsigned int ecc_data_bits_len;
    if(ptTrasmit_data == NULL || ptTrasmit_data_len == 0
       || p_ecc_data == NULL || p_ecc_check_error_bit_index == NULL) {
        ECC_FUNC_WARNING("[Error] Can't get valid Ecc check bits len\n");
        ret = ECC_CHECK_RESULT_IMPROPER_PARAM;
        return ret;
    }
    ecc_data_bits_len = get_ecc_check_bits_len(ptTrasmit_data_len);
    if(ecc_data_bits_len == 0) {
        ECC_FUNC_WARNING("[Error] Can't get valid Ecc check bits len\n");
        ret = ECC_CHECK_RESULT_IMPROPER_PARAM;
        return ret;
    }
    ECC_FUNC_DBG("ecc_data = %d\n", *p_ecc_data);
    // Compute syndrome
    syn = 0;
    paritybit_pattern = 0;
    get_bit_num_idx = 0;
    databit_idx = 0;
    for (codebit_idx = 0; codebit_idx < (ecc_data_bits_len + CHAR_BITS_SIZE * ptTrasmit_data_len); codebit_idx++) {
        if(codebit_idx != paritybit_pattern) {
            if(*(ptTrasmit_data + ((databit_idx & 0xfffffff8) >> 3)) & (1 << (databit_idx & 0x07))) {
                syn ^= (codebit_idx + 1);
                get_bit_num_idx++;
            }
            databit_idx++;
        } else {
            if(*p_ecc_data & (paritybit_pattern + 1)) {
                syn ^= (codebit_idx + 1);
                get_bit_num_idx++;
            }

            paritybit_pattern = ((paritybit_pattern + 1) << 1) - 1;
        }
    }
    ECC_FUNC_DBG("syndrome = %d\n", syn);

    // Correct error if needed
    if(syn <= (ecc_data_bits_len + CHAR_BITS_SIZE * ptTrasmit_data_len)) {
        if (syn) {
            paritybit_pattern = 1;
            for(codebit_idx = 0; codebit_idx < ecc_data_bits_len; codebit_idx++) {
                if(syn  <=      paritybit_pattern)
                    break;
                paritybit_pattern = paritybit_pattern << 1;
            }

            if((get_bit_num_idx & 0x01) != ((*p_ecc_data >> ecc_data_bits_len) & 0x01)) {
                if(paritybit_pattern == syn) {
                    *p_ecc_data ^= paritybit_pattern;
                    *p_ecc_check_error_bit_index = codebit_idx;
                    ret = ECC_CHECK_RESULT_FAIL_ECC_BUT_FIXED;

                } else {
                    *(ptTrasmit_data + (((syn - codebit_idx - 1) & 0xfffffff8) >> 3)) ^= (1 << ((syn - codebit_idx - 1) & 0x07));
                    *p_ecc_check_error_bit_index = (syn - codebit_idx - 1);
                    ret = ECC_CHECK_RESULT_FAIL_DATA_BUT_FIXED;
                }
                ECC_FUNC_DBG("[Info] repaired ok\n");
            } else {
                ECC_FUNC_DBG("[Error] unrepaired\n");
                ret = ECC_CHECK_RESULT_FAIL_BUT_NOT_FIXED;
            }
        } else {
            ECC_FUNC_DBG("[Info] No error for data\n");
            ret = ECC_CHECK_RESULT_SUCC;
        }
    } else {
        ECC_FUNC_DBG("[Error] Get Error syndrome\n");
        ret = ECC_CHECK_RESULT_FAIL_BUT_NOT_FIXED;
    }
    return ret;
}




