#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/fcntl.h>
#include <linux/random.h>
#include "rtk_otp_sw_crc.h"
#include "rtk_otp_util.h"
#include <drivers/otp/rtk_otp_reg.h>


#define OTP_DEVICE_UNIQUE_KEY_OFST              0x28
#define OTP_DEVICE_UNIQUE_KEY_LEN               16


extern unsigned char builtin_otp_map_array[];
extern unsigned char builtin_otp_map_array_end;

int otp_write(int fd,unsigned int ofst,unsigned int val)
{
    char buffer[256] = {0};
    //printf("0x%08x = 0x%08x\n", OTP_DDR_MAP_BASE + ofst, val);
    if(val == 0)
        return 0;

    sprintf(buffer, "\n\n\n#0x%08x = 0x%08x\n", OTP_DDR_MAP_BASE + ofst, val);
    write(fd, buffer, strlen(buffer));


    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_PROG_STATUS1, 0x0);
    write(fd, buffer, strlen(buffer));   
    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_DDR_MAP_BASE + ofst, val);
    write(fd, buffer, strlen(buffer));
    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_CTRL_PROG, OTP_CTRL_PROG_EN_PROG_EFUSE);
    write(fd, buffer, strlen(buffer));
        
    sprintf(buffer, "ScalerTimer_DelayXms(2);\n");
    write(fd, buffer, strlen(buffer));
        
    sprintf(buffer, "rtd_inl(0x%08x);\n", OTP_PROG_STATUS1);
    write(fd, buffer, strlen(buffer));
    sprintf(buffer, "rtd_inl(0x%08x);\n", OTP_PROG_STATUS2);
    write(fd, buffer, strlen(buffer));

        
    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_PROG_STATUS1, 0x0);
    write(fd, buffer, strlen(buffer));    
    return 0;    
}



int otp_program_with_bitmap(int fd)
{
    int i = 0;
    int ret;
    char buffer[256] = {0};
    int len = 0;
    unsigned int *map_head = (unsigned int *)builtin_otp_map_array;
    unsigned char device_unique_key[128] = {0};
    unsigned int device_unique_key_ecc_value = 0;
    unsigned int device_unique_key_ecc_value_bits = 0;
    if(fd < 0)
        return;

    memset(device_unique_key, 0, 128);
    for(i = 0; i < OTP_DEVICE_UNIQUE_KEY_LEN ; i++)
        device_unique_key[i] = rand() % 0xFF;
    
    memcpy(builtin_otp_map_array + OTP_DEVICE_UNIQUE_KEY_OFST,
           device_unique_key, OTP_DEVICE_UNIQUE_KEY_LEN);
    
    sprintf(buffer, "#the device unique key = 0x");
    for(i = OTP_DEVICE_UNIQUE_KEY_LEN - 1; i >= 0; i--)
        sprintf(buffer + strlen(buffer), "%02x", device_unique_key[i]);
    sprintf(buffer, "\n#it's ecc value = 0x%08x\n", device_unique_key_ecc_value);
    write(fd, buffer, strlen(buffer));
    
    len = (&builtin_otp_map_array_end - builtin_otp_map_array) / sizeof(unsigned int); 
    for(i = 0; i < len; i++)
        otp_write(fd, 4 * i, map_head[i]);
    return 0;

}


int otp_program_with_bitmap_with_verify(int fd)
{
    int i = 0;
    int ret;
    char buffer[256] = {0};
    int len = 0;
    time_t t;
    time(&t);
    unsigned int sw_crc = 0x1;
    unsigned int *map_head = (unsigned int *)builtin_otp_map_array;
    unsigned char device_unique_key[16] = {0};
    unsigned int device_unique_key_ecc_value = 0;
    unsigned int device_unique_key_ecc_value_bits = 0;
    if(fd < 0)
        return;
    srand(t);
    memset(device_unique_key, 0, 16);
    for(i = 0; i < OTP_DEVICE_UNIQUE_KEY_LEN ; i++) {
        unsigned char tmp_v = 0xa5;
        while(1) {
            tmp_v = rand() % 0xFF;
            if(tmp_v)
                break;
        }
        device_unique_key[i] = tmp_v;
    }
    
    memcpy(builtin_otp_map_array + OTP_DEVICE_UNIQUE_KEY_OFST,
           device_unique_key, OTP_DEVICE_UNIQUE_KEY_LEN);

    sprintf(buffer, "#the device unique key = 0x");
    for(i = OTP_DEVICE_UNIQUE_KEY_LEN - 1; i >= 0; i--)
        sprintf(buffer + strlen(buffer), "%02x", device_unique_key[i]);
    sprintf(buffer + strlen(buffer), "\n");
    write(fd, buffer, strlen(buffer));

    len = (&builtin_otp_map_array_end - builtin_otp_map_array) / sizeof(unsigned int); 
    for(i = 0; i < len; i++)
        otp_write(fd, 4 * i, map_head[i]);  

    count_sw_crc(CRC_TYPE_CRC16, builtin_otp_map_array,
                         len * sizeof(unsigned int), &sw_crc);
    sprintf(buffer, "\n#The expected whole CRC value == 0x%08x", sw_crc);
    write(fd, buffer, strlen(buffer));
    return 0;
}





/*-----------------------------------------------------------------------------
 * Func : otp_crc_partial_region
 *
 * Desc : otp_crc_partial_region
 *
 * Parm : start_ofst:the offset of start reg you want to verify.
 *      end_ofst:the offset of end reg you want to verify.
 *      pval:the address of the crc value
 * Retn : OTP words
 *-----------------------------------------------------------------------------*/
int otp_crc_partial_region(int fd, unsigned int start_ofst, unsigned int len, int is_margin_read)
{
    int ret = -1;
    int times = 200;
    char buffer[256] = {0};
    if (start_ofst >= MAX_OTP_OFST || (start_ofst + len) > MAX_OTP_OFST || fd < 0) {
        printf("params are input error\n");
        return ret;
    }
    if(is_margin_read)
        sprintf(buffer, "\n\n#Do hw margin read crc value now\n");
    else
        sprintf(buffer, "\n\n#Do hw normal read crc value now\n");
    write(fd, buffer, strlen(buffer));
    
    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_CRC, OTP_CRC_CRC_INI);
    write(fd, buffer, strlen(buffer));

    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_AUTO_RD_REGION, OTP_AUTO_RD_REGION_START_ADDR(start_ofst)
             | OTP_AUTO_RD_REGION_END_ADDR(start_ofst + len - 1));
    write(fd, buffer, strlen(buffer));
    
    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_CTRL_CRC_EN, OTP_CRC_EN(1));
    write(fd, buffer, strlen(buffer));
    
    if(is_margin_read)
        sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_CTRL, (OTP_CTRL_AUTO_RD_REQ | OTP_CTRL_WRITE_EN0) | 0x10);
    else
        sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_CTRL, (OTP_CTRL_AUTO_RD_REQ | OTP_CTRL_WRITE_EN0));
    write(fd, buffer, strlen(buffer));
    
    sprintf(buffer, "ScalerTimer_DelayXms(10);\n");
    write(fd, buffer, strlen(buffer));
    
    sprintf(buffer, "rtd_inl(0x%08x);\n", OTP_CTRL);
    write(fd, buffer, strlen(buffer));

    sprintf(buffer, "\n\n#The Hw crc value is::\n");
    write(fd, buffer, strlen(buffer));
    
    sprintf(buffer, "rtd_part_inl(0x%08x, %d, %d);\n\n\n", OTP_CRC, 15, 0);
    write(fd, buffer, strlen(buffer));
    
    sprintf(buffer, "rtd_outl(0x%08x, 0x%08x);\n", OTP_CTRL_CRC_EN, OTP_CRC_EN(0));
    write(fd, buffer, strlen(buffer));

    ret = 0;
    return ret;
}

/*-----------------------------------------------------------------------------
 * Func : otp_crc_auto_region
 *
 * Desc : opt crc auto region
 *
 * Parm : pval:pval:the address of the crc value
 *
 * Retn : OTP words
 *-----------------------------------------------------------------------------*/

int otp_crc_auto_region(int fd, int is_margin_read)
{
    return otp_crc_partial_region(fd, 0x0, MAX_OTP_OFST, is_margin_read);
}

