#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "generate_config.h"
#include "util.h"
#include "ver.h"

#define OPTION_GEN_NONE	0	
#define OPTION_GEN_CONFIG_FILE		1
#define OPTION_GEN_BITMAP_STRUCT		2
#define OPTION_GEN_CONFIG_ARRAY		3
#define OPTION_GEN_ID_ARRAY		4
#define OPTION_GEN_OTP_ID_HEADER_FILE	5
#define OPTION_GEN_OTP_ID_PARAM_FILE	6
#define OPTION_GEN_ALL		7

static void show_usage(void)
{
#define INDENT_ "\n\t"
	printf("\nUsage:\n"
		"\n"
		"<1>./otp_config_generator gen_config <config_file>\n"
		INDENT_
		"- Get Config file\n"
		"\n"
		"<2>./otp_config_generator gen_struct <config_file>\n"
		INDENT_
		"- Get bitmap struct \n"
		"\n"
		"<3>./otp_config_generator gen_config_array <config_file>\n"
		INDENT_
		"- Get Config array \n"
		"\n"
		"<4>./otp_config_generator gen_id_array <config_file>\n"
		INDENT_
		"- Get Config array \n"
		"<5>./otp_config_generator gen_id_header <config_file>\n"
		INDENT_
		"- Get otp_id.h \n"
		"<6>./otp_config_generator gen_param_header <config_file>\n"
		INDENT_
		"- Get otp_param.h \n"
		"<7>./otp_config_generator gen_all <config_file>\n"
		INDENT_
		"- Get all \n"
		);
#undef INDENT_
}

static int get_option(int argc, char **argv, int *option,
	char **config_file, int *start_ofst, int *len)
{
	int i, res;
	char *ptr;

	*option = OPTION_GEN_NONE;
	*config_file = NULL;
	*start_ofst = 0;
	*len = 0;
	res = -1;

	for (i = 0; i <argc; i++) {
		ptr = "gen_config";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0
			&& strncmp("gen_config_array", argv[i], strlen("gen_config_array")) != 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_GEN_CONFIG_FILE;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
		ptr = "gen_struct";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_GEN_BITMAP_STRUCT;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
		ptr = "gen_config_array";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_GEN_CONFIG_ARRAY;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
		ptr = "gen_id_array";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_GEN_ID_ARRAY;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
		ptr = "gen_id_header";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}
			*option = OPTION_GEN_OTP_ID_HEADER_FILE;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
		ptr = "gen_param_header";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}
			*option = OPTION_GEN_OTP_ID_PARAM_FILE;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
		ptr = "gen_all";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_GEN_ALL;
			*config_file = argv[i+1];
			res = 0;
			break;
		}
	}
	return res;
}
void show_attention_info_about_this_program()
{
	printf("\n***********************************TIPS*********************************\n");
	printf("Program version %s (compiled at %s).\n", VERSION_OTP_BITMAP_GENERATOR, __DATE__);
	printf("  Attention:\n");
	printf("    1)The format of the item in the config file should be:end_bits start_bits name desc.\n");
	printf("    2)The name of the item sould be a legal name.\n");
	printf("    3)The desc of the item sould be a line or be contained in double quotation marks.\n");
	printf("    4)The encoding of the config file should only be ASCII txt.\n");
	printf("    5)The output file will be contained at the sub fold output/.\n");
	printf("**************************************************************************\n\n");
	
}
int main(int argc, char **argv)
{
	int ret = -1;
	int option;
	int len,start_ofst;
	char *config_file;
	OTP_CONFIG_TYPE type = OTP_CONFIG_TYPE_NONE;
	show_attention_info_about_this_program();
	
	// check argument
	if (get_option(argc, argv, &option, &config_file, &start_ofst, &len)) {
		show_usage(); 
		return ret;
	}
	switch(option) {
	case OPTION_GEN_CONFIG_FILE:
		type = OTP_CONFIG_TYPE_CONFIG_FILE;
       	break;
	case OPTION_GEN_BITMAP_STRUCT:
		type = OTP_CONFIG_TYPE_BITMAP_STRUCT;
		break;
	case OPTION_GEN_CONFIG_ARRAY:
		type = OTP_CONFIG_TYPE_CONFIG_ARRAY;
		break;
	case OPTION_GEN_ID_ARRAY:
		type = OTP_CONFIG_TYPE_ID_ARRAY;
		break;
	case OPTION_GEN_OTP_ID_HEADER_FILE:
		type = OTP_CONFIG_OTP_ID_HEADER_FILE;
		break;
	case OPTION_GEN_OTP_ID_PARAM_FILE:
		type = OTP_CONFIG_OTP_PARAM_HEADER_FILE;
		break;
	case OPTION_GEN_ALL:
		type = OTP_CONFIG_TYPE_NONE;
	default:
		type = OTP_CONFIG_TYPE_NONE;
		break;
	}
	system("rm -rf "OUTPUT_FOLD);
	system("mkdir "OUTPUT_FOLD);
	if(type != OTP_CONFIG_TYPE_NONE) {
		ret = generate_config(config_file, type);
	}
	else if(option == OPTION_GEN_ALL) {
		ret = generate_config(config_file, OTP_CONFIG_TYPE_CONFIG_FILE);
		ret = generate_config(config_file, OTP_CONFIG_TYPE_BITMAP_STRUCT);
		ret = generate_config(config_file, OTP_CONFIG_TYPE_CONFIG_ARRAY);
		ret = generate_config(config_file, OTP_CONFIG_TYPE_ID_ARRAY);
		ret = generate_config(config_file, OTP_CONFIG_OTP_ID_HEADER_FILE);
		ret = generate_config(config_file, OTP_CONFIG_OTP_PARAM_HEADER_FILE);
	}
	if(ret == 0)
		printf("[OTP_CONFIG_GENERATOR]:generate sucess\n");
	else
		printf("[OTP_CONFIG_GENERATOR]:generate fail\n");
		
	return 0;
}

