#ifndef __RTK_PWM_LIB_H__
#define __RTK_PWM_LIB_H__

#include  <drivers/pwm/pwm_reg.h>

#define PWM_DUTY_MAX   255  //Base
#define PWM_RESOLUTION 2047

#define ISO_PWM_MAX    2
#define MISC_PWM_MAX    8
#define MIN_PWM_FREQ        (4)
#define MAX_PWM_FREQ        (105600) 
#define N_MAX  (4096)
#define M_MAX  (3)
#define CONFIG_CPU_XTAL_FREQ_27MHZ  27000000
#define CONFIG_CPU_PLL_FREQ_196MHZ  196000000
#define CONFIG_CPU_PLL_FREQ_147MHZ  147000000
#define CONFIG_CPU_XTAL_NS_27MHZ   (37)   /*1/27000000*/
#define CONFIG_CPU_PLL_NS_196MHZ   (5)    /*1/196000000*/
#define CONFIG_CPU_PLL_NS_147MHZ   (7)    /*1/147000000*/
#define ABS(x, y)    ((x) > (y) ? ((x)-(y)) : ((y)-(x)))

#define TRUE 1
#define FALSE 0

#define PWM_OK 0
#define PWM_FAILE (-1)

#define GET_PIN_TYPE(x)     ((PCB_PIN_TYPE_T) (x & 0xFF))
#define GET_PIN_INDEX(x)    ((unsigned char) ((x >> 8) & 0xFF))
#define GET_PIN_PARAM1(x)   ((unsigned char) ((x >> 16) & 0xFF))
#define GET_PIN_PARAM2(x)   ((unsigned char) ((x >> 24) & 0xFF))
#define GET_PIN_PARAM3(x)   ((unsigned char) ((x >> 32) & 0xFF))
#define GET_PIN_PARAM4(x)   ((unsigned char) ((x >> 40) & 0xFF))
#define GET_PIN_PARAM5(x)   ((unsigned char) ((x >> 48) & 0xFF))
#define GET_PIN_PARAM6(x)   ((unsigned char) ((x >> 56) & 0xFF))
#define GET_PWM_INVERT(x)   ((GET_PIN_PARAM1(x) >> 7) & 0x01)
#define GET_PWM_FREQ(x)     ((((unsigned int) (GET_PIN_PARAM1(x) & 0x7F)) << 16) | (((unsigned int) GET_PIN_PARAM2(x)) << 8) | (GET_PIN_PARAM3(x)))
#define GET_PWM_DUTY_MAX(x)  GET_PIN_PARAM4(x)
#define GET_PWM_INIT_DUTY(x) GET_PIN_PARAM5(x)


#define GET_PWM_PINMUX_GPIO_NUM(x)        ((unsigned int)  (((x)) & 0x1FF))
#define GET_PWM_PINMUX_GPIO_INIT_VALUE(x) ((unsigned int)  (((x) >> 9) & 0x1))
#define GET_PWM_PINMUX_GPIO_PINTYPE(x)    ((unsigned int)  (((x) >> 10) & 0xF))
#define GET_PWM_PINMUX_GPIOVALUE(x)       ((unsigned char) (((x) >> 14) & 0xF))
#define GET_PWM_PINMUX_PWMVALUE(x)        ((unsigned char) (((x) >> 18) & 0xF))
#define GET_PWM_PINMUX_REG_ENDBIT(x)      ((unsigned char) (((x) >> 22) & 0x1F))
#define GET_PWM_PINMUX_REG_STARTBIT(x)    ((unsigned char) (((x) >> 27) & 0x1F))
#define GET_PWM_PINMUX_ADDR(x)            ((unsigned int)  (((x) >> 32) & 0xFFFFFFFF))

typedef enum {
    PWM_CRYSTAL = 0,
    PWM_PLL     = 1,
} PWM_CLK_SRC;

typedef enum {
    CLK_27MHZ = 0,
    CLK_196MHZ= 1,
    D_CLK_DIV4= 2,
    CLK_147MHZ=3,
} PWM_CLK_SEL;

typedef enum
{
    PWM_ISO = 0,
    PWM_MISC,
    PWM_UNKNOW,
}RTK_PWM_TYPE;

typedef enum{
    PWM_MODE   = 0,
    GPIO_MODE  = 1,
}PIN_MODE_T;

typedef enum {
    PWM0_DB0 = 0,
    PWM1_DB0,   
    PWM2_DB0,   
    PWM3_DB0,   
    PWM4_DB0,   
    PWM5_DB0,
    PWM6_DB0,   
    PWM7_DB0,   
    PWM_DB0_MAX,
} RTK_PWM_DB0_GROUP;

struct rtk_pwm_reg_map_t{
    union {
        unsigned long MISC_CTRL;
        unsigned long ISO_CTRL;
    };
    union {
        unsigned long MISC_TIMING_CTRL;
        unsigned long ISO_TIMING_CTRL;
    };
    union {
        unsigned long MISC_DUTY_SET;
        unsigned long ISO_DUTY_SET;
    };
    union {
        unsigned long DVS_PERIOD;
        unsigned long ALIGN_SET;
    };
    union {
        unsigned long DV_MONITOR;
        unsigned long TRACKVALUE_SET;
    };
    union {
        unsigned long MONITOR;
        unsigned long PWM_UNUSED1;
    };
    union {
        unsigned long ERR_STATUS;
        unsigned long PWM_UNUSED2;
    };
    union {
        unsigned long TRACK_SET;
        unsigned long PWM_UNUSED3;
    };
    union {
        unsigned long TRACK_FSM;
        unsigned long PWM_UNUSED4;
    };
    union {
        unsigned long TRACK_PTI;
        unsigned long PWM_UNUSED5;
    };
    union {
        unsigned long FREQ0_SET;
        unsigned long PWM_UNUSED6;
    };
    union {
        unsigned long FREQ1_SET;
        unsigned long PWM_UNUSED7;
    };
    union {
        unsigned long PHASE0_SET;
        unsigned long PWM_UNUSED8;
    };
    union {
        unsigned long PHASE1_SET;
        unsigned long PWM_UNUSED9;
    };
    union {
        unsigned long HW_PREDICT;
        unsigned long PWM_UNUSED10;
    };
    union {
        unsigned long SW_PREDICT;
        unsigned long PWM_UNUSED11;
    };
    union {
        unsigned long PREDICT_RATIO;
        unsigned long PWM_UNUSED12;
    };
};

typedef struct rtk_pwm_reg_map_t rtk_pwm_reg_map;

struct rtk_pwm_phy_t {
    const rtk_pwm_reg_map *p_reg_map;
};

typedef struct rtk_pwm_phy_t rtk_pwm_phy;

struct rtk_pwm_pin_t {
    const char* pcb_name;
    rtk_pwm_phy pwm_phy;
    int index;
    int type;
};
typedef struct rtk_pwm_pin_t rtk_pwm_pin;

static const rtk_pwm_reg_map iso_pwm0_reg = {
    .ISO_CTRL = ISO_PWM0_CTRL_VADDR,
    .ISO_TIMING_CTRL = ISO_PWM0_TIMING_CTRL_VADDR,
    .ISO_DUTY_SET = ISO_PWM0_DUTY_SET_VADDR,
    .ALIGN_SET = ISO_PWM0_ALIGN_VADDR,
    .TRACKVALUE_SET = ISO_PWM0_TRACKVALUE_VADDR,
    .PWM_UNUSED1  = ISO_PWM_NONE,
    .PWM_UNUSED2  = ISO_PWM_NONE,
    .PWM_UNUSED3  = ISO_PWM_NONE,
    .PWM_UNUSED4  = ISO_PWM_NONE,
    .PWM_UNUSED5  = ISO_PWM_NONE,
    .PWM_UNUSED6  = ISO_PWM_NONE,
    .PWM_UNUSED7  = ISO_PWM_NONE,
    .PWM_UNUSED8  = ISO_PWM_NONE,
    .PWM_UNUSED9  = ISO_PWM_NONE,
    .PWM_UNUSED10 = ISO_PWM_NONE,
    .PWM_UNUSED11 = ISO_PWM_NONE,
    .PWM_UNUSED12 = ISO_PWM_NONE,
};

static const rtk_pwm_reg_map iso_pwm1_reg = {
    .ISO_CTRL = ISO_PWM1_CTRL_VADDR,
    .ISO_TIMING_CTRL = ISO_PWM1_TIMING_CTRL_VADDR,
    .ISO_DUTY_SET = ISO_PWM1_DUTY_SET_VADDR,
    .ALIGN_SET = ISO_PWM1_ALIGN_VADDR,
    .TRACKVALUE_SET = ISO_PWM1_TRACKVALUE_VADDR,
    .PWM_UNUSED1  = ISO_PWM_NONE,
    .PWM_UNUSED2  = ISO_PWM_NONE,
    .PWM_UNUSED3  = ISO_PWM_NONE,
    .PWM_UNUSED4  = ISO_PWM_NONE,
    .PWM_UNUSED5  = ISO_PWM_NONE,
    .PWM_UNUSED6  = ISO_PWM_NONE,
    .PWM_UNUSED7  = ISO_PWM_NONE,
    .PWM_UNUSED8  = ISO_PWM_NONE,
    .PWM_UNUSED9  = ISO_PWM_NONE,
    .PWM_UNUSED10 = ISO_PWM_NONE,
    .PWM_UNUSED11 = ISO_PWM_NONE,
    .PWM_UNUSED12 = ISO_PWM_NONE,
};

static const rtk_pwm_reg_map mis_pwm0_reg = {
    .MISC_CTRL = MIS_PWM0_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM0_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM0_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM0_DVS_PERIOD_VADDR,
    .DV_MONITOR = MIS_PWM0_DVS_MONITOR_VADDR,
    .MONITOR = MIS_PWM0_MONITOR_VADDR,
    .ERR_STATUS = MIS_PWM0_ERR_STATUS_VADDR,
    .TRACK_SET = MIS_PWM0_TRACK_SET_VADDR,
    .TRACK_FSM = MIS_PWM0_TRACK_FSM_VADDR,
    .TRACK_PTI = MIS_PWM0_TRACK_PTI_VADDR,
    .FREQ0_SET = MIS_PWM0_FREQ0_VADDR,
    .FREQ1_SET = MIS_PWM0_FREQ1_VADDR,
    .PHASE0_SET = MIS_PWM0_PHASE0_VADDR,
    .PHASE1_SET = MIS_PWM0_PHASE1_VADDR,
    .HW_PREDICT = MIS_PWM0_HW_PREDICT_VADDR,
    .SW_PREDICT = MIS_PWM0_SW_PREDICT_VADDR,
    .PREDICT_RATIO = MIS_PWM0_PREDICT_RATIO_VADDR,
};

static const rtk_pwm_reg_map mis_pwm1_reg = {
    .MISC_CTRL = MIS_PWM1_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM1_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM1_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM1_DVS_PERIOD_VADDR,
    .DV_MONITOR = MIS_PWM1_DVS_MONITOR_VADDR,
    .MONITOR = MIS_PWM1_MONITOR_VADDR,
    .ERR_STATUS = MIS_PWM1_ERR_STATUS_VADDR,
    .TRACK_SET = MIS_PWM1_TRACK_SET_VADDR,
    .TRACK_FSM = MIS_PWM1_TRACK_FSM_VADDR,
    .TRACK_PTI = MIS_PWM1_TRACK_PTI_VADDR,
    .FREQ0_SET = MIS_PWM1_FREQ0_VADDR,
    .FREQ1_SET = MIS_PWM1_FREQ1_VADDR,
    .PHASE0_SET = MIS_PWM1_PHASE0_VADDR,
    .PHASE1_SET = MIS_PWM1_PHASE1_VADDR,
    .HW_PREDICT = MIS_PWM1_HW_PREDICT_VADDR,
    .SW_PREDICT = MIS_PWM1_SW_PREDICT_VADDR,
    .PREDICT_RATIO = MIS_PWM1_PREDICT_RATIO_VADDR,
};

static const rtk_pwm_reg_map mis_pwm2_reg = {
    .MISC_CTRL = MIS_PWM2_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM2_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM2_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM_NONE,
    .DV_MONITOR = MIS_PWM_NONE,
    .MONITOR = MIS_PWM_NONE,
    .ERR_STATUS = MIS_PWM_NONE,
    .TRACK_SET = MIS_PWM_NONE,
    .TRACK_FSM = MIS_PWM_NONE,
    .TRACK_PTI = MIS_PWM_NONE,
    .FREQ0_SET = MIS_PWM_NONE,
    .FREQ1_SET = MIS_PWM_NONE,
    .PHASE0_SET = MIS_PWM_NONE,
    .PHASE1_SET = MIS_PWM_NONE,
    .HW_PREDICT = MIS_PWM_NONE,
    .SW_PREDICT = MIS_PWM_NONE,
    .PREDICT_RATIO = MIS_PWM_NONE,
};

static const rtk_pwm_reg_map mis_pwm3_reg = {
    .MISC_CTRL = MIS_PWM3_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM3_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM3_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM_NONE,
    .DV_MONITOR = MIS_PWM_NONE,
    .MONITOR = MIS_PWM_NONE,
    .ERR_STATUS = MIS_PWM_NONE,
    .TRACK_SET = MIS_PWM_NONE,
    .TRACK_FSM = MIS_PWM_NONE,
    .TRACK_PTI = MIS_PWM_NONE,
    .FREQ0_SET = MIS_PWM_NONE,
    .FREQ1_SET = MIS_PWM_NONE,
    .PHASE0_SET = MIS_PWM_NONE,
    .PHASE1_SET = MIS_PWM_NONE,
    .HW_PREDICT = MIS_PWM_NONE,
    .SW_PREDICT = MIS_PWM_NONE,
    .PREDICT_RATIO = MIS_PWM_NONE,
};

static const rtk_pwm_reg_map mis_pwm4_reg = {
    .MISC_CTRL = MIS_PWM4_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM4_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM4_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM_NONE,
    .DV_MONITOR = MIS_PWM_NONE,
    .MONITOR = MIS_PWM_NONE,
    .ERR_STATUS = MIS_PWM_NONE,
    .TRACK_SET = MIS_PWM_NONE,
    .TRACK_FSM = MIS_PWM_NONE,
    .TRACK_PTI = MIS_PWM_NONE,
    .FREQ0_SET = MIS_PWM_NONE,
    .FREQ1_SET = MIS_PWM_NONE,
    .PHASE0_SET = MIS_PWM_NONE,
    .PHASE1_SET = MIS_PWM_NONE,
    .HW_PREDICT = MIS_PWM_NONE,
    .SW_PREDICT = MIS_PWM_NONE,
    .PREDICT_RATIO = MIS_PWM_NONE,
};

static const rtk_pwm_reg_map mis_pwm5_reg = {
    .MISC_CTRL = MIS_PWM5_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM5_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM5_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM_NONE,
    .DV_MONITOR = MIS_PWM_NONE,
    .MONITOR = MIS_PWM_NONE,
    .ERR_STATUS = MIS_PWM_NONE,
    .TRACK_SET = MIS_PWM_NONE,
    .TRACK_FSM = MIS_PWM_NONE,
    .TRACK_PTI = MIS_PWM_NONE,
    .FREQ0_SET = MIS_PWM_NONE,
    .FREQ1_SET = MIS_PWM_NONE,
    .PHASE0_SET = MIS_PWM_NONE,
    .PHASE1_SET = MIS_PWM_NONE,
    .HW_PREDICT = MIS_PWM_NONE,
    .SW_PREDICT = MIS_PWM_NONE,
    .PREDICT_RATIO = MIS_PWM_NONE,
};

static const rtk_pwm_reg_map mis_pwm6_reg = {
    .MISC_CTRL = MIS_PWM6_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM6_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM6_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM_NONE,
    .DV_MONITOR = MIS_PWM_NONE,
    .MONITOR = MIS_PWM_NONE,
    .ERR_STATUS = MIS_PWM_NONE,
    .TRACK_SET = MIS_PWM_NONE,
    .TRACK_FSM = MIS_PWM_NONE,
    .TRACK_PTI = MIS_PWM_NONE,
    .FREQ0_SET = MIS_PWM_NONE,
    .FREQ1_SET = MIS_PWM_NONE,
    .PHASE0_SET = MIS_PWM_NONE,
    .PHASE1_SET = MIS_PWM_NONE,
    .HW_PREDICT = MIS_PWM_NONE,
    .SW_PREDICT = MIS_PWM_NONE,
    .PREDICT_RATIO = MIS_PWM_NONE,
};

static const rtk_pwm_reg_map mis_pwm7_reg = {
    .MISC_CTRL = MIS_PWM7_CTRL_VADDR,
    .MISC_TIMING_CTRL = MIS_PWM7_TIMING_CTRL_VADDR,
    .MISC_DUTY_SET = MIS_PWM7_DUTY_SET_VADDR,
    .DVS_PERIOD = MIS_PWM_NONE,
    .DV_MONITOR = MIS_PWM_NONE,
    .MONITOR = MIS_PWM_NONE,
    .ERR_STATUS = MIS_PWM_NONE,
    .TRACK_SET = MIS_PWM_NONE,
    .TRACK_FSM = MIS_PWM_NONE,
    .TRACK_PTI = MIS_PWM_NONE,
    .FREQ0_SET = MIS_PWM_NONE,
    .FREQ1_SET = MIS_PWM_NONE,
    .PHASE0_SET = MIS_PWM_NONE,
    .PHASE1_SET = MIS_PWM_NONE,
    .HW_PREDICT = MIS_PWM_NONE,
    .SW_PREDICT = MIS_PWM_NONE,
    .PREDICT_RATIO = MIS_PWM_NONE,
};

static const rtk_pwm_phy pwm_iso_phy[] = {
    {&iso_pwm0_reg},
    {&iso_pwm1_reg},
};

static const rtk_pwm_phy pwm_mis_phy[] = {
    {&mis_pwm0_reg},
    {&mis_pwm1_reg},
    {&mis_pwm2_reg},
    {&mis_pwm3_reg},
    {&mis_pwm4_reg},
    {&mis_pwm5_reg},
    {&mis_pwm6_reg},
    {&mis_pwm7_reg},
};
/*
static rtk_pwm_pin g_pwm_phy_avlb[] = {
    {"PIN_ISO_PWM_DIM1", &RTD287x_iso_pwm0_reg, -1},
    {"PIN_ISO_PWM_DIM2", &RTD287x_iso_pwm1_reg, -1},
    {"PIN_PWM_DIM1", &RTD287x_mis_pwm0_reg, -1},
    {"PIN_PWM_DIM2", &RTD287x_mis_pwm1_reg, -1},
    {"PIN_PWM_DIM3", &RTD287x_mis_pwm2_reg, -1},
    {"PIN_PWM_DIM4", &RTD287x_mis_pwm3_reg, -1},
    {"PIN_PWM_DIM5", &RTD287x_mis_pwm4_reg, -1},
    {"PIN_PWM_DIM6", &RTD287x_mis_pwm5_reg, -1},
    {"PIN_PWM_DIM7", &RTD287x_mis_pwm6_reg, -1},
    {"PIN_PWM_DIM8", &RTD287x_mis_pwm7_reg, -1},
};
#define RTK_PWM_CANDIDATE_CNT  sizeof(g_pwm_phy_avlb)/sizeof(g_pwm_phy_avlb[0])*/

typedef union {
    unsigned long long val;
    struct {
        unsigned long long pwm_type:8;
        unsigned long long pwm_index:8;
        unsigned long long freq_h:7;
        unsigned long long invert:1;
        unsigned long long freq_l:8;
        unsigned long long freq:8;
        unsigned long long duty_max:8;
        unsigned long long init_duty:8;
        unsigned long long res1:8;
    };
}PWM_PIN_INFO;

typedef union {
    unsigned long long val;
    struct {
        unsigned long long gpio_num:9;
        unsigned long long init_val:1;
        unsigned long long gpio_group:4;
        unsigned long long gpio_value:4;
        unsigned long long pwm_value:4;
        unsigned long long end_bit:5;
        unsigned long long start_bit:5;
        unsigned long long addr:32;
    };
}PWM_PINMUX_INFO;

typedef struct _RTK_PWM_INFO {
    char *name;
    PWM_PIN_INFO pinInfo;
    PWM_PINMUX_INFO pin_mux_info;
    int mode;
    unsigned int   gid;
}RTK_PWM_INFO;

enum pwm_cmd {
    PWM_FREQ_GET = 0,
    PWM_FREQ_SET,
    PWM_DUTY_GET,
    PWM_DUTY_SET,
    PWM_ENABLE_GET,
    PWM_ENABLE_SET,
    PWM_SYNC_GET,
    PWM_SYNC_SET,
    PWM_MODE_SET,
    PWM_MODE_GET,
    PWM_LIST,
    PWM_DUMP,
    PWM_MENU,
};

//#define PWM_DEBUG_PRINTF
#ifdef PWM_DEBUG_PRINTF
#define pwmDebug(fmt,args...)        printf(fmt,args...)
#else
#define pwmDebug(fmt,args...)
#endif

void rtk_hw_get_divider(int pwm_freq_hz, int duty_max, int *totalcnt,
            unsigned short *M_best, unsigned short *N_best, int clk_src);

/** these function code in driver/pwm/pwm.c **/
int get_pwm_frequency(RTK_PWM_TYPE type, int index);
int set_pwm_frequency(RTK_PWM_TYPE type, int index, int freq);

int get_pwm_duty(RTK_PWM_TYPE type, int index);
int set_pwm_duty(RTK_PWM_TYPE type, int index, int duty);

int get_pwm_enable(RTK_PWM_TYPE type, int index);
int set_pwm_enable(RTK_PWM_TYPE type, int index, int enable);

int get_pwm_sync(RTK_PWM_TYPE type, int index);
int set_pwm_sync(RTK_PWM_TYPE type, int index, int rtk_vsync);

int get_pwm_polarity(RTK_PWM_TYPE type, int index);
int set_pwm_polarity(RTK_PWM_TYPE type, int index, int polarity);

/** these function code in driver/pwm/pwm.c **/

int pwm_init(RTK_PWM_TYPE type, int index, int pwm_freq_hz, int duty_max, int duty, int clk_src,int invert);
int pwm_set(int index, int type, int duty);
int set_pwm_gpio_mode(char *name,int mode, int value);
int get_pwm_gpio_mode(char *name,int *mode, int *value);

void rtk_pwm_pcb_info_init(void);
char* rtk_pwm_gpio_group_get(int group);
char* rtk_pwm_name_get(int index);
int rtk_pwm_list(void);
int rtk_pwm_index_get(RTK_PWM_TYPE type,int i);
int rtk_pwm_info_dump(void);
int rtk_pwm_gpio_mode_check(char *name);
RTK_PWM_TYPE rtk_pwm_type_get(int i);


#endif
