/*=============================================================
 * Copyright (c)      Realtek Semiconductor Corporation, 2016
 *
 * All rights reserved.
 *
 *============================================================*/

/*======================= Description ============================
 *
 * file: 		hdmiHdmi.c
 *
 * author: 	Captain America, Steve Rogers
 * date:
 * version: 	3.0
 *
 *============================================================*/

/*========================Header Files============================*/
#include "hdmi_common.h"
#if 0
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/module.h>
#include <linux/fs.h>		/* everything... */
#include <linux/cdev.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/sched.h>
#include <linux/sched/rt.h>
#include <linux/kthread.h>
#include <linux/string.h>
#include <rbus/stb_reg.h>
#include <rtk_kdriver/RPCDriver.h>
#include <rtk_kdriver/rtk_hdcp_otp_util.h>
#include <mach/rtk_platform.h>
#endif
#include <rbus/timer_reg.h>
#include "hdmi_reg.h"


#include "hdmi_hdcp.h"
#include "hdmi_scdc.h"

#if 0
#include "hdmi_ddc.h"
#include "hdmi_edid.h"
#include "hdmi_hdr.h"
#include "hdmi_info_packet.h"
#endif




/**********************************************************************************************
*
*	Marco or Definitions
*
**********************************************************************************************/
#define CLK_STABLE_CNT			8// 2  [WOSQRTK-8394 -Green transient appears - SONY S570/S470]
#define TMDS_CLOCK_TOLERANCE(port)	MAX(hdmi_rx[port].timing_t.tmds_clk_b>>7, 5)     // 6G tolerance should be larger
//#define NO_INFO_MAX			6
#define NO_DRM_INFO_MAX		6
#define NO_AVI_INFO_MAX		6
#define NO_VSI_INFO_MAX		6
#define NO_SPD_INFO_MAX		25	// 40ms * 25 = 1000 ms
#define NO_SPD_CLEAR_PACKET	0	//if no SPD packet will clear packet
#define NO_AUDIO_INFO_MAX		6
#define HDCP_RESET_CNT			30     //The counter for keeping HDCP 1.4 or 2.2 in no signal state, if timeout, turn on both 1.4 & 2.2


#define _AUDIO_128_TIMES        1
#define _AUDIO_256_TIMES        2


unsigned char hdmi_good_timing_ready = FALSE;//when hdmi get good timing, it will be true

/**********************************************************************************************
*
*	Const Declarations
*
**********************************************************************************************/

const char *colorimetry_name[]  = {

	"HDMI_COLORIMETRY_NOSPECIFIED",
	"HDMI_COLORIMETRY_601",
	"HDMI_COLORIMETRY_709",
	"HDMI_COLORIMETRY_XYYCC601",
	"HDMI_COLORIMETRY_XYYCC709",
	"HDMI_COLORIMETRY_SYCC601",
	"HDMI_COLORIMETRY_ADOBE_YCC601",
	"HDMI_COLORIMETRY_ADOBE_RGB",

};

const char *depth_name[] = {

	"HDMI_COLOR_DEPTH_8B",
	"HDMI_COLOR_DEPTH_10B",
	"HDMI_COLOR_DEPTH_12B",
	"HDMI_COLOR_DEPTH_16B",

};

const char* _hdmi_color_depth_str(unsigned char depth)
{
    return (depth < sizeof(depth_name)/sizeof(char*)) ? depth_name[depth] : "Unknown";
}

const char *colorspace_name[] = {
	"COLOR_RGB",
	"COLOR_YUV422",
	"COLOR_YUV444",
	"COLOR_YUV420",
	"COLOR_UNKNOW"
};

const char* _hdmi_color_space_str(unsigned char color_space)
{
    if (color_space >= sizeof(colorspace_name)/sizeof(char*))
        color_space =  sizeof(colorspace_name)/sizeof(char*)-1;  // force to unknow

    return colorspace_name[color_space];
}

const char *hdmi_3d_name[] = {
	"HDMI3D_FRAME_PACKING",
	"HDMI3D_FIELD_ALTERNATIVE",
	"HDMI3D_LINE_ALTERNATIVE",
	"HDMI3D_SIDE_BY_SIDE_FULL",
	"HDMI3D_L_DEPTH",

	"HDMI3D_L_DEPTH_GPX",
	"HDMI3D_TOP_AND_BOTTOM",
	"HDMI3D_RSV0",
	"HDMI3D_SIDE_BY_SIDE_HALF",
	"HDMI3D_LINE_BY_LINE",

	"HDMI3D_VERTICAL_STRIPE",
	"HDMI3D_CHECKER_BOARD",
	"HDMI3D_REALID",
	"HDMI3D_SENSIO",
	"HDMI3D_RSV1",

	"HDMI3D_2D_ONLY",
};


/**********************************************************************************************
*
*	Variables
*
**********************************************************************************************/
HDMI_PORT_INFO_T	hdmi_rx[HDMI_PORT_TOTAL_NUM];
HDMI_CHANNEL_T		pcb_info[HDMI_PORT_TOTAL_NUM];
HDCP_KEY_T		hdcpkey;
unsigned char 		current_port = 0;

//unsigned int 		hpd_low_ms = 100; //default value (+ resume already had 500ms low)
unsigned int 		hpd_low_ms = HPD_DEFAULT_LOW_MS; //default value (+ resume already had 500ms low)
unsigned int 		hpd_target_ch = 0xff;
//unsigned int 		hpd_low_is_running = 0;

static unsigned int bNeedDebBeforeDisp = 0;
static unsigned int DispDebCnt = 0;
static unsigned int DispDebCondition = 0;


#if HDMI_TOGGLE_HPD_Z0_ENABLE
unsigned char hdmi_toggle_hpd_z0_flag[HDMI_PORT_TOTAL_NUM] = {0};
HDMI_DC_ON_STATE_E hdmi_dc_on_delay_state[HDMI_PORT_TOTAL_NUM] = {HDMI_DC_ON_NONE};
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

#if HDMI_STR_TOGGLE_HPD_ENABLE
HDMI_STR_HPD_TOGGLE_STATE_E hdmi_str_toggle_hpd_state[HDMI_PORT_TOTAL_NUM] = {HDMI_STR_HPD_TOGGLE_NONE};
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE

#if HDMI_DISPLAY_DELAY_ENABLE
unsigned char hdmi_delay_display_enable[HDMI_PORT_TOTAL_NUM] = {_DISABLE};
HDMI_DISPLAY_DELAY_STATE_STATE_E hdmi_delay_display_state[HDMI_PORT_TOTAL_NUM] = {HDMI_DISPLAY_DELAY_NONE};
static unsigned char hdmi_first_display_flag = FALSE;

#endif //#if HDMI_DISPLAY_DELAY_ENABLE

#if 1 //#if HDMI_DISPLAY_DELAY_ENABLE
HDMI_SPECIFIC_DEVICE_E hdmi_specific_device[HDMI_PORT_TOTAL_NUM] = {HDMI_SPECIFIC_DEVICE_NONE};
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

#if HDCP_STATUS_CHECK_ENABLE
HDMI_HDCP_TOGGLE_STATUS_T	hdmi_hdcp_check_toggle_state[HDMI_PORT_TOTAL_NUM] = {HDMI_HDCP_TOGGLE_NONE};
#endif //#if HDCP_STATUS_CHECK_ENABLE

#if 0 //fixed me
extern DOLBY_HDMI_VSIF_T pre_dolby_hdmi_vsif_mode;
#endif

#if HDMI2p1
	//lane_align_state lane_st[4];  //bgrc
#endif
/**********************************************************************************************
*
*	Funtion Declarations
*
**********************************************************************************************/

void newbase_hdmi_edid_init(void);//Need Move to Hdmi_edid.c
void newbase_hdmi_hdcp_init(void);//Need Move to Hdmi_hdcp.c


void newbase_hdmi_fix_green_line(unsigned char nport);
void newbase_hdmi_set_scaler_info(HDMI_TIMING_T *timing);

void newbase_hdmi_hpd_delay_work_func(void);


unsigned char newbase_hdmi_update_vsi(unsigned char port);
unsigned char newbase_hdmi_check_spd_name(unsigned char port, unsigned char *vendor_name);
unsigned char newbase_hdmi_check_spd_prod(unsigned char port, unsigned char *prod_desc);
unsigned char newbase_hdmi_check_aksv(unsigned char port, unsigned char *aksv);

unsigned char newbase_hdmi_compatibility_check(unsigned char port);
unsigned char newbase_hdmi_check_avi_colorimetry_info(unsigned char port, unsigned char ucInit);
unsigned char newbase_hdmi_no_SPD_packetInfo(unsigned char nport);
unsigned char newbase_hdmi_check_deb_condition_beforeDisp(unsigned char nport, unsigned char ucInit);
unsigned char newbase_hdmi_check_hdcp_condition(unsigned char nport, unsigned char ucInit);
unsigned char newbase_check_hdmi_spd_info(unsigned char nport, unsigned char enable);


/**********************************************************************************************
*
*	Mutex/Semaphore
*
**********************************************************************************************/

#if 0

struct semaphore sem_timing_detect;

void _init_timing_detect_sem(void)   { sema_init(&sem_timing_detect, 1); }
void _lock_timing_detect_sem(void)   { down(&sem_timing_detect); }
void _unlock_timing_detect_sem(void) { up(&sem_timing_detect); }

struct semaphore sem_info_pack;

void  _init_info_pack_sem(void)      { sema_init(&sem_info_pack, 1); }
void  _lock_info_pack_sem(void)      { down(&sem_info_pack); }
void  _unlock_info_pack_sem(void)    { up(&sem_info_pack); }

struct semaphore sem_dolby_vsif;

void  _init_dolby_vsif_sem(void)      { sema_init(&sem_dolby_vsif, 1); }
void  _lock_dolby_vsif_sem(void)      { down(&sem_dolby_vsif); }
void  _unlock_dolby_vsif_sem(void)    { up(&sem_dolby_vsif); }


/**********************************************************************************************
*
*	Info Frmae Thread 
*
**********************************************************************************************/


static bool infoframe_flag = false;
static struct task_struct *infoframe_task;
unsigned char ucInfoframe_Thread_Stop = _FALSE;


void hdmi_infoframe(void)
{
#ifdef SUPPORT_DOLBY_VSIF
	newbase_hdmi_update_infoframe(0);
	newbase_hdmi_update_infoframe(1);
	newbase_hdmi_update_infoframe(2);
	newbase_hdmi_update_infoframe(3);
#endif 
}

unsigned char hdmi_get_infoframe_thread_stop(void)
{
#ifdef SUPPORT_DOLBY_VSIF
	return ucInfoframe_Thread_Stop;
#endif 

	return _FALSE;
}

static int hdmi_infoframe_thread(void* arg)
{
	while (!kthread_should_stop() && infoframe_flag) 
	{

#ifdef SUPPORT_DOLBY_VSIF
		_lock_dolby_vsif_sem();
		
		if (_FALSE == hdmi_get_infoframe_thread_stop())
		{
			hdmi_infoframe();
		}
		_unlock_dolby_vsif_sem();
#else
		int i;
		for( i = 0 ; i < HDMI_PORT_TOTAL_NUM; ++i )		
			newbase_hdmi_update_infoframe(i);		  // update info frame for each port
#endif
		msleep(40);        
	}

	return 0;
}

void hdmi_set_infoframe_thread_stop(unsigned char ucStop)
{
	_lock_dolby_vsif_sem();
	ucInfoframe_Thread_Stop = ucStop;
	_unlock_dolby_vsif_sem();
}

void hdmi_set_infoframe_thread_stop_nonseamphore(unsigned char ucStop)// for ISR use
{
	ucInfoframe_Thread_Stop = ucStop;
}


void newbase_hdmi_infoframe_thread_enable(unsigned char on)
{
	if (on) 
	{    	
    		if (infoframe_flag!=true)
		{
			infoframe_task = kthread_create(hdmi_infoframe_thread, NULL, "infoframe_task");
        	        
#ifdef SUPPORT_DOLBY_VSIF
			hdmi_set_infoframe_thread_stop(_FALSE);
#endif 
        
			if (infoframe_task)
			{
				infoframe_flag = true;
				wake_up_process(infoframe_task);
			}
			else
			{
				HDMI_EMG("Unable to start infopack thread \n");
			}
		}
	}
	else
	{
		if (infoframe_flag) 
		{
			if (kthread_stop(infoframe_task)==0)
				HDMI_PRINTF("hdmi info thread stopped\n");			
			else
				HDMI_EMG("hdmi info thread stopped error\n");
			
			infoframe_flag = false;
		}
	}
}


/**********************************************************************************************
*
*	HDP Work Queue
*
**********************************************************************************************/

static struct workqueue_struct *hpdwq = NULL;
static struct delayed_work hpd_delay_work;

void _init_hpd_delay_wq(void *work_queue_func)
{
	if (NULL == hpdwq) 
    {
		hpdwq = create_workqueue("HPD workqueue");
		if (NULL == hpdwq) {
			HDMI_EMG("[%s] ERROR: Create HPD workqueue failed!\n", __func__ );
		} else {
			HDMI_EMG("[%s] Create HPD workqueue successful!\n", __func__ );
			INIT_DELAYED_WORK(&hpd_delay_work, (void *)work_queue_func);
		}
	}
}

int _start_hpd_delay_wq(unsigned int delayms)
{
	int ret = -1;
	if (hpdwq) {
		ret = queue_delayed_work(hpdwq, &hpd_delay_work, msecs_to_jiffies(delayms));
	}
	return ret;
}

void _cancel_hpd_delay_wq(void)
{
	if (hpdwq) {
		cancel_delayed_work(&hpd_delay_work);
		flush_workqueue(hpdwq);
	}
}

void _destroy_hpd_delay_wq(void)
{
	if (hpdwq) {
		cancel_delayed_work(&hpd_delay_work);
		flush_workqueue(hpdwq);
		destroy_workqueue(hpdwq);
		hpdwq = NULL;
	}
}

#endif

/**********************************************************************************************
*
*	Function Body
*
**********************************************************************************************/


HDMI_CHANNEL_T*  newbase_hdmi_get_pcbinfo(unsigned int port)
{
	return (port < HDMI_PORT_TOTAL_NUM) ? &pcb_info[port] : NULL;
}

void newbase_hdmi_save_pcbinfo(unsigned char port, const HDMI_CHANNEL_T_PCB_INFO *pcb)
{
	HDMI_CHANNEL_T* p_ch = newbase_hdmi_get_pcbinfo(port);

	if (p_ch==NULL) {
		HDMI_EMG("%s failed, port %d doesn't exist\n", __FUNCTION__ , port);
		return ;
	}
	
	p_ch->phy_port     = pcb->phy_port;
	p_ch->ddc_selected = pcb->ddc_selected;
	p_ch->edid_type    = pcb->edid_type;
	p_ch->arc_support  = pcb->arc_support;
	p_ch->dedicate_hpd_det  = pcb->dedicate_hpd_det;
	p_ch->dedicate_hpd_det_sel  = pcb->dedicate_hpd_det_sel;
	p_ch->hpd          = pcb->hpd;
	p_ch->det_5v       = pcb->det_5v;
}

unsigned char newbase_hdmi_phy_port_status(unsigned char port)
{
	return (port < HDMI_PORT_TOTAL_NUM) ? pcb_info[port].phy_port : HDMI_PHY_PORT_NOTUSED;
}

int newbase_hdmi_ddc_channel_status(unsigned char port)
{
	return (port < HDMI_PORT_TOTAL_NUM) ? pcb_info[port].ddc_selected : -1;
}

void newbase_hdmi_save_edid(unsigned char port, const unsigned char *edid, const unsigned int len)
{
	HDMI_CHANNEL_T* p_ch = newbase_hdmi_get_pcbinfo(port);
	
	if (p_ch==NULL) {
		HDMI_EMG("%s failed, port %d doesn't exist\n", __FUNCTION__ , port);
		return ;
	}		
	
	memcpy(p_ch->edid_table, edid, len);
	
	p_ch->edid_initialized = 1;
	
	HDMI_EMG("Customer EDID:%d,%d,%d,%d,%d,port=%d\n", 
	    p_ch->edid_table[0], p_ch->edid_table[1], 
	    p_ch->edid_table[2], p_ch->edid_table[128], 
	    p_ch->edid_table[255], port);
}


int newbase_hdmi_edid_initialized(unsigned char port)
{
	HDMI_CHANNEL_T* p_ch = newbase_hdmi_get_pcbinfo(port);
	
	if (p_ch==NULL) {
		HDMI_EMG("%s failed, port %d doesn't exist\n", __FUNCTION__ , port);
		return 0;
	}		
	
	return (p_ch->edid_initialized) ? 1 : 0;	
}


//QC need
#if 0

void newbase_hdmi_init(void)
{
	unsigned char j;

	HDMI_EMG("newbase_hdmi_init (HW Ver: %d)\n", get_ic_version());

//	spin_lock_init(&hdmi_spin_lock);

	hdmi_first_display_flag = TRUE;
	lib_hdmi_sys_crt_on();
	lib_hdmi_crt_reset();
	lib_hdmiddc_crt_on();

//	newbase_hdmi_ddc_ch_remap();

	for (j=0; j<HDMI_PORT_TOTAL_NUM; j++)
	{
		unsigned char ddc_ch = 0 ;

		if (newbase_hdmi_phy_port_status(j)==HDMI_PHY_PORT_NOTUSED) {
			lib_hdmi_mac_crt_perport_off(j);               // disable power of HDMI mac
			lib_hdmi_mac_sram_light_sleep_enable(j, 1);    // enable mac light speep mode (SRAM's power domain is different from mac, power off mac won't power down sram)
			HDMI_INFO("HDMI[p%d] is not used, power off it\n", j);
			continue;
		}

		newbase_hdmi_set_hpd(j, 0);
		lib_hdmi_power(j, 0);

		if (newbase_hdmi_ddc_channel_status(j,&ddc_ch) != 0 )
			continue;

		lib_hdmiddc_sda2gpio(ddc_ch);
		lib_hdmiddc_set_sda_delay(ddc_ch, 0x1); //K2L, one NB no audio, due to IIC issue. Q2879/3094/3222
		lib_hdmiddc_sda_dir_out(ddc_ch, 0x1);  //Q9305: first plug in, no tx_clk in due to IIC issue.
		lib_hdmiddc_gpio2sda(ddc_ch);
	}
	newbase_hdmi_port_var_init_all();
//	newbase_hdmi_flow_cfg_info_init();

	newbase_hdmi_edid_init();
	newbase_hdmi_hdcp_init();

	lib_hdmi_phy_init();
	lib_hdmi21_phy_init();

	_init_timing_detect_sem();

//	newbase_hdmi_power_saving_init();
//	newbase_hdmi_infoframe_rx_init();

	newbase_rxphy_isr_enable(1);
//	newbase_hdmi_infoframe_rx_enable(1);
	newbase_hdmi_audio_thread_enable(1);

	_init_hpd_delay_wq();

	// requeset interrupt handler
#if 0
	if (irq_init==0)
	{
		if (request_irq(IRQ_GET_KERNEL_IRQ_NUM(IRQ_HDMI), newbase_hdmi_irq_handler, IRQF_SHARED, "hdmi", newbase_hdmi_irq_handler))
		{
			HDMI_EMG("!!!!!!!!!!!!!!!!! Register HDMI Interrupt handler failed - IRQ %d !!!!!!!!!!!!!!!!\n", IRQ_HDMI);
		}
		else
		{
			HDMI_EMG("!!!!!!!!!!!!!!!!! Register HDMI Interrupt handler successed - IRQ %d !!!!!!!!!!!!!!!!\n", IRQ_HDMI);
			irq_init=1;
		}
	}
#endif	
}

#else
void newbase_hdmi_init(void)
{
	unsigned char j;

	HDMI_PRINTF("%s\n", __func__);

	//lib_hdmi_crt_on();
	lib_hdmi_sys_crt_on();
	lib_hdmi_crt_reset();	
	lib_hdmiddc_crt_on();
	//lib_hdmi_reg_init();

	for (j=0; j<HDMI_PORT_TOTAL_NUM; j++)
	{
		unsigned int ddc_ch;
		
		if (newbase_hdmi_phy_port_status(j)==HDMI_PHY_PORT_NOTUSED)
			continue;

		newbase_hdmi_set_hpdNew(j, 0, INIT_HPD_OFF);
		lib_hdmi_power(j, 0);
		
		ddc_ch = newbase_hdmi_ddc_channel_status(j);
		if (ddc_ch < 0)
			continue;
		
		lib_hdmiddc_sda2gpio(ddc_ch);
		lib_hdmi_ddc_sel(j, ddc_ch);
		lib_hdmiddc_set_sda_delay(ddc_ch, 0x1); //K2L, one NB no audio, due to IIC issue. Q2879/3094/3222
		lib_hdmiddc_sda_dir_out(ddc_ch, 0x1);  //Q9305: first plug in, no tx_clk in due to IIC issue.
		lib_hdmiddc_gpio2sda(ddc_ch);

		newbase_hdmi_check_hdcp_condition(j, _TRUE);
	}

	newbase_hdmi_edid_init();
	newbase_hdmi_hdcp_init();

	lib_hdmi_phy_init();
	lib_hdmi21_phy_init();

	newbase_hdmi_port_var_init_all();
	
	_init_timing_detect_sem();
	_init_info_pack_sem();	

#ifdef SUPPORT_DOLBY_VSIF
	_init_dolby_vsif_sem();
#endif


	newbase_rxphy_isr_enable(1);
	newbase_hdmi_infoframe_thread_enable(1);
	newbase_hdmi_audio_thread_enable(1);
}
#endif
void newbase_hdmi_uninit(void)
{
	_destroy_hpd_delay_wq();

	newbase_rxphy_isr_enable(0);
	newbase_hdmi_infoframe_thread_enable(0);
	newbase_hdmi_audio_thread_enable(0);
}


//QC need
void newbase_hdmi_init_source(unsigned char port)
{
	if (newbase_hdmi_phy_port_status(port) == HDMI_PHY_PORT_NOTUSED)
	{
		HDMI_EMG("This channel %d is Disable\n", port);
		return;
	}

	HDMI_PRINTF("%s[port%d] INITSRC_HPD_ON=%d\n", __func__, port, INITSRC_HPD_ON);

//0715	lib_hdmi_crt_on();
//0715	#if PCB_HDMI_4K120
//0715	rtd_outl(0xb800d628,0x3fe00000);
//0715	rtd_outl(0xb800d62c,0x3fe00000);
//0715	#endif
	//newbase_hdmi_port_select(port);
	lib_hdmi_power(port, 1);
	lib_hdmi_tmds_init(port);
	lib_hdmi_mac_init(port);	//need to check: move to newbase_drvif_hdmi_Init, ok?

	newbase_hdmi_port_var_init(port);
	newbase_hdmi_audio_close(port);
#if HDMI_STR_HPD_SPEED_UP_ENABLE
	if(0 == lib_hdmi_cec_is_enabled())
		newbase_hdmi_set_hpdNew(port , 1, INITSRC_HPD_ON);  //need to separate AC/DC case, move to resume and init function
#else
	newbase_hdmi_set_hpdNew(port , 1, INITSRC_HPD_ON);  //need to separate AC/DC case, move to resume and init function
#endif//HDMI_STR_HPD_SPEED_UP_ENABLE

	newbase_rxphy_init_struct(port);
	newbase_rxphy_isr_set(port, 1);


	lib_hdmi_gcp_ackg_header_parsing_mode(port, FALSE);

	lib_hdmi_valid_format_condition(port);

	lib_hdmi_misc_variable_initial(port);
}

void newbase_hdmi_release_source(unsigned char port)
{

	if (port >= HDMI_PORT_TOTAL_NUM)  {
		HDMI_EMG("%s: err port=%d\n", __func__, port);
		return;
	}

	newbase_rxphy_isr_set(port, 0);
	lib_hdmi_power(port, 0);

	newbase_hdmi_disconnect_source(port);
	_cancel_hpd_delay_wq();
}

void newbase_hdmi_disconnect_source(unsigned char port)
{	// keep PHY_ISR in active, keep power on

	lib_hdmi_video_output(port, 0);  // blanking screen in step 1.
	newbase_hdmi_port_var_init(port);

	lib_hdmi_video_output(port, 0); //close again, avoid re-open in detect_mode
	newbase_hdmi_audio_close(port);
}

//QC need
void newbase_hdmi_port_select(unsigned char port)	// reset all mac functions
{
	lib_hdmi_mac_select_port(port);
	lib_hdmi_video_output(port, 0);
	newbase_hdmi_set_current_display_port(port);
}

//QC need
void newbase_hdmi_port_var_init_all(void)
{
	unsigned char i;
	for (i=0; i<HDMI_PORT_TOTAL_NUM; i++)
	{
		newbase_hdmi_port_var_init(i);
	}
}

//QC need
void newbase_hdmi_port_var_init(unsigned char port)
{
	memset(&hdmi_rx[port], 0, sizeof(HDMI_PORT_INFO_T));

	SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
	SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_START);

	SET_H_INTERLACE(port, 0);
	SET_H_MODE(port, MODE_HDMI);
	SET_H_COLOR_DEPTH(port, HDMI_COLOR_DEPTH_8B);
	SET_H_COLOR_SPACE(port, COLOR_RGB);

	SET_H_3DFORMAT(port, HDMI3D_2D_ONLY);

	newbase_hdcp_init_timer(port);

	newbase_hdmi_audio_var_init(port);

}

//USER:LewisLee DATE:2017/12/19
//to save the value that don't need to reset
void newbase_hdmi_port_var_init_skip(unsigned char port, HDMI_RX_VAR_INIT_SKIP Skip_Case, unsigned char ucRecover)
{
	static UINT8 conn_st = 0;

	switch(Skip_Case)
	{
		case HDMI_RX_VAR_INIT_SKIP_CASE1:
			if(_TRUE == ucRecover)
			{
				//Recover latest value
				hdmi_rx[port].cable_conn = conn_st;
			}
			else //if(_FALSE == ucRecover)
			{
				//Remember latest value
				conn_st = hdmi_rx[port].cable_conn;
			}
		break;

		default:
		break;
	}
}

//QC need
#if 1 //Need Move to Hdmi_edid.c
void newbase_hdmi_edid_init(void)
{
	unsigned char j;

	for (j=0; j<HDMI_PORT_TOTAL_NUM; j++)
	{
		HDMI_CHANNEL_T* p_ch = newbase_hdmi_get_pcbinfo(j);

		if (p_ch == NULL || p_ch->phy_port == HDMI_PHY_PORT_NOTUSED)  
		{
			HDMI_EMG("%s: init port %d failed - port not used, skip it\n", __func__, j);
			continue;
		}

		if (p_ch->edid_initialized)
		{
			HDMI_EMG("[HDMI] vfe_hdmi_drv_write_edid_apply Write Customer EDID that before vfe_hdmi_drv_init\n");
		}
		
		HDMI_EMG("edid_init EDID:%d,%d,%d,%d,%d,Port=%d\n",
			p_ch->edid_table[0], p_ch->edid_table[1], p_ch->edid_table[2],
			p_ch->edid_table[128], p_ch->edid_table[255], j);

		newbase_hdmi_write_edid(p_ch->ddc_selected, p_ch->edid_table, 256);

		if (p_ch->edid_type == HDMI_EDID_ONCHIP)
			newbase_hdmi_edid_enable(p_ch->ddc_selected, 1);
	}
}
#endif
//QC need
#if 1 //Need Move to Hdmi_hdcp.c
void newbase_hdmi_hdcp_init(void)
{
	unsigned char j;
#if defined(PROJECT_LGE)
#if defined(CONFIG_RTK_KDRV_HDCP_KEY_FROM_OTP)
	unsigned char hdcp14_buf[325];
	if (get_hdcp_14_otp(hdcp14_buf, 325) == 0) {
		HDMI_PRINTF("[HDCP] HDCP1.4 OTP-KEY  OK\n");
		memcpy(hdcpkey.BKsv , hdcp14_buf,5);
		memcpy(hdcpkey.Key , hdcp14_buf+5, 320);
	} else {
		HDMI_EMG("[HDCP] NO HDCP1.4 OTP-KEY\n");
	}
#else
	HDMI_EMG("[HDCP] NO HDCP1.4 OTP-KEY (UNDEFINED)\n");
#endif
#endif

	for (j=0; j<HDMI_PORT_TOTAL_NUM; j++) {
		if (newbase_hdmi_phy_port_status(j)==HDMI_PHY_PORT_NOTUSED)
			continue;

		lib_hdmi_hdcp_init(j);
	}

	newbase_hdmi_write_hdcp14_key(hdcpkey.BKsv, hdcpkey.Key);

	

#if HDCP2_2
	for (j=0; j<HDMI_PORT_TOTAL_NUM; j++) {
		if (newbase_hdmi_phy_port_status(j) == HDMI_PHY_PORT_NOTUSED)
			continue;

		lib_hdmi_hdcp22_init(j);
	}
#endif
}
#endif
const char* _hdmi_video_fsm_str(unsigned int fsm)
{
    switch (fsm) {
    case MAIN_FSM_HDMI_WAIT_SYNC:       return "WAIT_SYNC";
    case MAIN_FSM_HDMI_SETUP_VEDIO_PLL: return "SETUP_VEDIO_PLL";
    case MAIN_FSM_HDMI_MEASURE:         return "MEASURE";		
    case MAIN_FSM_HDMI_DISPLAY_ON:      return "DISPLAY_ON";					
    case MAIN_FSM_HDMI_VIDEO_READY:     return "VIDEO_READY";			
    default:
    return "UNKNOW";	    
    }
}
#if 1 //Need Move to Hdmi_audio.c
const char* _hdmi_audio_fsm_str(unsigned int fsm)
{
    switch (fsm) {
    case AUDIO_FSM_AUDIO_START:          return "START";
    case AUDIO_FSM_FREQ_DETECT:          return "DETECT";
    case AUDIO_FSM_AUDIO_WAIT_TO_START:  return "WAIT TO START";
    case AUDIO_FSM_AUDIO_START_OUT:      return "START OUT";
    case AUDIO_FSM_AUDIO_WAIT_PLL_READY: return "WAIT PLL READY";
    case AUDIO_FSM_AUDIO_CHECK:          return "CHECK";
    default:                             return "UNKNOW";
    }
}
#endif
unsigned char newbase_hdmi_detect_mode(unsigned char port)
{
	HDMIRX_PHY_STRUCT_T* p_phy_st;
	p_phy_st = newbase_rxphy_get_status(port);

	if (port >= HDMI_PORT_TOTAL_NUM) {
		return _MODE_NOSIGNAL;
	}

	if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_26_BYPASS_DETECT) {
		return _MODE_NOSIGNAL;
	}

	if (GET_H_VIDEO_FSM(port) > MAIN_FSM_HDMI_SETUP_VEDIO_PLL) {
		if (newbase_hdmi_check_condition_change(port)) {
			SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
		}
	}
	
#if HDMI2p0
	// only enable SCDC CED when signal is stable
	lib_hdmi_scdc_ced_enable(port, (GET_H_VIDEO_FSM(port)==MAIN_FSM_HDMI_VIDEO_READY) ? 1 : 0); 
#endif

	switch (GET_H_VIDEO_FSM(port)) {
		case MAIN_FSM_HDMI_WAIT_SYNC:
			if(hdmi_good_timing_ready)
				hdmi_good_timing_ready = FALSE;
			if (newbase_hdmi_wait_sync(port, p_phy_st->frl_mode)) {
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_SETUP_VEDIO_PLL);
			} else {
				return _MODE_NOSIGNAL;
			}
		break;

		case MAIN_FSM_HDMI_SETUP_VEDIO_PLL:
			if (newbase_hdmi_setup_video_pll(port, p_phy_st->frl_mode)) {
				newbase_hdmi_check_deb_condition_beforeDisp(port, _TRUE);
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_MEASURE);
			} else {
				return _MODE_NOSIGNAL;
			}
		break;

		case MAIN_FSM_HDMI_MEASURE:
			if (newbase_hdmi_measure(port, p_phy_st->frl_mode)) {
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_DISPLAY_ON);		
			} else {
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
				return _MODE_DETECT;
			}
		break;

		case MAIN_FSM_HDMI_DISPLAY_ON:
			if (newbase_hdmi_display_on(port)) {
				if (lib_hdmi_get_avmute(port) && !lib_hdmi_get_bch_2bit_error(port)) {

					HDMI_PRINTF("[WARNING] AVMUTE...\n");
					//USER:willychou DATE:2016/08/30
					//to prevent [WOSQRTK-8372]APPLE TV BOX display AV mute timing 1920X 1084 P
					SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_SETUP_VEDIO_PLL);
					return _MODE_NOSIGNAL;
				}

				if (newbase_hdmi_check_condition_change(port)) {
					SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
					return _MODE_NOSIGNAL;
				}

				if (!newbase_hdmi_compatibility_check(port)) {
					SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_SETUP_VEDIO_PLL);
					return _MODE_DETECT;
				}

				lib_hdmi_char_err_rst(port, 0);
				HDMI_PRINTF("FSM_HDMI_DISPLAY_ON[%d] OK\n", port);
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_VIDEO_READY);
				if(!hdmi_good_timing_ready)
					hdmi_good_timing_ready = TRUE;
				return _MODE_SUCCESS;
			} else {
				hdmi_good_timing_ready = FALSE;
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
				return _MODE_DETECT;
			}
		break;

		case MAIN_FSM_HDMI_VIDEO_READY:
			if (newbase_hdmi_video_ready(port)) {
				// ...
				if(!hdmi_good_timing_ready)
					hdmi_good_timing_ready = TRUE;
				return _MODE_SUCCESS;
			} else {
				hdmi_good_timing_ready = FALSE;
				SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
				return _MODE_NOSIGNAL;
			}
		break;

		default:
		break;
	}

	return _MODE_NOSIGNAL;
}

unsigned char newbase_hdmi_check_condition_change(unsigned char port)
{
unsigned int phy_clk;

	if (!newbase_rxphy_is_clk_stable(port)) {
		return TRUE;
	}

	phy_clk = newbase_rxphy_get_clk(port);

	if (phy_clk < VALID_MIN_CLOCK) {
		HDMI_EMG("invalid clock\n");
		return TRUE;
	}

	if (HDMI_ABS(phy_clk, hdmi_rx[port].timing_t.tmds_clk_b) > TMDS_CLOCK_TOLERANCE(port)) {
		HDMI_EMG("clock change\n");
		return TRUE;
	}

	if (MODE_HDMI == GET_H_MODE(port)) {
		if (GET_H_COLOR_SPACE(port) != newbase_hdmi_get_colorspace_reg(port)) {
			HDMI_EMG("colorspace chg from %d to %d\n", GET_H_COLOR_SPACE(port), newbase_hdmi_get_colorspace_reg(port));
			return TRUE;
		}

		if (GET_H_COLOR_DEPTH(port) != lib_hdmi_get_color_depth(port)) {
			HDMI_EMG("color depth chage\n");
			return TRUE;
		}
	}

	if (GET_H_MODE(port) != newbase_hdmi_get_hdmi_mode_reg(port)) {
		HDMI_EMG("HDMI/DVI chg from %d to %d\n", GET_H_MODE(port), (!GET_H_MODE(port)));
		return TRUE;
	}

	return FALSE;
}

unsigned char newbase_hdmi_wait_sync(unsigned char port,unsigned char frl_mode)
{
	unsigned int curr_clk;

#if HDMI_TOGGLE_HPD_Z0_ENABLE
	lib_hdmi_toggle_hpd_z0_check(port, _FALSE);
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

	curr_clk = newbase_rxphy_get_clk(port);
	if ((curr_clk < VALID_MIN_CLOCK) ||
		(HDMI_ABS(hdmi_rx[port].timing_t.tmds_clk_b, curr_clk) > TMDS_CLOCK_TOLERANCE(port))) {
		// Set false if clock is invalid or changed
		hdmi_rx[port].timing_t.tmds_clk_b = curr_clk;
		hdmi_rx[port].timing_t.tmds_clk_stable_cnt = 0;
		if (hdmi_rx[port].timing_t.is_interlace) {
			lib_hdmi_clear_interlace_reg(port);
			hdmi_rx[port].timing_t.is_interlace = 0;
		}
		return FALSE;
	}

	if (hdmi_rx[port].timing_t.tmds_clk_stable_cnt < CLK_STABLE_CNT) {
		// Set false if clock is unstable
		hdmi_rx[port].timing_t.tmds_clk_stable_cnt++;
		return FALSE;
	}
	hdmi_rx[port].timing_t.tmds_clk_b = curr_clk;

	if (!newbase_rxphy_get_setphy_done(port)) {
		//HDMI_PRINTF("Wait for set_phy(p%d)\n", port);
		return FALSE;
	}

	newbase_hdmi_open_err_detect(port);
	lib_hdmi_set_repeat_auto(port);
	lib_hdmi_set_colorspace_auto(port);

	lib_hdmi_bch_error_clr(port);

	newbase_hdmi_audio_var_init(port);
	newbase_hdmi_reset_all_infoframe(port);	

#ifdef HDMI_AVI_INFO_COLORIMETRY_ENABLE
	newbase_hdmi_check_avi_colorimetry_info(port, _TRUE);
#endif //#ifdef HDMI_AVI_INFO_COLORIMETRY_ENABLE

	return TRUE;
}

//QC need
unsigned char newbase_hdmi_setup_video_pll(unsigned char port, unsigned char frl_mode)
{
	HDMI_PRINTF("FSM_HDMI_SETUP_VEDIO_PLL[%d]\n", port);

	//HDMIRX_PHY_STRUCT_T* p_phy_st;

	//HDMI_PRINTF("FSM_HDMI_SETUP_VEDIO_PLL[%d][frl_mode:(%s)%d][hdmi21en:%d]\n", port, _hdmi_hd21_frl_mode_str(frl_mode), frl_mode, hdmi_rx[port].hdmi_2p1_en);

	newbase_hdmi_port_select(port);

	SET_H_COLOR_DEPTH(port, (HDMI_COLOR_DEPTH_T)lib_hdmi_get_color_depth(port));
	SET_H_MODE(port, (HDMI_DVI_MODE_T)newbase_hdmi_get_hdmi_mode_reg(port));
	SET_H_PIXEL_REPEAT(port, lib_hdmi_get_pixelrepeat(port));

	if (GET_H_MODE(port) == MODE_DVI) {
		lib_hdmi_set_repeat_manual(port, 0);
		lib_hdmi_set_colorspace_manual(port, 0);
	} else {
		lib_hdmi_set_repeat_auto(port);
		lib_hdmi_set_colorspace_auto(port);
	}

	SET_H_COLOR_SPACE(port, (HDMI_COLOR_SPACE_T)newbase_hdmi_get_colorspace_reg(port));
	SET_H_INTERLACE(port, lib_hdmi_get_is_interlace(port, HDMI_MS_DIRECT));
	lib_hdmi_clear_interlace_reg(port);

	HDMI_INFO("[setup_video_pll] port [%d], bIsHDMIDVI: %d, PixelRepeat: %d, ColorDepth: %s, ColorSpace: %s, BCH error (1bit,2bit) = (%d,%d)\n",
		port,
		GET_H_MODE(port),
		GET_H_PIXEL_REPEAT(port),
		GET_H_COLOR_DEPTH(port) < (sizeof(depth_name)/4)  ? depth_name[GET_H_COLOR_DEPTH(port)] : "UNDEFINED",
		GET_H_COLOR_SPACE(port) < (sizeof(colorspace_name)/4)  ? colorspace_name[GET_H_COLOR_SPACE(port)] : "UNDEFINED",
		hdmi_rx[port].bch_err_detect,
		hdmi_rx[port].bch_err_cnt_continue);


	//p_phy_st = newbase_rxphy_get_status(port);
	
	//HDMI_INFO("[setup_video_pll] port =%d, p_phy_st->frl_mode =%d\n", port, frl_mode);

	if (!lib_hdmi_set_video_pll(port,
		hdmi_rx[port].timing_t.tmds_clk_b,
		frl_mode,
		hdmi_rx[port].timing_t.colordepth,
		hdmi_rx[port].timing_t.is_interlace,
		hdmi_rx[port].timing_t.colorspace==COLOR_YUV420))
		return FALSE;

	return TRUE;
}

unsigned char newbase_hdmi_measure(unsigned char port, unsigned char frl_mode)
{
	HDMI_PRINTF("FSM_HDMI_MEASURE[12.01][%d]\n", port);

	//lib_hdmi_meas_select_port(port,MODE_TMDS);
	lib_hdmi_meas_select_port(port);
	lib_hdmi_video_output(port, 1);
	lib_hdmi_crc_check(port);

	if (!newbase_hdmims_measure_video(port, &hdmi_rx[port].timing_t, frl_mode)) {
		hdmi_rx[port].meas_retry_cnt++;
		if (hdmi_rx[port].meas_retry_cnt > MEAS_RETRY_LIMIT) {
			newbase_rxphy_reset_setphy_proc(port);

#if HDMI2p0
			//USER:LewisLee DATE:2016/11/22
			//to prevent Tx unstable, pixel clock unstable, but large than VALID_MIN_CLOCK
			//it will let toggle function NG
			//pixel clock stable + measure NG
			newbase_hdmi2p0_inc_scdc_toggle(port);
#endif

#if HDMI_STR_TOGGLE_HPD_ENABLE
			if(HDMI_STR_HPD_TOGGLE_READY == lib_hdmi_get_str_toggle_hpd_state(port))
			{
				newbase_hdmi_set_hpd(port , 0, INIT_HPD_OFF);
				msleep(HDMI_STR_TOGGLE_HPD_MS);
				HDMI_EMG("STR on HPD Toggle\n");
				newbase_hdmi_set_hpd(port , 1, INIT_HPD_OFF);
				lib_hdmi_set_str_toggle_hpd_state(port, HDMI_STR_HPD_TOGGLE_NONE);
			}
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE
		}
		return FALSE;
	}

	newbase_hdmi_check_crc_0(port);

	return TRUE;
}

unsigned char newbase_hdmi_display_on(unsigned char port)
{
	//HDMI_PRINTF("FSM_HDMI_DISPLAY_ON[%d]\n", port);

	SET_MODE_IHCOUNT(hdmi_rx[port].timing_t.h_total);
	SET_MODE_IVCOUNT(hdmi_rx[port].timing_t.v_total);
	SET_MODE_IHFREQ(hdmi_rx[port].timing_t.h_freq);
	SET_MODE_IVFREQ(hdmi_rx[port].timing_t.v_freq);

	hdmi_rx[port].timing_t.modetable_index = Hdmi_Search_Mode(&hdmi_rx[port].timing_t.mode_id);

	if (GET_H_COLOR_SPACE(port) != newbase_hdmi_get_colorspace_reg(port)) {
		HDMI_EMG("D-CS chg from %d to %d\n", GET_H_COLOR_SPACE(port), newbase_hdmi_get_colorspace_reg(port));
		return FALSE;
	}
	if (GET_H_COLOR_DEPTH(port) != lib_hdmi_get_color_depth(port)) {
		HDMI_EMG("D-CD chg from %d to %d\n", GET_H_COLOR_DEPTH(port), lib_hdmi_get_color_depth(port));
		return FALSE;
	}
	if (GET_H_PIXEL_REPEAT(port) != lib_hdmi_get_pixelrepeat(port)) {
		HDMI_EMG("D-PR chg from %d to %d\n", GET_H_PIXEL_REPEAT(port), lib_hdmi_get_pixelrepeat(port));
		return FALSE;
	}
	if (GET_H_INTERLACE(port) != lib_hdmi_get_is_interlace(port, HDMI_MS_DIRECT)) {
		HDMI_EMG("D-IP chg from %d to %d\n", GET_H_INTERLACE(port), (!GET_H_INTERLACE(port)));
		return FALSE;
	}

	newbase_hdmi_set_scaler_info(&hdmi_rx[port].timing_t);

	newbase_hdmi_fix_green_line(port);

#if HDR10_MD_INFO_CHECK
#ifdef CONFIG_FORCE_RUN_I3DDMA
	if (newbase_hdmi_is_dispsrc(SLR_MAIN_DISPLAY))
#endif
		newbase_hdmi_dv_hdr_enable(_ENABLE);//Minchay@20160607-a add dolby version hdr
#endif

	if (GET_H_MODE(port)) {
		lib_hdmi_audio_set_thread_run(port, _TRUE);

		SET_H_VIDEO_FORMAT(port, (HDMI_HVF_E)newbase_hdmi_get_video_format_reg(port));
		SET_H_3DFORMAT(port, (HDMI_3D_T)newbase_hdmi_get_3d_structure_reg(port));
	}

	HDMI_PRINTF("bHDMIColorSpace = %s\n", GET_H_COLOR_SPACE(port) < (sizeof(colorspace_name)/4)  ? colorspace_name[GET_H_COLOR_SPACE(port)] : "UNDEFINED");
	HDMI_PRINTF("IsInterlaced = %d\n", GET_H_INTERLACE(port));
	HDMI_PRINTF("bIsHDMIDVI = %d\n", GET_H_MODE(port));
	HDMI_PRINTF("PixelRepeat = %d\n", GET_H_PIXEL_REPEAT(port));
	HDMI_PRINTF("ColorDepth = %s\n", GET_H_COLOR_DEPTH(port) < (sizeof(depth_name)/4)  ? depth_name[GET_H_COLOR_DEPTH(port)] : "UNDEFINED");
	HDMI_PRINTF("ColorMetry = %s\n", newbase_hdmi_get_colorimetry(port) < (sizeof(colorimetry_name)/4)  ? colorimetry_name[newbase_hdmi_get_colorimetry(port)]: "UNDEFINED");
	HDMI_PRINTF("3D Format  = %s\n", GET_H_3DFORMAT(port) < (sizeof(hdmi_3d_name)/4) ? hdmi_3d_name[GET_H_3DFORMAT(port)] : "UNDEFINED");

	return TRUE;
}

unsigned char newbase_hdmi_video_ready(unsigned char port)
{
	//USER:LewisLee DATE:2016/08/14
	//to prevent picture have short time garbage before display
	if (lib_hdmi_get_avmute(port) && !lib_hdmi_get_bch_2bit_error(port)) {
		HDMI_PRINTF("[WARNING] AVMUTE 2...\n");
		SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
		return FALSE;
	}

	return TRUE;
}


void newbase_hdmi_set_info(unsigned char port){
	newbase_hdmi_set_scaler_info(&hdmi_rx[port].timing_t);
}

unsigned char newbase_hdmi_check_mode(void)
{
unsigned char port = current_port;
	unsigned int r_err = 0, g_err = 0, b_err = 0;

	if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_25_BYPASS_CHECK) {
		return TRUE;
	}

	if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_17_DUMP_REG) {
		lib_hdmi_dbg_reg_dump(port);
	}

	// KTASKWBS-5703
	// Send Avmute( once or 150ms )but no timing change when  KT STB play HDR video play again
	if (lib_hdmi_get_avmute(port) && (!lib_hdmi_get_bch_2bit_error(port))) {
		lib_hdmi_video_output(port, 0);
		lib_hdmi_audio_output(port, 0);
		HDMI_EMG("check_mode: [WARNING] AVMUTE 3...\n");
		//SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);
		return FALSE;
	}

	if (GET_H_VIDEO_FSM(port) != MAIN_FSM_HDMI_VIDEO_READY) {
		HDMI_EMG("check_mode: changed_1(p=%d)\n", port);
		return FALSE;
	}

	if (newbase_hdmi_check_condition_change(port)) {
		lib_hdmi_video_output(port, 0);
		lib_hdmi_audio_output(port, 0);
		HDMI_EMG("check_mode: changed_2(p=%d)\n", port);
		return FALSE;
	}

#if HDMI_INT_PRO_CHG_FLAG_ENABLE
	if(TRUE == lib_hdmi_get_is_interlace_change(port))
	{
		HDMI_EMG("interlace chg (p=%d)\n", port);
		return FALSE;
	}
#else //#if HDMI_INT_PRO_CHG_FLAG_ENABLE
	if (GET_H_INTERLACE(port) != newbase_hdmi_get_interlace_conti_mode(port)) {
		HDMI_EMG("check_mode: interlace chg from %d to %d\n", GET_H_INTERLACE(port), (!GET_H_INTERLACE(port)));
		return FALSE;
	}
#endif //#if HDMI_INT_PRO_CHG_FLAG_ENABLE

		if (newbase_hdmi_char_err_detection(port,&r_err,&g_err,&b_err) && (!newbase_hdmi_err_detect_stop(port)))

#if BITERR_PATCH
	if (!(hdmi_in(DEBUG_REG_ADDR) & DEBUG_23_BITERR_DET)) {
		if (hdmi_rx[port].timing_t.tmds_clk_b > 2900) {	// > 3G
			//if (newbase_hdmi_char_err_detection(port)) {
			if (newbase_hdmi_char_err_detection(port,&r_err,&g_err,&b_err) && (!newbase_hdmi_err_detect_stop(port))){
				if (newbase_hdmi_is_long_cable(port)) {
					newbase_hdmi_dfe_recovery_6g_long_cable(port);
				}
			}
		} else if (hdmi_rx[port].timing_t.tmds_clk_b > 1042) {	// > 1.1G

			//if (newbase_hdmi_bit_err_detection(port)) {
		      if (newbase_hdmi_bit_err_detection(port,&r_err,&g_err,&b_err) && (!newbase_hdmi_err_detect_stop(port))){

			//	newbase_hdmi_dfe_recovery_mi_speed(port);
			}
		}
		else if (hdmi_rx[port].timing_t.tmds_clk_b > 430) {  // > 450M~1.1G
			//if (newbase_hdmi_bit_err_detection(port)) {
		      if (newbase_hdmi_bit_err_detection(port,&r_err,&g_err,&b_err) && (!newbase_hdmi_err_detect_stop(port))){

				newbase_hdmi_dfe_recovery_midband_long_cable(port);
			}
		}
	}
#endif

#if HDR10_MD_INFO_CHECK
	//Minchay@20160607-a add hdr10 md check
	if (!newbase_hdmi_hdr10_md_info_check()) {
		HDMI_PRINTF("checkmode: [HDR10] fail\n");
		return FALSE;
	}
#endif

#if HDCP_STATUS_CHECK_ENABLE
	if(_FALSE == lib_hdmi_hdcp_status_check(port, _FALSE))
		return FALSE;
#endif //#if HDCP_STATUS_CHECK_ENABLE

#if BIST_PHY_SCAN
if ((hdmi_in(DEBUG_REG_ADDR) & DEBUG_22_PHY_SCAN_TEST)) {
	if (!newbase_get_phy_scan_done(port)) {
		HDMI_PRINTF("########################################OK, scan contine !!!");
		return FALSE;
	}
}
#endif

	return TRUE;
}

void newbase_hdmi_reset_video_state(unsigned char port)
{
	SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_WAIT_SYNC);

	SET_H_INTERLACE(port, 0);
	SET_H_MODE(port, MODE_HDMI);
	SET_H_COLOR_DEPTH(port, HDMI_COLOR_DEPTH_8B);
	SET_H_COLOR_SPACE(port, COLOR_RGB);

	SET_H_3DFORMAT(port, HDMI3D_2D_ONLY);
}

unsigned char newbase_hdmi_get_video_state(unsigned char port)
{
	return GET_H_VIDEO_FSM(port);
}

void newbase_hdmi_reset_meas_counter(unsigned char port)
{
	hdmi_rx[port].meas_retry_cnt = 0;
}

void newbase_hdmi_check_crc_0(unsigned char port)
{
	if (lib_hdmi_read_crc(port)) {
		return;
	}

	HDMI_EMG("CRC=00000000000, cs=%d\n", GET_H_COLOR_SPACE(port));

	if (GET_H_COLOR_SPACE(port) == COLOR_YUV420) {
//		hdmi_mask(DEBUG_REG_ADDR, ~DEBUG_27_BYPASS_PHY_ISR, DEBUG_27_BYPASS_PHY_ISR);
//		while (lib_hdmi_read_crc(port) == 0) {
//			msleep(10);
//		}
//		hdmi_mask(DEBUG_REG_ADDR, ~DEBUG_27_BYPASS_PHY_ISR, 0);

		lib_hdmi_420_en(port, 0);
		msleep(20);
		lib_hdmi_420_en(port, 1);
	}
}
#if 1//Need Move to Hdmi_hdcp.c
void newbase_hdcp_init_timer(unsigned char port)
{
#if HDCP2_2 && HDCP22_PATCH_DEFAULT_EN
	hdmi_rx[port].hdcp_init_period = HDCP_RESET_CNT;
#endif
}


void lib_hdmi_hdcp14_clear_ri(unsigned char nport)
{
    hdmi_out(HDMI_STB_HDCP_CR_reg, hdmi_in(HDMI_STB_HDCP_CR_reg) | HDMI_STB_HDCP_CR_clr_ri_mask); // clear Ri
}


void newbase_hdcp_handler(unsigned char port)
{
	unsigned char aksv_rcv = lib_hdcp14_check_aksv_reg(port);
	//unsigned char tx_rd_bksv = lib_hdcp14_check_bksv_reg(port);

	if (aksv_rcv) { //Tx write Aksv
		lib_hdmi_hdcp14_enable(port, 1);
#if HDCP2_2
		lib_hdmi_hdcp22_enable(port, 0);
#endif

		if (aksv_rcv) {
			lib_hdcp14_clear_aksv_reg(port);
		}

		newbase_hdcp_init_timer(port);
		hdmi_rx[port].hdcp_authing = HDCP14;
		hdmi_rx[port].tx_aksv[0] = lib_hdmi_hdcp_port_read(port, 0x10);
		hdmi_rx[port].tx_aksv[1] = lib_hdmi_hdcp_port_read(port, 0x11);
		hdmi_rx[port].tx_aksv[2] = lib_hdmi_hdcp_port_read(port, 0x12);
		hdmi_rx[port].tx_aksv[3] = lib_hdmi_hdcp_port_read(port, 0x13);
		hdmi_rx[port].tx_aksv[4] = lib_hdmi_hdcp_port_read(port, 0x14);

		HDMI_PRINTF("\n[HDCP] AKSV Flag = %x, %x ,%x ,%x, %x\n", hdmi_rx[port].tx_aksv[0], hdmi_rx[port].tx_aksv[1], hdmi_rx[port].tx_aksv[2], hdmi_rx[port].tx_aksv[3], hdmi_rx[port].tx_aksv[4]);
//		HDMI_PRINTF("\nHDMI An = %x  %x %x  %x  %x %x %x  %x\n",
//			lib_hdmi_hdcp_port_read(port, 0x18),
//			lib_hdmi_hdcp_port_read(port, 0x19),
//			lib_hdmi_hdcp_port_read(port, 0x1a),
//			lib_hdmi_hdcp_port_read(port, 0x1b),
//			lib_hdmi_hdcp_port_read(port, 0x1c),
//			lib_hdmi_hdcp_port_read(port, 0x1d),
//			lib_hdmi_hdcp_port_read(port, 0x1e),
//			lib_hdmi_hdcp_port_read(port, 0x1f));
	}

#if HDCP2_2 && HDCP22_PATCH_DEFAULT_EN
    
	// No HDCP1.4 and No clock for a while, then reset to HDCP2.2
	if (newbase_rxphy_get_clk(port) < VALID_MIN_CLOCK) 
	{
		if (hdmi_rx[port].hdcp_init_period) {
			hdmi_rx[port].hdcp_init_period--;
		} else {

			if (!lib_hdmi_hdcp22_is_onoff(port) || !lib_hdmi_hdcp14_is_onoff(port)) {
				lib_hdmi_hdcp22_enable(port, 0); // First, connect MSHG-800/HDCP2.2, then plug out cable to QD882, HDCP CTS fail.
				lib_hdmi_hdcp14_enable(port, 1);
				lib_hdmi_hdcp22_enable(port, 1);
				lib_hdmi_hdcp14_clear_ri(port);   // [RL6557-575] clear ri when no clock detected
				hdmi_rx[port].hdcp_authing = NO_HDCP;
				lib_hdmi_hdcp_port_write(port, 0xC4, 0x0);   //[WOSQRTK-9185]SKbox transient garbage
				HDMI_PRINTF("[HDCP] reset to ALL TURN ON\n");
			}
		}
	}
#endif
#if HDCP2_2
	if (lib_hdmi_hdcp22_get_auth_done(port)) {
		hdmi_rx[port].hdcp_authing = HDCP22;
	}
#endif

}

unsigned char newbase_hdcp_get_auth_mode(unsigned char port)
{
	return hdmi_rx[port].hdcp_authing;
}
#endif

#if HDMI2p0
//QC need
void newbase_hdmi2p0_check_tmds_config(unsigned char port)
{
	unsigned char cfg, hail_hydra;
	
	lib_hdmi_scdc_data_sel(port,0); // sel hdmi2p0

	cfg = lib_hdmi_scdc_get_tmds_config(port);

	hail_hydra = (hdmi_rx[port].scramble_flag|hdmi_rx[port].clock40x_flag);

	if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_24_FORCE_6G) {
		#if USE_H5X_PTG
		hdmi_rx[port].scramble_flag = 1;
		#else
		hdmi_rx[port].scramble_flag = 0;
		#endif
		hdmi_rx[port].clock40x_flag = 1;
	} else {
		if (hdmi_rx[port].hdmi2p0_tmds_toggle_flag > 10) {
			hdmi_rx[port].scramble_flag = ((cfg&_BIT0) != 0);
			hdmi_rx[port].clock40x_flag = ((cfg&_BIT1) != 0);
		} else if (hdmi_rx[port].hdmi2p0_tmds_toggle_flag > 2) {
			hdmi_rx[port].scramble_flag = ((cfg&_BIT0) == 0);
			hdmi_rx[port].clock40x_flag = ((cfg&_BIT1) == 0);
			//HDMI_PRINTF("HDMI2.0 swap\n");
		} else {
			hdmi_rx[port].scramble_flag = ((cfg&_BIT0) != 0);
			hdmi_rx[port].clock40x_flag = ((cfg&_BIT1) != 0);
		}
	}

	if (!hail_hydra && (hdmi_rx[port].scramble_flag||hdmi_rx[port].clock40x_flag)) {
		HDMI_PRINTF("HDMI2.0 SCDC: got it(p%d)\n", port);
	}

}

//QC need
unsigned char newbase_hdmi2p0_get_scramble_flag(unsigned char port)
{
	return hdmi_rx[port].scramble_flag;
}

//QC need
unsigned char newbase_hdmi2p0_get_clock40x_flag(unsigned char port)
{
	return hdmi_rx[port].clock40x_flag;
}

//QC need
void newbase_hdmi2p0_detect_config(unsigned char port)
{
	lib_hdmi2p0_scramble_enable(port, hdmi_rx[port].scramble_flag);

}

void newbase_hdmi2p0_reset_scdc_toggle(unsigned char port)
{
	hdmi_rx[port].hdmi2p0_tmds_toggle_flag = 0;
}

void newbase_hdmi2p0_inc_scdc_toggle(unsigned char port)
{
	if(hdmi_rx[port].hdmi2p0_tmds_toggle_flag >= 0xfe)
		hdmi_rx[port].hdmi2p0_tmds_toggle_flag = 0;
	else
		hdmi_rx[port].hdmi2p0_tmds_toggle_flag++;
}


#endif


void newbase_hdmi_set_current_display_port(unsigned char port)
{
	current_port = port;
}

unsigned char newbase_hdmi_get_current_display_port(void)
{
	return current_port;
}



unsigned char newbase_hdmi_get_timing(unsigned char port, HDMI_TIMING_T *ptiming)
{
	_lock_timing_detect_sem();

	if (GET_H_VIDEO_FSM(port) == MAIN_FSM_HDMI_VIDEO_READY) {
		memcpy(ptiming, &hdmi_rx[port].timing_t, sizeof(HDMI_TIMING_T));
		_unlock_timing_detect_sem();
		return TRUE;
	} else {
		memset(ptiming, 0, sizeof(HDMI_TIMING_T));
		_unlock_timing_detect_sem();
		return FALSE;
	}

	return FALSE;
}

unsigned char newbase_hdmi_get_hvstart(unsigned char port, HDMI_TIMING_T *ptiming, unsigned char vsc_ch)
{
	if (GET_H_VIDEO_FSM(port) == MAIN_FSM_HDMI_VIDEO_READY) {
		lib_hdmims_get_hvstart(ptiming, vsc_ch);
		return TRUE;
	} else {
		ptiming->h_act_sta = 0;
		ptiming->v_act_sta = 0;
		return FALSE;
	}

}

unsigned char newbase_hdmi_get_interlace_conti_mode(unsigned char nport)
{
unsigned char is_intc= 0;

	if (hdmi_rx[nport].vs_cnt > 3) {
		is_intc = lib_hdmi_get_is_interlace(nport, HDMI_MS_DIRECT);
		lib_hdmi_clear_interlace_reg(nport);
		hdmi_rx[nport].vs_cnt = 0;
		return is_intc;
	} else {
		lib_hdmi_vsync_checked(nport);
		hdmi_rx[nport].vs_cnt++;
		return hdmi_rx[nport].timing_t.is_interlace;
	}

}

unsigned char newbase_hdmi_get_is_interlace(unsigned char port)
{
	return GET_H_INTERLACE(port);
}

//QC need
unsigned char newbase_hdmi_get_hdmi_mode_reg(unsigned char port)
{
	if ((hdmi_rx[port].hdmi_2p1_en == FALSE) && (newbase_hdmi2p0_get_clock40x_flag(port) || newbase_hdmi2p0_get_scramble_flag(port))) {
		return TRUE;
	}

	return lib_hdmi_is_hdmi_mode(port);

}

unsigned char newbase_hdmi_get_hdmi_mode(unsigned char port)
{
	return GET_H_MODE(port);
}

unsigned char newbase_hdmi_get_colordepth(unsigned char port)
{
	return GET_H_COLOR_DEPTH(port);
}

//QC need
unsigned char newbase_hdmi_get_colorspace_reg(unsigned char port)
{
	if (!newbase_hdmi_get_hdmi_mode_reg(port)) {
		return COLOR_RGB;
	}

	return lib_hdmi_get_colorspace(port);
}

unsigned char newbase_hdmi_get_colorspace(unsigned char port)
{
	if (GET_H_COLOR_SPACE(port) == COLOR_YUV420) {  //follow K2L
		return COLOR_YUV444;
	} else {
		return GET_H_COLOR_SPACE(port);
	}
}

unsigned char newbase_hdmi_get_video_format_reg(unsigned char port)
{
	if (hdmi_rx[port].vsi_t.Length == 0) {	//no vsinfo
		return HVF_NO;
	}

	return lib_hdmi_get_video_format(port);
}

unsigned char newbase_hdmi_get_video_format(unsigned char port)
{
	return GET_H_VIDEO_FORMAT(port);
}

unsigned char newbase_hdmi_get_3d_structure_reg(unsigned char port)
{
	if (hdmi_rx[port].vsi_t.Length == 0) {	//no vsinfo
		return HDMI3D_2D_ONLY;
	}

	if (lib_hdmi_get_video_format(port) != HVF_3D) {
		return HDMI3D_2D_ONLY;
	} else {
		return lib_hdmi_get_3d_structure(port);
	}
}


unsigned char newbase_hdmi_get_3d_structure(unsigned char port)
{
	if (hdmi_rx[port].vsi_t.Length == 0)	{	//no vsinfo
		return HDMI3D_2D_ONLY;
	}
	return GET_H_3DFORMAT(port);
}

unsigned char newbase_hdmi_get_colorimetry(unsigned char port)
{
	if (hdmi_rx[port].avi_t.len == 0) return 0;

	return (hdmi_rx[port].avi_t.C == 3)?
		(hdmi_rx[port].avi_t.EC+HDMI_COLORIMETRY_XYYCC601):hdmi_rx[port].avi_t.C;
}

unsigned char newbase_hdmi_get_color_range(unsigned char port)
{
	if (hdmi_rx[port].avi_t.len == 0) return 0;

	if (hdmi_rx[port].avi_t.Y == 0) {	// RGB
		return hdmi_rx[port].avi_t.Q;
	} else if (hdmi_rx[port].avi_t.Y == 1 || hdmi_rx[port].avi_t.Y == 2 || hdmi_rx[port].avi_t.Y == 3) {	// YUV
		return (hdmi_rx[port].avi_t.YQ<3)?(hdmi_rx[port].avi_t.YQ+1):MODE_RAG_DEFAULT;
	} else {
		return MODE_RAG_DEFAULT;
	}
}

unsigned int newbase_hdmi_get_tmds_clockx10(unsigned char port)
{
	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_MEASURE)
		return 0;
	return (lib_hdmi_get_clock(port)*270)>>8;
}

void newbase_hdmi_fix_green_line(unsigned char nport) //k3l need to check
{
	if (GET_H_COLOR_SPACE(nport) == COLOR_RGB) {
		lib_hdmi_tmds_out_ctrl(nport, 1, 0, 0, 0);
	} else {
		lib_hdmi_tmds_out_ctrl(nport, 0, 0x8000, 0x1000, 0x8000);
	}
}


void newbase_hdmi_set_scaler_info(HDMI_TIMING_T *timing)
{
	SET_MODE_IVSTA(timing->v_act_sta);
	SET_MODE_IHSTA(timing->h_act_sta);
	SET_MODE_IVHEIGHT(timing->v_act_len);      // Get Input VSYNC Period Measurement Result ;match resolution(cause measure result will lose one line)
	SET_MODE_IHWIDTH(timing->h_act_len);  // Input HSYNC High Period Measurement Result; match resolution(cause measure result will lose one line)
	SET_MODE_IVTOTAL(timing->v_total);
	SET_MODE_IHTOTAL(timing->h_total);
	SET_MODE_IHCOUNT(timing->h_total);
	SET_MODE_IVCOUNT(timing->v_total);
	SET_MODE_IVFREQ(timing->v_freq);
	SET_MODE_IHFREQ(timing->h_freq);
	SET_SCALER_POLARITY(timing->polarity);
	SET_SCALER_IVTOTAL(timing->v_total);
	SET_SCALER_IHTOTAL(timing->h_total);
	SET_SCALER_IVHEIGHT(timing->v_act_len);
	SET_SCALER_IVHEIGHT_PRE(timing->v_act_len);
	SET_SCALER_IHWIDTH(timing->h_act_len);
	SET_SCALER_IHWIDTH_PRE(timing->h_act_len);
	SET_SCALER_IVSTA(timing->v_act_sta);
	SET_SCALER_IVSTA_PRE(timing->v_act_sta);
	SET_SCALER_IHSTA(timing->h_act_sta);
	SET_SCALER_IHSTA_PRE(timing->h_act_sta);
	SET_SCALER_IVFREQ(timing->v_freq);
	SET_SCALER_IHFREQ(timing->h_freq);
	SET_SCALER_IVCOUNT(timing->v_total);
	SET_SCALER_IHCOUNT(timing->h_total);
	SET_SCALER_ISINTERLACE((timing->is_interlace) == 1);
	SET_SCALER_IS422(timing->colorspace == COLOR_YUV422);

	SET_SCALER_COLORSPACE(timing->colorspace);
	SET_SCALER_COLORDEPTH(timing->colordepth);
	SET_SCALER_COLORIMEYRY(timing->colorimetry);
}





#ifdef SUPPORT_DOLBY_VSIF
unsigned char newbase_hdmi_dolby_vsi(unsigned char port)
{	// Dolby VSIF (0x82)
	lib_hdmi_set_rsv_packet_type(port, 2, 0x81);

	if (!lib_hdmi_is_rsv_packet2_received(port))
		return FALSE;

	return TRUE;
}
#endif //#ifdef SUPPORT_DOLBY_VSIF

/*
unsigned char newbase_hdmi_reg_table(HDMI_TABLE_TYPE index, void *ptr,unsigned char port)
{

	if ( !ptr )
		return FALSE;

	//HDMI_PRINTF("%s %d %x\n", __func__, index, (unsigned int) ptr);
	switch ( index )
	{
		case HDMI_HDCP_KEY_TABLE:
                                          HDCP_KEY_T* hdcp_ptr =  (HDCP_KEY_T*)ptr;
                                         	memcpy(hdcpkey.BKsv, hdcp_ptr->BKsv, 5) ;
                                          memcpy(hdcpkey.Key, hdcp_ptr->Key, 320) ;
			break;
		case HDMI_CHANNEL:
			HDMI_CHANNEL_T* hdmi_ch_ptr = (HDMI_CHANNEL_T*)ptr;
			pcb_info[port].phy_port = hdmi_ch_ptr->phy_port;
			pcb_info[port].ddc_selected = hdmi_ch_ptr->ddc_selected;
			break;

#if MHL_SUPPORT
		case CBUS_TABLE:
		//cbus = (CBUS_CHANNEL_T*) ptr;

		break;
#endif

		default:

			return FALSE;
	}
	return TRUE;
}
*/


void newbase_hdmi_read_aksv(unsigned char port, unsigned char *pAKSV)
{
	unsigned char i;
	for (i=0; i<5; i++) {
		pAKSV[i] = hdmi_rx[port].tx_aksv[i];
	}
}

void newbase_hdmi_read_hdcp14_key(unsigned char *bksv, unsigned char *hdcp_key)
{
	lib_hdmi_read_hdcpkey(0, bksv, hdcp_key);
}

void newbase_hdmi_write_hdcp14_key(unsigned char *bksv, unsigned char *hdcp_key)
{
	unsigned char i;
	for (i=0; i<HDMI_PORT_TOTAL_NUM; i++) {
		lib_hdmi_write_hdcpkey(i, bksv, hdcp_key);
	}
}


#if 1//Need move to Hdmi_edid.c
void newbase_hdmi_read_edid(unsigned char port, unsigned char *edid, unsigned int len)
{
	if (port >= HDMI_PORT_TOTAL_NUM) {
		HDMI_EMG("%s: err port=%d\n", __func__, port);
		return;
	}
	lib_hdmiddc_read(port, edid, len);
}

void newbase_hdmi_write_edid(unsigned char port, unsigned char *edid, unsigned int len)
{
	if (port >= HDMI_PORT_TOTAL_NUM) {
		HDMI_EMG("%s: err port=%d\n", __func__, port);
		return;
	}
	lib_hdmiddc_write(port, edid, len);
}

void newbase_hdmi_edid_enable(unsigned char port, unsigned char enable)
{
	if (port >= HDMI_PORT_TOTAL_NUM) {
		HDMI_EMG("%s: err port=%d\n", __func__, port);
		return;
	}
	lib_hdmiddc_enable(port, enable);
}
#endif

const unsigned int dedicate_hpd_det_reg[] =
{
    PINMUX_ST_HPD_R1K_Ctrl0_reg,
    PINMUX_ST_HPD_R1K_Ctrl1_reg,
    PINMUX_ST_HPD_R1K_Ctrl2_reg,
    PINMUX_ST_HPD_R1K_Ctrl3_reg
};

#define DEDICATE_HPD_DET_CNT (sizeof(dedicate_hpd_det_reg)/sizeof(unsigned int))


unsigned char newbase_hdmi_get_5v_state(unsigned char port)
{
    HDMI_CHANNEL_T* p_info = newbase_hdmi_get_pcbinfo(port);

    if (p_info==NULL)
        return 0;

    if (p_info->dedicate_hpd_det)
    {
        if (p_info->dedicate_hpd_det_sel >= DEDICATE_HPD_DET_CNT)
            return 0;
            
        return PINMUX_ST_HPD_R1K_Ctrl0_get_hdmirx_p0_h5vdet(hdmi_in(dedicate_hpd_det_reg[p_info->dedicate_hpd_det_sel]));
    }
    else
    {
        rtk_gpio_set_dir(p_info->det_5v, 0);
        return rtk_gpio_input(p_info->det_5v);
    }
}

unsigned char newbase_hdmi_get_5v_stateNew(unsigned char port)
{
   // HDMI_CHANNEL_T* p_info = newbase_hdmi_get_pcbinfo(port);
	unsigned hpd_det_sel;
    if (port >= HDMI_PORT_TOTAL_NUM)
        return 0;

	switch(port)
	{
	    case HDMI_PORT0:
		hpd_det_sel=2;
	        break;
	    case HDMI_PORT1:
		hpd_det_sel=3;
	        break;
	    case HDMI_PORT2:
		hpd_det_sel=0;
	        break;
	    case HDMI_PORT3:
		hpd_det_sel=1;
	        break;
	    default:
	        // do nothing...
	        break;
	}
       return PINMUX_ST_HPD_R1K_Ctrl0_get_hdmirx_p0_h5vdet(hdmi_in(dedicate_hpd_det_reg[hpd_det_sel]));

}



void newbase_hdmi_set_hpd(unsigned char nport, unsigned char high, unsigned char controllable)
{
    HDMI_CHANNEL_T* p_info = newbase_hdmi_get_pcbinfo(nport);

    if (p_info==NULL)
        return;

    if (!controllable)
        return;

    if (high)
    {
#if HDMI2p0
        lib_hdmi_scdc_reset(nport, 0);        
#endif
        HDMI_EMG("HOTPLUG[%d]=HIGH\n", nport);

    } 
    else 
    {
#if HDMI2p0
        lib_hdmi_scdc_reset(nport, 1);        
#endif
        HDMI_EMG("HOTPLUG[%d]=LOW\n", nport);
    }

    if (p_info->dedicate_hpd_det)
    {        
        unsigned int val;
        
        if (p_info->dedicate_hpd_det_sel >= DEDICATE_HPD_DET_CNT)
            return;

        val = hdmi_in(dedicate_hpd_det_reg[p_info->dedicate_hpd_det_sel]) & ~(PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_r1k_en_mask|PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_pd_mask);
        
        if (high)    
            val |= PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_r1k_en_mask;    // enable 1K pull up and disable pull down
        else                
            val |= PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_pd_mask;        // disable 1K pull up and enable pull down
            
        hdmi_out(dedicate_hpd_det_reg[p_info->dedicate_hpd_det_sel], val);
    }
    else
    {
    	 if (high) {
		 SET_HDMIRX_HPD(nport, 1);
    	 }
	else {
		 SET_HDMIRX_HPD(nport, 0);
	}
	
	 rtk_gpio_output(p_info->hpd, (high) ? 0 : 1);
        rtk_gpio_set_dir(p_info->hpd, 1);
    }
}


void newbase_hdmi_set_hpdNew(unsigned char nport, unsigned char high, unsigned char controllable)
{
   // HDMI_CHANNEL_T* p_info = newbase_hdmi_get_pcbinfo(port);
	unsigned hpd_det_sel;
       unsigned int val;
    if (nport >= HDMI_PORT_TOTAL_NUM)
        return 0;

	switch(nport)
	{
	    case HDMI_PORT0:
		hpd_det_sel=2;
	        break;
	    case HDMI_PORT1:
		hpd_det_sel=3;
	        break;
	    case HDMI_PORT2:
		hpd_det_sel=0;
	        break;
	    case HDMI_PORT3:
		hpd_det_sel=1;
	        break;
	    default:
	        // do nothing...
	        break;
	}

    if (!controllable)
        return;

    if (high)
    {
#if HDMI2p0
        lib_hdmi_scdc_reset(nport, 0);        
#endif
        HDMI_EMG("HOTPLUG[%d]=HIGH\n", nport);

    } 
    else 
    {
#if HDMI2p0
        lib_hdmi_scdc_reset(nport, 1);        
#endif
        HDMI_EMG("HOTPLUG[%d]=LOW\n", nport);
    }
     
        
        val = hdmi_in(dedicate_hpd_det_reg[hpd_det_sel]) & ~(PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_r1k_en_mask|PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_pd_mask);
        
        if (high)    
            val |= PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_r1k_en_mask;    // enable 1K pull up and disable pull down
        else                
            val |= PINMUX_ST_HPD_R1K_Ctrl0_hdmirx_p0_pd_mask;        // disable 1K pull up and enable pull down
            
        hdmi_out(dedicate_hpd_det_reg[hpd_det_sel], val);

}


void newbase_hdmi_hpd_delay_work_init(void)
{
	hpd_target_ch = 0xff;
	
	_destroy_hpd_delay_wq();	
	_init_hpd_delay_wq(&newbase_hdmi_hpd_delay_work_func);
	
	if (0 == _start_hpd_delay_wq(hpd_low_ms)) {
		//hpd_low_is_running = 1;
	}

	hpd_low_ms = HPD_DEFAULT_LOW_MS;
}

void newbase_hdmi_hpd_delay_work_func(void)
{
	int nPortCnt = 0;

	//hpd_low_is_running = 0;

	//do what you want to do after 'hpd_delayms' delay
	if (hpd_target_ch == 0xff) { // for all channel
		for (nPortCnt=0; nPortCnt<HDMI_PORT_TOTAL_NUM; nPortCnt++) {
			newbase_hdmi_set_hpd(nPortCnt, 1, INIT_HPD_OFF); //HPD pull high
		}
	} else {
		//for certain channel
	}
	HDMI_PRINTF("%s is actived(%x)\n", __func__, hpd_target_ch);
}

void newbase_hdmi_check_connection_state(unsigned char port)
{
	unsigned char conn_st;

	conn_st = newbase_hdmi_get_5v_state(port);

	if (conn_st != hdmi_rx[port].cable_conn)
	{
		if (conn_st) {
			lib_hdmi_char_err_rst(port, 1);  //reset CED until measure done
			#if HDMI_FRL_TRANS_DET
			lib_hdmi_lane_rate_detect_start(port);
			#endif
		}
		else
		{
			int ddc_ch;

#if HDMI2p0            
			lib_hdmi_scdc_reset_toggle(port);
			lib_hdmi2p0_scramble_enable(port, 0);	// First, connect MSHG-800/4K60(#474), then plug out cable to QD882, HDCP CTS will check Bstatus.
#endif
			lib_hdmi_clear_avmute(port);
			lib_hdmi_hdcp_port_write(port, 0xC4, 0x0);   //[WOSQRTK-9185]SKbox transient garbage

			//---------------------------------------------                            
			// ddc sync_reset, patch for poor design
			//---------------------------------------------
			ddc_ch = newbase_hdmi_ddc_channel_status(port);

			if (ddc_ch >=0)
			{
				lib_hdmiddc_sync_reset(ddc_ch);
				lib_hdmiddc_set_sda_delay(ddc_ch, 0x1); //K2L, one NB no audio, due to IIC issue. Q2879/3094/3222
				lib_hdmiddc_enable(ddc_ch, 1);
			}
			
			//---------------------------------------------
			// reset hdcp 
			//---------------------------------------------
#if HDCP2_2
			newbase_hdmi_hdcp2_clear_status(port);
#endif
#if HDMI_TOGGLE_HPD_Z0_ENABLE
			lib_hdmi_set_toggle_hpd_z0_flag(port, _FALSE);
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

			lib_hdmi_misc_variable_initial(port);
		}
	}

	hdmi_rx[port].cable_conn = conn_st;


#ifdef CONFIG_POWER_SAVING_MODE


	if (!hdmi_rx[port].cable_conn) {
		if (port < HD20_PORT) {		
			if (hdmi_rx[port].hdmi_2p1_en) {
				newbase_hdmi21_phy_pw_saving(port);
			} else {
				if (!newbase_rxphy_is_clk_stable(port))  //5v disable & clock unstable
					newbase_hdmi21_phy_pw_saving(port);
			}
		}
		else {
			if (!newbase_rxphy_is_clk_stable(port))
				newbase_hdmi_phy_pw_saving(port);
		}
	}
#endif
}


void newbase_hdmi_save_hdcp(const HDCP_KEY_T *key)
{
	memcpy(&hdcpkey, key, sizeof(HDCP_KEY_T));
}


#if 0  //fixed me

void newbase_hdmi_reset_all_infoframe(unsigned char port)
{
	memset(&hdmi_rx[port].vsi_t, 0, sizeof(HDMI_VSI_T));
	hdmi_rx[port].no_vsi_cnt = 0;
	lib_hdmi_clear_fvs_received_status(port);
	lib_hdmi_clear_vsi_received_status(port);

	memset(&hdmi_rx[port].avi_t, 0, sizeof(HDMI_AVI_T));
	hdmi_rx[port].no_avi_cnt = 0;
	lib_hdmi_clear_avi_received_status(port);

	memset(&hdmi_rx[port].spd_t, 0, sizeof(HDMI_SPD_T));
	hdmi_rx[port].no_spd_cnt = 0;
	lib_hdmi_clear_spd_received_status(port);

	memset(&hdmi_rx[port].audiopkt_t, 0, sizeof(HDMI_AUDIO_T));
	hdmi_rx[port].no_audiopkt_cnt = 0;
	lib_hdmi_clear_audiopkt_received_status(port);

	memset(&hdmi_rx[port].drm_t, 0, sizeof(HDMI_DRM_T));
	hdmi_rx[port].no_drm_cnt = 0;
	lib_hdmi_clear_drm_received_status(port);

}

void newbase_hdmi_update_infoframe(unsigned char port)
{
	unsigned char pkt_buf[31];
	unsigned char is_drm_here = hdmi_rx[port].drm_t.type_code;
	unsigned char is_avi_here = hdmi_rx[port].avi_t.type_code;
	unsigned char is_spd_here = hdmi_rx[port].spd_t.type_code;
	unsigned char is_audio_here = hdmi_rx[port].audiopkt_t.type_code;
	unsigned char is_vsi_here = hdmi_rx[port].vsi_t.VSIF_Version;  // vsi has 2 version-type //861-G

	if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_19_BYPASS_PACKET_DET)
		return;

	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_MEASURE || GET_H_MODE(port) != MODE_HDMI)
		return;

	if (lib_hdmi_is_drm_received(port)) 
	{
		lib_hdmi_read_drm_packet(port, pkt_buf);		

		((unsigned char*)(&hdmi_rx[port].drm_t))[0] = pkt_buf[0]; // Info Frame Type
		((unsigned char*)(&hdmi_rx[port].drm_t))[1] = pkt_buf[1]; // version
		((unsigned char*)(&hdmi_rx[port].drm_t))[2] = pkt_buf[2]; // len
		hdmi_rx[port].drm_t.eEOTFtype  = pkt_buf[3];
		hdmi_rx[port].drm_t.eMeta_Desc = pkt_buf[4];
		hdmi_rx[port].drm_t.display_primaries_x0 = pkt_buf[5]|(pkt_buf[6]<<8);
		hdmi_rx[port].drm_t.display_primaries_y0 = pkt_buf[7]|(pkt_buf[8]<<8);
		hdmi_rx[port].drm_t.display_primaries_x1 = pkt_buf[9]|(pkt_buf[10]<<8);
		hdmi_rx[port].drm_t.display_primaries_y1 = pkt_buf[11]|(pkt_buf[12]<<8);
		hdmi_rx[port].drm_t.display_primaries_x2 = pkt_buf[13]|(pkt_buf[14]<<8);
		hdmi_rx[port].drm_t.display_primaries_y2 = pkt_buf[15]|(pkt_buf[16]<<8);
		hdmi_rx[port].drm_t.white_point_x = pkt_buf[17]|(pkt_buf[18]<<8);
		hdmi_rx[port].drm_t.white_point_y = pkt_buf[19]|(pkt_buf[20]<<8);
		hdmi_rx[port].drm_t.max_display_mastering_luminance = pkt_buf[21]|(pkt_buf[22]<<8);
		hdmi_rx[port].drm_t.min_display_mastering_luminance = pkt_buf[23]|(pkt_buf[24]<<8);
		hdmi_rx[port].drm_t.maximum_content_light_level = pkt_buf[25]|(pkt_buf[26]<<8);
		hdmi_rx[port].drm_t.maximum_frame_average_light_level = pkt_buf[27]|(pkt_buf[28]<<8);	
		hdmi_rx[port].no_drm_cnt = 0;
	} 
	else 
	{
		if (++hdmi_rx[port].no_drm_cnt > NO_DRM_INFO_MAX) {
			memset(&hdmi_rx[port].drm_t, 0, sizeof(HDMI_DRM_T));
			hdmi_rx[port].no_drm_cnt = 0;
		}
	}

	if (lib_hdmi_is_avi_received(port)) 
	{
		lib_hdmi_read_avi_info_packet(port, pkt_buf);

		memcpy(&hdmi_rx[port].avi_t, pkt_buf, sizeof(HDMI_AVI_T));
		hdmi_rx[port].no_avi_cnt = 0;
	} 
	else 
	{
		if (++hdmi_rx[port].no_avi_cnt > NO_AVI_INFO_MAX) {
			memset(&hdmi_rx[port].avi_t, 0, sizeof(HDMI_AVI_T));
			hdmi_rx[port].no_avi_cnt = 0;
		}
	}

	if (newbase_hdmi_update_vsi(port)) {
		hdmi_rx[port].no_vsi_cnt = 0;
	} else {
		if (++hdmi_rx[port].no_vsi_cnt > NO_VSI_INFO_MAX) {
			memset(&hdmi_rx[port].vsi_t, 0, sizeof(HDMI_VSI_T));
			hdmi_rx[port].no_vsi_cnt = 0;
		}
	}	
	
	if (lib_hdmi_is_spd_received(port)) 
	{
		lib_hdmi_read_spd_info_packet(port, pkt_buf);
		memcpy(&hdmi_rx[port].spd_t, pkt_buf, sizeof(HDMI_SPD_T));
		hdmi_rx[port].no_spd_cnt = 0;
	} 
	else 
	{
#if NO_SPD_CLEAR_PACKET
		if (++hdmi_rx[port].no_spd_cnt > NO_SPD_INFO_MAX) {
			memset(&hdmi_rx[port].spd_t, 0, sizeof(HDMI_SPD_T));
			hdmi_rx[port].no_spd_cnt = 0;
		}
#endif //#if NO_SPD_CLEAR_PACKET
	}

	if (lib_hdmi_is_audiopkt_received(port)) 
	{	    
		lib_hdmi_read_audio_packet(port, pkt_buf);
        	memcpy(&hdmi_rx[port].audiopkt_t, pkt_buf, 13);			
		hdmi_rx[port].no_audiopkt_cnt = 0;
	}
	else 
	{
		if (++hdmi_rx[port].no_audiopkt_cnt > NO_AUDIO_INFO_MAX) {
			memset(&hdmi_rx[port].audiopkt_t, 0, sizeof(HDMI_AUDIO_T));
			hdmi_rx[port].no_audiopkt_cnt = 0;
		}
	}

	if (is_drm_here != hdmi_rx[port].drm_t.type_code) {
		HDMI_PRINTF("DRM INFO=%d\n", hdmi_rx[port].drm_t.type_code);
	}
	if (is_avi_here != hdmi_rx[port].avi_t.type_code) {
		HDMI_PRINTF("AVI INFO=%d\n", hdmi_rx[port].avi_t.type_code);
	}
	if (is_spd_here != hdmi_rx[port].spd_t.type_code) {
		HDMI_PRINTF("SPD INFO=%d\n", hdmi_rx[port].spd_t.type_code);
	}
	if (is_audio_here != hdmi_rx[port].audiopkt_t.type_code) {
		HDMI_PRINTF("AUD INFO=%d\n", hdmi_rx[port].audiopkt_t.type_code);
	}
	if (is_vsi_here != hdmi_rx[port].vsi_t.VSIF_Version) {
		HDMI_PRINTF("VSI INFO=%d (%x,%x,%x)\n", hdmi_rx[port].vsi_t.VSIF_Version, hdmi_rx[port].vsi_t.Reg_ID[0], hdmi_rx[port].vsi_t.Reg_ID[1], hdmi_rx[port].vsi_t.Reg_ID[2]);
		// 0xd8, 0x5d, 0xc4 is hdmi2.0 FVSPS	                    (QD980 Forum VS)
		// 0x03, 0x0c, 0x00 is hdmi1.4 VSPS or Dolby         (QD980 LLC VS)
		// QD980 has a bug, it needs to toggle & apply twice then send correct info.
	}

}

unsigned char newbase_hdmi_get_hdmi_mode_infoframe(unsigned char port)
{
	if(GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_DISPLAY_ON)
		return 0;

	if(MODE_HDMI == GET_H_MODE(port))
		return 1;

	return 0;
}

unsigned char newbase_hdmi_get_vs_infoframe(unsigned char port, HDMI_VSI_T *p_vsi_t)
{
	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_MEASURE || GET_H_MODE(port) != MODE_HDMI) {
		return 0;
	}

	if (hdmi_rx[port].vsi_t.VSIF_TypeCode == 0) {
		return 0;
	}

	if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_21_FORCE_NO_INFOPKT) {
		return 0;
	}

	memcpy(p_vsi_t, &hdmi_rx[port].vsi_t, sizeof(HDMI_VSI_T));
	return 1;
}

unsigned char newbase_hdmi_get_avi_infoframe(unsigned char port, HDMI_AVI_T *p_avi_t)
{
	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_MEASURE || GET_H_MODE(port) != MODE_HDMI) {
		return 0;
	}

	if (hdmi_rx[port].avi_t.type_code == 0) {
		return 0;
	}

	if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_21_FORCE_NO_INFOPKT) {
		return 0;
	}

	memcpy(p_avi_t, &hdmi_rx[port].avi_t, sizeof(HDMI_AVI_T));
	return 1;
}

unsigned char newbase_hdmi_get_spd_infoframe(unsigned char port, HDMI_SPD_T *p_spd_t)
{
	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_MEASURE || GET_H_MODE(port) != MODE_HDMI) {
		return 0;
	}

	if (hdmi_rx[port].spd_t.type_code == 0) {
		return 0;
	}

	if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_21_FORCE_NO_INFOPKT) {
		return 0;
	}

	memcpy(p_spd_t, &hdmi_rx[port].spd_t, sizeof(HDMI_SPD_T));
	return 1;
}

unsigned char newbase_hdmi_get_audio_infoframe(unsigned char port, HDMI_AUDIO_T *p_audio_t)
{
	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_MEASURE || GET_H_MODE(port) != MODE_HDMI) {
		return 0;
	}

	if (hdmi_rx[port].audiopkt_t.type_code == 0) {
		return 0;
	}

	if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_21_FORCE_NO_INFOPKT) {
		return 0;
	}

	memcpy(p_audio_t, &hdmi_rx[port].audiopkt_t, sizeof(HDMI_AUDIO_T));
	return 1;
}

unsigned char newbase_hdmi_get_drm_infoframe(unsigned char port, HDMI_DRM_T *p_drm_t)
{
	if (GET_H_VIDEO_FSM(port) < MAIN_FSM_HDMI_VIDEO_READY || GET_H_MODE(port) != MODE_HDMI) {
		return 0;
	}

	if (hdmi_rx[port].drm_t.type_code == 0) {
		return 0;
	}

	if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_20_FORCE_NO_DRMPKT) {
		return 0;
	}

	memcpy(p_drm_t, &hdmi_rx[port].drm_t, sizeof(HDMI_DRM_T));
	return 1;
}

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#ifdef SUPPORT_DOLBY_VSIF
unsigned char newbase_hdmi_update_vsi(unsigned char port)
{
	#define VSI_PKT_LEN			31
	#define VSI_PKT_CONT_LEN	12
	unsigned char sta_pos, len, pkt_buf[VSI_PKT_LEN]={0}, len_extend;
	unsigned char Is_DolbyVSIF = _FALSE;
	unsigned char Is_VSIF_14 = FALSE;//check is vsif 1.4 or not
	unsigned char Check_PB6_PB24_result = TRUE; //True: pb6-pb24 all 0. False: not all 0
	unsigned char check_dolby_h14b_result = FALSE; //True: h1.4b dolby format  FALSE: not dolby format
	int idx_PB; // HDMI 1.4b VSIF PB[] index
	
	lib_hdmi_set_rsv_packet_type(port, 2, 0x81);        // use reserved2 to receive Dolby VSIF
	
#if HDMI2p0
	if (lib_hdmi_is_fvs_received(port)) 
	{
		lib_hdmi_clear_fvs_received_status(port);
		sta_pos = 253;
		len = 11;
	} else
#endif //#if HDMI2p0
	if (lib_hdmi_is_vsi_received(port))
	{
		lib_hdmi_clear_vsi_received_status(port);
		sta_pos = 203;
		len = 18;
		Is_VSIF_14 = TRUE;
	}
	else if(lib_hdmi_is_rsv_packet2_received(port))
	{
		//lib_hdmi_clear_rsv_packet2_status(port); //it will clear later
		Is_DolbyVSIF = _TRUE;
		len = 30;
	}
	else {
		return FALSE;
	}

	if(_TRUE == Is_DolbyVSIF)
	{
		if (lib_hdmi_read_rsv2_packet(port, pkt_buf, len)<0)
		{
			lib_hdmi_clear_rsv_packet2_status(port);
			return FALSE;
		}

		lib_hdmi_clear_rsv_packet2_status(port);

		// DolbyVSIF 24-bit IEEE OUI (=0x00D046)
		if((0x00 == pkt_buf[6]) && (0xD0 == pkt_buf[5]) && (0x46 == pkt_buf[4]))
		{
			hdmi_rx[port].vsi_t.VSIF_TypeCode=0x81;

			len = pkt_buf[2] + 3;
			if(len > 31)
				len = 31;
			memcpy(&hdmi_rx[port].vsi_t.VSIF_Version, &pkt_buf[1], len); //byte 0 is BCH					
			if(0x2 == pkt_buf[7])
				{
			    set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_STD);//10
				}
			else if(0x3 == pkt_buf[7])
				{
				//if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_16_PRINT_DRM)
			    set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_LL);//11
				}
			else
				{
				//if (hdmi_in(DEBUG_REG_ADDR)&DEBUG_16_PRINT_DRM)
			    set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_DISABLE);//00 or 01
				}

            if(hdmi_get_infoframe_thread_stop()==_FALSE)
			    pre_dolby_hdmi_vsif_mode=get_HDMI_Dolby_VSIF_mode();
			
			return TRUE;
		}
		set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_DISABLE);
		
		 if(hdmi_get_infoframe_thread_stop()==_FALSE)
			pre_dolby_hdmi_vsif_mode=get_HDMI_Dolby_VSIF_mode();
		 
		return FALSE;
	}
	else
	{
		set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_DISABLE);
		
	    	if (hdmi_get_infoframe_thread_stop()==_FALSE)
		    pre_dolby_hdmi_vsif_mode=get_HDMI_Dolby_VSIF_mode();		   
	}

	lib_hdmi_clear_rsv_packet2_status(port);

	lib_hdmi_read_packet_sram(port, sta_pos, len, pkt_buf);

	//=====willychou 20170712 modify Keep checksume information=======

	//pkt_buf[0];	=>version
	//pkt_buf[1];	=>len
	//pkt_buf[2];	=>checksum

	if(203 == sta_pos)
	{
		if(pkt_buf[1] > 15)//len > 15
		{
			len_extend = pkt_buf[1] - 15; //len_extend = len -15

			if(len_extend >= VSI_PKT_CONT_LEN)
				lib_hdmi_read_packet_sram(port, 323, VSI_PKT_CONT_LEN, &pkt_buf[18]);
			else// if(len_extend < VSI_PKT_CONT_LEN)
				lib_hdmi_read_packet_sram(port, 323, len_extend, &pkt_buf[18]);

			len = pkt_buf[1] + 3;
		}
	}
	hdmi_rx[port].vsi_t.VSIF_TypeCode=0x81;
	memcpy(&hdmi_rx[port].vsi_t.VSIF_Version, pkt_buf, len); //use len for memcpy because copy size will be 11 or 18

	if (Is_VSIF_14)
	{
		if(pkt_buf[1] == 24)//lenth is 0x18
		{
			  // HDMI1.4b VSIF 24-bit IEEE OUI (=0x000C03)
			 if((0x00 == pkt_buf[5]) && (0x0C == pkt_buf[4]) && (0x03 == pkt_buf[3]))
			 {
				 for(idx_PB=8 ; idx_PB < 27 ; idx_PB++ )//PB6-PB24
				 {
					 if(pkt_buf[idx_PB]!=0)
					 {
						 Check_PB6_PB24_result = FALSE;
						 break;
					 }							 	
				 }
				 
				 if(Check_PB6_PB24_result)
				 {
					check_dolby_h14b_result = TRUE;
				 } 	
			 } 
		}
		
		if(check_dolby_h14b_result)
		{//h14b dolby format
			set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_h14B_VSIF);

			if(hdmi_get_infoframe_thread_stop()==_FALSE)
				pre_dolby_hdmi_vsif_mode = get_HDMI_Dolby_VSIF_mode();	
		}
	    else
		{//h14b not dolby
			set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_DISABLE);
			if(hdmi_get_infoframe_thread_stop()==_FALSE)
				pre_dolby_hdmi_vsif_mode = get_HDMI_Dolby_VSIF_mode();	
		}	
	}
	else
	{//hdmi 2.0
		set_HDMI_Dolby_VSIF_mode(DOLBY_HDMI_VSIF_DISABLE);

		if(hdmi_get_infoframe_thread_stop()==_FALSE)
			pre_dolby_hdmi_vsif_mode = get_HDMI_Dolby_VSIF_mode();	
	}
	return TRUE;
}
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#else //#ifdef SUPPORT_DOLBY_VSIF
unsigned char newbase_hdmi_update_vsi(unsigned char port)
{
#if 1
	#define VSI_PKT_LEN			31
	#define VSI_PKT_CONT_LEN	12
	unsigned char sta_pos, len, pkt_buf[VSI_PKT_LEN]={0}, len_extend;

#if HDMI2p0
	if (lib_hdmi_is_fvs_received(port)) {
		lib_hdmi_clear_fvs_received_status(port);
		sta_pos = 253;
		len = 11;
	} else
#endif //#if HDMI2p0
	if (lib_hdmi_is_vsi_received(port)) {
		lib_hdmi_clear_vsi_received_status(port);
		sta_pos = 203;
		len = 18;
	} else {
		return FALSE;
	}


	lib_hdmi_read_packet_sram(port, sta_pos, len, pkt_buf);

	//=====willychou 20170712 modify Keep checksume information=======

	//pkt_buf[0];	=>version
	//pkt_buf[1];	=>len
	//pkt_buf[2];	=>checksum

	if(203 == sta_pos)
	{
		if(pkt_buf[1] > 15)//len > 15
		{
			len_extend = pkt_buf[1] - 15; //len_extend = len -15

			if(len_extend >= VSI_PKT_CONT_LEN)
				lib_hdmi_read_packet_sram(port, 323, VSI_PKT_CONT_LEN, &pkt_buf[18]);
			else// if(len_extend < VSI_PKT_CONT_LEN)
				lib_hdmi_read_packet_sram(port, 323, len_extend, &pkt_buf[18]);

			len = pkt_buf[1] + 3;
		}
	}
	hdmi_rx[port].vsi_t.VSIF_TypeCode=0x81;
	memcpy(&hdmi_rx[port].vsi_t.VSIF_Version, pkt_buf, len); //use len for memcpy because copy size will be 11 or 18

	return TRUE;

#else //#if MERLIN3_MODIFY
#define VSI_PKT_LEN	31
	unsigned char sta_pos, len, pkt_buf[VSI_PKT_LEN]={0};

	if (lib_hdmi_is_fvs_received(port)) {
		lib_hdmi_clear_fvs_received_status(port);
		sta_pos = 253;
		len = 11;
	} else if (lib_hdmi_is_vsi_received(port)) {
		lib_hdmi_clear_vsi_received_status(port);
		sta_pos = 203;
		len = 18;
	} else {
		return FALSE;
	}


	lib_hdmi_read_packet_sram(port, sta_pos, len, pkt_buf);
	pkt_buf[2] = pkt_buf[1];	//len
	pkt_buf[1] = pkt_buf[0];	//version
	pkt_buf[0] = 0x81;

	memcpy(&hdmi_rx[port].vsi_t, pkt_buf, len); //use len for memcpy because copy size will be 11 or 18
	return TRUE;
#endif //#if MERLIN3_MODIFY
}
#endif //#ifdef SUPPORT_DOLBY_VSIF


unsigned char newbase_hdmi_check_spd_name(unsigned char port, unsigned char *vendor_name)
{
	unsigned char i;

	if (hdmi_rx[port].spd_t.type_code == 0)
		return 0;

#if 0
	HDMI_PRINTF("VendorName [%x, %x, %x, %x, %x, %x, %x, %x]\n",
		hdmi_rx[port].spd_t.VendorName[0], hdmi_rx[port].spd_t.VendorName[1],
		hdmi_rx[port].spd_t.VendorName[2], hdmi_rx[port].spd_t.VendorName[3],
		hdmi_rx[port].spd_t.VendorName[4], hdmi_rx[port].spd_t.VendorName[5],
		hdmi_rx[port].spd_t.VendorName[6], hdmi_rx[port].spd_t.VendorName[7]);
#endif

	for (i=0; i<8; i++) {
		if (hdmi_rx[port].spd_t.VendorName[i] != vendor_name[i]) {
			return 0;
		}
	}

	return 1;
}

unsigned char newbase_hdmi_check_spd_prod(unsigned char port, unsigned char *prod_desc)
{
	unsigned char i;

	if (hdmi_rx[port].spd_t.type_code == 0)
		return 0;

#if 0
	HDMI_PRINTF("ProductDesc [%x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x]\n",
		hdmi_rx[port].spd_t.ProductDesc[0], hdmi_rx[port].spd_t.ProductDesc[1],
		hdmi_rx[port].spd_t.ProductDesc[2], hdmi_rx[port].spd_t.ProductDesc[3],
		hdmi_rx[port].spd_t.ProductDesc[4], hdmi_rx[port].spd_t.ProductDesc[5],
		hdmi_rx[port].spd_t.ProductDesc[6], hdmi_rx[port].spd_t.ProductDesc[7],
		hdmi_rx[port].spd_t.ProductDesc[8], hdmi_rx[port].spd_t.ProductDesc[9],
		hdmi_rx[port].spd_t.ProductDesc[10], hdmi_rx[port].spd_t.ProductDesc[11],
		hdmi_rx[port].spd_t.ProductDesc[12], hdmi_rx[port].spd_t.ProductDesc[13],
		hdmi_rx[port].spd_t.ProductDesc[14], hdmi_rx[port].spd_t.ProductDesc[15]);
#endif

	for (i=0; i<16; i++) {
		if (hdmi_rx[port].spd_t.ProductDesc[i] != prod_desc[i]) {
			return 0;
		}
	}

	return 1;
}


#endif

unsigned char newbase_hdmi_check_aksv(unsigned char port, unsigned char *aksv)
{
	unsigned char i;

#if 0
	HDMI_PRINTF("AKSV [%x, %x, %x, %x, %x]\n",
	hdmi_rx[port].tx_aksv[0], hdmi_rx[port].tx_aksv[1],
	hdmi_rx[port].tx_aksv[2], hdmi_rx[port].tx_aksv[3],
	hdmi_rx[port].tx_aksv[4]);
#endif

	for (i = 0; i < 5; i++) {
		if (hdmi_rx[port].tx_aksv[i] != aksv[i]) {
			return 0;
		}
	}

	return 1;
}



#if 0


unsigned char newbase_hdmi_compatibility_check(unsigned char port)
{
	HDMI_POOR_DEV_T btv_stb = {{0x42, 0x72, 0x6f, 0x61, 0x64, 0x63, 0x6f, 0x6d},
				{0x53, 0x54, 0x42, 0x20, 0x52, 0x65, 0x66, 0x73, 0x77, 0x20, 0x44, 0x65, 0x73, 0x69, 0x67, 0x6e}};

#if HDMI_TOGGLE_HPD_Z0_ENABLE
	// [STB-14] KT UHD STB (IC1000)
	HDMI_POOR_DEV_T kt_stb_IC1000 = {{0x4b, 0x61, 0x6f, 0x6e, 0x00, 0x63, 0x6f, 0x6d},
				{0x49, 0x43, 0x31, 0x30, 0x30, 0x30, 0x00, 0x63, 0x65, 0x20, 0x42, 0x6f, 0x61, 0x72, 0x64, 0x00}};

#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE
#if HDMI_STR_TOGGLE_HPD_ENABLE
	// [STB-20] IC1100 - KT olleh GiGA UHD tv STB
	HDMI_POOR_DEV_T kt_stb_IC1100 = {{0x4b, 0x61, 0x6f, 0x6e, 0x00, 0x63, 0x6f, 0x6d},
				{0x49, 0x43, 0x31, 0x31, 0x30, 0x30, 0x00, 0x63, 0x65, 0x20, 0x42, 0x6f, 0x61, 0x72, 0x64, 0x00}};

	HDMI_POOR_DEV_T kt_stb_GX_KT600EJ = {{0x53, 0x61, 0x6d, 0x73, 0x75, 0x6e, 0x67, 0x0},
				{0x47, 0x58, 0x2d, 0x4b, 0x54, 0x36, 0x30, 0x30, 0x45, 0x4a, 0x0, 0x6f, 0x61, 0x72, 0x64, 0x0}};
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE

#if HDMI_DISPLAY_DELAY_ENABLE
	// [HOME-05] LHB755 - LG
	HDMI_POOR_DEV_T LHB755 = {{0x4d, 0x54, 0x4b, 0x00, 0x00, 0x00, 0x00, 0x00},
				{0x42, 0x44, 0x20, 0x50, 0x4c, 0x41, 0x59, 0x45, 0x52, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}};

//	unsigned char LHB755_NG_aksv[5] = {0};

//	lib_hdmi_set_delay_display_enable(port, _DISABLE); //move to lib_hdmi_misc_variable_initial(), to prevent Tx SPD and AKSV unstable

	// Roku Ultra 4640R
	HDMI_POOR_DEV_T Roku_Ultra_4640R = {{0x4d, 0x00, 0x74, 0x00, 0x72, 0x00, 0x20, 0x00},
				{0x48, 0x00, 0x4d, 0x00, 0x20, 0x00, 0x78, 0x00, 0x44, 0x00, 0x6d, 0x00, 0x00, 0x00, 0x00, 0x00}};

	HDMI_POOR_DEV_T Samsung_GX_HC630CH = {{0x42, 0x72, 0x6f, 0x61, 0x64, 0x63, 0x6f, 0x6d},
				{0x53, 0x54, 0x42, 0x20, 0x47, 0x58, 0x36, 0x33, 0x30, 0x43, 0x48, 0x0, 0x61, 0x72, 0x64, 0x0}};

	// [STB-32] KAO_CT1100-LB(GiGA Genie) (KT STB)
//	HDMI_POOR_DEV_T GiGa_Genie_KAO_CT1100_LB = {{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
//				{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}};
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

//	lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_NONE); //move to lib_hdmi_misc_variable_initial(), to prevent Tx SPD and AKSV unstable
#if HDMI_TOGGLE_HPD_Z0_ENABLE
	lib_hdmi_set_toggle_hpd_z0_flag(port, _FALSE);
#endif

#if HDMI_STR_TOGGLE_HPD_ENABLE
	lib_hdmi_set_str_toggle_hpd_state(port, HDMI_STR_HPD_TOGGLE_NONE);
#endif
	hpd_low_ms = HPD_DEFAULT_LOW_MS;

	if (newbase_hdmi_check_spd_name(port, btv_stb.spd_vn_name) && newbase_hdmi_check_spd_prod(port, btv_stb.psd_pd_desc)) {
		//[WOSQRTK-9185]SKbox transient garbage
		HDMI_EMG("This is SKBOX!!!!!!!\n");
		lib_hdmi_hdcp_port_write(port, 0xC4, 0x8);
		lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_SK_STB);
	}
#if HDMI_TOGGLE_HPD_Z0_ENABLE
	else if (newbase_hdmi_check_spd_name(port, kt_stb_IC1000.spd_vn_name) && newbase_hdmi_check_spd_prod(port, kt_stb_IC1000.psd_pd_desc)) {
		//[WOSQRTK-9185]SKbox transient garbage
		HDMI_EMG("This is KT STB IC1000 !!!!!!!\n");
		lib_hdmi_set_toggle_hpd_z0_flag(port, _TRUE);
		lib_hdmi_dc_on_delay_check(port, _TRUE);
		lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_KT_STB_IC1000);
	}
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE
#if HDMI_STR_TOGGLE_HPD_ENABLE
	else if (newbase_hdmi_check_spd_name(port, kt_stb_IC1100.spd_vn_name) && newbase_hdmi_check_spd_prod(port, kt_stb_IC1100.psd_pd_desc)) {
		//[WOSQRTK-10334] KT STB and TV STR off/on, can not display
		HDMI_EMG("This is KT STB IC1100 !!!!!!!\n");
		lib_hdmi_set_str_toggle_hpd_state(port, HDMI_STR_HPD_TOGGLE_ENTER);
		lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_KT_STB_IC1100);
	}

	else if (newbase_hdmi_check_spd_name(port, kt_stb_GX_KT600EJ.spd_vn_name) && newbase_hdmi_check_spd_prod(port, kt_stb_GX_KT600EJ.psd_pd_desc)) {
		//[WOSQRTK-10596] KT STB and TV STR off/on, no signal
		HDMI_EMG("This is KT STB kt_stb_GX_KT600EJ !!!!!!!\n");
		lib_hdmi_set_str_toggle_hpd_state(port, HDMI_STR_HPD_TOGGLE_ENTER);
		lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_KT_STB_KT600EJ);
	}

#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE
#if HDMI_DISPLAY_DELAY_ENABLE
	else if (newbase_hdmi_check_spd_name(port, LHB755.spd_vn_name) && newbase_hdmi_check_spd_prod(port, LHB755.psd_pd_desc)) {
		//[WOSQRTK-10006] [HOME-05] LHB755 - LG, change timing flicker
		HDMI_EMG("This is DVD LHB755 !!!!!!!\n");
		lib_hdmi_set_delay_display_enable(port, _ENABLE);
		lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_LHB_755);

		if(HDMI_DISPLAY_DELAY_NONE == lib_hdmi_get_delay_display_state(port))
		{
			lib_hdmi_set_delay_display_state(port, HDMI_DISPLAY_DELAY_INIT);
			lib_hdmi_delay_check_before_display(port, _TRUE);
		}
	}
	else if (newbase_hdmi_check_spd_name(port, Samsung_GX_HC630CH.spd_vn_name) && newbase_hdmi_check_spd_prod(port, Samsung_GX_HC630CH.psd_pd_desc)) {
		//[WOSQRTK-10599] HDMI_SPECIFIC_DEVICE_SAMSUNG_GX_HC630CH change resolution flicker
		HDMI_EMG("This is SAMSUNG_GX_HC630CH !!!!!!!\n");
		lib_hdmi_set_delay_display_enable(port, _ENABLE);
		lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_SAMSUNG_GX_HC630CH);

		if(HDMI_DISPLAY_DELAY_NONE == lib_hdmi_get_delay_display_state(port))
		{
			lib_hdmi_set_delay_display_state(port, HDMI_DISPLAY_DELAY_INIT);
			lib_hdmi_delay_check_before_display(port, _TRUE);
		}
	}
#if HDCP2_2
	else if (newbase_hdmi_check_spd_name(port, Roku_Ultra_4640R.spd_vn_name) && newbase_hdmi_check_spd_prod(port, Roku_Ultra_4640R.psd_pd_desc)) {
		//[WOSQRTK-10397] [WOSQRTK-10396] [WOSQRTK-10389] [WOSQRTK-10385] Roku Ultra 4640R
		if(_ENABLE == lib_hdmi_hdcp22_is_onoff(port))
		{
			HDMI_EMG("This is Roku Ultra 4640R !!!!!!!\n");
			lib_hdmi_set_delay_display_enable(port, _ENABLE);
			lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_ROKU_ULTRA_4640R);

			if(HDMI_DISPLAY_DELAY_NONE == lib_hdmi_get_delay_display_state(port))
			{
				lib_hdmi_set_delay_display_state(port, HDMI_DISPLAY_DELAY_INIT);
				lib_hdmi_delay_check_before_display(port, _TRUE);
			}
		}
		else// if(_DISABLE == lib_hdmi_hdcp22_is_onoff(port))
		{
			HDMI_EMG("This is Roku Ultra 4640R-Not HDCP22\n");
		}
#if HDCP_STATUS_CHECK_ENABLE
		lib_hdmi_hdcp_status_check(port, _TRUE);
#else// #if HDCP_STATUS_CHECK_ENABLE
		HDMI_EMG("skip lib_hdmi_hdcp_status_check\n");
#endif //#if HDCP_STATUS_CHECK_ENABLE
		hpd_low_ms = HPD_ROKU_ULTRA_4640R_LOW_MS;
	}
#endif //#if HDCP2_2
	else if(0 == hdmi_rx[port].spd_t.type_code) {
		//if (newbase_hdmi_check_spd_name(port, GiGa_Genie_KAO_CT1100_LB.spd_vn_name) && newbase_hdmi_check_spd_prod(port, GiGa_Genie_KAO_CT1100_LB.psd_pd_desc)) {
		if((3840 == hdmi_rx[port].timing_t.h_act_len) && (2160 == hdmi_rx[port].timing_t.v_act_len) && (97 == hdmi_rx[port].avi_t.VIC))
		{
			if(_TRUE == newbase_hdmi_no_SPD_packetInfo(port))
			{
#if HDCP2_2
				if( (_ENABLE == lib_hdmi_hdcp14_is_onoff(port)) && (_ENABLE ==lib_hdmi_hdcp22_is_onoff(port)) )
#else //#if HDCP2_2
				if (_ENABLE == lib_hdmi_hdcp14_is_onoff(port))
#endif //#if HDCP2_2
				{
					//LewisLee DATE:2017/04/13
					//[WOSQRTK-10803] [STB-32] KAO_CT1100-LB(GiGA Genie)
					//for 3840x2160x60Hz, YUV420 10bit case
					//STB on, TV set DC off/on or Plug in/out
					//It will have short time green garbage
					//for this case, the HDCP 1.4/2.2 is on, it means Tx/Rx had not HDCP handshake
					//It make picture garbage
					//similar with newbase_hdmi_check_deb_condition_beforeDisp
					if ((COLOR_YUV420 == GET_H_COLOR_SPACE(port)) && (HDMI_COLOR_DEPTH_10B == GET_H_COLOR_DEPTH(port)))
					{
						HDMI_EMG("This is GiGa_Genie_KAO_CT1100_LB @ 4K2K !!!!!!!\n");
						lib_hdmi_set_delay_display_enable(port, _ENABLE);
						lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_KT_STB_KAO_CT1100);

						if(HDMI_DISPLAY_DELAY_NONE == lib_hdmi_get_delay_display_state(port))
						{
							lib_hdmi_set_delay_display_state(port, HDMI_DISPLAY_DELAY_INIT);
							lib_hdmi_delay_check_before_display(port, _TRUE);
						}
					}
				}
			}
		}
	}
#endif //#if HDMI_DISPLAY_DELAY_ENABLE
	else {
		// init variable
	}

#ifdef HDMI_AVI_INFO_COLORIMETRY_ENABLE
	if(_FALSE == newbase_hdmi_check_avi_colorimetry_info(port, _FALSE)) {
//					SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_MEASURE);
		SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_SETUP_VEDIO_PLL);
		return FALSE;
	}
#endif //#ifdef HDMI_AVI_INFO_COLORIMETRY_ENABLE

#ifdef HDMI_CHECK_HDCP_ENABLE
	if(_FALSE == newbase_hdmi_check_hdcp_condition(port, _FALSE)) {
//					SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_MEASURE);
		SET_H_VIDEO_FSM(port, MAIN_FSM_HDMI_SETUP_VEDIO_PLL);
		return FALSE;
	}
#endif //#ifdef HDMI_CHECK_HDCP_ENABLE

	newbase_hdmi_check_deb_condition_beforeDisp(port, _FALSE);

	return TRUE;

}


#endif

//USER:Lewislee DATE:2016/08/26
//to fix blu-ray player, NS-BRDVD short time unstable
unsigned char newbase_hdmi_check_stable_before_display(void)
{
	unsigned char port = current_port;


	if (bNeedDebBeforeDisp == 1)
	{
		if( DispDebCnt > 0 ) {
			HDMI_PRINTF("------>debouncing =%d, %d\n", bNeedDebBeforeDisp,  DispDebCnt);
			DispDebCnt--;
			return FALSE;
		}
	}
	bNeedDebBeforeDisp = 0;
	DispDebCondition = 0;

#if HDMI_DISPLAY_DELAY_ENABLE
	if(_FALSE == lib_hdmi_delay_check_before_display(port, _FALSE))
	{
		if(_TRUE == lib_hdmi_delay_display_action(port))
		{
			//need to debounce, skip now
//			return _FALSE;
		}
	}

	lib_hdmi_set_delay_display_enable(port, _DISABLE);
	lib_hdmi_set_delay_display_state(port, HDMI_DISPLAY_DELAY_NONE);
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

	if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_25_BYPASS_CHECK)
	{
		return TRUE;
	}

	if (GET_H_VIDEO_FSM(port) != MAIN_FSM_HDMI_VIDEO_READY)
	{
		HDMI_EMG("B-check_mode_changed_1(p=%d)\n", port);
		return FALSE;
	}

#if HDMI_TOGGLE_HPD_Z0_ENABLE
	if(_FALSE == lib_hdmi_dc_on_delay_check(port, _FALSE))
		return FALSE;
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

	if (newbase_hdmi_check_condition_change(port))
	{
		lib_hdmi_video_output(port, 0);
		lib_hdmi_audio_output(port, 0);
		HDMI_EMG("B-check_mode_changed_2(p=%d)\n", port);
		return FALSE;
	}

#if HDMI_INT_PRO_CHG_FLAG_ENABLE
	if(TRUE == lib_hdmi_get_is_interlace_change(port))
	{
		HDMI_EMG("B-interlace chg (p=%d)\n", port);
		return FALSE;
	}
#else //#if HDMI_INT_PRO_CHG_FLAG_ENABLE
	if (GET_H_INTERLACE(port) != newbase_hdmi_get_interlace_conti_mode(port))
	{
		HDMI_EMG("B-interlace chg from %d to %d\n", GET_H_INTERLACE(port), (!GET_H_INTERLACE(port)));
		return FALSE;
	}
#endif //#if HDMI_INT_PRO_CHG_FLAG_ENABLE

#if HDR10_MD_INFO_CHECK
	//Minchay@20160607-a add hdr10 md check
	if (!newbase_hdmi_hdr10_md_info_check())
	{
		HDMI_PRINTF("B-[HDR10] check mode fail\n");
		return FALSE;
	}
#endif //#if HDR10_MD_INFO_CHECK

	return TRUE;
}



//USER:LewisLee Date:2016/09/30
//to fix colorimetry NG case
unsigned char newbase_hdmi_check_avi_colorimetry_info(unsigned char port, unsigned char ucInit)
{
#ifdef HDMI_AVI_INFO_COLORIMETRY_ENABLE
	static unsigned char avi_debounce[HDMI_PORT_TOTAL_NUM];

	if(_TRUE == ucInit)
	{
		avi_debounce[port] = HDMI_AVI_INFO_COLORIMETRY_DEBOUNCE_CNT;
		return _TRUE;
	}

	if (port >= HDMI_PORT_TOTAL_NUM)
	{
		return _TRUE;
	}

	if (MODE_DVI == GET_H_MODE(port))
	{
		return _TRUE;
	}

	// 160ms/times
	if(2 == hdmi_rx[port].avi_t.type_code)
	{
		if(0 == hdmi_rx[port].avi_t.len)
		{
			//for other case
		}
		else// if(0 != hdmi_rx[port].avi_t.len)
		{
			if((0 == hdmi_rx[port].avi_t.EC) && (0 == hdmi_rx[port].avi_t.C))
			{
				if(avi_debounce[port])
				{
					//Just for 2 depth = LG Bluray BP-550 + Speaker YAMAHA YSP-5600
					//special case, wait more time
					if((0x59 == hdmi_rx[port].spd_t.VendorName[0]) && (0x61 == hdmi_rx[port].spd_t.VendorName[1])
						&& (0x6d == hdmi_rx[port].spd_t.VendorName[2]) && (0x61 == hdmi_rx[port].spd_t.VendorName[3])
						&& (0x68 == hdmi_rx[port].spd_t.VendorName[4]) && (0x61 == hdmi_rx[port].spd_t.VendorName[5])
						&& (0x00 == hdmi_rx[port].spd_t.VendorName[6]) && (0x00 == hdmi_rx[port].spd_t.VendorName[7]))
					{
						avi_debounce[port]--;
						HDMI_EMG("EC /C avi_debounce1=%x\n", avi_debounce[port]);
						return _FALSE;
					}
#if 0
					HDMI_PRINTF("VendorName [%x, %x, %x, %x, %x, %x, %x, %x]\n", hdmi_rx[port].spd_t.VendorName[0],
						hdmi_rx[port].spd_t.VendorName[1], hdmi_rx[port].spd_t.VendorName[2],
						hdmi_rx[port].spd_t.VendorName[3], hdmi_rx[port].spd_t.VendorName[4],
						hdmi_rx[port].spd_t.VendorName[5], hdmi_rx[port].spd_t.VendorName[6],
						hdmi_rx[port].spd_t.VendorName[7]);

					HDMI_EMG("EC /C avi_debounce1 exit\n");
#endif //#if 0
				}
			}
		}
	}
#endif //#ifdef HDMI_AVI_INFO_COLORIMETRY_ENABLE

	return _TRUE;
}

//==========================================

unsigned char newbase_hdmi_no_SPD_packetInfo(unsigned char nport)
{
	if( (0 == hdmi_rx[nport].spd_t.VendorName[0]) && (0 == hdmi_rx[nport].spd_t.VendorName[1])
		&& (0 == hdmi_rx[nport].spd_t.VendorName[2]) && (0 == hdmi_rx[nport].spd_t.VendorName[3])
		&& (0 == hdmi_rx[nport].spd_t.VendorName[4]) && (0 == hdmi_rx[nport].spd_t.VendorName[5])
		&& (0 == hdmi_rx[nport].spd_t.VendorName[6]) && (0 == hdmi_rx[nport].spd_t.VendorName[7]))
	{
		if((0 == hdmi_rx[nport].spd_t.ProductDesc[0]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[1])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[2]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[3])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[4]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[5])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[6]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[7])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[8]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[9])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[10]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[11])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[12]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[13])
			&& (0 == hdmi_rx[nport].spd_t.ProductDesc[14]) && (0 == hdmi_rx[nport].spd_t.ProductDesc[15]))
		{
			return _TRUE;
		}
	}
	return _FALSE;
}

//==========================================

unsigned char newbase_hdmi_check_deb_condition_beforeDisp(unsigned char nport, unsigned char ucInit)
{
//#ifdef HDMI_CHECK_HDCP_ENABLE

	unsigned int cur_v_act_len = hdmi_rx[nport].timing_t.v_act_len;
	unsigned int cur_h_act_len = hdmi_rx[nport].timing_t.h_act_len;

	if(_TRUE == ucInit)
	{
		if(bNeedDebBeforeDisp == 0 ) {
			DispDebCnt = 0;
		}
		return _TRUE;
	}

	if (nport >= HDMI_PORT_TOTAL_NUM)
	{
		return _TRUE;
	}

	if( ((cur_h_act_len==1280 && cur_v_act_len == 720) ||(cur_h_act_len==1360 && cur_v_act_len == 768)
		||(cur_h_act_len==1920 && cur_v_act_len == 540) ||(cur_h_act_len == 3840 && cur_v_act_len == 2160))
#if HDCP2_2
		&& ( (_ENABLE == lib_hdmi_hdcp14_is_onoff(nport)) && (_ENABLE ==lib_hdmi_hdcp22_is_onoff(nport)) )
#else
		&& (_ENABLE == lib_hdmi_hdcp14_is_onoff(nport))
#endif
		&& (_TRUE == newbase_hdmi_no_SPD_packetInfo(nport)) )
	{
		HDMI_PRINTF("...............checking h/v= (%d, %d) [%d-%d], c=%d, vic=%d\n", cur_h_act_len, cur_v_act_len, bNeedDebBeforeDisp, DispDebCondition, DispDebCnt, hdmi_rx[nport].avi_t.VIC);

		//deb start point: condition1
		if( (cur_h_act_len==1360 && cur_v_act_len == 768) && (bNeedDebBeforeDisp == 0) && (hdmi_rx[nport].avi_t.VIC == 0))
		{
			bNeedDebBeforeDisp = 1;
			DispDebCnt = DISP_DEB_CNT_APPLETV;
			DispDebCondition = 1;

			HDMI_PRINTF("**************************** disp deb start1!!\n");
		}

		//deb again: condition2 =1360x768->1280x720
		if((cur_h_act_len==1280 && cur_v_act_len == 720) && (DispDebCondition==1))
		{
			DispDebCnt = DISP_DEB_CNT_APPLETV;
			DispDebCondition = 2;

			HDMI_PRINTF("**************************** disp deb start2!!\n");
		}

		//LewisLee DATE:2016/12/22
		//WOSQRTK-10354, PC40000
		// 3840x2160 timing, HDMI cable plug in/out will flicker
		if( (cur_h_act_len == 3840 && cur_v_act_len == 2160) && (bNeedDebBeforeDisp == 0) && (hdmi_rx[nport].avi_t.VIC == 0))
		{
			bNeedDebBeforeDisp = 1;
			DispDebCnt = DISP_DEB_CNT_PC40000;
			DispDebCondition = 3;

			HDMI_PRINTF("**************************** disp deb start3!!\n");
		}
	}
	else {

		DispDebCondition = 0;
		DispDebCnt = 0;
		bNeedDebBeforeDisp = 0;
	}
//#endif //#ifdef HDMI_CHECK_HDCP_ENABLE

	return _TRUE;
}

//==========================================

//USER:LewisLee Date:2016/10/08
//Fix TV power on, PS 4 flocker
//No HDCP => stable clock => display => HDCP => flicker => display
//QC need


unsigned char newbase_hdmi_check_hdcp_condition(unsigned char nport, unsigned char ucInit)
{
#ifdef HDMI_CHECK_HDCP_ENABLE
	hdmi_stb_p0_hdcp_flag1_RBUS hdmi_stb_p0_hdcp_flag1_reg;
	static unsigned char HDCP_debounce[HDMI_PORT_TOTAL_NUM];

	if(_TRUE == ucInit)
	{
		HDCP_debounce[nport] = HDMI_HDCP_DEBOUNCE_CNT;
		return _TRUE;
	}

	if (nport >= HDMI_PORT_TOTAL_NUM)
	{
		return _TRUE;
	}

	if (MODE_DVI == GET_H_MODE(nport))
	{
		return _TRUE;
	}

	if(0 == HDCP_debounce[nport])
		return _TRUE;

#if 0
	HDMI_PRINTF("VendorName [%x, %x, %x, %x, %x, %x, %x, %x]\n", hdmi_rx[nport].spd_t.VendorName[0],
		hdmi_rx[nport].spd_t.VendorName[1], hdmi_rx[nport].spd_t.VendorName[2],
		hdmi_rx[nport].spd_t.VendorName[3], hdmi_rx[nport].spd_t.VendorName[4],
		hdmi_rx[nport].spd_t.VendorName[5], hdmi_rx[nport].spd_t.VendorName[6],
		hdmi_rx[nport].spd_t.VendorName[7]);

	HDMI_PRINTF("ProductDesc [%x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x]\n",
		hdmi_rx[nport].spd_t.ProductDesc[0], hdmi_rx[nport].spd_t.ProductDesc[1],
		hdmi_rx[nport].spd_t.ProductDesc[2], hdmi_rx[nport].spd_t.ProductDesc[3],
		hdmi_rx[nport].spd_t.ProductDesc[4], hdmi_rx[nport].spd_t.ProductDesc[5],
		hdmi_rx[nport].spd_t.ProductDesc[6], hdmi_rx[nport].spd_t.ProductDesc[7],
		hdmi_rx[nport].spd_t.ProductDesc[8], hdmi_rx[nport].spd_t.ProductDesc[9],
		hdmi_rx[nport].spd_t.ProductDesc[10], hdmi_rx[nport].spd_t.ProductDesc[11],
		hdmi_rx[nport].spd_t.ProductDesc[12], hdmi_rx[nport].spd_t.ProductDesc[13],
		hdmi_rx[nport].spd_t.ProductDesc[14], hdmi_rx[nport].spd_t.ProductDesc[15] );
#endif //#if 0

	// 160ms/times
	// VendorName = SCEI
	if((0x53 == hdmi_rx[nport].spd_t.VendorName[0]) && (0x43 == hdmi_rx[nport].spd_t.VendorName[1])
		&& (0x45 == hdmi_rx[nport].spd_t.VendorName[2]) && (0x49 == hdmi_rx[nport].spd_t.VendorName[3])
		&& (0x00 == hdmi_rx[nport].spd_t.VendorName[4]) && (0x00 == hdmi_rx[nport].spd_t.VendorName[5])
		&& (0x00 == hdmi_rx[nport].spd_t.VendorName[6]) && (0x00 == hdmi_rx[nport].spd_t.VendorName[7]))
	{
		// ProductDesc = PS4
		if((0x50 == hdmi_rx[nport].spd_t.ProductDesc[0]) && (0x53 == hdmi_rx[nport].spd_t.ProductDesc[1])
			&& (0x34 == hdmi_rx[nport].spd_t.ProductDesc[2]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[3])
			&& (0x00 == hdmi_rx[nport].spd_t.ProductDesc[4]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[5])
			&& (0x00 == hdmi_rx[nport].spd_t.ProductDesc[6]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[7])
			&& (0x00 == hdmi_rx[nport].spd_t.ProductDesc[8]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[9])
			&& (0x00 == hdmi_rx[nport].spd_t.ProductDesc[10]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[11])
			&& (0x00 == hdmi_rx[nport].spd_t.ProductDesc[12]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[13])
			&& (0x00 == hdmi_rx[nport].spd_t.ProductDesc[14]) && (0x00 == hdmi_rx[nport].spd_t.ProductDesc[15]))
			{
				hdmi_stb_p0_hdcp_flag1_reg.regValue = hdmi_in(HDMI_STB_HDCP_FLAG1_reg);

				if((0 == hdmi_stb_p0_hdcp_flag1_reg.wr_aksv_flag) && (0 == hdmi_stb_p0_hdcp_flag1_reg.rd_ri_flag))
				{
					//Need check HDCP 1.4 or 2.2
					HDCP_debounce[nport]--;

					HDMI_EMG("[HDCP] HDCP_debounce=%x\n", HDCP_debounce[nport]);
					if(_ENABLE == lib_hdmi_hdcp14_is_onoff(nport))
						return _FALSE;
				}
			}
	}

	HDCP_debounce[nport] = 0; // need check it can clear in here or not

//	HDMI_EMG("[HDCP] HDCP_debounce exit\n");
#endif //#ifdef HDMI_CHECK_HDCP_ENABLE

	return _TRUE;
}



#ifdef CONFIG_FORCE_RUN_I3DDMA
unsigned char newbase_hdmi_is_dispsrc(unsigned char disp)
{
	if(VSC_INPUTSRC_HDMI == Get_DisplayMode_Src(disp))
		return TRUE;
	else
		return FALSE;
}
#endif

// NOT GOOD DEMO FOR FUNCTION CALL, PLEASE USE newbase_hdmi_compatibility_check & newbase_hdmi_check_spd_name.
//Btv_STB_patch
unsigned char newbase_check_hdmi_spd_info(unsigned char nport, unsigned char enable)
{

	if (nport >= HDMI_PORT_TOTAL_NUM)
	{
		return _FALSE;
	}

	if (enable == _FALSE)
	{
		return _FALSE;
	}

#if 0
	HDMI_PRINTF("VendorName [%x, %x, %x, %x, %x, %x, %x, %x]\n", hdmi_rx[nport].spd_t.VendorName[0],
		hdmi_rx[nport].spd_t.VendorName[1], hdmi_rx[nport].spd_t.VendorName[2],
		hdmi_rx[nport].spd_t.VendorName[3], hdmi_rx[nport].spd_t.VendorName[4],
		hdmi_rx[nport].spd_t.VendorName[5], hdmi_rx[nport].spd_t.VendorName[6],
		hdmi_rx[nport].spd_t.VendorName[7]);

	HDMI_PRINTF("ProductDesc [%x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x]\n",
		hdmi_rx[nport].spd_t.ProductDesc[0], hdmi_rx[nport].spd_t.ProductDesc[1],
		hdmi_rx[nport].spd_t.ProductDesc[2], hdmi_rx[nport].spd_t.ProductDesc[3],
		hdmi_rx[nport].spd_t.ProductDesc[4], hdmi_rx[nport].spd_t.ProductDesc[5],
		hdmi_rx[nport].spd_t.ProductDesc[6], hdmi_rx[nport].spd_t.ProductDesc[7],
		hdmi_rx[nport].spd_t.ProductDesc[8], hdmi_rx[nport].spd_t.ProductDesc[9],
		hdmi_rx[nport].spd_t.ProductDesc[10], hdmi_rx[nport].spd_t.ProductDesc[11],
		hdmi_rx[nport].spd_t.ProductDesc[12], hdmi_rx[nport].spd_t.ProductDesc[13],
		hdmi_rx[nport].spd_t.ProductDesc[14], hdmi_rx[nport].spd_t.ProductDesc[15] );
#endif //#if 0

	//Btv STB
	//VendorName
	if((0x42 == hdmi_rx[nport].spd_t.VendorName[0]) && (0x72 == hdmi_rx[nport].spd_t.VendorName[1])
		&& (0x6f == hdmi_rx[nport].spd_t.VendorName[2]) && (0x61 == hdmi_rx[nport].spd_t.VendorName[3])
		&& (0x64 == hdmi_rx[nport].spd_t.VendorName[4]) && (0x63 == hdmi_rx[nport].spd_t.VendorName[5])
		&& (0x6f == hdmi_rx[nport].spd_t.VendorName[6]) && (0x6d == hdmi_rx[nport].spd_t.VendorName[7]))
	{
		// ProductDesc
		if((0x53 == hdmi_rx[nport].spd_t.ProductDesc[0]) && (0x54 == hdmi_rx[nport].spd_t.ProductDesc[1])
			&& (0x42 == hdmi_rx[nport].spd_t.ProductDesc[2]) && (0x20 == hdmi_rx[nport].spd_t.ProductDesc[3])
			&& (0x52 == hdmi_rx[nport].spd_t.ProductDesc[4]) && (0x65 == hdmi_rx[nport].spd_t.ProductDesc[5])
			&& (0x66 == hdmi_rx[nport].spd_t.ProductDesc[6]) && (0x73 == hdmi_rx[nport].spd_t.ProductDesc[7])
			&& (0x77 == hdmi_rx[nport].spd_t.ProductDesc[8]) && (0x20 == hdmi_rx[nport].spd_t.ProductDesc[9])
			&& (0x44 == hdmi_rx[nport].spd_t.ProductDesc[10]) && (0x65 == hdmi_rx[nport].spd_t.ProductDesc[11])
			&& (0x73 == hdmi_rx[nport].spd_t.ProductDesc[12]) && (0x69 == hdmi_rx[nport].spd_t.ProductDesc[13])
			&& (0x67 == hdmi_rx[nport].spd_t.ProductDesc[14]) && (0x6e == hdmi_rx[nport].spd_t.ProductDesc[15]))
			{
				HDMI_EMG("HDMI Check conditon SKBOX\n");
				return _TRUE;
			}
	}

	return _FALSE;
}


/*********************************************************************************
*
*	hdmi_lib.c
*
*********************************************************************************/


HDMI_PORT_INFO_T* newbase_hdmi_get_rx_port_info(unsigned char port)
{
	return (port < HDMI_PORT_TOTAL_NUM) ? &hdmi_rx[port] : NULL;
}

/*********************************************************************************
*
*	hdmi_lib.c
*
*********************************************************************************/



#if HDMI_STR_HPD_SPEED_UP_ENABLE
//QC need
unsigned char lib_hdmi_cec_is_enabled(void)
{
#ifdef CONFIG_RTK_KDRV_CEC
	return (CEC_CEC_CR0_get_cec_mode(hdmi_in(CEC_CEC_CR0_reg))==1?1:0);
#else
	return 0;
#endif
}
#endif

#if 1 // Need Move to Hdmi_mac.c



#if 1//Need replaced
unsigned char lib_hdmi_get_is_interlace(unsigned char nport, HDMI_MS_T ms_mode)
{
	unsigned char i;


    if (hdmi_rx[nport].hdmi_2p1_en) {
        unsigned int frl_measure_a =hdmi_in(HD21_MEAS_FRL_MEASURE_A_reg);
        HDMI_INFO("lib_hdmi_get_is_interlace, field_tog=%d, interlaced_field=%d, interlaced_vtotal=%d\n",
             HD21_MEAS_FRL_MEASURE_A_get_field_tog(frl_measure_a),
             HD21_MEAS_FRL_MEASURE_A_get_interlaced_field(frl_measure_a),
             HD21_MEAS_FRL_MEASURE_A_get_interlaced_vtotal(frl_measure_a));

        return HD21_MEAS_FRL_MEASURE_A_get_interlaced_field(hdmi_in(HD21_MEAS_FRL_MEASURE_A_reg));
    }
	if (ms_mode == HDMI_MS_ONESHOT) {
		lib_hdmi_clear_interlace_reg(nport);
		for (i=0; i<4; i++)	{
			lib_hdmi_vsync_checked(nport);
		}
	}

	if (HDMI_HDMI_VCR_get_se(hdmi_in(HDMI_HDMI_VCR_reg))) {
		lib_hdmi_clear_interlace_reg(nport);
		return 0;
	} else {
		return HDMI_HDMI_VCR_get_eot(hdmi_in(HDMI_HDMI_VCR_reg));
	}
}
#endif
unsigned char lib_hdmi_get_is_interlace_change(unsigned char nport)
{
    if (hdmi_rx[nport].hdmi_2p1_en) {
        return FALSE; //HDMI2.1 use interlace realtime update. no need to consider chg flag
    }

    if (HDMI_HDMI_VCR_get_int_pro_chg_flag(hdmi_in(HDMI_HDMI_VCR_reg)))
        return TRUE;

    return FALSE;
}


//QC need



void lib_hdmi_bch_error_clr(unsigned char nport)
{
	hdmi_mask(HDMI_HDMI_BCHCR_reg , ~(HDMI_HDMI_BCHCR_bches_mask|HDMI_HDMI_BCHCR_bches2_mask), (HDMI_HDMI_BCHCR_bches_mask|HDMI_HDMI_BCHCR_bches2_mask));
}


#if HDMI2p0


void lib_hdmi2p0_scdc_update_flag(unsigned char nport) //HDMI2.0 spec: 10.4.1.3
{
	// this function will be called by newbase_rxphy_isr each 10 ms...
	if ((lib_hdmi_scdc_get_update_flags(nport) & SCDC_UPDATE_FLAGS_RR_TEST)) 
	{
		// humm... not really know what it doing 
		lib_hdmi_scdc_set_update_flags(nport, 0x00);
		lib_hdmi_scdc_set_update_flags(nport, SCDC_UPDATE_FLAGS_RR_TEST);
	}
}

#endif



#endif

void lib_hdmi_set_toggle_hpd_z0_flag(unsigned char nport, unsigned char uc_flag)
{
#if HDMI_TOGGLE_HPD_Z0_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_toggle_hpd_z0_flag[%d]=%d\n", nport, uc_flag);
	hdmi_toggle_hpd_z0_flag[nport] = uc_flag;
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE
}

unsigned char lib_hdmi_get_toggle_hpd_z0_flag(unsigned char nport)
{
#if HDMI_TOGGLE_HPD_Z0_ENABLE
	return hdmi_toggle_hpd_z0_flag[nport];
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

	return _FALSE;
}

unsigned char lib_hdmi_toggle_hpd_z0_check(unsigned char nport, unsigned char ucInit)
{
#if HDMI_TOGGLE_HPD_Z0_ENABLE
	static unsigned int start_time = 0;
	static unsigned int current_time = 0;

	if(_FALSE == lib_hdmi_get_toggle_hpd_z0_flag(nport))
		return _TRUE;

	if(_TRUE == ucInit)
	{
		// need to think, if start time almost overflow
		start_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;
		current_time = start_time;
		HDMI_EMG("lib_hdmi_toggle_hpd_z0_check nport=%x, start=%d\n", nport, start_time);
		return _TRUE;
	}

	current_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;

	if(current_time >= start_time)
	{
		if(current_time - start_time >= HDMI_START_TOGGLE_HPD_Z0_MS)
		{
			newbase_hdmi_set_hpd(nport , 0, INIT_HPD_OFF);
			lib_hdmi_z0_set(nport, LN_ALL, 0);
			msleep(HDMI_TOGGLE_HPD_Z0_MS);
			lib_hdmi_z0_set(nport, LN_ALL, 1);
			newbase_hdmi_set_hpd(nport , 1, INIT_HPD_OFF);
			HDMI_EMG("lib_hdmi_toggle_hpd_z0_check nport=%x, time=%d\n", nport, current_time);
			lib_hdmi_set_dc_on_delay_state(nport, HDMI_DC_ON_ENABLE_DELAY);
			lib_hdmi_set_toggle_hpd_z0_flag(nport, _FALSE);
		}
	}
	else// if(current_time <= start_time)
	{
		HDMI_EMG("lib_hdmi_toggle_hpd_z0_check over, start_time=%d, current_time=%d\n", start_time, current_time);
		lib_hdmi_set_toggle_hpd_z0_flag(nport, _FALSE);
	}
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

	return _TRUE;
}

void lib_hdmi_set_dc_on_delay_state(unsigned char nport, HDMI_DC_ON_STATE_E state)
{
#if HDMI_TOGGLE_HPD_Z0_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_dc_on_delay_state[%d]=%d\n", nport, state);
	hdmi_dc_on_delay_state[nport] = state;
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE
}

HDMI_DC_ON_STATE_E lib_hdmi_get_dc_on_delay_state(unsigned char nport)
{
#if HDMI_TOGGLE_HPD_Z0_ENABLE
	return hdmi_dc_on_delay_state[nport];
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

	return HDMI_DC_ON_NONE;
}

unsigned char lib_hdmi_dc_on_delay_check(unsigned char nport, unsigned char ucInit)
{
#if HDMI_TOGGLE_HPD_Z0_ENABLE
	static unsigned int start_time = 0;
	static unsigned int current_time = 0;

	if(_FALSE == lib_hdmi_get_toggle_hpd_z0_flag(nport))
		return _TRUE;

	if(_TRUE == ucInit)
	{
		if(HDMI_DC_ON_ENABLE_DELAY != lib_hdmi_get_dc_on_delay_state(nport))
			return _TRUE;

		lib_hdmi_set_dc_on_delay_state(nport, HDMI_DC_ON_START_TIME);
		// need to think, if start time almost overflow
		start_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;
		current_time = start_time;
		HDMI_EMG("lib_hdmi_dc_on_delay_check nport=%x, start=%d\n", nport, start_time);
		return _TRUE;
	}

	if(HDMI_DC_ON_START_TIME != lib_hdmi_get_dc_on_delay_state(nport))
		return _TRUE;

	current_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;

	if(current_time >= start_time)
	{
		if(current_time - start_time >= HDMI_DC_ON_DELAY_MS)
		{
			HDMI_EMG("lib_hdmi_dc_on_delay_check nport=%x, time=%d\n", nport, current_time);
			lib_hdmi_set_dc_on_delay_state(nport, HDMI_DC_ON_FINISH);
			return _TRUE;
		}
	}
	else// if(current_time <= start_time)
	{
		HDMI_EMG("lib_hdmi_dc_on_delay_check over, start_time=%d, current_time=%d\n", start_time, current_time);
		lib_hdmi_set_dc_on_delay_state(nport, HDMI_DC_ON_NONE);
		return _TRUE;
	}

	return _FALSE;
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

	return _TRUE;
}

void lib_hdmi_set_str_toggle_hpd_state(unsigned char nport, HDMI_STR_HPD_TOGGLE_STATE_E state)
{
#if HDMI_STR_TOGGLE_HPD_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_str_toggle_hpd_state[%d]=%d\n", nport, state);
	hdmi_str_toggle_hpd_state[nport] = state;
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE
}

HDMI_STR_HPD_TOGGLE_STATE_E lib_hdmi_get_str_toggle_hpd_state(unsigned char nport)
{
#if HDMI_STR_TOGGLE_HPD_ENABLE
	return hdmi_str_toggle_hpd_state[nport];
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE

	return HDMI_STR_HPD_TOGGLE_NONE;
}

void lib_hdmi_set_delay_display_enable(unsigned char nport, unsigned char ucEnable)
{
#if HDMI_DISPLAY_DELAY_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_delay_display_enable[%d]=%d\n", nport, ucEnable);
	hdmi_delay_display_enable[nport] = ucEnable;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE
}

unsigned char lib_hdmi_get_delay_display_enable(unsigned char nport)
{
#if HDMI_DISPLAY_DELAY_ENABLE
	return hdmi_delay_display_enable[nport];
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

	return _DISABLE;
}

void lib_hdmi_set_delay_display_state(unsigned char nport, HDMI_DISPLAY_DELAY_STATE_STATE_E state)
{
#if HDMI_DISPLAY_DELAY_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_delay_display_state[%d]=%d\n", nport, state);
	hdmi_delay_display_state[nport] = state;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE
}

HDMI_DISPLAY_DELAY_STATE_STATE_E lib_hdmi_get_delay_display_state(unsigned char nport)
{
#if HDMI_DISPLAY_DELAY_ENABLE
	return hdmi_delay_display_state[nport];
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

	return HDMI_DISPLAY_DELAY_NONE;
}

unsigned char lib_hdmi_delay_check_before_display(unsigned char nport, unsigned char ucInit)
{
#if HDMI_DISPLAY_DELAY_ENABLE
	static unsigned int start_time = 0;
	static unsigned int current_time = 0;
	unsigned int Delay_Time = 0;

	if(_DISABLE == lib_hdmi_get_delay_display_enable(nport))
		return _TRUE;

	if(_TRUE == ucInit)
	{
		if(HDMI_DISPLAY_DELAY_INIT == lib_hdmi_get_delay_display_state(nport))
		{
			lib_hdmi_set_delay_display_state(nport, HDMI_DISPLAY_DELAY_START);
			// need to think, if start time almost overflow
			start_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;
			current_time = start_time;
			HDMI_EMG("lib_hdmi_delay_check_before_display nport=%x, start=%d\n", nport, start_time);
		}
		return _TRUE;
	}

	if(HDMI_DISPLAY_DELAY_START != lib_hdmi_get_delay_display_state(nport))
		return _TRUE;

	Delay_Time = lib_hdmi_get_delay_display_time(nport);
	current_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;

	if(current_time >= start_time)
	{
		if(current_time - start_time >= Delay_Time)
		{
			HDMI_EMG("lib_hdmi_delay_check_before_display nport=%x, time=%d\n", nport, current_time);
			lib_hdmi_set_delay_display_state(nport, HDMI_DISPLAY_DELAY_FINISH);
			return _TRUE;
		}
	}
	else// if(current_time <= start_time)
	{
		HDMI_EMG("lib_hdmi_delay_check_before_display over, start_time=%d, current_time=%d\n", start_time, current_time);
		lib_hdmi_set_delay_display_state(nport, HDMI_DISPLAY_DELAY_FINISH);
		return _TRUE;
	}

	return _FALSE;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

	return _TRUE;
}

unsigned int lib_hdmi_get_delay_display_time(unsigned char nport)
{
	unsigned int DelayTime = 0; //ms

	switch(lib_hdmi_get_specific_device(nport))
	{
		case HDMI_SPECIFIC_DEVICE_LHB_755:
			DelayTime = HDMI_DISPLAY_DELAY_LHB_755_MS;
		break;

		case HDMI_SPECIFIC_DEVICE_ROKU_ULTRA_4640R:
			DelayTime = HDMI_DISPLAY_DELAY_ROKU_ULTRA_4640R_MS;
		break;

		case HDMI_SPECIFIC_DEVICE_SAMSUNG_GX_HC630CH:
			DelayTime = HDMI_DISPLAY_DELAY_SAMSUNG_GX_HC630CH_MS;
		break;

		case HDMI_SPECIFIC_DEVICE_KT_STB_KAO_CT1100:
			DelayTime = HDMI_DISPLAY_DELAY_KT_STB_KAO_CT1100_MS;
		break;

//		case HDMI_SPECIFIC_DEVICE_NONE:
		default:
			//other device, all 0
			DelayTime = 0;
		break;
	}

	return DelayTime;
}

//USER:LewisLee DATE:2016/12/27
//select delay by debounce or delay function
//don't delay too long
unsigned char lib_hdmi_delay_display_action(unsigned char nport)
{
#if HDMI_DISPLAY_DELAY_ENABLE
	unsigned char Need_Debounce = _FALSE;

	switch(lib_hdmi_get_specific_device(nport))
	{
		case HDMI_SPECIFIC_DEVICE_LHB_755:
			msleep(HDMI_DISPLAY_DELAY_LHB_755_MS_2); //OK
			Need_Debounce = _FALSE;
			HDMI_EMG("LHB_755 delay\n");
		break;

		case HDMI_SPECIFIC_DEVICE_ROKU_ULTRA_4640R:
			msleep(HDMI_DISPLAY_DELAY_ROKU_ULTRA_4640R_MS); //OK
			Need_Debounce = _FALSE;
			HDMI_EMG("Roku Ultra 4640R\n");
		break;

		case HDMI_SPECIFIC_DEVICE_SAMSUNG_GX_HC630CH:
			msleep(HDMI_DISPLAY_DELAY_SAMSUNG_GX_HC630CH_MS);
			Need_Debounce = _FALSE;
			HDMI_EMG("SAMSUNG_GX_HC630CH\n");
		break;

		case HDMI_SPECIFIC_DEVICE_KT_STB_KAO_CT1100:
			msleep(HDMI_DISPLAY_DELAY_KT_STB_KAO_CT1100_MS);
			Need_Debounce = _FALSE;
			HDMI_EMG("KT_STB_KAO_CT1100\n");
		break;

//		case HDMI_SPECIFIC_DEVICE_NONE:
		default:
			//other device, all 0
			Need_Debounce = _FALSE;
		break;
	}

	return Need_Debounce;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

	return _FALSE;
}

void lib_hdmi_set_specific_device(unsigned char nport, HDMI_SPECIFIC_DEVICE_E device)
{
#if 1 //#if HDMI_DISPLAY_DELAY_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_specific_device[%d]=%d\n", nport, device);
	hdmi_specific_device[nport] = device;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE
}

HDMI_SPECIFIC_DEVICE_E lib_hdmi_get_specific_device(unsigned char nport)
{
#if 1 //#if HDMI_DISPLAY_DELAY_ENABLE
	return hdmi_specific_device[nport];
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

	return HDMI_SPECIFIC_DEVICE_NONE;
}

void lib_hdmi_misc_variable_initial(unsigned char port)
{
	lib_hdmi_set_specific_device(port, HDMI_SPECIFIC_DEVICE_NONE);

#if HDMI_DISPLAY_DELAY_ENABLE
	lib_hdmi_set_delay_display_enable(port, _DISABLE);
	lib_hdmi_set_delay_display_state(port, HDMI_DISPLAY_DELAY_NONE);
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

#if HDCP_STATUS_CHECK_ENABLE
	lib_hdmi_set_hdcp_check_toggle_state(port, HDMI_HDCP_TOGGLE_NONE);
#endif //#if HDCP_STATUS_CHECK_ENABLE
}

void lib_hdmi_set_hdcp_check_toggle_state(unsigned char nport, HDMI_HDCP_TOGGLE_STATUS_T status)
{
#if HDCP_STATUS_CHECK_ENABLE
//	HDMI_PRINTF("lib_hdmi_set_delay_display_enable[%d]=%d\n", nport, ucEnable);
	hdmi_hdcp_check_toggle_state[nport] = status;
#endif //#if HDCP_STATUS_CHECK_ENABLE
}

HDMI_HDCP_TOGGLE_STATUS_T lib_hdmi_get_hdcp_check_toggle_state(unsigned char nport)
{
#if HDCP_STATUS_CHECK_ENABLE
	return hdmi_hdcp_check_toggle_state[nport];
#endif //#if HDCP_STATUS_CHECK_ENABLE

	return HDMI_HDCP_TOGGLE_NONE;
}

unsigned char lib_hdmi_hdcp_status_check(unsigned char nport, unsigned char IsInit)
{
#if HDCP2_2
#if HDCP_STATUS_CHECK_ENABLE
	static unsigned int start_time = 0;
	static unsigned int current_time = 0;

	if(_TRUE == IsInit)
	{
		// need to think, if start time almost overflow
		start_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;
		current_time = start_time;
		lib_hdmi_set_hdcp_check_toggle_state(nport, HDMI_HDCP_TOGGLE_ENABLE);
		HDMI_EMG("[HDCP] lib_hdmi_hdcp_status_check nport=%x, start=%d\n", nport, start_time);

		//Write clear ADNE_FLAG
		hdmi_out(HDMI_STB_HDCP_FLAG1_reg, _BIT7);
		return _TRUE;
	}

	if(lib_hdmi_get_hdcp_check_toggle_state(nport) != HDMI_HDCP_TOGGLE_ENABLE)
		return _TRUE;

	if(_ENABLE == lib_hdmi_hdcp22_is_onoff(nport))
	{
		if(HDMI_STB_HDCP_2p2_SR0_get_state(hdmi_in(HDMI_STB_HDCP_2p2_SR0_reg)) >= HDMI_HDCP_2P2_AUTHENTICATED)
				return _TRUE;
		}
	else if(_ENABLE == lib_hdmi_hdcp14_is_onoff(nport))
		{
		if(1 == HDMI_STB_HDCP_FLAG1_get_adne_flag(hdmi_in(HDMI_STB_HDCP_FLAG1_reg)))
			return _TRUE;
		}
	else
	{
		return _TRUE;
	}


	current_time = hdmi_in(TIMER_SCPU_CLK90K_LO_reg)/90;

	if(current_time >= start_time)
	{
		if(current_time - start_time >= HDCP_STATUS_CHECK_TOGGLE_MS)
		{
			HDMI_EMG("[HDCP] lib_hdmi_hdcp_status_check nport=%x, time=%d\n", nport, current_time);
			if(_TRUE == lib_hdmi_hdcp_toggle_action(nport))
				return _FALSE;

			return _TRUE;
		}
	}
	else// if(current_time <= start_time)
	{
		HDMI_EMG("[HDCP] lib_hdmi_hdcp_status_check over, start_time=%d, current_time=%d\n", start_time, current_time);
		lib_hdmi_set_hdcp_check_toggle_state(nport, HDMI_HDCP_TOGGLE_EXIT);
		return _TRUE;
	}
#endif //#if HDCP_STATUS_CHECK_ENABLE
#endif //#if HDCP2_2

	return _TRUE;
}

//USER:LewisLee DATE:2017/01/04
//In different device, use different solution
unsigned char lib_hdmi_hdcp_toggle_action(unsigned char nport)
{
#if HDCP_STATUS_CHECK_ENABLE
	unsigned char result = _FALSE;

	switch(lib_hdmi_get_specific_device(nport))
	{
		case HDMI_SPECIFIC_DEVICE_ROKU_ULTRA_4640R:
#if 1
			newbase_hdmi_set_hpd(nport , 0, INIT_HPD_OFF);
			msleep(HDCP_STATUS_CHECK_ROKU_ULTRA_4640R_HPD_MS);
			newbase_hdmi_set_hpd(nport , 1, INIT_HPD_OFF);
#else //#if 1
			newbase_hdmi_set_hpd(nport , 0, INIT_HPD_OFF);
			if (0 == _start_hpd_delay_wq(HDCP_STATUS_CHECK_ROKU_ULTRA_4640R_HPD_MS)) {
				HDMI_EMG("Roku Ultra 4640R toggle HPD\n");
			}
#endif //#if 1
			result = _TRUE;
		break;

//		case HDMI_SPECIFIC_DEVICE_NONE:
		default:
			result = _FALSE;
		break;
	}

	return result;
#endif //#if HDCP_STATUS_CHECK_ENABLE

	return _FALSE;
}

#if 1//Need Move Hdmi_mac_2p1.c

#if HDMI2p1





//QC need
void lib_hdmi_2p1_path_sel(unsigned char nport, unsigned char frl_mode, unsigned lane_mode)
{
	if (nport >= HD20_PORT)
		return;
	
	if (frl_mode == MODE_TMDS) {
		hdmi_mask(HD21_CR_reg,~(HDMI21_P0_HD21_CR_hd21_en_mask),0x0);

	} else {
		if (lane_mode == HDMI_4LANE)
			hdmi_mask(HD21_CR_reg,~HDMI21_P0_HD21_CR_lane_mode_fw_mask,HDMI21_P0_HD21_CR_lane_mode_fw_mask);
		else
			hdmi_mask(HD21_CR_reg,~HDMI21_P0_HD21_CR_lane_mode_fw_mask,0x0);

		hdmi_mask(HD21_CR_reg,~(HDMI21_P0_HD21_CR_hd21_en_mask),HDMI21_P0_HD21_CR_hd21_en_mask);
	}
}

#endif


//-------------------------------------------------------------------------
// HDR
//-------------------------------------------------------------------------


/*---------------------------------------------------
 * Func : lib_hdm_hdr_emp_sram_sel
 *
 * Desc : share sram for HDMI2.0 or HDMI2.1
 *
 * Para : nport : HDMI port number
 *        hd21  : 0 : share SRAM for HDMI 2.0
 *                1 : share SRAM for HDMI 2.1
 *
 * Retn : 0 : success
 *--------------------------------------------------*/
int lib_hdm_hdr_emp_sram_sel(
    unsigned char           nport,
    unsigned char           hd21
    )
{
    hdmi_mask(HDMI_EM_CT_reg, HDMI_em_ct_hdr_hd20_hd21_sel_mask, (hd21) ? HDMI_em_ct_hdr_hd20_hd21_sel_mask : 0);
    return 0;
}

//QC need
void newbase_set_frl_mode(unsigned char port, unsigned char frl_mode)
{
	hdmi_rx[port].frl_mode = frl_mode;
}

//QC need
void newbase_set_lane_num(unsigned char port, unsigned char lane_num)
{
	hdmi_rx[port].lane_num = lane_num;
}

//QC need
unsigned char newbase_get_lane_num(unsigned char port)
{
        return hdmi_rx[port].lane_num;
}

//QC need
unsigned char newbase_get_frl_mode(unsigned char port) //scdc
{
	return hdmi_rx[port].frl_mode;
}


//QC need
unsigned char newbase_hdmi_is_legacy_mode(unsigned char port)
{
	if (hdmi_rx[port].lts == LTS_P)
		return 0;
	else
		return 1;

}

//QC need
unsigned char newbase_hdmi_get_lts_state(unsigned char port)
{
	return hdmi_rx[port].lts;
}

//QC need
unsigned char newbase_hdmi_force_lts_pass(unsigned char port)
{
	hdmi_rx[port].lts = LTS_P;
}

void newbase_hdmi_frl_start(unsigned char nport,unsigned char frl_mode)
{
	unsigned char scdc_status;

	if (nport >= HD20_PORT)
		return;
	if (frl_mode == MODE_TMDS) 
		return;
	
	scdc_status= lib_hdmi_scdc_get_update_flags(nport);
	scdc_status = scdc_status | FRL_start_mask;
	lib_hdmi_scdc_set_update_flags(nport,scdc_status);

}



static unsigned char _wait_scdc_flt_update_cleared(unsigned char port, unsigned int timeout)
{
    while (timeout--) 
    {
        if (lib_hdmi_scdc_get_flt_update(port)==0) 
        {
            HDMI_EMG("Source cleared flt_update time=%d!!\n",timeout);
            return 1;
        }
        mdelay(1);
    }

    HDMI_EMG("Wait flt update clear timeout!!\n");
    return 0;
}



void polling_flt_done(unsigned char nport)
{

	unsigned int k_cnt = 5000;

	while ((lib_hdmi_scdc_get_update_flags(nport) & FLT_update_mask) && (k_cnt > 0)){
		udelay(10);
		k_cnt--;
		
	}

	if (!k_cnt) {
			HDMI_EMG("polling flt timeout\n");
	}

}

#if (!defined(RDC_SINGLE_LANE))

int hdmi_link_communicate(unsigned char nport,unsigned char lane)
{
	#define CLR_ERR_CNT_MASK	0x10
	#define MAX_STABLE_CNT	1
	#define B_LANE	0x1
	#define G_LANE	0x2
	#define R_LANE	0x4
	#define C_LANE	0x8
	#define ALL_3LANE 0x7
	#define ALL_4LANE 0xf
	
	unsigned char lock_st = 0, pop_st = 0, pass_st = 0;
	unsigned int err_b = 0, err_g = 0, err_r = 0, err_c = 0;
	unsigned char stable_b = 0, stable_g = 0,stable_r = 0,stable_c = 0;
	unsigned char scdc_status;
	unsigned int timeout_cnt = 4096;
	unsigned int rate;
	unsigned char ltp3_sel;
	unsigned int timeout = 200;
	unsigned int i = 0;
	
	hdmi_mask(HD21_LT_B_reg,~HDMI21_P0_HD21_LT_B_reset_mask,HDMI21_P0_HD21_LT_B_reset_mask);
	hdmi_mask(HD21_LT_G_reg,~HDMI21_P0_HD21_LT_G_reset_mask,HDMI21_P0_HD21_LT_G_reset_mask);
	hdmi_mask(HD21_LT_R_reg,~HDMI21_P0_HD21_LT_R_reset_mask,HDMI21_P0_HD21_LT_R_reset_mask);
	
	if (lane == HDMI_4LANE)
		hdmi_mask(HD21_LT_C_reg,~HDMI21_P0_HD21_LT_C_reset_mask,HDMI21_P0_HD21_LT_C_reset_mask);
	
	udelay(5);
	hdmi_mask(HD21_LT_B_reg,~HDMI21_P0_HD21_LT_B_reset_mask,0x0);
	hdmi_mask(HD21_LT_G_reg,~HDMI21_P0_HD21_LT_G_reset_mask,0x0);
	hdmi_mask(HD21_LT_R_reg,~HDMI21_P0_HD21_LT_R_reset_mask,0x0);
	
	if (lane == HDMI_4LANE)
		hdmi_mask(HD21_LT_C_reg,~HDMI21_P0_HD21_LT_C_reset_mask,0x0);

	hdmi_mask(HD21_LT_B_reg,~HDMI21_P0_HD21_LT_B_ln_req_mask,HDMI21_P0_HD21_LT_B_ln_req(0x5));
	hdmi_mask(HD21_LT_G_reg,~HDMI21_P0_HD21_LT_G_ln_req_mask,HDMI21_P0_HD21_LT_G_ln_req(0x6));
	hdmi_mask(HD21_LT_R_reg,~HDMI21_P0_HD21_LT_R_ln_req_mask,HDMI21_P0_HD21_LT_R_ln_req(0x7));
	
	if (lane == HDMI_4LANE)
		hdmi_mask(HD21_LT_C_reg,~HDMI21_P0_HD21_LT_C_ln_req_mask,HDMI21_P0_HD21_LT_C_ln_req(0x8));

	//ltp enable
	switch(hdmi_rx[nport].frl_mode) {
		
		case MODE_TMDS:
			ltp3_sel = 4;
			break;
		case MODE_FRL_3G_3_LANE:
			ltp3_sel = 4;
			break;
		case MODE_FRL_6G_3_LANE:
			ltp3_sel = 3;
			break;
		case MODE_FRL_6G_4_LANE:
			ltp3_sel = 3;
			break;
		case MODE_FRL_8G_4_LANE:
			ltp3_sel = 2;
			break;
		case MODE_FRL_10G_4_LANE:
			ltp3_sel = 1;
			break;
		case MODE_FRL_12G_4_LANE:
			ltp3_sel = 0;
			break;

	}

         timeout *= 27000;
	
	
	lib_hdmi_scdc_set_ln01_req(nport,0x65);
	lib_hdmi_scdc_set_ln23_req(nport,0x87);

	// 4. wait source to send data
    //-----------------------------------------------------
    	_wait_scdc_flt_update_cleared(nport, 10);


	  hdmi_out(HDMI21_P0_HD21_LT_ALL_reg, HDMI21_P0_HD21_LT_ALL_ltp_en(1)|
                                        HDMI21_P0_HD21_LT_ALL_ltp3_sel(ltp3_sel)|
                                        HDMI21_P0_HD21_LT_ALL_timeout_set(timeout));


#if 0
	for (i = 0 ; i < 100 ; ++i) {
		HDMI_EMG("[2]%x\n",hdmi_in(HD21_LT_B_reg));
		hdmi_mask(HD21_LT_B_reg,~(HDMI21_P0_HD21_LT_B_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_B_popup_mask);
		//mdelay(1);
		HDMI_EMG("[3]%x\n",hdmi_in(HD21_LT_B_reg));

	}
	
	return 1;
#endif	

	while(--timeout_cnt) {
		
		lock_st = lock_st | HDMI21_P0_HD21_LT_B_get_lock_valid(hdmi_in(HD21_LT_B_reg));
		lock_st = lock_st | (HDMI21_P0_HD21_LT_G_get_lock_valid(hdmi_in(HD21_LT_G_reg)) << 1);
		lock_st = lock_st | (HDMI21_P0_HD21_LT_R_get_lock_valid(hdmi_in(HD21_LT_R_reg)) << 2);
		if (lane == HDMI_4LANE) {
			lock_st = lock_st | (HDMI21_P0_HD21_LT_C_get_lock_valid(hdmi_in(HD21_LT_C_reg)) << 3);
			
				
		}
	
		if ((lock_st & B_LANE)  && (!(pass_st & B_LANE))){
			hdmi_mask(HD21_LT_B_reg,~(HDMI21_P0_HD21_LT_B_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_B_popup_mask);
			pop_st = pop_st |B_LANE;
		}

#if 1
		
		if ((lock_st & G_LANE) && (!(pass_st & G_LANE))) {
			hdmi_mask(HD21_LT_G_reg,~(HDMI21_P0_HD21_LT_G_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_G_popup_mask);
			pop_st = pop_st |G_LANE;
		}
		if ((lock_st & R_LANE) && (!(pass_st & R_LANE))){
			hdmi_mask(HD21_LT_R_reg,~(HDMI21_P0_HD21_LT_R_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_R_popup_mask);
			pop_st = pop_st |R_LANE;
		}
		if ((lock_st & C_LANE)&& (!(pass_st & C_LANE)))  {
			hdmi_mask(HD21_LT_C_reg,~(HDMI21_P0_HD21_LT_C_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_C_popup_mask);
			pop_st = pop_st |C_LANE;
		}
#endif
		if (pop_st & B_LANE) {
			if (!HDMI21_P0_HD21_LT_B_get_popup(hdmi_in(HD21_LT_B_reg))){
				err_b = HDMI21_P0_HD21_LT_B_get_err_cnt(hdmi_in(HD21_LT_B_reg));

				//HDMI_EMG("err_b=%d reg=%x\n",err_b,hdmi_in(HD21_LT_B_reg));
				
				if (err_b == 0) 
					stable_b++;
				else {
					HDMI_EMG("B lane link error=%d\n",err_b);
					
					scdc_status = lib_hdmi_scdc_get_ln01_req(nport);
					scdc_status = (scdc_status & 0xf0) | 0x0e;
					lib_hdmi_scdc_set_ln01_req(nport,scdc_status);
					polling_flt_done(nport);
					stable_b = 0;
				}
				if (stable_b >= MAX_STABLE_CNT) {
					scdc_status = lib_hdmi_scdc_get_ln01_req(nport);
					scdc_status = (scdc_status & 0xf0);   //LTP_REQ = 0
			//		lib_hdmi_scdc_set_ln01_req(port,scdc_status);
			//		polling_flt_done(port);
					
					pass_st = pass_st | B_LANE;
				}
			}
		}

#if 1
		if (pop_st & G_LANE) {
			if (!HDMI21_P0_HD21_LT_G_get_popup(hdmi_in(HD21_LT_G_reg))) {
				err_g = HDMI21_P0_HD21_LT_G_get_err_cnt(hdmi_in(HD21_LT_G_reg));
				if (err_g == 0)
					stable_g++;	
				else {
					HDMI_EMG("G lane link error=%d\n",err_g);
					scdc_status = lib_hdmi_scdc_get_ln01_req(nport);
					scdc_status = (scdc_status & 0x0f) | 0xe0;
					lib_hdmi_scdc_set_ln01_req(nport,scdc_status);
					polling_flt_done(nport);
					stable_g = 0;
					
				}
				if (stable_g >= MAX_STABLE_CNT) {
					scdc_status = lib_hdmi_scdc_get_ln01_req(nport);
					scdc_status = (scdc_status & 0x0f);   //LTP_REQ = 0
			//		lib_hdmi_scdc_set_ln01_req(port,scdc_status);
			//		polling_flt_done(port);
					pass_st = pass_st | G_LANE;
				}
			}
		}

		if (pop_st & R_LANE) {
			if (!HDMI21_P0_HD21_LT_R_get_popup(hdmi_in(HD21_LT_R_reg))) {
				err_r = HDMI21_P0_HD21_LT_R_get_err_cnt(hdmi_in(HD21_LT_R_reg));
				if (err_r == 0)
					stable_r++;
				else {
					HDMI_EMG("R lane link error=%d\n",err_r);
					scdc_status = lib_hdmi_scdc_get_ln23_req(nport);
					scdc_status = (scdc_status & 0xf0) | 0x0e;
					lib_hdmi_scdc_set_ln23_req(nport,scdc_status);
					polling_flt_done(nport);
					stable_r = 0;
				}
				if (stable_r >= MAX_STABLE_CNT) {
					
					scdc_status = lib_hdmi_scdc_get_ln23_req(nport);
					scdc_status = (scdc_status & 0xf0);   //LTP_REQ = 0
			//		lib_hdmi_scdc_set_ln23_req(port,scdc_status);
			//		polling_flt_done(port);
					pass_st = pass_st | R_LANE;
				}
			}
		}

		if (pop_st & C_LANE) {
			if (!HDMI21_P0_HD21_LT_C_get_popup(hdmi_in(HD21_LT_C_reg))) {
				err_c = HDMI21_P0_HD21_LT_C_get_err_cnt(hdmi_in(HD21_LT_C_reg));
				if (err_c == 0) 
					stable_c++;
				else {
					HDMI_EMG("CK lane link error=%d\n",err_c);
					scdc_status = lib_hdmi_scdc_get_ln23_req(nport);
					scdc_status = (scdc_status & 0x0f) | 0xe0;
					lib_hdmi_scdc_set_ln23_req(nport,scdc_status);
					polling_flt_done(nport);
					stable_c = 0;
				}
				if (stable_c >= MAX_STABLE_CNT) {
					
					scdc_status = lib_hdmi_scdc_get_ln23_req(nport);
					scdc_status = (scdc_status & 0x0f);  //LTP_REQ = 0
					//lib_hdmi_scdc_set_ln23_req(port,scdc_status);
			//		polling_flt_done(port);
					pass_st = pass_st | C_LANE;
				}
			}
		}
#endif

		if ((lane ==HDMI_4LANE) && ((pass_st & ALL_4LANE) == ALL_4LANE))
			break;
		if ((lane == HDMI_3LANE) && ((pass_st & ALL_3LANE) == ALL_3LANE))
			break;

                 udelay(5);
		//detectt tmds return fail;
	}

	if (!timeout_cnt) {
			HDMI_EMG("link training timeout\n");
			return 0;
	}

	HDMI_EMG("polling status....\n");
	lib_hdmi_scdc_set_ln01_req(nport,0x00);
	lib_hdmi_scdc_set_ln23_req(nport,0x00);
//	_wait_scdc_flt_update_cleared(port, 10);
	polling_flt_done(nport);

#if 0	
	if (pass_st & B_LANE) {
		HDMI_EMG("B LANE PASS\n");
	}
	else  {
		if ((lock_st & B_LANE)) {
			HDMI_EMG("B LANE FAIL err_cnt=%d\n", err_b);
		} else {
			HDMI_EMG("B LANE FAIL no lock\n");
		}
	}
	
	if (pass_st & G_LANE) {
		HDMI_EMG("G LANE PASS\n");
	}
	else {
		if ((lock_st & G_LANE)) {
			HDMI_EMG("G LANE FAIL err_cnt=%d\n", err_g);
		} else {
			HDMI_EMG("G LANE FAIL no lock\n");
		}
	}

	if (pass_st & R_LANE) {
		HDMI_EMG("R LANE PASS\n");
	}
	else {
		if ((lock_st & R_LANE)) {
			HDMI_EMG("R LANE FAIL err_cnt=%d\n", err_r);
		} else {
			HDMI_EMG("R LANE FAIL no lock\n");
		}
	}

	if (pass_st & C_LANE) {
		HDMI_EMG("C LANE PASS\n");
	}
	else{
		if ((lock_st & C_LANE)) {
			HDMI_EMG("C LANE FAIL err_cnt=%d\n", err_c);
		} else {
			HDMI_EMG("C LANE FAIL no lock\n");
		}
	}
#endif
	//ltp disable
	hdmi_mask(HD21_LT_ALL_reg,~HDMI21_P0_HD21_LT_ALL_ltp_en_mask,0x0);

	return 1;

}

#else

int hdmi_link_communicate(unsigned char nport,unsigned char lane)
{
	#define CLR_ERR_CNT_MASK	0x10
	#define MAX_STABLE_CNT	10
	#define B_LANE	0x1
	#define G_LANE	0x2
	#define R_LANE	0x4
	#define C_LANE	0x8
	#define ALL_3LANE 0x7
	#define ALL_4LANE 0xf
	
	unsigned char lock_st = 0, pop_st = 0, pass_st = 0;
	unsigned int err_b, err_g, err_r, err_c;
	unsigned char stable_b = 0, stable_g = 0,stable_r = 0,stable_c = 0;
	unsigned char scdc_status;
	unsigned int timeout_cnt = 256;
	unsigned char cnt = 0;
	
	hdmi_mask(HD21_LT_B_reg,~HDMI21_P0_HD21_LT_B_reset_mask,HDMI21_P0_HD21_LT_B_reset_mask);
	hdmi_mask(HD21_LT_G_reg,~HDMI21_P0_HD21_LT_G_reset_mask,HDMI21_P0_HD21_LT_G_reset_mask);
	hdmi_mask(HD21_LT_R_reg,~HDMI21_P0_HD21_LT_R_reset_mask,HDMI21_P0_HD21_LT_R_reset_mask);
	
	if (lane == HDMI_4LANE)
		hdmi_mask(HD21_LT_C_reg,~HDMI21_P0_HD21_LT_C_reset_mask,HDMI21_P0_HD21_LT_C_reset_mask);
	
	udelay(5);
	hdmi_mask(HD21_LT_B_reg,~HDMI21_P0_HD21_LT_B_reset_mask,0x0);
	hdmi_mask(HD21_LT_G_reg,~HDMI21_P0_HD21_LT_G_reset_mask,0x0);
	hdmi_mask(HD21_LT_R_reg,~HDMI21_P0_HD21_LT_R_reset_mask,0x0);
	
	if (lane == HDMI_4LANE)
		hdmi_mask(HD21_LT_C_reg,~HDMI21_P0_HD21_LT_C_reset_mask,0x0);

	hdmi_mask(HD21_LT_B_reg,~HDMI21_P0_HD21_LT_B_ln_req_mask,HDMI21_P0_HD21_LT_B_ln_req(0x5));
	hdmi_mask(HD21_LT_G_reg,~HDMI21_P0_HD21_LT_G_ln_req_mask,HDMI21_P0_HD21_LT_G_ln_req(0x6));
	hdmi_mask(HD21_LT_R_reg,~HDMI21_P0_HD21_LT_R_ln_req_mask,HDMI21_P0_HD21_LT_R_ln_req(0x7));
	
	if (lane == HDMI_4LANE)
		hdmi_mask(HD21_LT_C_reg,~HDMI21_P0_HD21_LT_C_ln_req_mask,HDMI21_P0_HD21_LT_C_ln_req(0x8));

	//ltp enable
	hdmi_mask(HD21_LT_ALL_reg,~HDMI21_P0_HD21_LT_ALL_ltp_en_mask,HDMI21_P0_HD21_LT_ALL_ltp_en_mask);

	while(--timeout_cnt) {
		lock_st = lock_st | HDMI21_P0_HD21_LT_B_get_lock_valid(hdmi_in(HD21_LT_B_reg));
		lock_st = lock_st | (HDMI21_P0_HD21_LT_G_get_lock_valid(hdmi_in(HD21_LT_G_reg)) << 1);
		lock_st = lock_st | (HDMI21_P0_HD21_LT_R_get_lock_valid(hdmi_in(HD21_LT_R_reg)) << 2);
		if (lane == HDMI_4LANE) {
			lock_st = lock_st | (HDMI21_P0_HD21_LT_C_get_lock_valid(hdmi_in(HD21_LT_C_reg)) << 3);
		}
	
		if ((lock_st & B_LANE)  && (!(pass_st & B_LANE))){
			hdmi_mask(HD21_LT_B_reg,~(HDMI21_P0_HD21_LT_B_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_B_popup_mask);
			pop_st = pop_st |B_LANE;
		}
		if ((lock_st & G_LANE) && (!(pass_st & G_LANE))) {
			hdmi_mask(HD21_LT_G_reg,~(HDMI21_P0_HD21_LT_G_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_G_popup_mask);
			pop_st = pop_st |G_LANE;
		}
		if ((lock_st & R_LANE) && (!(pass_st & R_LANE))){
			hdmi_mask(HD21_LT_R_reg,~(HDMI21_P0_HD21_LT_R_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_R_popup_mask);
			pop_st = pop_st |R_LANE;
		}
		if ((lock_st & C_LANE)&& (!(pass_st & C_LANE)))  {
			hdmi_mask(HD21_LT_C_reg,~(HDMI21_P0_HD21_LT_C_popup_mask|CLR_ERR_CNT_MASK),HDMI21_P0_HD21_LT_C_popup_mask);
			pop_st = pop_st |C_LANE;
		}

		if (pop_st & B_LANE) {
			if (!HDMI21_P0_HD21_LT_B_get_popup(hdmi_in(HD21_LT_B_reg))){
				err_b = HDMI21_P0_HD21_LT_B_get_err_cnt(hdmi_in(HD21_LT_B_reg));
				if (err_b == 0) 
					stable_b++;
				else {
					stable_b = 0;
				}
				if (stable_b >= MAX_STABLE_CNT) {
					pass_st = pass_st | B_LANE;
				}
			}
		}

		if (pop_st & G_LANE) {
			if (!HDMI21_P0_HD21_LT_G_get_popup(hdmi_in(HD21_LT_G_reg))) {
				err_g = HDMI21_P0_HD21_LT_G_get_err_cnt(hdmi_in(HD21_LT_G_reg));
				if (err_g == 0)
					stable_g++;	
				else {
					stable_g = 0;
				}
				if (stable_g >= MAX_STABLE_CNT) {
					pass_st = pass_st | G_LANE;
				}
			}
		}

		if (pop_st & R_LANE) {
			if (!HDMI21_P0_HD21_LT_R_get_popup(hdmi_in(HD21_LT_R_reg))) {
				err_r = HDMI21_P0_HD21_LT_R_get_err_cnt(hdmi_in(HD21_LT_R_reg));
				if (err_r == 0)
					stable_r++;
				else {
					stable_r = 0;
				}
				if (stable_r >= MAX_STABLE_CNT) {
					pass_st = pass_st | R_LANE;
				}
			}
		}

		if (pop_st & C_LANE) {
			if (!HDMI21_P0_HD21_LT_C_get_popup(hdmi_in(HD21_LT_C_reg))) {
				err_c = HDMI21_P0_HD21_LT_C_get_err_cnt(hdmi_in(HD21_LT_C_reg));
				if (err_c == 0) 
					stable_c++;
				else {
					stable_c = 0;
				}
				if (stable_c >= MAX_STABLE_CNT) {
					pass_st = pass_st | C_LANE;
				}
			}
		}

		udelay(5);
		//detectt tmds return fail;
	}


	if (pass_st & B_LANE) {
		HDMI_EMG("B LANE PASS\n");
		cnt++;
	}
	else  {
		if ((lock_st & B_LANE)) {
			HDMI_EMG("B LANE FAIL err_cnt=%d\n", err_b);
		} else {
			HDMI_EMG("B LANE FAIL no lock\n");
		}
	}
	
	if (pass_st & G_LANE) {
		HDMI_EMG("G LANE PASS\n");
		cnt++;
	}
	else {
		if ((lock_st & G_LANE)) {
			HDMI_EMG("G LANE FAIL err_cnt=%d\n", err_g);
		} else {
			HDMI_EMG("G LANE FAIL no lock\n");
		}
	}

	if (pass_st & R_LANE) {
		HDMI_EMG("R LANE PASS\n");
		cnt++;
	}
	else {
		if ((lock_st & R_LANE)) {
			HDMI_EMG("R LANE FAIL err_cnt=%d\n", err_r);
		} else {
			HDMI_EMG("R LANE FAIL no lock\n");
			
		}
	}

	if (pass_st & C_LANE) {
		HDMI_EMG("C LANE PASS\n");
		cnt++;
	}
	else{
		if ((lock_st & C_LANE)) {
			HDMI_EMG("C LANE FAIL err_cnt=%d\n", err_c);
		} else {
			HDMI_EMG("C LANE FAIL no lock\n");
		}
	}

	//ltp disable
//	hdmi_mask(HD21_LT_ALL_reg,~HDMI21_P0_HD21_LT_ALL_ltp_en_mask,0x0);

	if (cnt == 0)
		return 0;
	else
		return 1;
	
}

#endif


//QC need
unsigned char newbase_hdmi_is_force_frl_rate(unsigned char port, unsigned char *frl_mode)
{
//	*frl_mode = MODE_FRL_12G_4_LANE;;
//	return 1;

	if (port >= HD20_PORT) 
	{
		*frl_mode = MODE_TMDS;
		return 0;
	}
	else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_10_FORCE_FRL_3G_3_LANE) {
		*frl_mode = MODE_FRL_3G_3_LANE;
		return 1;
	}
	else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_11_FORCE_FRL_6G_3_LANE) {
		*frl_mode = MODE_FRL_6G_3_LANE;
		return 1;
	}
	else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_12_FORCE_FRL_6G_4_LANE) {
		*frl_mode = MODE_FRL_6G_4_LANE;
		return 1;
	} else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_13_FORCE_FRL_8G_4_LANE) {
		*frl_mode = MODE_FRL_8G_4_LANE;
		return 1;
	}else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_14_FORCE_FRL_10G_4_LANE) {
		*frl_mode = MODE_FRL_10G_4_LANE;
		return 1;
	}else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_15_FORCE_FRL_12G_4_LANE) {
		*frl_mode = MODE_FRL_12G_4_LANE;
		return 1;
	}
//#if PCB_HDMI_4K120
	else
	{
#if PCB_HDMI_4K120
		*frl_mode = MODE_FRL_12G_4_LANE;
#else
		*frl_mode = MODE_TMDS;
#endif
		return 1;
	}
//#endif	
	return 0;

}

//QC need
unsigned char newbase_hdmi_is_bypass_ltp(unsigned char port)
{
	if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_16_FORCE_LTP_BYPASS)
		return 1;
	else
		return 0;
}

//QC need
void newbase_hdmi_bypass_link_training(unsigned char nport, unsigned char frl_mode)
{
	unsigned int b;

	HDMI_PRINTF("%s @ nport\n", __func__ ,nport);
		
	if (nport >= HD20_PORT)
		return;
#if 0
	
	newbase_hdmi_set_frl_info(nport,frl_mode);
	//newbase_hdmi21_rxphy_set_frl_info(nport,frl_mode);

	hdmi_rx[nport].frl_mode = frl_mode;
	hdmi_rx[nport].lane_num = (frl_mode <3)?3:4;
	if (frl_mode > 0) {

		b = newbase_rxphy_get_frl_info(nport,hdmi_rx[nport].frl_mode);			
		newbase_hdmi_set_phy_eq(nport,b,hdmi_rx[nport].frl_mode,hdmi_rx[nport].lane_num);
		lib_hdmi_2p1_path_sel(nport,frl_mode,hdmi_rx[nport].lane_num);
	}
#else

	hdmi_rx[nport].frl_mode = frl_mode;
	hdmi_rx[nport].lane_num = (frl_mode <3)?3:4;
	if (frl_mode > 0) {

		b = newbase_rxphy_get_frl_info(nport,hdmi_rx[nport].frl_mode);			
		newbase_hdmi_set_phy_eq(nport,b,hdmi_rx[nport].frl_mode,hdmi_rx[nport].lane_num);
		lib_hdmi_2p1_path_sel(nport,frl_mode,hdmi_rx[nport].lane_num);
	}

        // HDCP reset
 //       newbase_hdmi_hdcp_reset_fsm(port);    //reset hdcp
        // Initail global variable
        lib_hdmi_misc_variable_initial(nport);    //reset global variable
        
	HDMI_PRINTF("%s %d\n", __func__ ,__LINE__);

        // 2.1mac enable
        newbase_hdmi_hd21_mac_enable(nport);

        // lane mode setting
        lib_hdmi_hd21_set_lane_mode_fw(nport,frl_mode);

        // set phy
		#if PCB_HDMI_4K60
        newbase_hdmi_frl_set_phy(nport,frl_mode);
        msleep(40);
		#endif

        //error detect initailize
        lib_hdmi_hd21_fw_disparity_symbol_error_enable(nport);

        lib_hdmi_scdc_ced_enable(nport, 1); //for hdmi2.1 need open more early

        lib_hdmi_hd21_fec_rs_cnt_keep_enable(nport, TRUE); //keep rs cnt
        lib_hdmi_hd21_fec_rs_decode_enable(nport, TRUE);
        lib_hdmi_hd21_fec_scdc_rs_cnt_enable(nport, TRUE);

        lib_hdmi_hd21_fw_disparity_error_reset(nport);
        lib_hdmi_hd21_fw_symbol_error_reset(nport);
        lib_hdmi_scdc_char_err_rst(nport, 0);
#endif
}

//QC need
unsigned int newbase_hdmi_link_training(unsigned char nport)
{
	int scdc_status;
	unsigned int b;
	unsigned char frl_mode;

	lib_hdmi_scdc_data_sel(nport,1); // sel hdmi2p1
		
	switch(hdmi_rx[nport].lts) {

	case LTS_1:
		hdmi_rx[nport].lts = LTS_2;

	case LTS_2:
		if (hdmi_rx[nport].frl_mode == MODE_TMDS) {
			hdmi_rx[nport].lts = LTS_L;
			break;
		}
		scdc_status = lib_hdmi_scdc_get_status_flags(nport);
		scdc_status = scdc_status|FLT_ready_mask;
		lib_hdmi_scdc_set_status_flags(nport,scdc_status);

		mdelay(1);
		scdc_status = lib_hdmi_scdc_get_config_1(nport);
		frl_mode = scdc_status & FRL_Rate_mask;

		if (frl_mode == 0) {  //may be try 2 times
			HDMI_EMG("scdc not get frl mode\n");
			//use transition detect
	#if HDMI_FRL_TRANS_DET		
			lib_hdmi_lane_rate_detect_start(nport);
			lib_hdmi_lane_rate_detect_stop(nport);
			if (lib_hdmi_lane_rate_is_ready(nport)) {
				frl_mode = lib_hdmi_get_lane_rate(nport);

			}
	#endif
		}
		
		if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_10_FORCE_FRL_3G_3_LANE) 
			frl_mode = MODE_FRL_3G_3_LANE;
		else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_11_FORCE_FRL_6G_3_LANE)
			frl_mode = MODE_FRL_6G_3_LANE;
		else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_12_FORCE_FRL_6G_4_LANE)
			frl_mode = MODE_FRL_6G_4_LANE;
		else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_13_FORCE_FRL_8G_4_LANE)
			frl_mode = MODE_FRL_8G_4_LANE;
		else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_14_FORCE_FRL_10G_4_LANE)
			frl_mode = MODE_FRL_10G_4_LANE;
		else if (hdmi_in(DEBUG_REG_ADDR) & DEBUG_15_FORCE_FRL_12G_4_LANE)
			frl_mode = MODE_FRL_12G_4_LANE;
		
		newbase_hdmi_set_frl_info(nport,frl_mode);  //phy frl mode
		//newbase_hdmi21_rxphy_set_frl_info(nport,frl_mode);
		hdmi_rx[nport].frl_mode = frl_mode;
		hdmi_rx[nport].lane_num = (frl_mode <3)?3:4;

		if (frl_mode > 0) {
			lib_hdmi_2p1_path_sel(nport,frl_mode,hdmi_rx[nport].lane_num);
			b = newbase_rxphy_get_frl_info(nport,hdmi_rx[nport].frl_mode);			
			newbase_hdmi_set_phy_eq(nport,b,hdmi_rx[nport].frl_mode,hdmi_rx[nport].lane_num);
			hdmi_rx[nport].lts = LTS_3;
		}
		else {
			hdmi_rx[nport].lts = LTS_L;
		}
		
		break;
	case LTS_3:
		if (hdmi_rx[nport].frl_mode == MODE_TMDS) {
			hdmi_rx[nport].lts = LTS_L;
			scdc_status = lib_hdmi_scdc_get_status_flags(nport);
			scdc_status = scdc_status & ~FLT_ready_mask;
			lib_hdmi_scdc_set_status_flags(nport,scdc_status);
			
			break;
		}
/*		
		scdc_status= lib_hdmi_scdc_get_update_flags(port);
		scdc_status = scdc_status & ~FRL_start_mask;
		lib_hdmi_scdc_set_update_flags(port,scdc_status);
*/
		if (hdmi_link_communicate(nport,hdmi_rx[nport].lane_num))
			hdmi_rx[nport].lts = LTS_P;
		else
			hdmi_rx[nport].lts = LTS_L;
		
		break;
	case LTS_4:	//for source
		
		break;
	case LTS_P:
		scdc_status= lib_hdmi_scdc_get_update_flags(nport);
		if (!(scdc_status & FRL_start_mask)) {
			scdc_status = scdc_status | FRL_start_mask;
			lib_hdmi_scdc_set_update_flags(nport,scdc_status);
				//frl_start setting after set phy
		}
		if (hdmi_rx[nport].lts == LTS_P)
			return 1;
		//add FRL rate change and detect bit error to recovery
		break;
	case LTS_L:
		scdc_status = lib_hdmi_scdc_get_status_flags(nport);
		scdc_status = scdc_status|FLT_ready_mask;
		lib_hdmi_scdc_set_status_flags(nport,scdc_status);

		udelay(20); //wait tx set frl rate
		scdc_status = lib_hdmi_scdc_get_config_1(nport);
		scdc_status = scdc_status & FRL_Rate_mask;

		if (scdc_status > 0) {
			newbase_hdmi_set_frl_info(nport,scdc_status);
			//newbase_hdmi21_rxphy_set_frl_info(nport,frl_mode);
			hdmi_rx[nport].frl_mode = scdc_status;
			hdmi_rx[nport].lane_num = (scdc_status <3)?3:4;
			hdmi_rx[nport].lts = LTS_3;
		}

		if (hdmi_rx[nport].lts == LTS_L)
			return 1;
		
		break;

	}

	return 0;
	
	
}



#endif

//QC need
unsigned char lib_scdc_get_frl_mode(unsigned char nport)
{
	unsigned char scdc_status;
	unsigned char i = 0;
	unsigned char frl_mode;
	unsigned char tx_write_config_31 = 0;

	if (nport >= HD20_PORT)
		return 0;
	
	scdc_status = lib_hdmi_scdc_get_status_flags(nport);
	scdc_status = scdc_status|FLT_ready_mask;
	lib_hdmi_scdc_set_status_flags(nport,scdc_status);

	//	lib_hdmi_scdc_data_sel(nport,1); // sel hdmi2p1

       for(i = 0 ; i < 200 ; ++i) {

		if (lib_hdmi_scdc_is_sink_config_31_updated(nport)) {
			tx_write_config_31 = 1;
			break;
		}
		mdelay(10);
	}

	if (tx_write_config_31) {
		scdc_status = lib_hdmi_scdc_get_config_1(nport);
		 frl_mode = scdc_status & FRL_Rate_mask;
		 lib_hdmi_scdc_clr_sink_config_31_update_status(nport);
		 HDMI_EMG("frl mode = %d cnt=%d\n",frl_mode, i);  //cnt = 34~124
	} else {
//		HDMI_EMG("timeout frl mode not get\n");
		frl_mode = 0;
	}

	return frl_mode;
         
}

void reset_hdmi_timing_ready(void)
{//auto run scaler to use. reset timing ready
	hdmi_good_timing_ready = FALSE;
}

